/*
 * (C) Copyright 2001
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2001-2004
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2003
 * Texas Instruments, <www.ti.com>
 * Kshitij Gupta <Kshitij@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#if 0
#include <common.h>
#include <linux/byteorder/swab.h>
#endif
#include "flash.h"
#include "defs.h"

flash_info_t flash_info[CFG_MAX_FLASH_BANKS];	/* info for FLASH chips    */

#define mb() __asm__ __volatile__ ("" : : : "memory")


ulong (*flash_get_size_ptr) (FPW * addr, flash_info_t * info);
int (*flash_unprotect_sectors_ptr) (FPWV * addr);
int (*flash_erase_ptr) (flash_info_t * info, int s_first, int s_last);
int (*write_buff_ptr) (flash_info_t * info, uchar * src, ulong addr, ulong cnt);
int (*write_data_ptr) (flash_info_t * info, ulong dest, FPW data);
int (*buffered_write_data_ptr) (flash_info_t * info, ulong dest, FPW *data);
void (*reset_timer_masked_ptr) (void);
ulong (*get_timer_masked_ptr) (void);
void (*spin_wheel_ptr) (void);

ulong monitor_flash_len;

unsigned int instruction_area [2048];

/* Flash Organization Structure */
typedef struct OrgDef {
	unsigned int sector_number;
	unsigned int sector_size;
} OrgDef;


/* Flash Organizations */
OrgDef OrgIntel_28F128J3[] = {
	{128, 128 * 1024},		/* 128 * 128kBytes sectors */
};


/*-----------------------------------------------------------------------
 * Functions
 */
unsigned long flash_init (void);
static ulong flash_get_size (FPW * addr, flash_info_t * info);
static int write_data (flash_info_t * info, ulong dest, FPW data);
static void flash_get_offsets (ulong base, flash_info_t * info);
void inline spin_wheel (void);
void flash_print_info (flash_info_t * info);
int flash_unprotect_sectors (FPWV * addr);
int flash_erase (flash_info_t * info, int s_first, int s_last);
int write_buff (flash_info_t * info, uchar * src, ulong addr, ulong cnt);
static int buffered_write_data (flash_info_t * info, ulong dest, FPW *data);

/*-----------------------------------------------------------------------
 */

static void flash_vpp(int on)
{
}

unsigned long flash_init (void)
{
	int i;
	ulong size = 0;
	for (i = 0; i < CFG_MAX_FLASH_BANKS; i++) {
		switch (i) {
		case 0:
		    flash_vpp(1);
			(flash_get_size_ptr) ((FPW *) PHYS_FLASH_1, &flash_info[i]);
			flash_get_offsets (PHYS_FLASH_1, &flash_info[i]);
			flash_vpp(0);
			break;
		default:
			putString ("configured too many flash banks!\n\r");
			break;
		}
		size += flash_info[i].size;
	}
#if 0
	/* Protect monitor and environment sectors
	 */
	flash_protect (FLAG_PROTECT_SET,
			CFG_FLASH_BASE,
			CFG_FLASH_BASE + monitor_flash_len - 1, &flash_info[0]);
#endif

	return size;
}

/*-----------------------------------------------------------------------
 */
static void flash_get_offsets (ulong base, flash_info_t * info)
{
	int i;
	OrgDef *pOrgDef;

	pOrgDef = OrgIntel_28F128J3;
	if (info->flash_id == FLASH_UNKNOWN) {
		return;
	}

	if ((info->flash_id & 0xFFFF) == 0x8918) {
		for (i = 0; i < info->sector_count; i++) {
			if (i > 255) {
				info->start[i] = base + (i * 0x8000);
				info->protect[i] = 0;
			} else {
				info->start[i] = base + (i * PHYS_FLASH_SECT_SIZE);
				info->protect[i] = 0;
			}
		}
	}
}

/*-----------------------------------------------------------------------
 */
void flash_print_info (flash_info_t * info)
{
	int i;

	if (info->flash_id == FLASH_UNKNOWN) {
		putString ("missing or unknown FLASH type\n\r");
		return;
	}

	switch (info->flash_id & 0xFFFF) {
	case 0x8918:
		putString ("INTEL StrataFlash 28F128J3\n\r");
		break;
	default:
		putString ("Unknown Chip Type (ID=0x");
		print_hex (info->flash_id);
		putString (")\n\r");
		break;
	}

	putString ("  Size: ");
	print_decimal (info->size);
	putString (" MB in ");
	print_decimal (info->sector_count);
	putString (" Sectors\n\r");

	putString ("  Sector Start Addresses:");
	for (i = 0; i < info->sector_count; ++i) {
		if ((i % 5) == 0)
			putString ("\n\r   ");
		print_hex (info->start [i]);
		if (info->protect[i])
			putString (" (RO)");
		else
			putString ("     ");
	}
	putString ("\n\r");
	return;
}

/*
 * The following code cannot be run from FLASH!
 */
static ulong flash_get_size (FPW * addr, flash_info_t * info)
{
	volatile FPW value;

	/* Write auto select command: read Manufacturer ID */
	addr[0x0] = (FPW) 0x00AA;
	addr[0x0] = (FPW) 0x0055;
	addr[0x0] = (FPW) 0x0090;

	mb ();
	value = addr[0];
#if 0
	putString ("Manufacture ID: ");
        print_hex4 (value);
	putString ("\n\r");
#endif
	switch (value) {
	case (FPW) 0x0089:
		mb ();
		value = addr[1];	/* device ID        */
#if 0
	putString ("Device ID: ");
        print_hex4 (value);
	putString ("\n\r");
#endif
		switch (value) {
		case (FPW) (0x0018):
			info->flash_id = 0x8918;
			info->sector_count = 128;
			info->size = 0x01000000;
			break;			/* => 16 MB     */
		default:
			info->flash_id = FLASH_UNKNOWN;
			break;
		}
		break;

	default:
		info->flash_id = FLASH_UNKNOWN;
		info->sector_count = 0;
		info->size = 0;
		addr[0] = (FPW) 0x00FF;	/* restore read mode */
		return (0);		/* no or unknown flash  */
	}


	if (info->sector_count > CFG_MAX_FLASH_SECT) {
		putString ("** ERROR: sector count ");
		print_decimal (info->sector_count);
		putString (" > max (");
		print_decimal (CFG_MAX_FLASH_SECT);
		putString (") **\n\r");
		info->sector_count = CFG_MAX_FLASH_SECT;
	}

	addr[0] = (FPW) 0x00FF;	/* restore read mode */

	return (info->size);
}


/* unprotects a sector for write and erase
 * on some intel parts, this unprotects the entire chip, but it
 * wont hurt to call this additional times per sector...
 */
int flash_unprotect_sectors (FPWV * addr)
{
	FPW status;

	/* arm simple, non interrupt dependent timer */
	(reset_timer_masked_ptr) ();

	*addr = (FPW) 0x0050;	/* clear status register */
	*addr = (FPW) 0x0060; 	/* this sends the clear lock bit command */
	*addr = (FPW) 0x00D0;

	while (((status = *addr) & (FPW) 0x0080) != (FPW) 0x0080) 
	{
			if ((get_timer_masked_ptr) () > CFG_FLASH_ERASE_TOUT) 
			{
#if 0
				putString ("Timeout\n\r");
#endif
				/* reset to read mode */
				*addr = (FPW) 0x00FF;
				return 0x0008;
			}
	}
        return 0;
}


/*-----------------------------------------------------------------------
 */

int flash_erase (flash_info_t * info, int s_first, int s_last)
{
	int prot, sect;
	ulong type;
	int rcode = 0;

	if ((s_first < 0) || (s_first > s_last)) {
		if (info->flash_id == FLASH_UNKNOWN) {
			putString ("- missing\n\r");
		} else {
			putString ("- no sectors to erase\n\r");
		}
		return 0x0001;
	}

	type = (info->flash_id & 0xFFFF);
	if ((type != 0x8918)) {
		putString ("Can't erase unknown flash type ");
		print_hex (info->flash_id);
		putString (" - aborted\n\r");
		return 0x0002;
	}

	prot = 0;
	for (sect = s_first; sect <= s_last; ++sect) {
		if (info->protect[sect]) {
			prot++;
		}
	}

	if (prot) {
		putString ("- Warning: \n\r");
		print_decimal (prot);
		putString (" protected sectors will not be erased!\n\r");
	}

	flash_vpp(1);

	/* Start erase on unprotected sectors */
	for (sect = s_first; sect <= s_last; sect++) {
		if (info->protect[sect] == 0) {	/* not protected */
			FPWV *addr = (FPWV *) (info->start[sect]);
			FPW status;

#if 0
			putString ("Erasing sector ");
			print_decimal (sect);
			putString (" ... ");
#endif

			rcode = (flash_unprotect_sectors_ptr) (addr);

			if (rcode)
				return rcode;

			/* arm simple, non interrupt dependent timer */
			(reset_timer_masked_ptr) ();

			*addr = (FPW) 0x0050;/* clear status register */
			*addr = (FPW) 0x0020;/* erase setup */
			*addr = (FPW) 0x00D0;/* erase confirm */

			while (((status = *addr) & (FPW) 0x0080) != (FPW) 0x0080) 
			{
					if ((get_timer_masked_ptr) () > CFG_FLASH_ERASE_TOUT) 
					{
						/* suspend erase     */
						*addr = (FPW) 0x00B0;
						/* reset to read mode */
						*addr = (FPW) 0x00FF;
#if 0
						putString ("Timeout\n\r");
#endif
						rcode = 0x0010;
						break;
					}
			}
			/* clear status register cmd.   */
			*addr = (FPW) 0x0050;
			*addr = (FPW) 0x00FF;/* resest to read mode */

			if (status & 0x7F)
			{
#if 0
				putString("\n\rError! Status = 0x");
				print_hex (status);
				putString("\n\r");
#endif
				rcode = 0x0020;
			}
			else
			{
#if 0
				putString (" done\n\r");
#endif
			}
		}
	}

	flash_vpp(0);

	return rcode;
}

/*-----------------------------------------------------------------------
 * Copy memory to flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 * 4 - Flash not identified
 */
int write_buff (flash_info_t * info, uchar * src, ulong addr, ulong cnt)
{
	ulong cp, wp;
	FPW data;
	int i, j, l, rc, port_width;

	if (info->flash_id == FLASH_UNKNOWN) {
		return 0x0001;
	}
/* get lower word aligned address */
	wp = (addr & ~1);
	port_width = 2;

	flash_vpp(1);

	/*
	 * handle unaligned start bytes
	 */
	if (addr & 0x01) 
	{
/* will not happen, do not use it */
		data = 0;
		for (i = 0, cp = wp; i < l; ++i, ++cp) {
			data = (data << 8) | (*(uchar *) cp);
		}
		for (; i < port_width && cnt > 0; ++i) {
			data = (data << 8) | *src++;
			--cnt;
			++cp;
		}
		for (; cnt == 0 && i < port_width; ++i, ++cp) {
			data = (data << 8) | (*(uchar *) cp);
		}

		if ((rc = (write_data_ptr) (info, wp, SWAP (data))) != 0) {
			flash_vpp(0);
			return (rc);
		}
		wp += port_width;
	}

	/*
	 * handle word aligned part
	 */
	while (cnt)
	{
		FPW buff[16];					// 32bytes
		for (j=0;j<16;j++)
		{
			buff[j] = *src++;
			buff[j] = (buff[j] << 8) | *src++;
		}

		if ((rc = (buffered_write_data_ptr) (info, wp, buff)) != 0) {
			flash_vpp(0);
			return (rc);
		}
		wp += 32;
		cnt -= 32;
#if 0
		spin_wheel ();
#endif
	}

	if (cnt == 0) {
		flash_vpp(0);
		return (0);
	}

	/*
	 * handle unaligned tail bytes
	 */
	data = 0;
	for (i = 0, cp = wp; i < port_width && cnt > 0; ++i, ++cp) {
		data = (data << 8) | *src++;
		--cnt;
	}
	for (; i < port_width; ++i, ++cp) {
		data = (data << 8) | (*(uchar *) cp);
	}

	rc = (write_data_ptr) (info, wp, SWAP (data));

	flash_vpp(0);

	return rc;
}

/*-----------------------------------------------------------------------
 * Write a word or halfword to Flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 */
static int write_data (flash_info_t * info, ulong dest, FPW data)
{
	FPWV *addr = (FPWV *) dest;
	ulong status;
	int rcode;

	/* Check if Flash is (sufficiently) erased */
	if ((*addr & data) != data) {
		putString ("not erased at ");
		print_hex ((uint) addr);
		putString (" (0x");
		print_hex (*addr);
		putString (")\n\r");
		return 0x0002;
	}

	flash_vpp(1);

	rcode = (flash_unprotect_sectors_ptr) (addr);
	if (rcode)
	  	return rcode;
	*addr = (FPW) 0x0040;	/* write setup */
	*addr = data;

	/* arm simple, non interrupt dependent timer */
	(reset_timer_masked_ptr) ();

	/* wait while polling the status register */
	while (((status = *addr) & (FPW) 0x0080) != (FPW) 0x0080) {
		if ((get_timer_masked_ptr) () > CFG_FLASH_WRITE_TOUT) {
			*addr = (FPW) 0x00FF;	/* restore read mode */
			flash_vpp(0);
			return 0x0010;
		}
	}
	*addr = (FPW) 0x00FF;	/* restore read mode */
	flash_vpp(0);
	return (0);
}

/*-----------------------------------------------------------------------
 * Write a buffer (32bytes) to Flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 */
static int buffered_write_data (flash_info_t * info, ulong dest, FPW *data)
{
	FPWV *addr = (FPWV *) dest;
	ulong status, i;

	if (*addr != 0xFFFF) 
	{
		putString ("not erased at ");
		print_hex ((ulong) addr);
		putString (" (0x");
		print_hex (*addr);
		putString (")\n\r");
		return 0x0100;
	}

	flash_vpp(1);

	*addr = (FPW) 0x00E8;	/* write to buffer */
	/* wait while polling the status register */
	while (((status = *addr) & (FPW) 0x00FF) != (FPW) 0x0080) {
		if ((get_timer_masked_ptr) () > CFG_FLASH_WRITE_TOUT) {
			*addr = (FPW) 0x00FF;	/* restore read mode */
			flash_vpp(0);
			return 0x0200;
		}
	}

	// set word count (word count - 1 = 16word - 1 = 15)
	*addr = 0x0F;

	// write data (32bytes)
	for (i=0;i<16;i++) 
	{
		addr[i] = SWAP(data[i]);
	}

	*addr = 0xD0;

	/* arm simple, non interrupt dependent timer */
	(reset_timer_masked_ptr) ();

	/* wait while polling the status register */
	while (((status = *addr) & (FPW) 0x00FF) != (FPW) 0x0080) {
		if ((get_timer_masked_ptr) () > CFG_FLASH_WRITE_TOUT) {
			*addr = (FPW) 0x00FF;	/* restore read mode */
			flash_vpp(0);
			return 0x0400;
		}
	}

	*addr = (FPW) 0x00FF;	/* restore read mode */
	flash_vpp(0);
	return (0);
}

/* 25 MHz */
void reset_timer_masked (void)
{
	timer_mask_value = (*FREE_RUN);
}

ulong get_timer_masked (void)
{
	return ((ulong) ((*FREE_RUN) - timer_mask_value));
}

void inline spin_wheel (void)
{
	static int p = 0;
	static char w[] = "\\/-";

	putDebugChar (8);
	putDebugChar (w[p]);
	(++p == 3) ? (p = 0) : 0;
}

void init_flash_environment (void)
{
  ulong size;

  if (((ulong) init_flash_environment) - ((ulong) flash_get_size) >
	sizeof (instruction_area)) 
  {
    putString ("instruction_area [] is to small\n\r");
    return;
  }
  memcpy (instruction_area, flash_get_size, ((ulong) init_flash_environment) - ((ulong) flash_get_size));
  flash_get_size_ptr =
    (ulong (*) (FPW *, flash_info_t *)) instruction_area;
  flash_unprotect_sectors_ptr =
    (int (*) (FPWV *)) ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) flash_unprotect_sectors);
  flash_erase_ptr =
    (int (*) (flash_info_t *, int, int)) ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) flash_erase);
  write_buff_ptr =
    (int (*) (flash_info_t *, uchar *, ulong, ulong))
    ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) write_buff);
  write_data_ptr =
    (int (*) (flash_info_t *, ulong, FPW))
    ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) write_data);
  buffered_write_data_ptr =
    (int (*) (flash_info_t *, ulong, FPW *))
    ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) buffered_write_data);
  reset_timer_masked_ptr =
    (void (*) (void)) instruction_area -
    (ulong) flash_get_size + (ulong) reset_timer_masked;
  get_timer_masked_ptr =
    (ulong (*) (void)) instruction_area -
    (ulong) flash_get_size + (ulong) get_timer_masked;
  spin_wheel_ptr =
    (void (*) (void)) ((ulong) instruction_area -
    (ulong) flash_get_size + (ulong) spin_wheel);

  monitor_flash_len = CFG_MONITOR_LEN;
  size = flash_init ();
  putString ("flash size is 0x");
  print_hex (size);
  putString (" bytes\n\r");
}

