/* $Id: //depot/swdev/LAN/Aldebaran/mainline/bootldr/main.c#1 $
 *
 * sh-ipl+g/main.c
 *
 *  Copyright (C) 2000  Niibe Yutaka
 *
 * This file is subject to the terms and conditions of the GNU Lesser
 * General Public License.  See the file "COPYING.LIB" in the main
 * directory of this archive for more details.
 *
 */

#include "config.h"
#include "defs.h"
#include "string.h"
#include "io.h"
#include "net.h"
#include "flash.h"
#include "polaris.h"

#define VERSION_STRING		"SMSC modification v1.01"

#define PECR   (volatile unsigned short *)0xA4000108
#define PEDR   (volatile unsigned char *)0xA4000128

#define PCCR   (volatile unsigned short *)0xA4000104
#define PCDR   (volatile unsigned char *)0xA4000124

#define	CCR			(volatile unsigned long *)0xFFFFFFEC
#define	CCR_INIT 	(0x05)				// Write-Back

#define	BIT0		(1<<0)
#define	BIT1		(1<<1)
#define	BIT2		(1<<2)
#define	BIT3		(1<<3)
#define	BIT4		(1<<4)
#define	BIT5		(1<<5)
#define	BIT6		(1<<6)
#define	BIT7		(1<<7)
static int shutdown (unsigned int);
static int rtc (unsigned int, unsigned int);

/*
  Runs at P2 Area

  VBR = 0xa0000000

  ROM:
   0xa0000000 ---> [ .text  ]

  RAM:
  		   [ .data  ]  <-------   RAM = 0xa8000000
		   [ .bss   ]
   init_stack ---> [ .stack ]  RAM+0x0a00
		      :
		   [        ]
		   [        ]  RAM+0x0eff <--- init_sp (= initial R15)
   stub_stack ---> [ .stack ]  RAM+0x0F00
		      :
		   [        ]
		   [        ]  RAM+0x1000 <--- stub_sp (= stub R15) 
 */

#define stub_stack_size	64
#define init_stack_size	320
static int init_stack[init_stack_size]
	__attribute__ ((section (".stack"),unused));
int stub_stack[stub_stack_size]
	__attribute__ ((section (".stack")));

int *stub_sp;

static const char * const banner = "\n\
SH IPL+g version 0.9, Copyright (C) 2000 Free Software Foundation, Inc.\n\
\n\
This software comes with ABSOLUTELY NO WARRANTY; for details type `w'.\n\
This is free software, and you are welcome to redistribute it under\n\
certain conditions; type `l' for details.\n\n";

static const char * const license_message = "\n\
SH IPL+g is free software; you can redistribute it and/or modify it under\n\
the terms of the GNU Lesser General Public License as published by \n\
the Free Software Foundatin; either version 2.1 of the License, or (at your\
option) any later version.\n";

static const char * const warranty_message = "\n\
SH IPL+g is distributed in the hope that it will be useful, but\n\
WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n\
Lesser General Public License for more details.\n";

static const char * const help_message = "\n\
SH IPL+g version 0.9, Copyright (C) 2000 Free Software Foundation, Inc.\n\
    help         --- Show this message (HELP)\n\
    boot         --- Boot the system\n\
    g            --- Invoke GDB stub\n\
    license      --- Show about license\n\
    warranty     --- Show about (no)warranty\n\
    ouripaddr    --- enter our ip address\n\
    serveripaddr --- enter server ip address\n\
    subnet_mask  --- enter subnet mask\n\
    info         --- display ip addresses and subnet mask\n\
    tftp_kernel  --- start tftp process to get kernel file and burn\n\
    tftp_fs      --- start tftp process to get file system and burn\n";

static char *prompt;

static inline unsigned long
get_boot_signature (void)
{
  unsigned long r5, r6, r7;

  asm ("stc	r5_bank, %0":"=r" (r5));
  asm ("stc	r6_bank, %0":"=r" (r6));
  asm ("stc	r7_bank, %0":"=r" (r7));

  if ((r5 == r6) && (r6 == r7))
    return r7;
  else
    return 0;
}

static inline void
set_boot_signature (unsigned long sig)
{
  asm volatile ("ldc	%0, r5_bank": : "r" (sig));
  asm volatile ("ldc	%0, r6_bank": : "r" (sig));
  asm volatile ("ldc	%0, r7_bank": : "r" (sig));
}

char token [4][64];
int num_args;
extern char BootFile [128];
IPaddr_t env_NetServerIP, env_NetOurIP, env_NetOurSubnetMask;

void parse_string (char *cmd)
{
  char *ptr, *ptr2;
  int i;
  num_args = 0;

  ptr = cmd;
  while (1) {
    while (*ptr != '\0'  && *ptr != '\n' && *ptr != '\r' && *ptr == ' ')		/* skip space */
      ptr++;
    if (*ptr == '\0' || *ptr == '\n' || *ptr == '\r')
      break;
    
    ptr2 = ptr;
    i = 0;
    while (*ptr != '\0'  && *ptr != '\n'&& *ptr != '\r' && *ptr != ' ') {	/* copy token */
      if (i < 63) {
        token [num_args][i++] = *ptr;      
        token [num_args][i] = '\0';      
      }
      ptr++;
    }
    num_args++;
    if (num_args >= 4)
      break;
  }
}

unsigned short cksum (unsigned short *ptr, int len)
{
  ulong xsum;
  int i;
  
  xsum = 0;
  for (i = 0; i < len; i++) {
    xsum += *ptr++;
    xsum = (xsum & 0xffff) + (xsum >> 16);
  }
  return ((unsigned short) xsum);
}

void
start_main (void)
{
  extern void start ();		/* In entry.S */
  void (*startlinux)();
  char cmd [128];
  volatile unsigned short pecr;
  volatile unsigned char pedr;
  volatile unsigned char SW2;
  ulong len, flash_addr, flash_addr_end, remaining, sofar;
  ulong old_percent, new_percent, ram_source_addr, flash_dest_addr;
  unsigned int i1;
  int i, begin_sector, end_sector, must_reset;
  unsigned long mread_addr;
  char *e;

  len = flash_addr = flash_addr_end = 0;
  in_nmi = 0;
  dofault = 1;
  stepped = 0;
  prompt = "> ";

  stub_sp = stub_stack + stub_stack_size;
  putString (banner);

#ifdef	CONFIG_POLARIS_2
  putString("Modified by SMSC for PolarisII 7709S Demo Board ");
#else
  putString("Modified by SMSC for Polaris 7709S Demo Board ");
#endif
  putString(" Built at ");
  putString(__DATE__);
  putString(" ");
  putString(__TIME__);
  putString("\n");

  putString(VERSION_STRING);
  putString(", Copyright (c) 2004 - 2005, SMSC\n");

  // Setup PE1 to Output HIGH to deactivate nFLASH_LOCK
  pecr = *PECR;
  pecr &= ~0x000C;		// clear bit3,2 (PE1)
  pecr |= 0x0004;			// set bit 3,2(PE1) to output (01xx)
  p4_out(PECR, pecr);
  pedr = *PEDR;
  pedr |= 0x02;			// set bit1(PE1) to High(One)
  p4_out(PEDR, pedr);

  // Set Cache Mode
  *CCR = CCR_INIT;

#ifdef	CONFIG_POLARIS_2
  // Turn off LEDs
  p4_out(PCDR, 0x00);
  // read SW1-4
  SW2 = (p4_in(PCDR) >> 4) & 0x0F;
  if (SW2 & BIT0)
  {
	  // switch 4 at SW2 is ON
	  putString("copy linux kernel from Flash to SDRAM\n");
	  memcpy((void *)0xAC210000, (void *)0xA0100000, 0x100000);
	  putString("Jump to 0x8C210000\n");
	  startlinux = (void *)0x8C210000;
	  (*startlinux)();
	  putString("Oops! Should not reach here.\n");
	  while (1);
  }
#endif

  init_net ();
  init_util ();
  init_malloc ();
  init_flash_environment ();

  env_NetOurSubnetMask = string_to_ip ("255.255.255.0");
  env_NetServerIP = string_to_ip ("192.168.1.100");
  env_NetOurVLAN = htons(-1);
  env_NetOurNativeVLAN = htons(-1);
  env_NetOurIP = bd.bi_ip_addr = string_to_ip ("192.168.1.118");

  must_reset = 0;
  while (1)
    {
      putString (prompt);
      getDebugString (cmd, 127);
      parse_string (cmd);
      switch (num_args) {
        case 0:
	  break;
	default:
	  /* Help */
	  putString ("invalid command\n");
	  putString (help_message);
	  break;
        case 1:
	  if (!strcmp (token [0], "help")) {
	    /* Help */
	    putString (help_message);
          }
	  else if (!strcmp (token [0], "boot")) {
            if (must_reset) {
              putString ("please reset the polaris board before enter boot command\n\r");
            }
            else {
	      /* Boot */
	      putString("copy linux kernel from Flash to SDRAM\n");
	      memcpy((void *)0xAC210000, (void *)0xA0100000, 0x100000);
	      putString("Jump to 0x8C210000\n");
	      startlinux = (void *)0x8C210000;
	      (*startlinux)();
            }
	  }
#if 0
	  else if (!strcmp (token [0], "jump")) {
	    /* Jump */
	    putString("Jump to 0x8C210000 (software reset)\n");
	    startlinux = (void *)0x8C210000;
	    (*startlinux)();
  	  }
#endif
	  else if ((!strcmp (token [0], "license")) || (!strcmp (token [0], "l"))) {
	    /* License */
	    putString (license_message);
	  }
	  else if ((!strcmp (token [0], "g")) || (!strcmp (token [0], "#"))) {
	    /* GDB Stub */
	    sti ();
	    breakpoint ();
	  }
	  else if ((!strcmp (token [0], "warranty")) || (!strcmp (token [0], "w"))) {
	    /* Warranty */
	    putString (warranty_message);
	  }
	  else if (!strcmp (token [0], "info")) {
	    ip_to_string (env_NetOurIP, token [1]);
            putString ("\rOur IP address:    ");
            putString (token [1]);
            putString ("\n\r");
	    ip_to_string (env_NetServerIP, token [1]);
            putString ("Server IP address: ");
            putString (token [1]);
            putString ("\n\r");
	    ip_to_string (env_NetOurSubnetMask, token [1]);
            putString ("Subnet mask: ");
            putString (token [1]);
            putString ("\n\r");
	  }
	  else if (!strcmp (token [0], "tftp_kernel")) {
	    strcpy (BootFile, "zImage");
	    load_addr = 0xAD000000;
	    NetBootFileXferSize = 0;

	    len = NetLoop (TFTP);
 	    if (len == 0)
              putString ("error!\n\r");
            else {
	      putString ("xfer completes with ");
              print_decimal (len);
	      putString (" bytes\n\r");
            }
	    flash_addr = 0xA0100000;
	    flash_addr_end = 0xA0600000;
	    putString ("zImage checksum from RAM = ");
            print_hex4 (cksum ((unsigned short *) 0xAD000000, len >> 1));
            putString ("\n\r");
            if (flash_addr + len > flash_addr_end) {
              putString ("kernel file 'zImage' is too big\n");
            }
            else {
	      putString ("erase from addr: 0x");
	      print_hex (flash_addr);
	      putString (" to 0x");
	      print_hex (flash_addr + len - 1);
	      putString ("\n\r");
	      putString ("erase from sectors: ");
	      print_decimal ((flash_addr & 0xfffffff) >> 17);
              putString (" to ");
	      print_decimal ((((flash_addr + len + PHYS_FLASH_SECT_SIZE - 1) & 0xfffffff) >> 17) - 1);
              putString ("\n\r");

              begin_sector = ((flash_addr & 0xfffffff) >> 17);
              end_sector = (((flash_addr + len + PHYS_FLASH_SECT_SIZE - 1) & 0xfffffff) >> 17) - 1;
              while (begin_sector <= end_sector) {
                putString ("\n\rerase sector: ");
	        print_decimal (begin_sector);
                putString (" ... ");

                i1 = (flash_erase_ptr) (&flash_info [0], begin_sector, begin_sector);
	        if (i1) {
                  putString ("\n\rerase failed, err_code = 0x");
	          print_hex ((ulong) i1);
		  break;
                }
                else
                  putString ("done");
	        begin_sector++;
              }
              putString ("\n\r");

              putString ("write flash: source = 0xAD000000, dest = 0x");
              print_hex (flash_addr);
              putString (", len = ");
              print_decimal (len);
	      putString ("\n\r");
	      i1 = (write_buff_ptr) (&flash_info [0], (uchar *) 0xAD000000, flash_addr, ((len + 31) & 0xffffffe0));
	      if (i1) {
                putString ("write flash failed, err_code = 0x");
	        print_hex ((ulong) i1);
                putString ("\n\r");
              }
              else
	        putString ("write flash done\n\r");

	      putString ("checksum from flash = ");
              print_hex4 (cksum ((unsigned short *) flash_addr, len >> 1));
              putString ("\n\r");
              putString ("please reset the polaris board before enter boot command\n\r");
            }
            must_reset = 1;
	  }
	  else if (!strcmp (token [0], "tftp_fs")) {
	    strcpy (BootFile, "polaris.jffs2");
	    load_addr = 0xAD000000;
	    NetBootFileXferSize = 0;

	    len = NetLoop (TFTP);
 	    if (len == 0)
              putString ("error!\n\r");
            else {
	      putString ("xfer completes with ");
              print_decimal (len);
	      putString (" bytes\n\r");
            }
	    flash_addr = 0xA0600000;
	    flash_addr_end = 0xA1000000;
	    putString ("polaris.jffs2 checksum from RAM = ");
            print_hex4 (cksum ((unsigned short *) 0xAD000000, len >> 1));
            putString ("\n\r");
            if (flash_addr + len > flash_addr_end) {
              putString ("file system 'polaris.jffs2' is too big\n");
            }
            else {
	      putString ("erase from addr: 0x");
	      print_hex (flash_addr);
	      putString (" to 0x");
	      print_hex (flash_addr + len - 1);
	      putString ("\n\r");
	      putString ("erase from sectors: ");
	      print_decimal ((flash_addr & 0xfffffff) >> 17);
              putString (" to ");
	      print_decimal ((((flash_addr + len + PHYS_FLASH_SECT_SIZE - 1) & 0xfffffff) >> 17) - 1);
              putString ("\n\r");

              begin_sector = ((flash_addr & 0xfffffff) >> 17);
              end_sector = (((flash_addr + len + PHYS_FLASH_SECT_SIZE - 1) & 0xfffffff) >> 17) - 1;
              while (begin_sector <= end_sector) {
                putString ("erase sector: ");
	        print_decimal (begin_sector);
                putString (" ...");

                i1 = (flash_erase_ptr) (&flash_info [0], begin_sector, begin_sector);
	        if (i1) {
                  putString (" erase failed, err_code = 0x");
	          print_hex ((ulong) i1);
                  putString ("\n\r");
		  break;
                }
                else
                  putString (" done\n\r");
	        begin_sector++;
              }

              putString ("write flash: source = 0xAD000000, dest = 0x");
              print_hex (flash_addr);
              putString (", len = ");
              print_decimal (len);
	      putString ("\n\r");

              ram_source_addr = 0xAD000000;
              flash_dest_addr = flash_addr;
              remaining = len;
              sofar = 0;
              old_percent = 0;
              new_percent = 0;
              while (remaining) {
                if (remaining < 128 * 1024) {
	          i1 = (write_buff_ptr) (&flash_info [0], (uchar *) ram_source_addr, flash_dest_addr, ((remaining + 31) & 0xffffffe0));
	          putString ("write flash done\n\r");
                  break;
                }
                else
	          i1 = (write_buff_ptr) (&flash_info [0], (uchar *) ram_source_addr, flash_dest_addr, 128 * 1024);
	        if (i1) {
                  putString ("write flash failed, err_code = 0x");
	          print_hex ((ulong) i1);
                  putString ("\n\r");
                  break;
                }
                ram_source_addr += 128 * 1024;
                flash_dest_addr += 128 * 1024;
                remaining -= 128 * 1024;
                sofar += 128 * 1024;
                new_percent = do_divide (sofar * 100, len);
                if (new_percent != old_percent) {
                  putString ("percentage complete = ");
                  print_decimal (new_percent);
                  putString ("%\n\r");
                  old_percent = new_percent;
                }
              }

	      putString ("checksum from flash = ");
              print_hex4 (cksum ((unsigned short *) flash_addr, len >> 1));
              putString ("\n\r");
              putString ("please reset the polaris board before enter boot command\n\r");
            }
            must_reset = 1;
	  }
#if 0
	  else if (!strcmp (token [0], "burn")) {
	    putString ("erase from addr: 0x");
	    print_hex (flash_addr);
	    putString (" to 0x");
	    print_hex (flash_addr_end-1);
	    putString ("\n\r");
	    putString ("erase from sectors: ");
	    print_decimal ((flash_addr & 0xfffffff) >> 17);
            putString (" to ");
	    print_decimal (((flash_addr_end & 0xfffffff) >> 17) - 1);
            putString ("\n\r");
           
            begin_sector = ((flash_addr & 0xfffffff) >> 17);
            end_sector = ((flash_addr_end & 0xfffffff) >> 17) - 1;
            while (begin_sector <= end_sector) {
	      putString ("erase sector: ");
	      print_decimal (begin_sector);
              putString (" ... ");

              i1 = (flash_erase_ptr) (&flash_info [0], begin_sector, begin_sector);
	      if (i1) {
                putString ("erase failed, err_code = 0x");
	        print_hex ((ulong) i1);
                putString ("\n\r");
		break;
              }
              else
                putString ("erase done\n\r");
	      begin_sector++;
            }
            putString ("write flash: source = 0xAD000000, dest = 0x");
            print_hex (flash_addr);
            putString (", len = ");
            print_decimal (len);
	    putString ("\n\r");
	    i1 = (write_buff_ptr) (&flash_info [0], (uchar *) 0xAD000000, flash_addr, ((len + 31) & 0xffffffe0));
	    if (i1) {
              putString ("write flash failed, err_code = 0x");
	      print_hex ((ulong) i1);
              putString ("\n\r");
            }
            else
	      putString ("write flash done\n\r");
	    putString ("data dump from flash:\n\r");
            for (i = 0; i < 20; i++) {
	      print_hex4 (((unsigned short *) flash_addr) [i]);
	      putString ("\n\r");
	    }
	    putString ("checksum from flash = ");
            print_hex4 (cksum ((unsigned short *) flash_addr, len >> 1));
            putString ("\n\r");
          }
#endif
	  else {
	    /* Help */
	    putString ("invalid command\n");
	    putString (help_message);
          }
	  break;
        case 2:
	  if (!strcmp (token [0], "ouripaddr")) {
	    env_NetOurIP = bd.bi_ip_addr = string_to_ip (token [1]);
	  }
	  else if (!strcmp (token [0], "subnet_mask")) {
	    env_NetOurSubnetMask = string_to_ip (token [1]);
	  }
	  else if (!strcmp (token [0], "serveripaddr")) {
	    /* server UDP port is well known 69 */
	    env_NetServerIP = string_to_ip (token [1]);
	  }
	  else if (!strcmp (token [0], "mread_hex")) {
   	    mread_addr = simple_strtoul (token [1], &e, 16);            
	    for (i = 0; i < 20; i++) {
              print_hex (mread_addr+i);
 	      putString ("    ");
	      print_hex4 (((unsigned short *) mread_addr) [i]);
	      putString ("\n\r");
	    }
          }
	  else if (!strcmp (token [0], "mread_dec")) {
   	    mread_addr = simple_strtoul (token [1], &e, 10);            
	    for (i = 0; i < 20; i++) {
              print_decimal (mread_addr+i);
 	      putString ("    ");
	      print_hex4 (((unsigned short *) mread_addr) [i]);
	      putString ("\n\r");
	    }
          }
	  else {
	    /* Help */
	    putString ("invalid command\n");
	    putString (help_message);
          }
	  break;
      }
    }

  asm volatile ("jmp @r0; nop"
		: : "z" (start));
}

static void
serial_output(const char *p, unsigned int len)
{
  if (ingdbmode)
    {
      /* encode the output in gdb protocol & wait for gdb response */
      remcomOutBuffer[0] = 'O';
      mem2hex (p, remcomOutBuffer+1, len);
      putpacket (remcomOutBuffer);
    }
  else
    {
      /* send the output directly to the serial port */
      while (len--)
	{
	  if (*p == '\n')
	    putDebugChar ('\r');
	  putDebugChar (*p++);
	}
    }
}

/* 
   R0: Function Number
   R4-R7: Input Arguments
   R0: Return value
 */
void
handle_bios_call (void)
{
  unsigned int func = registers[R0];
  unsigned int arg0 = registers[R4];
  unsigned int arg1 = registers[R5];
  int ret;
  char ch;

  /* When this is called, R0 will contain the BIOS function, and
   * R4 through R7 the arguments.
   */
  switch (func)
    {
    case 0:
      /* Serial output */
      serial_output((char *)arg0, arg1);
      ret = 0;
      break;

    case 1:
      /* Serial input */
      ret = getDebugCharTimeout ((int)arg0);
      break;

    case 2:
      /* Second storage access */
      ret = -1;
      break;

    case 3:
      { /* Feature query */
	/*
	  b15-b8: Machine #
	       0: Unknown
	       1: CqREEK with CQ IDE/ISA Bridge
	       2: Reserved
	       3: SolutionEngine
	       4: CAT68701
	       ...
	  b7:    RESERVED
	  b6:    RESERVED
	  b5:    RESERVED
	  b4:    RESERVED
	  b3:    Heartbeat LED support 1: exists 0: none
	  b2-b0: Serial interface type
	       0: SCI
	       1: SCIF
	       2: IRDA
	       3: Reserved
	       4: PC style serial (? not yet)
	       ...
	 */
	int feature;
	int machine_type = 0;
	int serial_type = 0;

#if defined(CONFIG_CQ_BRIDGE)
	machine_type = 1;
#elif defined(CONFIG_SOLUTION_ENGINE)
	machine_type = 3;
#elif defined(CONFIG_CAT68701)
	machine_type = 4;
#endif

#if defined(CONFIG_SCI)
	serial_type = 0;
#elif defined(CONFIG_SCIF)
	serial_type = 1;
#endif

	feature = (machine_type << 8) | serial_type;

	ret = feature;
      }
      break;
#if defined(CONFIG_MEMORY_SIZE)
    case 4:
      /* Memory size in bytes */
      ret = CONFIG_MEMORY_SIZE;
      break;
#endif
#if defined(CONFIG_IO_BASE)
    case 5:
      /* IO Base */
      ret = CONFIG_IO_BASE;
      break;
#endif
    case 6:
      /* Cache control */
      ret = cache_control (arg0);
      break;
#if defined(CONFIG_ETHERNET)
    case 7:
      /* Ethernet: RESET */
      ret = eth_reset (arg0);
      break;
    case 8:
      /* Ethernet: RECEIVE */
      ret = eth_receive ((char *)arg0, (unsigned int *)arg1);
      break;
    case 9:
      /* Ethernet: TRANSMIT */
      ret = eth_transmit ((const char *)arg0, arg1, arg2, (const char *)arg3);
      break;
    case 10:
      /* Ethernet: NODE Address */
      ret = eth_node_addr (arg0, (char *)arg1);
      break;
#endif
    case 11:
      /* Shutdown (Run Monitor again, Reboot, Powerdown (if supported)... */
      ret = shutdown (arg0);
      break;
    case 12:
      /* RTC handling: Initialization, start/stop, set/get, sleep */
      ret = rtc (arg0, arg1);
      break;

    case 31:
      /* Serial output of one char:  Emits the lowest 8 bits of R4 */
      ch = arg0 & 0xff; /* Take the lower bits of R4 */
      serial_output(&ch, 1);
      ret = 0;
      break;

    case 32:
      /* Serial string output */
      {
	char *p = (char *)arg0;
	serial_output(p, strlen (p));
	ret = 0;
	break;
      }

    case 254:	/* Query gdb mode variable */
      /*
       * Return a *pointer* to the stub's ingdbmode variable.
       * This allows the kernel to subsequently check that variable
       * efficiently (which it needs to do for every character
       * written to the console).
       */
      ret = (int)&ingdbmode;
      break;

    case 255:
      /* Detach gdb mode */
      if (ingdbmode)
	{
	  ingdbmode = 0;
	  /*
	   * Tell gdb on the host that the inferior exited with status 0
	   * which is not the same as a real detach but as close as we can get.
	   */
	  putpacket ("W00");
	  /* Wait for user to type a character at terminal program on host. */
	  getDebugChar ();
	}
      ret = 0;
      break;

    default:
      /* Do nothing */
      ret = -1;
      break;
    }

  registers[R0] = (unsigned int)ret;
}

void
printouthex(int x)
{
  remcomOutBuffer[0] = highhex (x);
  remcomOutBuffer[1] = lowhex (x);
  remcomOutBuffer[2] = ' ';
  remcomOutBuffer[3] = '\n';
  remcomOutBuffer[4] = '\0';
  putString (remcomOutBuffer);
}

void
printouthex32(unsigned int x)
{
  int i;
  const char* hexdigits = "0123456789abcdef";
  for (i=0; i<8; i++)
    {
      remcomOutBuffer[i] = hexdigits[x >> 28];
      x<<=4;
    }
  remcomOutBuffer[9] = '\0';
  putString (remcomOutBuffer);
}

static inline void
set_BL (void)
{
  unsigned long __dummy;

  asm volatile ("stc	sr, %0\n\t"
		 "or	%1, %0\n\t"
		 "ldc	%0, sr"
		: "=&r" (__dummy)
		: "r" (0x10000000)
		: "memory");
}

static inline void
set_RB (void)
{
  unsigned long __dummy;

  asm volatile ("stc	sr, %0\n\t"
		"or	%1, %0\n\t"
		"ldc	%0, sr"
		: "=&r" (__dummy)
		: "r" (0x20000000)
		: "memory");
}



static void disable_MMU (void);
static void reset_interrupt_request_sources (void);

static int
shutdown (unsigned int func)
{
  extern void start ();		/* In entry.S */

  set_BL ();
  disable_MMU ();
  reset_interrupt_request_sources ();	/* Stop timer and others  */
  /* If possible, put RESET signal down to cause hardware reset */
  /* If possible, we need reset all: bus, modules, and the peripherals */
  /* XXX: Reset all variables and memory... */
  switch (func)
    {
    default:
    case 0:			/* HALT */
      set_boot_signature (0xdeadbeef);
      break;
    case 1:			/* REBOOT */
      set_boot_signature (0xbabeface);
      break;
    /* ACPI support? */
    }

  set_RB ();
  asm volatile ("jmp @r0; nop"
		: : "z" (start));

  /* Never reached */ 
  return 0;
}

#include "io.h"

#if defined(__sh3__)
#define MMUCR		0xFFFFFFE0	/* MMU Control Register */
#define MMU_CONTROL_INIT_DISABLE	0x006	/* SV=0, TF=1, IX=1, AT=0 */
#elif defined(__SH4__)
#define MMUCR		0xFF000010	/* MMU Control Register */
#define MMU_CONTROL_INIT_DISABLE	0x204	/* SQMD=1, SV=0, TI=1, AT=0 */
#endif

static void
disable_MMU (void)
{
  p4_outl (MMUCR, MMU_CONTROL_INIT_DISABLE);
}

#if defined(__sh3__)
#define TMU_TSTR	0xfffffe92	/* Byte access */
#elif defined(__SH4__)
#define TMU_TSTR	0xffd80004	/* Byte access */
#endif

static void
reset_interrupt_request_sources (void)
{
  /* Stop the timer */
  p4_outb(TMU_TSTR, 0);

  /* ... and others */
}

#if defined(__sh3__)
#define R64CNT  	0xfffffec0
#define RSECCNT 	0xfffffec2
#define RMINCNT 	0xfffffec4
#define RHRCNT  	0xfffffec6
#define RCR1    	0xfffffedc

#define RTC_BIT_CHANGE 0x00 /* No bug */
#elif defined(__SH4__)
#define R64CNT  	0xffc80000
#define RSECCNT 	0xffc80004
#define RMINCNT 	0xffc80008
#define RHRCNT  	0xffc8000c
#define RCR1    	0xffc80038

#define RTC_BIT_CHANGE 0x40	/* We have bug on SH-4 */
#endif

#define RCR1_CF		0x80	/* Carry Flag             */

void
sleep128 (unsigned int count)
{
  unsigned int cur128 = p4_inb (R64CNT) ^ RTC_BIT_CHANGE;
  unsigned int n = (cur128 + count) % 128;
  unsigned int m = (cur128 + count) / 128;

  if (count == 0)
    return;

  while (m != 0)
    {
      /* Wait next one tick */
      while ((p4_inb (R64CNT) ^ RTC_BIT_CHANGE) == cur128)
	/* Do nothing */
	asm volatile ("" : : :"memory");

      m--;

      /* Wait next 127 ticks */
      while ((p4_inb (R64CNT) ^ RTC_BIT_CHANGE) != cur128)
	/* Do nothing */
	asm volatile ("" : : :"memory");
    }

  while ((p4_inb (R64CNT) ^ RTC_BIT_CHANGE) != n)
    /* Do nothing */
    asm volatile ("" : : :"memory");
}

/*
 * Unit is 1/128 sec.
 * It doesn't overflow within a day = 24 * 60 * 60 * 128
 */
static unsigned long tick;

#define BCD_TO_BIN(val) ((val)=((val)&15) + ((val)>>4)*10)

static unsigned long get_tick_1 (void)
{
  unsigned int cnt128, sec, min, hour;

  while (1)
    {
      p4_outb (RCR1, 0);  /* Clear CF-bit */

      cnt128 = (p4_inb (R64CNT) ^ RTC_BIT_CHANGE);
#if RTC_BIT_CHANGE != 0
      /* RCR1_CF doesn't work well. */
      if (cnt128 == 0)
	continue;
#endif

      sec = p4_inb (RSECCNT);
      min = p4_inb (RMINCNT);
      hour = p4_inb (RHRCNT);

      if ((p4_inb (RCR1) & RCR1_CF) == 0)
	break;
    }

  BCD_TO_BIN(sec);
  BCD_TO_BIN(min);
  BCD_TO_BIN(hour);

  return cnt128 + 128*(sec+60*(min+60*hour));
}

static void reset_tick (void)
{
  tick = get_tick_1 ();
}

static int rtc_error;
static unsigned long last_tick;

static unsigned long get_tick (void)
{
  unsigned long raw;

  raw = get_tick_1 ();

  /* This bug was hit by SH-4's RTC */
  if (raw < last_tick && last_tick-raw < 128)
    {
      if (rtc_error == 0)
	{
	  putString("Time goes backward!  RTC Problem.\n");
	  rtc_error++;
	}

      /* Work around */
      raw = last_tick;
    }

  last_tick = raw;

  if (raw + (24*60*60*128/4) < tick)		/* Wrap around */
    return 24*60*60*128+raw-tick; 
  else
    return raw-tick;
}

/* XXX: This will be hardware dependent... */ 
static int
rtc (unsigned int func, unsigned int arg)
{
  switch (func)
    {
    case 0:	/* Initialize & start */
      return 0;			/* Be called, but not supported yet */

    case 1:	/* Stop */
    case 2:	/* Set */
    case 3:     /* Get */
      return -1;		/* Not supported yet */

    case 4:	/* Sleep128 */
      sleep128 (arg);
      return 0;

    case 5:
      reset_tick ();
      return 0;
    case 6:
      return get_tick ();

    default:
      return -1;
    }
}
