/*
 * (C) Copyright 2003
 * Texas Instruments.
 * Kshitij Gupta <kshitij@ti.com>
 * Configuation settings for the TI OMAP Innovator board.
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 * Configuration for Versatile PB.
 *
 * (C) Copyright 2005
 * SMSC
 * Woojung Huh, <Woojung.Huh@smsc.com>
 * Configuration for Foxboro
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include "smsc9118.h"

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_FOXBOROARM926EJS	1	/* This is an arm926ejs CPU core  */
#define CONFIG_FOXBORO			1	/* in Foxboro	*/
#define CONFIG_ARCH_FOXBORO		1	/* Specifically, a Foxboro	*/

#define	CONFIG_INIT_CRITICAL	1

//#define	CFG_ALT_MEMTEST			1	/* more complete alternative test */
#define CFG_MEMTEST_START       0xA0200000
#define CFG_MEMTEST_END         0xA8000000

#if 0
/* for ARM CPU, we are using lan9118 timer */
#define CFG_HZ                  (1000000)	/* 1MHZ */
#define CFG_TIMERBASE           0x50000020	/* Timer 1 base */
#define CFG_TIMER_CTRL          0x82		/* Enable, Clock/1, 32bit */
#endif

#define CONFIG_CMDLINE_TAG		1	/* enable passing of ATAGs  */
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_MISC_INIT_R		1	/* call misc_init_r during start up */
/*
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN	(CFG_ENV_SIZE + 128*1024)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

/*
 * Hardware drivers
 */

#define	CONFIG_DRIVER_SMSC9118
#define CONFIG_SMSC9118_BASE    	0x10010000

/*
 * NS16550 Configuration
 */
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE	4
#define CFG_NS16550_CLK			1843200
#define CFG_NS16550_COM1		0x50400000

#define CONFIG_BAUDRATE         115200
#define CFG_BAUDRATE_TABLE      { 9600, 19200, 38400, 57600, 115200 }
#define	CONFIG_CONS_INDEX		1

#define CONFIG_COMMANDS	(CFG_CMD_DHCP | CFG_CMD_IMI | CFG_CMD_NET | CFG_CMD_PING | CFG_CMD_BDI | CFG_CMD_MEMORY | CFG_CMD_FLASH | CFG_CMD_ENV)

/*#define CONFIG_COMMANDS	(CFG_CMD_IMI | CFG_CMD_BDI | CFG_CMD_MEMORY) */

#define CONFIG_BOOTP_MASK	CONFIG_BOOTP_DEFAULT

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#if 0
#include <cmd_confdefs.h>
#endif

#define CONFIG_BOOTDELAY	2
#define CONFIG_BOOTARGS 	""
/*#define CONFIG_BOOTCOMMAND "bootp ; bootm" */

/*
 * Static configuration when assigning fixed address
 */
#if 1
#define CONFIG_NETMASK	255.255.255.0		/* talk on MY local net */
#define CONFIG_IPADDR	192.168.1.118
#define CONFIG_SERVERIP	192.168.1.101
#define	CONFIG_ETHADDR	00:80:0F:11:70:00
#endif
#define CONFIG_BOOTFILE	118diag.rom			/* file to load */


/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP	/* undef to save memory     */
#define CFG_PROMPT	"Foxboro # "	/* Monitor Command Prompt   */
#define CFG_CBSIZE	256		/* Console I/O Buffer Size  */
/* Print Buffer Size */
#define CFG_PBSIZE	(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
#define CFG_MAXARGS	16		/* max number of command args   */
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size    */

#undef	CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */
#define CFG_LOAD_ADDR	0xA0008000	/* default load address */

/*-----------------------------------------------------------------------
 * Identification
 */
#define	CONFIG_FOXBORO_VERSION	"(v0.7)"
#define	CONFIG_IDENT_STRING	" for Foxboro by SMSC " CONFIG_FOXBORO_VERSION

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */
#undef	CONFIG_USE_IRQ
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */
#endif

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS    1	/* we have 1 bank of DRAM */
#define PHYS_SDRAM_1            0xA0000000	/* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE       0x08000000	/* 128 MB */

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */

#define CFG_FLASH_BASE          0xA0000000

#define	CFG_MONITOR_BASE		(CFG_FLASH_BASE)
#define	CFG_MONITOR_LEN			0x100000
#undef	CFG_ENV_ADDR_REDUND

#define CFG_MAX_FLASH_BANKS		1			/* max number of memory banks */
#define PHYS_FLASH_SIZE         0x1000000	/* 16MB */

/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(20*CFG_HZ)	/* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(20*CFG_HZ)	/* Timeout for Flash Write */
#define CFG_MAX_FLASH_SECT		(128)

#define PHYS_FLASH_1		(CFG_FLASH_BASE)

#define CFG_ENV_IS_IN_FLASH     1               /* env in flash instead of CFG_ENV_IS_NOWHERE */
#define CFG_ENV_SECT_SIZE       0x00020000      /* 128 KB sectors */
#define CFG_ENV_SIZE            (CFG_ENV_SECT_SIZE) /* Total Size of Environment Sector */
#define CFG_ENV_OFFSET          0x00040000      /* environment starts here  */
#define CFG_ENV_ADDR            (CFG_FLASH_BASE + CFG_ENV_OFFSET)

/* Flash banks JFFS2 should use */
#define CFG_JFFS2_FIRST_BANK    0
#define CFG_JFFS2_FIRST_SECTOR	2
#define CFG_JFFS2_NUM_BANKS     1
#endif							/* __CONFIG_H */
