/*------------------------------------------------------------------------
 . smsc9118.c
 . This is a driver for SMSC's 9118 single-chip Ethernet device.
 .
 . This program is free software; you can redistribute it and/or modify
 . it under the terms of the GNU General Public License as published by
 . the Free Software Foundation; either version 2 of the License, or
 . (at your option) any later version.
 .
 . This program is distributed in the hope that it will be useful,
 . but WITHOUT ANY WARRANTY; without even the implied warranty of
 . MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 . GNU General Public License for more details.
 .
 . You should have received a copy of the GNU General Public License
 . along with this program; if not, write to the Free Software
 . Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 .
 . History:
 ----------------------------------------------------------------------------*/
#include "net.h"
#include "defs.h"
#include "util.h"
#include "smsc9118.h"

#define CONFIG_DRIVER_SMSC9118

#ifdef CONFIG_DRIVER_SMSC9118

/*************************************************************************
   FUNCTION PROTOTYPES

*************************************************************************/
int lan9118Int(void);
int lan9118_open(bd_t *bis);
void lan9118_close(void);
int DumpDirRegs(void);
int DumpMacRegs(void);
int DumpPhyRegs(void);
static int sendToNet(uchar * txbp, int len);
static void lan9118_udelay(unsigned long usec);


/*************************************************************************
   LOCAL DEFINITIONS AND MACROS

*************************************************************************/
#define 	GPIO_OUT(val)	  		(*GPIO_CFG = ((*GPIO_CFG & ~GPIO_CFG_GPIOD_MSK) | (val & GPIO_CFG_GPIOD_MSK)))
#define 	ENET_MAX_MTU			PKTSIZE
#define 	ENET_MAX_MTU_ALIGNED	PKTSIZE_ALIGN
#define 	NUM_RX_BUFF 	  		PKTBUFSRX
#define		ENET_ADDR_LENGTH		6


/*************************************************************************
 GLOBAL DATA

*************************************************************************/
ulong RxPacketBuf[400];
ulong MaxRxFifoSz;
ulong SWIntTriggered = FALSE;
ulong TotalInts = 0;
ulong TotalRXE = 0;
ulong TotalRxDrop = 0;
ulong TotalPackets = 0;
ulong TotalRxPackets = 0;
ulong TotalWords = 0;
ulong TotalBytes = 0;
ulong EmptyReads = 0;
ulong TBLower1, TBLower2;

static char * txbp; 				// TX buffer pointer (only 1 buffer)
static volatile char * rxbp[PKTBUFSRX];   // Receiver buffer queue (IP layers)
static struct rxQue rxAvlQue[PKTBUFSRX]; // Receive buffer available queue
static int rxNdx = 0;				// Current receive buffer index
static int rxNdxIn = 0; 			// Used for input
static int rxNdxOut = 0;			// Used for output to protocol layer
static ushort lastTxTag = 0x0;
static unsigned char macAddr[ENET_ADDR_LENGTH];


//*************************************************************************
 // EXTERNS

//*************************************************************************
extern int use_smsc9118;

#define	_1_USEC			(25)		// = 25000000/1000000
static void lan9118_udelay(unsigned long usec)
{
	unsigned long	delta, start;

	start = *FREE_RUN;
	delta = (usec*_1_USEC);

	while (1) 
	{
		if (*FREE_RUN > start)
		{
			if ((*FREE_RUN - start)	> delta)
				break;
		}
		else
		{
			if ((0xFFFFFFFF - (start - *FREE_RUN)) > delta)
				break;
		}
	}
}

int MacBusy(ReqTO)
{
	  int timeout = ReqTO;
	  int RetVal = FALSE;	  // No timeout

	  while (timeout--) {
			if (!(*MAC_CSR_CMD & MAC_CSR_CMD_CSR_BUSY)) {
				  goto done;
			}
	  }
	  RetVal = TRUE;		  // Timeout
done:
	  return (RetVal);
}

int
PhyBusy(ReqTO)
{
	  int timeout = ReqTO;
	  int RetVal = FALSE;	  // No timeout

	  while (timeout--) {
			if (!(GetMacReg(MAC_MIIACC) & MAC_MIIACC_MII_BUSY)) {
				  goto done;
			}
	  }

	  RetVal = TRUE;		  // Timeout
done:
	  return (RetVal);
}

ulong
GetMacReg(int Reg)
{
	  ulong RegVal = 0xffffffff;

	  if (*MAC_CSR_CMD & MAC_CSR_CMD_CSR_BUSY) {
			LAN9118_WARN("GetMacReg: previous command not complete\n\r");
			goto done;
	  }

	  *MAC_CSR_CMD = MAC_RD_CMD(Reg);
	  DELAY(1);

	  if (MacBusy(MAC_TIMEOUT) == TRUE) {
			LAN9118_WARN("GetMacReg: timeout waiting for response "
				  "from MAC\n\r");
			goto done;
	  }

	  RegVal = *MAC_CSR_DATA;

done:
	  return (RegVal);
}

int
SetMacReg(int Reg, ulong Value)
{
	  int RetVal = FALSE;

	  if (*MAC_CSR_CMD & MAC_CSR_CMD_CSR_BUSY) {
			LAN9118_WARN("SetMacReg: previous command not complete\n\r");
			goto done;
	  }

	  *MAC_CSR_DATA = Value;
	  DELAY(1);
	  *MAC_CSR_CMD = MAC_WR_CMD(Reg);
	  DELAY(1);

	  if (MacBusy(MAC_TIMEOUT) == TRUE) {
			LAN9118_WARN("SetMacReg: timeout waiting for response "
				  "from MAC\n\r");
				  goto done;
	  }

	  RetVal = TRUE;
done:
	  return (RetVal);
}

ushort
GetPhyReg(uchar Reg)
{
	  ushort RegVal = 0xffff;

	  if (GetMacReg(MAC_MIIACC) & MAC_MIIACC_MII_BUSY) {
			LAN9118_WARN("GetPhyReg: MII busy\n\r");
			RegVal = 0;
			goto done;
	  }

	  SetMacReg(MAC_MIIACC, MAC_MII_RD_CMD((uchar)PHY_ADDR, Reg));
	  DELAY(1);

	  if (PhyBusy(PHY_TIMEOUT) == TRUE) {
			LAN9118_WARN("GetPhyReg: timeout waiting for MII command\n\r");
			goto done;
	  }

	  RegVal = (ushort)GetMacReg(MAC_MIIDATA);
done:
	  return (RegVal);
}

int
SetPhyReg(uchar Reg, ushort Value)
{
	  int RetVal = FALSE;

	  if (GetMacReg(MAC_MIIACC) & MAC_MIIACC_MII_BUSY) {
			LAN9118_WARN("SetPhyReg: MII busy\n\r");
			goto done;
	  }

	  SetMacReg(MAC_MIIDATA, Value);
	  DELAY(1);
	  SetMacReg(MAC_MIIACC, MAC_MII_WR_CMD((uchar)PHY_ADDR, Reg));
	  DELAY(1);

	  if (PhyBusy(PHY_TIMEOUT) == TRUE) {
			LAN9118_WARN("SetPhyReg: timeout waiting for MII command\n\r");
			goto done;
	  }

	  RetVal = TRUE;
done:
	  return (RetVal);
}

int
DumpDirRegs(void)
{
	  putString ("ID_REV:\t0x");
	  print_hex (*ID_REV);
	  putString ("\n\r");
	  putString ("IRQ_CFG:\t0x");
	  print_hex (*IRQ_CFG);
	  putString ("\n\r");
	  putString ("INT_STS:\t0x");
	  print_hex (*INT_STS);
	  putString ("\n\r");
	  putString ("INT_EN:\t0x");
	  print_hex (*INT_EN);
	  putString ("\n\r");
	  putString ("BYTE_TEST:\t0x");
	  print_hex (*BYTE_TEST);
	  putString ("\n\r");
	  putString ("FIFO_INT:\t0x");
	  print_hex (*FIFO_INT);
	  putString ("\n\r");
	  putString ("RX_CFG:\t0x");
	  print_hex (*RX_CFG);
	  putString ("\n\r");
	  putString ("TX_CFG:\t0x");
	  print_hex (*TX_CFG);
	  putString ("\n\r");
	  putString ("HW_CFG:\t0x");
	  print_hex (*HW_CFG);
	  putString ("\n\r");
	  putString ("RX_DP_CTL:\t0x");
	  print_hex (*RX_DP_CTL);
	  putString ("\n\r");
	  putString ("RX_FIFO_INF:\t0x");
	  print_hex (*RX_FIFO_INF);
	  putString ("\n\r");
	  putString ("TX_FIFO_INF:\t0x");
	  print_hex (*TX_FIFO_INF);
	  putString ("\n\r");
	  putString ("PWR_MGMT:\t0x");
	  print_hex (*PWR_MGMT);
	  putString ("\n\r");
	  putString ("GPIO_CFG:\t0x");
	  print_hex (*GPIO_CFG);
	  putString ("\n\r");
	  putString ("GPT_CFG:\t0x");
	  print_hex (*GPT_CFG);
	  putString ("\n\r");
	  putString ("GPT_CNT:\t0x");
	  print_hex (*GPT_CNT);
	  putString ("\n\r");
	  putString ("FPGA_REV:\t0x");
	  print_hex (*FPGA_REV);
	  putString ("\n\r");
	  putString ("ENDIAN:\t0x");
	  print_hex (*ENDIAN);
	  putString ("\n\r");
	  putString ("FREE_RUN\t0x");
	  print_hex (*FREE_RUN);
	  putString ("\n\r");
	  putString ("RX_DROP\t0x");
	  print_hex (*RX_DROP);
	  putString ("\n\r");
	  putString ("MAC_CSR_CMD\t0x");
	  print_hex (*MAC_CSR_CMD);
	  putString ("\n\r");
	  putString ("MAC_CSR_DATA\t0x");
	  print_hex (*MAC_CSR_DATA);
	  putString ("\n\r");
	  putString ("AFC_CFG\t0x");
	  print_hex (*AFC_CFG);
	  putString ("\n\r");
	  return (0);
}

int
DumpMacRegs(void)
{
	  putString ("MAC_CR\t0x");
	  print_hex (GetMacReg(MAC_CR));
	  putString ("\n\r");
	  putString ("MAC_ADDRH\t0x");
	  print_hex (GetMacReg(MAC_ADDRH));
	  putString ("\n\r");
	  putString ("MAC_ADDRL\t0x");
	  print_hex (GetMacReg(MAC_ADDRL));
	  putString ("\n\r");
	  putString ("MAC_HASHH\t0x");
	  print_hex (GetMacReg(MAC_HASHH));
	  putString ("\n\r");
	  putString ("MAC_HASHL\t0x");
	  print_hex (GetMacReg(MAC_HASHL));
	  putString ("\n\r");
	  putString ("MAC_MIIACC\t0x");
	  print_hex (GetMacReg(MAC_MIIACC));
	  putString ("\n\r");
	  putString ("MAC_MIIDATA\t0x");
	  print_hex (GetMacReg(MAC_MIIDATA));
	  putString ("\n\r");
	  putString ("MAC_FLOW\t0x");
	  print_hex (GetMacReg(MAC_FLOW));
	  putString ("\n\r");
	  putString ("MAC_VLAN1\t0x");
	  print_hex (GetMacReg(MAC_VLAN1));
	  putString ("\n\r");
	  putString ("MAC_VLAN2\t0x");
	  print_hex (GetMacReg(MAC_VLAN2));
	  putString ("\n\r");
	  putString ("MAC_WUFF\t0x");
	  print_hex (GetMacReg(MAC_WUFF));
	  putString ("\n\r");
	  putString ("MAC_WUCSR\t0x");
	  print_hex (GetMacReg(MAC_WUCSR));
	  putString ("\n\r");
	  return (0);
}

int
DumpPhyRegs(void)
{
	  putString ("PHY_BCR\t0x");
	  print_hex4 (GetPhyReg(PHY_BCR));
	  putString ("\n\r");
	  putString ("PHY_BSR\t0x");
	  print_hex4 (GetPhyReg(PHY_BSR));
	  putString ("\n\r");
	  putString ("PHY_ID1\t0x");
	  print_hex4 (GetPhyReg(PHY_ID1));
	  putString ("\n\r");
	  putString ("PHY_ID2\t0x");
	  print_hex4 (GetPhyReg(PHY_ID2));
	  putString ("\n\r");
	  putString ("PHY_ANAR\t0x");
	  print_hex4 (GetPhyReg(PHY_ANAR));
	  putString ("\n\r");
	  putString ("PHY_ANLPAR\t0x");
	  print_hex4 (GetPhyReg(PHY_ANLPAR));
	  putString ("\n\r");
	  putString ("PHY_ANEXPR\t0x");
	  print_hex4 (GetPhyReg(PHY_ANEXPR));
	  putString ("\n\r");
	  putString ("PHY_SILREV\t0x");
	  print_hex4 (GetPhyReg(PHY_SILREV));
	  putString ("\n\r");
	  putString ("PHY_MCSR\t0x");
	  print_hex4 (GetPhyReg(PHY_MCSR));
	  putString ("\n\r");
	  putString ("PHY_SPMODES\t0x");
	  print_hex4 (GetPhyReg(PHY_SPMODES));
	  putString ("\n\r");
	  putString ("PHY_CSIR\t0x");
	  print_hex4 (GetPhyReg(PHY_CSIR));
	  putString ("\n\r");
	  putString ("PHY_ISR\t0x");
	  print_hex4 (GetPhyReg(PHY_ISR));
	  putString ("\n\r");
	  putString ("PHY_IMR\t0x");
	  print_hex4 (GetPhyReg(PHY_IMR));
	  putString ("\n\r");
	  putString ("PHY_PHYSCSR\t0x");
	  print_hex4 (GetPhyReg(PHY_PHYSCSR));
	  putString ("\n\r");
	  return (0);
}

int
lan9118_open(bd_t *bis)
{
	  int RetVal = TRUE;
	  int timeout;
	  int i;
	  unsigned long chip_id, test;
	  unsigned short dummy;

#ifdef		DEBUG
	  TotalInts = 0;
	  TotalRXE = 0;
	  TotalBytes = 0;

	  if (bis->bi_bootflags & 0x40000000) {
			use_smsc9118 = 1;
	  }
#endif		//DEBUG

	  lastTxTag = 0;

	  // Does SoftReset to 118
	  *HW_CFG = HW_CFG_SRST;
	  DELAY(10);

	  // detect bus width to support 16/32
	  // first assume 32-bit bus
          *((volatile unsigned short *) 0xFFFFFF62) |= (1 << 10);
          test = *BYTE_TEST;
          dummy = *((volatile unsigned short *) (SMSC9118_BASE + 0x66));
	  if (test == 0x87654321) {
            putString ("32-bit bus width detected\n\r");
          }
          else {
	    // assume 16-bit bus
            *((volatile unsigned short *) 0xFFFFFF62) &= ~(1 << 10);
            test = *BYTE_TEST;
	    if (test == 0x87654321) {
              putString ("16-bit bus width detected\n\r");
            }
            else {
              putString ("neither 16-biti nor 32-bit bus width detected\n\r");
            }
          }
          
	  // Because we just came out of h/w reset we can't be sure that
	  // the chip has completed reset and may have to implement the
	  // workaround for Errata 5, stepping A0.	Therefore we need to
	  // check the ID_REV in little endian, the reset default.
	  chip_id = (*ID_REV & ID_REV_ID_MASK) >> 16;
	  if (chip_id == 0x0115 || chip_id == 0x0116 || chip_id == 0x0117 ||
		chip_id == 0x0118 || chip_id==0x118A || chip_id == 0x0112 || chip_id==0x117A) 
	  {
			putString ("LAN911x (0x");
			print_hex (*ID_REV);
			putString (") detected.\n\r");
	  }
	  else
	  {
			putString ("Failed to detect LAN9118. ID_REV = 0x");
			print_hex (*ID_REV);
			putString ("\n\r");
		    RetVal = FALSE;
		    goto done;
	  }

	  // Is the internal PHY running?
	  if ((*PWR_MGMT & PWR_MGMT_PM_MODE_MSK) != 0) {
			// Apparently not...
			*BYTE_TEST = 0x0; // Wake it up
			DELAY(1);
			timeout = PHY_TIMEOUT;
			while (timeout-- && ((*PWR_MGMT & PWR_MGMT_PME_READY) == 0)) {
				  lan9118_udelay(1);
			}
			if ((*PWR_MGMT & PWR_MGMT_PME_READY) == 0) {
				  LAN9118_WARN("LAN9118: PHY not ready");
				  LAN9118_WARN(" - aborting\n\r");
				  RetVal = FALSE;
				  goto done;
			}
	  }

	  // Reset the PHY
	  *PWR_MGMT |= PWR_MGMT_PHY_RST;
	  timeout = PHY_TIMEOUT;
	  lan9118_udelay(50*1000);	// > 50ms
	  while(timeout-- && ((*PWR_MGMT) & PWR_MGMT_PHY_RST) == PWR_MGMT_PHY_RST)
	  {
			lan9118_udelay(10);
	  }
	  if (timeout == 0)
	  {
			LAN9118_WARN("PHY reset incomplete\n\r");
			goto done;
	  }

	  // Setup and start auto negotiation
	  {
			ushort anar;
			ushort bcr;
			char * spddplx;

			anar = GetPhyReg(PHY_ANAR);
			anar &= ~PHY_ANAR_PAUSE_OP_MSK;
			anar |= PHY_ANAR_PAUSE_OP_BOTH;
			anar |= (PHY_ANAR_10_FDPLX | PHY_ANAR_10_ABLE |
				  PHY_ANAR_100_TX_FDPLX | PHY_ANAR_100_TX_ABLE);
			SetPhyReg(PHY_ANAR, anar);

			DELAY(2);
			bcr = GetPhyReg(PHY_BCR);
			bcr |= (PHY_BCR_SS | PHY_BCR_FDPLX);
			SetPhyReg(PHY_BCR, bcr);
			DELAY(2);

			putString ("start Auto negotiation... (take ~2sec)\n\r");
			bcr = GetPhyReg(PHY_BCR);
			bcr |= (PHY_BCR_ANE | PHY_BCR_RSTAN);
			SetPhyReg(PHY_BCR, bcr);
			DELAY(2);

			timeout = PHY_AN_TIMEOUT;
			while((timeout--) && ((GetPhyReg(PHY_BSR) & PHY_BSR_ANC) == 0)) {
				  lan9118_udelay(500000);
			}
			if ((GetPhyReg(PHY_BSR) & PHY_BSR_ANC) == 0) {
				  putString ("Auto negotiation failed\n\r");
				  goto done;
			}

			if ((GetPhyReg(PHY_BSR) & PHY_BSR_LINK_STATUS) == 0) {
				  putString ("Link down\n");
				  goto done;
			}

			switch ((GetPhyReg(PHY_PHYSCSR) & PHY_PHYSCSR_SPEED_MSK)>>2) {
				  case 0x01:
						spddplx = "10BaseT, half duplex";
						break;
				  case 0x02:
						spddplx = "100BaseTX, half duplex";
						break;
				  case 0x05:
						spddplx = "10BaseT, full duplex";
						break;
				  case 0x06:
						spddplx = "100BaseTX, full duplex";
						break;
				  default:
						spddplx = "Unknown";
						break;
			}
			putString ("Auto negotiation complete, ");
			putString (spddplx);
			putString ("\n\r");
	  }
#if 0
	  {
		  	// 10HD
			ushort bcr = 0;

			SetPhyReg(PHY_BCR, bcr);
			DELAY(2);
			putString ("Set to 10HD\n\r");
	  }

putString ("MAC_CR set half duplex\n\rMAC_CR = ");
	SetMacReg(MAC_CR, (GetMacReg(MAC_CR) & (~(MAC_CR_FDPX))));
	print_hex (GetMacReg(MAC_CR));
putString ("\n\r");
#endif

	  // If PHYs auto negotiated for full duplex, enable full duplex in MAC.
	  if ((GetPhyReg(PHY_ANAR) & GetPhyReg(PHY_ANLPAR)) & 0x0140) {
			SetMacReg(MAC_CR, (GetMacReg(MAC_CR) | 0x00100000));
	  }

	  // Setup TX and RX resources.

	  // There is one TX buffer.
	  if ((txbp = malloc(ENET_MAX_MTU_ALIGNED)) == NULL) {
			LAN9118_WARN("lan9118_open: can't get TX buffer\n\r");
			goto cleanup;
	  }

	  // The receive buffers are allocated and aligned by upper layer
	  // software.
	  for (i = 0; i < PKTBUFSRX; i++) {
			rxbp[i] = NetRxPackets[i];
			rxAvlQue[i].index = -1;
	  }

	  rxNdx = 0;
	  rxNdxIn = 0;
	  rxNdxOut = 0;

	  // Set TX Fifo Size
	  *HW_CFG = 0x00040000;   // 4K for TX

	  // This value is dependent on TX Fifo Size since there's a limited
	  // amount of Fifo space.
	  MaxRxFifoSz = 13440;			// Decimal

	  // Set automatic flow control.
	  *AFC_CFG = 0x008c46af;

	  // Flash LEDs.
	  *GPIO_CFG = 0x70700000;

	  // Disable interrupts until the rest of initialization is complete.
	  *INT_EN = 0x0;				// Clear interrupt enables
	  *INT_STS = 0xffffffff;		// Clear pending interrupts
	  *IRQ_CFG = 0x00000001;		// IRQ disable

	  // Enable flow control and pause frame time
	  SetMacReg(MAC_FLOW, 0xffff0002);

	  // Set MAC address, if octet 0 is non-null assume it's all good.
	  {
			unsigned mac_addrh;
			unsigned mac_addrl;
			memcpy(macAddr, bis->bi_enetaddr, ENET_ADDR_LENGTH);
			mac_addrh = macAddr[5] << 8 | macAddr[4];
			mac_addrl = macAddr[3] << 24 | macAddr[2] << 16 |
				  macAddr[1] << 8 | macAddr[0];
			if (mac_addrh != 0 || mac_addrl != 0) {
				  SetMacReg(MAC_ADDRH, mac_addrh);
				  SetMacReg(MAC_ADDRL, mac_addrl);
			}
	  }

	  // Dump old status and data
	  *RX_CFG = (RX_CFG_FORCE_DISCARD);

	  // Initialize Tx parameters
	  *HW_CFG = ((*HW_CFG & HW_CFG_TX_FIF_SZ_MSK) | HW_CFG_SF);
	  *FIFO_INT = FIFO_INT_TDAL_MSK;	  // Max out value
	  *INT_EN |= INT_EN_TDFA_INT_EN;
	  {
			// Disable MAC heartbeat SQE and enable MAC transmitter
			ulong macCR = GetMacReg(MAC_CR);
			macCR |= (MAC_CR_TXEN | MAC_CR_HBDIS);
			macCR &= ~MAC_CR_PRMS;	// Turn off promiscuous mode
			macCR |= MAC_CR_BCAST;	// Don't accept broadcast frames
			SetMacReg(MAC_CR, macCR);
	  }

	  // Initialize Rx parameters
	  *RX_CFG = 0x00000000;			// 4byte end-alignment
	  {
			// Enable receiver.
			ulong macCR = GetMacReg(MAC_CR);
			SetMacReg(MAC_CR, (macCR | MAC_CR_RXEN));
	  }
	  *FIFO_INT = ((*FIFO_INT & 0xffff0000) | 0x00000101);
	  *INT_EN |= (INT_EN_RSFL_INT_EN | INT_EN_RXE_INT_EN);
	  *INT_EN |= INT_EN_RXDFH_INT_EN;

	  // Initialize PHY parameters
	  if ((GetPhyReg(PHY_ID1) != PHY_ID1_LAN9118) || 
		((GetPhyReg(PHY_ID2) != PHY_ID2_LAN9118_1) &&
		(GetPhyReg(PHY_ID2) != PHY_ID2_LAN9118_2))) {
			LAN9118_WARN("Unknown PHY ID\n\r");
			print_hex4 (GetPhyReg(PHY_ID1));
 			putString ("  ");
			print_hex4 (GetPhyReg(PHY_ID2));
 			putString ("\n\r");
			goto done;
	  }

	  goto done;

cleanup:
	  if (txbp != NULL) {
			free(txbp);
	  }

done:
	  return (RetVal);
}

void lan9118_close(void)
{
	  // Release the TX buffer.
	  if (txbp != NULL) {
			free(txbp);
	  }
}

int lan9118_read()
{
	  int curBufNdx;
	  int loopCount = 0;
	  ulong rxStatus;
	  ulong count;
	  ulong len;
	  ulong tmp;
	  register ulong * rxbpl;
	  int ffwdOk = TRUE;
	  int timeout;
	  int ndx;
	  int handled = 0;

	  while((*RX_FIFO_INF & 0x00ff0000) != 0) {
			if (loopCount >= NUM_RX_BUFF) {
				  break;	  // Packet buffers full
			}

			curBufNdx = rxNdx;
			loopCount++;
			if (++rxNdx == NUM_RX_BUFF) {
				  rxNdx = 0;  // Wrap buffer slot #
			}

			rxStatus = *RX_STATUS_FIFO_PORT;
			len = count = rxStatus >> 16;

			if (count >= 4*sizeof(ulong)) {
				  ffwdOk = TRUE;	// Use h/w to toss packet
			} else {
				  ffwdOk = FALSE;	// Have to empty manually on error
			}

			if (count != 0) {
				  if (count > ENET_MAX_MTU) {
						count = 0;
				  } else {
						if ((rxStatus & TX_STATUS_FIFO_ES) != 0) {
							  count = 0;
						}
				  }
			}

			if (count == 0) {
				  if (ffwdOk == TRUE) {
						// Drain it the fast way
						*RX_DP_CTL = RX_DP_FFWD;
						timeout = FFWD_TIMEOUT;
						while (timeout-- && (*RX_DP_CTL & RX_DP_FFWD)) {
							  lan9118_udelay(1);
						}
						if ((*RX_DP_CTL & RX_DP_FFWD) != 0) {
							  LAN9118_WARN("lan9118_read: fast "
									"forward op failed\n\r");
							  break;
						}
				  } else {
						// Drain it manually
						while (len--) {
							  tmp = *RX_FIFO_PORT;
						}
				  }
			} else if (rxAvlQue[rxNdxIn].index != -1) {
				  LAN9118_WARN("lan9118_read: read buffers full!\n\r");
				  break;
			} else {
				  TotalRxPackets++;
				  TotalBytes += count;
				  rxAvlQue[rxNdxIn].index = curBufNdx;
				  rxAvlQue[rxNdxIn].len = count;
				  if (++rxNdxIn == NUM_RX_BUFF) {
						rxNdxIn = 0;
				  }

				  // Copy this packet to a NetRxPacket buffer
				  handled = 1;
				  EmptyReads = 0;
				  rxbpl = (ulong *)rxbp[curBufNdx];
				  for (ndx = 0; ndx < (count+3)/sizeof(ulong); ndx++) {
						*rxbpl++ = *RX_FIFO_PORT;
				  }
				  DELAY(3);
			}
	  }

	  if (handled) {
			for (;;) {
				  curBufNdx = rxAvlQue[rxNdxOut].index;
				  if (curBufNdx == -1) {
						len = -1;	// Nothing else received
						break;
				  }
				  len = rxAvlQue[rxNdxOut].len;
				  NetReceive(NetRxPackets[curBufNdx], len - 4);
				  rxAvlQue[rxNdxOut].index = -1;	  // Free buffer
				  if (++rxNdxOut == NUM_RX_BUFF) {
						rxNdxOut = 0;	  // Handle wrap
				  }
			}
	  } else {
			EmptyReads++;
			return (-1);	  // Nothing was received
	  }

	  return (len);
}

int lan9118_write(volatile void *ptr, int len)
{
	  ulong startTime;
	  ulong txStatus;
	  ulong timeout;

	  if (len > ENET_MAX_MTU) {
			len = ENET_MAX_MTU;
	  }

	  // Copy the packet.
	  memcpy((void *)txbp, (void *)ptr, len);

	  // Drain the TX status fifo just in case there are old (good) statuses.
	  for (timeout=0; timeout<30; timeout++)
	  {
		  if ((*TX_FIFO_INF & TX_FIFO_TXSUSED_MSK) == 0) {
				break;
		  }
		  putString ("lan9118_write: discarded old TX status\n\r");
	  }
	  if (timeout == 30)
	  {
		DumpDirRegs();
		DumpMacRegs();
		DumpPhyRegs();
	  }

	  if (sendToNet(txbp, len) == FALSE) {
			return (-1);
	  }

	  startTime = get_timer(0);
	  while (1) {
			if (((*TX_FIFO_INF) & TX_FIFO_TXSUSED_MSK) == 0) {
				  // No status yet
				  if ((get_timer(0) - startTime) > TX_TIMEOUT) {
						return ( -1);
				}
			} else {
				  if (((txStatus = *TX_STATUS_FIFO_PORT) &
						TX_STATUS_FIFO_ES) == TX_STATUS_FIFO_ES) {
						return (-1);
				  } else {
						*TX_CFG |= TX_CFG_STOP_TX; // Stop transmitter
						return (len);
				  }
			}
	  }
}

static int
sendToNet(uchar * txbp, int len)
{
	  ulong tx_cmd_a, tx_cmd_b;
	  int i;
	  ulong * txbpl = (ulong *)txbp;

	  lastTxTag++;

	  tx_cmd_a = (((ulong)txbp & 0x3) << 16) | 0x00003000 | len;
	  tx_cmd_b = (lastTxTag << 16) | len;

	  *TX_FIFO_PORT = tx_cmd_a;
	  *TX_FIFO_PORT = tx_cmd_b;

	  for (i = 0; i < ((len+3)>>2); i++ , txbpl++) 
	  {
			*TX_FIFO_PORT = *txbpl; 
	  }
	  *TX_CFG = TX_CFG_TX_ON;		// Enable transmitter
	  return (TRUE);
}

int	eth_init(bd_t *bd)
{
	return lan9118_open(bd);
}

void eth_halt()
{
	lan9118_close();
}

int	eth_rx()
{
	int	r;

	r = lan9118_read();

	return r;
}

int	eth_send(volatile void *packet, int length)
{
	return lan9118_write(packet, length);
}

ulong get_timer (ulong base)
{
  return (*FREE_RUN);
}

#endif		// #ifdef CONFIG_DRIVER_SMSC9118
