#include "net.h"
#include "util.h"

/*
 ****************************************************************************
 *	StdArg.h stuff: 
 ****************************************************************************
 *
 */
#define _M_IX86
#ifdef _M_IX86

#include "defs.h"

#ifndef _VA_LIST_DEFINED
typedef char *	va_list;
#define _VA_LIST_DEFINED
#endif


#define _INTSIZEOF(n)	( (sizeof(n) + sizeof(int) - 1) & ~(sizeof(int) - 1) )

#define va_start(ap,v)	( ap = (va_list)&v + _INTSIZEOF(v) )
#define va_arg(ap,t)	( *(t *)((ap += _INTSIZEOF(t)) - _INTSIZEOF(t)) )
#define va_end(ap)		( ap = (va_list)0 )

#else
#include "varargs.h"
#endif

/* other functions */

ushort swap16 (ushort value)
{
#if 1
	uchar byte1;
	uchar byte2;

	byte1 = (uchar) ((value >> 8) & 0xff);
	byte2 = (uchar) (value & 0xff);
	return ((ushort) byte2 << 8) | ((ushort) byte1);
#else
	return value;
#endif
}

ulong swap32 (ulong value)
{
#if 1
	uchar byte1;
	uchar byte2;
	uchar byte3;
	uchar byte4;

	byte1 = (uchar) ((value >> 24) & 0xff);
	byte2 = (uchar) ((value >> 16) & 0xff);
	byte3 = (uchar) ((value >> 8) & 0xff);
	byte4 = (uchar) (value & 0xff);

	return ((ulong) byte4 << 24) | ((ulong) byte3 << 16) | ((ulong) byte2 << 8) | ((ulong) byte1);
#else
	return value;
#endif
}

ulong htonl (ulong value)
{
	return swap32 (value);
}

ulong ntohl (ulong value)
{
	return swap32 (value);
}

ushort htons (ushort value)
{
	return swap16 (value);
}

ushort ntohs (ushort value)
{
	return swap16 (value);
}

void setenv (char *var, char *value)
{
}

int ctrlc (void)
{
	return 0;
}

uint  malloc_array [8][512];
int malloc_flag [8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

void init_malloc (void)
{
	int i;

	for (i = 0; i < 8; i++)
		malloc_flag [i] = 0;
}

void free (void *addr)
{
	int i;

	for (i = 0; i < 8; i++) {
		if (malloc_array [i] == addr) {
			malloc_flag [i] = 0;
			return;
		}
	}
}

void *malloc (int size)
{
	int i;

	if (size > 2048)
		return NULL;
	for (i = 0; i < 8; i++) {
		if (malloc_flag [i] == 0) {
			malloc_flag [i] = 1;
			return malloc_array [i];
		}
	}
	return NULL;
}

#define BASE_MAX	36
static const char digits[] = {
	"0123456789abcdefghijklmnopqrstuvwxyz" };
static const char ndigs[BASE_MAX+1] = {
	0, 0, 33, 21, 17, 14, 13, 12, 11, 11,
	10, 10, 9, 9, 9, 9, 9, 8, 8, 8,
       	8, 8, 8, 8, 7, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 	
};

int is_space (char c)
{
	if (c == ' ' || c == '\t')
		return 1;
	return 0;
}

char tolower (char c)
{
	if (c >= 'A' && c <= 'Z')
		return (c - 'A' + 'a');
	else
	return (c);
}

void *memchr (const void *s, int c, int n)
{
	const uchar uc = c;
	const uchar *su;

	for (su = s; 0 < n; ++su, --n)
		if (*su == uc)
			return ((void *) su);
	return NULL;
}

#if 0
void *memcpy (void *s1, const void *s2, int n)
{
	char *su1;
	const char *su2;

	for (su1 = s1, su2 = s2; 0 < n; ++su1, ++su2, --n)
		*su1 = *su2;
	return s1;
}

void *memset (void *s, int c, int n)
{
	const uchar uc = c;
	uchar *su;

	for (su = s; 0 < n; ++su, --n)
		*su = uc;
	return s;
}

int strlen (const char *s)
{
	const char *sc;

	for (sc = s; *sc != '\0'; ++sc);
	return (sc - s);
}
#endif

int memcmp (const void *s1, const void *s2, uint n)
{
	const uchar *su1, *su2;

	for (su1 = s1, su2 = s2; 0 < n; ++su1, ++su2, --n)
		if (*su1 != *su2)
			return ((*su1 < *su2) ? -1 : +1);
	return 0;
}

int strcmp (const char *s1, const char *s2)
{
	for (; *s1 == *s2; ++s1, ++s2)
		if (*s1 == '\0')
			return 0;
	return ((*(uchar *) s1 < *(uchar *) s2) ? -1 : +1);
}

char *strcpy (char *s1, const char *s2)
{
	char *s = s1;

	for (s = s1; (*s++ = *s2++) != '\0'; );
	return s1;
}

ulong do_divide (ulong x, ulong y)
{
	ulong result;

	result = 0;
	while (x >= y) {
		x -= y;
		result++;
	}
	return result;
}

ulong do_mod (ulong x, ulong y)
{
  ulong quotient;

  quotient = do_divide (x, y);
  return x - (quotient * y);
}

ulong _Stoul (const char *s, char **endptr, int base)
{
	const char *sc, *sd;
	const char *s1, *s2;
	char sign;
	int n;
	ulong x, y;

	for (sc = s; is_space (*sc); ++sc);
	sign = *sc == '-' || *sc == '+' ? *sc++ : '+';
	if (base < 0 || base == 1 || BASE_MAX < base)
	{
		if (endptr)
			*endptr = (char *) s;
		return 0;
	}
	else if (base) {
		if (base == 16 && *sc == '0' &&
			(sc [1] == 'x' || sc [1] == 'X'))
			sc += 2;
	}
	else if (*sc != '0')
		base = 10;
	else if (sc [1] == 'x' || sc [1] == 'X')
		base = 16, sc += 2;
	else
		base = 8;
	for (s1 = sc; *sc == '0'; ++sc);
	x = 0;
	y = 0;
	for (s2 = sc; (sd = memchr(digits, tolower(*sc), base)) != NULL; ++sc)
	{
		y = x;
		x = x * base + (sd - digits);
	}
	if (s1 == sc)
	{
		if (endptr)
			*endptr = (char *) s;
		return 0;
	}
	n = (int) (sc - s2 - ndigs [base]);
	if (n < 0)
		;
	else if (0 < n || x < x - sc [-1] ||
		do_divide ((x - sc [-1]), base) != y)
		x = 0x7fffffff;
	if (sign == '-')
		x = -x;
	if (endptr)
		*endptr = (char *) sc;
	return x;
}

ulong simple_strtoul (const char *nptr, char **endptr, int base)
{
	return _Stoul (nptr, endptr, base);
}

#ifndef NULL
#define NULL ((void *)0)
#endif

uchar hdigit [16] = "0123456789ABCDEF";
unsigned long dec_mask [10];

void init_util (void)
{
  int i;

  for (i = 0; i < 10; i++)
    hdigit [i] = (uchar) (i + '0');
  for (i = 10; i < 16; i++)
    hdigit [i] = (uchar) (i - 10 + 'A');
  dec_mask [0] = 1000000000;
  dec_mask [1] = 100000000;
  dec_mask [2] = 10000000;
  dec_mask [3] = 1000000;
  dec_mask [4] = 100000;
  dec_mask [5] = 10000;
  dec_mask [6] = 1000;
  dec_mask [7] = 100;
  dec_mask [8] = 10;
  dec_mask [9] = 1;
}

void print_hex (ulong num)
{
  uchar buffer [9];
  int i;

  buffer [8] = '\0';
  for (i = 7; i >= 0; i--) {
    buffer [7-i] = hdigit [((num >> (i << 2)) & 0xf)];
  }
  putString (buffer);
}

void print_hex4 (ushort num)
{
  uchar buffer [5];
  int i;

  buffer [4] = '\0';
  for (i = 4; i >= 0; i--) {
    buffer [3-i] = hdigit [((num >> (i << 2)) & 0xf)];
  }
  putString (buffer);
}

void print_hex2 (uchar num)
{
  uchar buffer [3];

  buffer [2] = '\0';
  buffer [0] = hdigit [(num >> 4) & 0xf];
  buffer [1] = hdigit [num & 0xf];
  putString (buffer);
}

void print_decimal (ulong num)
{
  int i;
  ulong digit;

  i = 0;
  while (num < dec_mask [i])
    i++;

  while (i <= 9) {
    digit = do_divide (num, dec_mask [i]);
    putDebugChar (digit + '0'); 
    num = num - (digit * dec_mask [i]);
    i++;
  }
}

uchar *uchar_to_buffer (uchar num, uchar *start)
{
  uint len;
  uchar *ptr;

  if (num == 0) {
    *start++ = '0';
    *start = '\0';
    return start;
  }

  if (num >= 100)
    len = 3;
  else if (num >= 10)
    len = 2;
  else 
    len = 1;

  *(start+len) = '\0';
  ptr = start+len;
  while (num != 0) {
    ptr--;
    *ptr = hdigit [do_mod ((uint) num, 10)];
    num = (uchar) do_divide ((uint) num, 10); 
  } 
  return (start+len);
}

uchar *uint_to_buffer (uint num, uchar *start)
{
  return start;
}

uchar *hex_to_buffer (uint num, uchar *start)
{
  int i;

  start [8] = '\0';
  for (i = 7; i >= 0; i--) {
    start [7-i] = hdigit [((num >> (i << 2)) & 0xf)];
  }
  return start+8;
}

