/*
 ****************************************************************************
 *  File:  931sci.c
 *
 *	Copyright 2004 Standard Microsystems Corporation.  All rights reserved.
 *
 *	Serial Register reader/writer for LAN9312/13 device.
 *
 ****************************************************************************
 *
 *	AH  1/5/2007
 *		Initial writing
 ****************************************************************************
 */

#include "931xControlInterface.h"
#include "sh3_polaris.h"
#include <linux/delay.h>


/*
 ****************************************************************************
 *	Macro
 ****************************************************************************
 *
 */



static DWORD  i2cMaxRetry = 0;

/*
 ****************************************************************************
 *	Static Function Prototypes
 ****************************************************************************
 *
 */


void SmiSetRegMsw(const DWORD dwLanBase,
                    DWORD regAddress,
                    DWORD wVal);
void SmiSetRegLsw(const DWORD dwLanBase,
                    DWORD regAddress,
                    DWORD wVal);
WORD SmiGetRegMsw(const DWORD dwLanBase,
                    DWORD regAddress
                    );
WORD SmiGetRegLsw(const DWORD dwLanBase,
                    DWORD regAddress
                    );

static BYTE BitSwap(const BYTE bVal);
static BOOLEAN SpiTxW(const WORD wVal);
static BOOLEAN SpiTxDW(const DWORD dwVal);
static BOOLEAN SpiRxDW(DWORD * dwVal);
static DWORD SpiSingleRead(const DWORD dwOffset);
static void SpiSingleWrite(const DWORD dwOffset, const DWORD dwVal);


static BOOLEAN I2cNotBusy(BYTE * bI2cState);
static BOOLEAN IsI2cStopped(void);
static BOOLEAN I2cLeoAccessCmd(const DWORD dwOffset, const BOOLEAN bI2cStopEnable);


inline void PlatformSetRegDW(
		DWORD dwLanBase,
		DWORD dwOffset,
		DWORD dwVal)

{
#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
        SMSC_TRACE(" !!!!!!!!!! Reg Offset=0x%lx, new Value=0x%lx", dwOffset, dwVal);
    }
#endif

	(*(volatile DWORD *)(dwLanBase+dwOffset))=dwVal;
}

inline DWORD PlatformGetRegDW(
		DWORD dwLanBase,
	   DWORD dwOffset)
{
#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
        DWORD dwVal = *(volatile DWORD *)(dwLanBase+dwOffset);
        SMSC_TRACE("  reg Offset=0x%lx, value=0x%lx", dwOffset, dwVal);
        return dwVal;
    }
#endif
	return (*(volatile DWORD *)(dwLanBase+dwOffset));
}


/*
 ****************************************************************************
 *	SMI Access Registers Reader/Writers
 ****************************************************************************
 *
 */
void SmiLanSetRegDW( const DWORD dwLanBase, const DWORD dwOffset, const DWORD dwVal)
{
	/** If register is for 9217 readi it directly **/

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
        SMSC_TRACE(" lanBase=0x%lx, dwOffset=0x%lx, new Value=0x%lx\n", dwLanBase,dwOffset, dwVal);
        }
#endif
	{
		SmiSetRegLsw( dwLanBase, dwOffset, dwVal );
		SmiSetRegMsw( dwLanBase, dwOffset, dwVal);
	}

}



DWORD SmiLanGetRegDW( const DWORD dwLanBase, const DWORD dwOffset)
{
	DWORD dwTemp;

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE("!!!!!!!!!!!!!!!!!!!!!! lanBase=0x%lx, dwOffset=0x%lx\n", dwLanBase,dwOffset);
      }
#endif
		dwTemp = (DWORD)SmiGetRegMsw( dwLanBase, dwOffset);
		dwTemp <<= 16UL;
		dwTemp |= (DWORD)SmiGetRegLsw( dwLanBase, dwOffset);

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!!!!!!!!!!!!!!!!!Value Read =0x%lx", dwTemp);
      }
#endif
	return dwTemp;
}



void SpiLanSetRegDW( const DWORD dwLanBase, const DWORD dwOffset, const DWORD dwVal)
{
	DWORD dwTemp;

	// Keep Lint Happy
	dwTemp = dwLanBase;
	dwTemp = dwTemp;

	SpiSingleWrite(dwOffset, dwVal);
}

DWORD SpiLanGetRegDW( const DWORD dwLanBase, const DWORD dwOffset)
{
	DWORD dwTemp;

	// Keep Lint Happy
	dwTemp = dwLanBase;
	dwTemp = dwTemp;

	return SpiSingleRead(dwOffset);
}
/*
 ****************************************************************************
 *	Helper Function for SPI Access Registers Reader/Writers
 ****************************************************************************
 *
 */
static BYTE BitSwap(const BYTE bVal)
{
	BYTE bTemp = (BYTE)0;
	int i;

	for (i=0; i<8; i++)
	{
		if (bVal & (0x80 >> i))
		{
			bTemp |= (0x01 << i);
		}
	}

	return bTemp;
}

static BOOLEAN SpiTxW(const WORD wVal)
{
#ifdef _SPI_EXTERNAL_CLK_MODE
	int j;
#else
	DWORD dwStartTick = 1000UL;
#endif
	WORD i;


	for (i=0; i<2; i++)
	{
#ifdef _SPI_EXTERNAL_CLK_MODE
		// Write Swaped Tx Byte to SCTDR
		SCTDR = BitSwap((BYTE)((wVal >> ((1 - i)*8)) & (WORD)0xFF));

		SCSSR &= (~SCSSR_TDRE_); // Clear TDRE
		GpioOut(0UL, GPIO3, GPIO_HI); // External Clock Enable
		for(j=0; j<100; j++)          // Needs tuning if SH3 internal frequency changed.
		{
		        continue;
		}
		GpioOut(0UL, GPIO3, GPIO_LO); // External Clock Disable
#else
		dwStartTick = 1000UL;
		// Wait for TDRE becomes set
		while (!(SCSSR & SCSSR_TDRE_))
		{
			//  mobbad last udelay(1);
			if (dwStartTick-- == 0)
			{
				SMSC_ERROR("ERROR: SPI Tx Done timeout in SpiTxW(), SCSSR = 0x%04X\n", SCSSR);
				return FALSE;
			}
		}

		// Write Swaped Tx Byte to SCTDR
		SCTDR = BitSwap((BYTE)((wVal >> ((1 - i)*8)) & (WORD)0xFF));

		SCSSR &= (~SCSSR_TDRE_); // Clear TDRE
#endif
	}
#ifndef _SPI_EXTERNAL_CLK_MODE

	// Wait for TEND becomes set
	dwStartTick = 1000UL;
	while (!(SCSSR & SCSSR_TEND_))
	{
		// mobbad last udelay(1);
		if (dwStartTick-- == 0)
		{
			SMSC_ERROR("ERROR: SPI Tx End timeout in SpiTxW(), SCSSR = 0x%04X\n", SCSSR);
			return FALSE;
		}
	}
#endif
	return TRUE;
}

static BOOLEAN SpiTxDW(const DWORD dwVal)
{
	// Tx HIWORD
	if (!(SpiTxW(HIWORD(dwVal))))
	{
		return FALSE;
	}

	// Tx LOWORD
	if (!(SpiTxW(LOWORD(dwVal))))
	{
		return FALSE;
	}

	return TRUE;
}

static BOOLEAN SpiRxDW(DWORD * dwVal)
{
	DWORD dwStartTick, i;
	BYTE bTemp, bRxBuf[4];

	// Clear Rx Ready Flag
	SCSSR &= (~SCSSR_RDRF_);

	// Fill with Dummy Data
	SCTDR = 0xFF;

	dwStartTick = 5000UL;
	for (i=0UL; i<4UL; i++)
	{
		// Tx Dummy Data to generate SpiClk
		SCSSR &= (~SCSSR_TDRE_); // Clear TDRE
#ifdef _SPI_EXTERNAL_CLK_MODE
		GpioOut(0UL, GPIO3, GPIO_HI);
#endif


		// Check RDRF before writing to SCRDR
		while (!(SCSSR & SCSSR_RDRF_))
		{
			// Check for overrun bit set at the enterance
			if (SCSSR & SCSSR_ORER_)
			{
				SMSC_ERROR("ERROR: SPI Rx Overrun error in SpiRxDW(), SCSSR = 0x%04X\n", SCSSR);

				// Check ORER bit for the 2nd time
				if (SCSSR & SCSSR_ORER_)
				{
					// purge any stall data in Rx Data Reg
					bTemp = SCRDR;
					bTemp = bTemp;

					// Reset ORER bit
					SCSSR &= (~SCSSR_ORER_);
#ifdef _SPI_EXTERNAL_CLK_MODE
					GpioOut(0UL, GPIO3, GPIO_LO);
#endif
					return FALSE;
				}
				else
				{
					SMSC_ERROR("Warning: ORER cleared after 2nd read!!!, SCSSR = 0x%04X\n", SCSSR);
					// Reset ORER bit
					SCSSR &= (~SCSSR_ORER_);
				}
			}
			else
			{
#ifdef _SPI_EXTERNAL_CLK_MODE
				if (SCSSR & SCSSR_RDRF_)
				{
				        GpioOut(0UL, GPIO3, GPIO_LO);
				}
#endif
				// a DW read should not be taken more than 5 msec at 500KHz Spi clk
				udelay(1);
				if (dwStartTick-- == 0)
				{
#ifdef _SPI_EXTERNAL_CLK_MODE
					GpioOut(0UL, GPIO3, GPIO_LO);
#endif
					SMSC_ERROR("ERROR: SPI Rx Data Ready reset timeout!, SCSSR = 0x%04X\n", SCSSR);
					return FALSE;
				}
			}
		}
#ifdef _SPI_EXTERNAL_CLK_MODE
		GpioOut(0UL, GPIO3, GPIO_LO);
#endif


		// Read Rx Data Reg.
		bRxBuf[i] = SCRDR;

		// Clear Rx Ready Flag
		SCSSR &= (~SCSSR_RDRF_);
	}

	for (i=0UL;i<4UL;i++)
	{
		bRxBuf[i] = BitSwap(bRxBuf[i]);
	}

	*dwVal = MAKEDWORD (MAKEWORD(bRxBuf[3],bRxBuf[2]), (DWORD)MAKEWORD(bRxBuf[1], bRxBuf[0]));

	return TRUE;
}


/**
 **/

static DWORD SpiSingleRead(const DWORD dwOffset)
{
	WORD wTemp;
	DWORD dwVal;

#ifndef INIT_SPI_AT_STARTUP
	ClearSpiCs();
#ifndef _SPI_EXTERNAL_CLK_MODE
	udelay(5UL); //Give 5uS of setup for Chip select
#endif

	// Enable SPI Tx
	EnableSpiTx();
#endif

	wTemp = (WORD)SPI_READ_CMD;
	wTemp <<= (WORD)8;
	wTemp |= (WORD)((dwOffset & 0x03FCUL) >> 2UL); // A9-A2 bits of the address

	// Transmit RD Command & Address
	if (!SpiTxW(wTemp))
	{
		SMSC_ERROR("ERROR: SpiTxW() failed in SpiReadSingle()\n");
		return 0xFFFFFFFFUL;
	}

#ifndef INIT_SPI_AT_STARTUP
	EnableSpiRx();
#endif

	if (!SpiRxDW(&dwVal))
	{
		SMSC_ERROR("ERROR: SpiRxDW() failed in SpiReadSingle()\n");
		return 0xFFFFFFFFUL;
	}

#ifndef INIT_SPI_AT_STARTUP
	DisableSpiRx();
	DisableSpiTx();
	SetSpiCs();
#endif
	return dwVal;
}

static void SpiSingleWrite(const DWORD dwOffset, const DWORD dwVal)
{
	WORD wTemp;

#ifndef INIT_SPI_AT_STARTUP
	ClearSpiCs();
#ifndef _SPI_EXTERNAL_CLK_MODE
	udelay(5UL); //Give 5uS of setup for Chip select
	// Enable SPI Tx
#endif
	EnableSpiTx();
#endif

	wTemp = (WORD)SPI_WRITE_CMD;
	wTemp <<= (WORD)8;
	wTemp |= (WORD)((dwOffset & 0x03FCUL) >> 2UL); // A9-A2 bits of the address


	// Transmit WR Command & Address
	if (!SpiTxW(wTemp))
	{
		SMSC_ERROR("ERROR: SpiTxW() failed in SpiReadSingle()\n");
		return;
	}

	// Transmit Data
	if (!SpiTxDW(dwVal))
	{
		SMSC_ERROR("ERROR: SpiTxDW() failed in SpiReadSingle()\n");
		return;
	}

#ifndef INIT_SPI_AT_STARTUP
	udelay(5UL); //Give 5uS of hold for Chip select
	DisableSpiTx();
	SetSpiCs();
#endif
}



BOOLEAN SpiMultipleRead(const BOOLEAN bDirection, const DWORD dwOffset, const DWORD dwSize, DWORD *pdwBuf)
{
	WORD wTemp;
	DWORD i;

#ifndef INIT_SPI_AT_STARTUP
	ClearSpiCs();
	udelay(5UL); //Give 5uS of setup for Chip select

	// Enable SPI Tx
	EnableSpiTx();
#endif

	if (bDirection == SPI_AUTO_INCREMENT)
	{
		wTemp = (WORD)SPI_READ_INC_CMD;
	}
	else
	{
		wTemp = (WORD)SPI_READ_DEC_CMD;
	}
	wTemp <<= (WORD)8;
	wTemp |= (WORD)((dwOffset & 0x03FCUL) >> 2UL); // A9-A2 bits of the address

	// Transmit RD Command & Address
	if (!SpiTxW(wTemp))
	{
		SMSC_ERROR("ERROR: SpiTxW() failed in SpiMultipleRead()\n");
		return FALSE;
	}
#ifndef INIT_SPI_AT_STARTUP
	EnableSpiRx();
#endif

	for (i=0UL; i<dwSize; i++)
	{
		if (!SpiRxDW(&pdwBuf[i]))
		{
			SMSC_ERROR("ERROR: SpiRxDW() failed in SpiMultipleRead()\n");
			return FALSE;
		}
	}

#ifdef INIT_SPI_AT_STARTUP
	DisableSpiRx();
	DisableSpiTx();
	SetSpiCs();
#endif

	return TRUE;
}

BOOLEAN SpiMultipleWrite(const BOOLEAN bDirection, const DWORD dwOffset, const DWORD dwSize, const DWORD * const pdwBuf)
{
	WORD wTemp;
	DWORD i;

#ifndef INIT_SPI_AT_STARTUP
	ClearSpiCs();
	udelay(5UL); //Give 5uS of setup for Chip select
	// Enable SPI Tx
	EnableSpiTx();
#endif

	if (bDirection == SPI_AUTO_INCREMENT)
	{
		wTemp = (WORD)SPI_WRITE_INC_CMD;
	}
	else
	{
		wTemp = (WORD)SPI_WRITE_DEC_CMD;
	}
	wTemp <<= (WORD)8;
	wTemp |= (WORD)((dwOffset & 0x03FCUL) >> 2UL); // A9-A2 bits of the address


	// Transmit WR Command & Address
	if (!SpiTxW(wTemp))
	{
		SMSC_ERROR("ERROR: SpiTxW() failed in SpiReadSingle()\n");
		return FALSE;
	}

	for (i=0UL; i<dwSize; i++)
	{
		// Transmit Data
		if (!SpiTxDW(pdwBuf[i]))
		{
			SMSC_ERROR("ERROR: SpiTxDW() failed in SpiReadSingle()\n");
			return FALSE;
		}
	}

#ifndef INIT_SPI_AT_STARTUP
	udelay(5UL); //Give 5uS of hold for Chip select
	DisableSpiTx();
	SetSpiCs();
#endif

	return TRUE;

}

/*
 ****************************************************************************
 *	I2C Access Registers Reader/Writers
 ****************************************************************************
 *
 */



void I2cLanSetRegDW( const DWORD dwLanBase, const DWORD dwOffset, const DWORD dwVal)
{
	DWORD dwTemp = dwLanBase;

	//Kepp lint happy
	dwTemp = dwTemp;

	if (!I2cMasterWrite(dwOffset, 1UL, &dwVal))
	{
		SMSC_ERROR("ERROR: I2cMasterWrite() failed in I2cLanSetRegDW()!\n");
	}

  // SMSC_TRACE ("----- i2cMaxRetry = %lu ", i2cMaxRetry);
}

DWORD I2cLanGetRegDW( const DWORD dwLanBase, const DWORD dwOffset)
{
	DWORD dwTemp = dwLanBase;

	//Kepp lint happy
	dwTemp = dwTemp;

	if (!I2cMasterRead(dwOffset, 1UL, &dwTemp))
	{
		SMSC_ERROR("ERROR: I2cMasterRead() failed in I2cLanGetRegDW()!\n");
	}

    // SMSC_TRACE ("----- i2cMaxRetry = 0x%lu ", i2cMaxRetry);
	return dwTemp;
}


/*
 ****************************************************************************
 *	Helper Function for I2C Access Registers Reader/Writers
 ****************************************************************************
 *
 */
BYTE I2cGetRegB(const DWORD dwI2cBase, const DWORD dwOffset)
{
	return (BYTE)(*(volatile BYTE *)(dwI2cBase + dwOffset));
}

void I2cSetRegB(const DWORD dwI2cBase, const DWORD dwOffset, const BYTE bVal)
{
	(*(volatile BYTE *)(dwI2cBase + dwOffset)) = bVal;
}

static BOOLEAN I2cNotBusy(BYTE * bI2cState)
{
	DWORD dwStartTick = 10000UL; //  was 1000UL

	while (!(I2cGetRegB(I2C_CSBASE, I2CCON) & I2CCON_SI_))
	{
		if (dwStartTick-- == 0)
		{
			*bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
			SMSC_ERROR("ERROR: Wait for SI bit set timeout in I2cNotBusy(), I2CCON=0x%02X, I2CSTA=0x%02X\n",
					I2cGetRegB(I2C_CSBASE, I2CCON), *bI2cState);

			return FALSE;
		}
	}

	*bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
	return TRUE;
}

BOOLEAN IsI2cStopped(void)
{
	DWORD dwStartTick= 1000UL;

	while (I2cGetRegB(I2C_CSBASE, I2CCON) & I2CCON_STO_)
	{
		if (dwStartTick-- == 0)
		{
			SMSC_ERROR("ERROR: Wait for STO bit clear timeout in IsI2cStopped(), I2CCON=0x%02X\n", I2cGetRegB(I2C_CSBASE, I2CCON));
			return FALSE;
		}
	}

	return TRUE;
}

BOOLEAN I2cLeoAccessCmd(const DWORD dwOffset, const BOOLEAN bI2cStopEnable)
{
	BYTE bTemp, bI2cState;
	BOOLEAN fResult = TRUE;
    DWORD retry = 0;
    BYTE i2cBaseAddr;

	// Generate Start Command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CCON);
	bTemp |= I2CCON_STA_;
	bTemp &= (~I2CCON_SI_);
	I2cSetRegB(I2C_CSBASE, I2CCON, bTemp);
	if (!I2cNotBusy(&bI2cState))
	{
		SMSC_ERROR("ERROR: I2cNotBusy() failed after setting STA in I2cMasterRead()\n");
		return FALSE;
	}

	// I2C state after start command
	if (bI2cState != (BYTE)0x08) // Start condition has been transmitted
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after start command in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
		return FALSE;
	}
	if(ACCESS_STRAP_REGISTER)
		i2cBaseAddr= LEO_I2C_STRAP_ADDR;
	else
	    i2cBaseAddr= LEO_I2C_ADDR;
	// Load SLA + nW
	I2cSetRegB(I2C_CSBASE, I2CDAT, (BYTE)((i2cBaseAddr << (BYTE)1) | I2C_WRITE_COMMAND));
	I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~(I2CCON_STA_ + I2CCON_SI_))));
	if (!I2cNotBusy(&bI2cState))
	{
		SMSC_ERROR("ERROR: I2cNotBusy() failed after setting SLA+W in I2cLeoAccessCmd()\n");
		return FALSE;
	}

	// Check possible I2C states
	if (bI2cState == (BYTE)0x20) // Not acknowledge received after SLA+W
	{
		SMSC_ERROR("ERROR: Not acknowledge received after SLA+W, I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_ACC_STOP;
	}
	else if (bI2cState == (BYTE)0x38)
	{
		SMSC_ERROR("ERROR: Arbitration lost in SLA or Ack bit after SLA+W, I2CSTA=0x%02X\n", bI2cState);
		return FALSE;
	}
	else if (bI2cState != (BYTE)0x18)
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after SLA+W in I2cLeoAccessCmd(), I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_ACC_STOP;
	}
	else
	{
		// Keep lint happy
	}

	// Leo Address Phase
	I2cSetRegB(I2C_CSBASE, I2CDAT, (BYTE)((dwOffset>>2UL) & 0xFFUL)); // A9-A2 bits
	I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~I2CCON_SI_)));
	if (!I2cNotBusy(&bI2cState))
	{
		SMSC_ERROR("ERROR: I2cNotBusy() failed after Leo Addrress phase in I2cLeoAccessCmd()\n");
		return FALSE;
	}

	if (bI2cState == (BYTE)0x30) // Not acknowledge received after Data Byte
	{
		SMSC_ERROR("ERROR: Not acknowledge received after Leo Addrress phase in I2cLeoAccessCmd(), I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_ACC_STOP;
	}
	else if (bI2cState == (BYTE)0x38)
	{
		SMSC_ERROR("ERROR: Arbitration lost in SLA or Ack bit in Leo Addrress phase, I2CSTA=0x%02X\n", bI2cState);
		return FALSE;
	}
	else if (bI2cState != (BYTE)0x28)
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after Leo Addrress phase in I2cLeoAccessCmd(), I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_ACC_STOP;
	}
	else
	{
		if (bI2cStopEnable == TRUE)
		{
			fResult = TRUE;
		}
		else
		{
			return TRUE;
		}
	}

_I2C_ACC_STOP:
	// Generate Stop Command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CCON);
	bTemp |= (I2CCON_STO_ + I2CCON_AA_);
	bTemp &= (~I2CCON_SI_);
	I2cSetRegB(I2C_CSBASE, I2CCON, bTemp);
	if (!IsI2cStopped())
	{
		SMSC_ERROR("ERROR: IsI2cStopped() failed after STOP command in I2cLeoAccessCmd()\n");
		return FALSE;
	}
    do {
	       bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
           retry++;

    } while ( (bI2cState != (BYTE)0xF8) && (retry++ < I2C_SPI_MAX_RETRY ) );

    if (retry >= I2C_SPI_MAX_RETRY)
    {
        SMSC_ERROR("ERROR 1: Unexpected I2C state after STOP command in I2cLeoAccessCmd(), I2CSTA=0x%02X\n", bI2cState);
    }

    if (i2cMaxRetry < retry)
    {
        i2cMaxRetry = retry;
    }
	return fResult;
}

BOOLEAN I2cMasterRead(const DWORD dwOffset, const DWORD dwSize, DWORD *pdwBuf)
{
	BYTE bTemp, bI2cState, bBuf[4];
	DWORD i;
	int j;
	BOOLEAN fResult = TRUE;
    DWORD retry = 0;
	BYTE i2cBaseAddr;

	// Check I2CSTA at the beginning
	bTemp = I2cGetRegB(I2C_CSBASE, I2CSTA);
	if (bTemp != (BYTE)0xF8) // Default State
	{
		SMSC_ERROR("ERROR: I2C state is not default at the beginning of I2cMasterRead(), I2CCON=0x%02X, I2CSTA=0x%02X\n", I2cGetRegB(I2C_CSBASE, I2CCON), bTemp);
		return FALSE;
	}

	// Access to Leo register command
	if (!I2cLeoAccessCmd(dwOffset, TRUE))
	{
		return FALSE;
	}

	// Check I2CSTA after Leo Reg Access command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CSTA);
	if (bTemp != (BYTE)0xF8) // Default State
	{
		SMSC_ERROR("ERROR: I2C state is not default after I2cLeoAccessCmd() in I2cMasterRead(), I2CCON=0x%02X, I2CSTA=0x%02X\n", I2cGetRegB(I2C_CSBASE, I2CCON), bTemp);
		return FALSE;
	}

	// Generate Start Command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CCON);
	bTemp |= I2CCON_STA_;
	bTemp &= (~I2CCON_SI_);
	I2cSetRegB(I2C_CSBASE, I2CCON, bTemp);
	if (!I2cNotBusy(&bI2cState))
	{
		SMSC_ERROR("ERROR: I2cNotBusy() failed after setting STA in I2cMasterRead()\n");
		return FALSE;
	}

	// I2C state after start command
	if (bI2cState != (BYTE)0x08) // Start condition has been transmitted
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after start command in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
		return FALSE;
	}
	if(ACCESS_STRAP_REGISTER)
		i2cBaseAddr= LEO_I2C_STRAP_ADDR;
	else
	    i2cBaseAddr= LEO_I2C_ADDR;

	// Load SLA + R
	I2cSetRegB(I2C_CSBASE, I2CDAT, (BYTE)((i2cBaseAddr << (BYTE)1) | I2C_READ_COMMAND));
	I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~(I2CCON_STA_ + I2CCON_SI_))));
	if (!I2cNotBusy(&bI2cState))
	{
		SMSC_ERROR("ERROR: I2cNotBusy() failed after setting SLA+R in I2cMasterRead()\n");
		return FALSE;
	}

	// Check possible I2C states
	if (bI2cState == (BYTE)0x48) // Not acknowledge received after SLA+R
	{
		SMSC_ERROR("ERROR: Not acknowledge received after SLA+R, I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_RD_STOP;
	}
	else if (bI2cState == (BYTE)0x38)
	{
		SMSC_ERROR("ERROR: Arbitration lost in SLA or Ack bit after SLA+R, I2CSTA=0x%02X\n", bI2cState);
		return FALSE;
	}
	else if (bI2cState != (BYTE)0x40)
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after SLA+R in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
		goto _I2C_RD_STOP;
	}
	else
	{
		// Keep lint happy
	}

	// Data Read Phase
	for (i=0UL; i<dwSize; i++)
	{
		// DWORD Read Only
		for (j=0; j<4; j++)
		{
			if ((i == (dwSize - 1UL)) && (j==3)) // Reading last byte of the last DWORD
			{
				I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~(I2CCON_AA_ + I2CCON_SI_)))); // No Ack for the last byte
			}
			else
			{
				I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~I2CCON_SI_)));
			}

			if (!I2cNotBusy(&bI2cState))
			{
				SMSC_ERROR("ERROR: I2cNotBusy() failed in data read phase in I2cMasterRead()\n");
				return FALSE;
			}

			if ((i == (dwSize - 1UL)) && (j==3)) // Reading last byte of the last DWORD
			{
				if (bI2cState == (BYTE)0x38)
				{
					SMSC_ERROR("ERROR: Arbitration lost in SLA or Ack bit in data read phase, I2CSTA=0x%02X\n", bI2cState);
					return FALSE;
				}
				else if (bI2cState != (BYTE)0x58)
				{
					SMSC_ERROR("ERROR: Unexpected I2C state in the last byte read in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
					fResult = FALSE;
					goto _I2C_RD_STOP;
				}
				else
				{
					// keep lint happy
				}
			}
			else
			{
				if (bI2cState != (BYTE)0x50) // Data byte has been received with Ack
				{
					SMSC_ERROR("ERROR: Unexpected I2C state in data read phase in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
					fResult = FALSE;
					goto _I2C_RD_STOP;
				}
			}

			// Read data byte
			bBuf[j] = I2cGetRegB(I2C_CSBASE, I2CDAT);
		}

		pdwBuf[i] = MAKEDWORD(MAKEWORD(bBuf[3],bBuf[2]), (DWORD)MAKEWORD(bBuf[1], bBuf[0]));
	}

_I2C_RD_STOP:
	// Generate Stop Command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CCON);
	bTemp |= (I2CCON_STO_ + I2CCON_AA_);
	bTemp &= (~I2CCON_SI_);
	I2cSetRegB(I2C_CSBASE, I2CCON, bTemp);
	if (!IsI2cStopped())
	{
		SMSC_ERROR("ERROR: IsI2cStopped() failed after STOP command in I2cMasterRead()\n");
		return FALSE;
	}

    retry = 0;
    do {
	       bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
       } while  ( (bI2cState != (BYTE)0xF8) && (retry++ < I2C_SPI_MAX_RETRY ) ) ;


    if  (retry  >= I2C_SPI_MAX_RETRY )
    {
        SMSC_ERROR("ERROR: Unexpected I2C state after STOP command in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
        fResult = FALSE;
    }

    if (i2cMaxRetry < retry)
    {
        i2cMaxRetry = retry;
    }


#if 0
	if (bI2cState != (BYTE)0xF8) // Default state
	{
		SMSC_ERROR("ERROR: Unexpected I2C state after STOP command in I2cMasterRead(), I2CSTA=0x%02X\n", bI2cState);
		fResult = FALSE;
	}
#endif

	return fResult;
}

BOOLEAN I2cMasterWrite(const DWORD dwOffset, const DWORD dwSize, const DWORD * const pdwBuf)
{
	BYTE bTemp, bI2cState, bBuf[4];
	DWORD i;
	int j;
	BOOLEAN fResult = TRUE;
    DWORD retry = 0;

	// Check I2CSTA at the beginning
	bTemp = I2cGetRegB(I2C_CSBASE, I2CSTA);
	if (bTemp != (BYTE)0xF8) // Default State
	{
		SMSC_ERROR("ERROR: I2C state is not default at the beginning of I2cMasterWrite(), I2CCON=0x%02X, I2CSTA=0x%02X\n",
				I2cGetRegB(I2C_CSBASE, I2CCON), bTemp);
		return FALSE;
	}

	// Access to Leo register command
	if (!I2cLeoAccessCmd(dwOffset, FALSE))
	{
		return FALSE;
	}

	// Data Write Phase
	for (i=0UL; i<dwSize; i++)
	{
		bBuf[3] = LOBYTE(LOWORD(pdwBuf[i]));
		bBuf[2] = HIBYTE(LOWORD(pdwBuf[i]));
		bBuf[1] = LOBYTE(HIWORD(pdwBuf[i]));
		bBuf[0] = HIBYTE(HIWORD(pdwBuf[i]));

		// DWORD Write Only
		for (j=0; j<4; j++)
		{
			// Write Data Byte
			I2cSetRegB(I2C_CSBASE, I2CDAT, bBuf[j]);
			I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2cGetRegB(I2C_CSBASE, I2CCON) & (~I2CCON_SI_)));

			if (!I2cNotBusy(&bI2cState))
			{
				SMSC_ERROR("ERROR: I2cNotBusy() failed in data read phase in I2cMasterWrite()\n");
				return FALSE;
			}

			if (bI2cState == (BYTE)0x30) // Not acknowledge received after Data Byte
			{
				SMSC_ERROR("ERROR: Not acknowledge received after data byte in I2cMasterWrite(), I2CSTA=0x%02X\n", bI2cState);
				fResult = FALSE;
				goto _I2C_WR_STOP;
			}
			else if (bI2cState == (BYTE)0x38)
			{
				SMSC_ERROR("ERROR: Arbitration lost in SLA or Ack bit in data write phase, I2CSTA=0x%02X\n", bI2cState);
				return FALSE;
			}
			else if (bI2cState != (BYTE)0x28)
			{
				SMSC_ERROR("ERROR: Unexpected I2C state after data byte in I2cMasterWrite(), I2CSTA=0x%02X\n", bI2cState);
				fResult = FALSE;
				goto _I2C_WR_STOP;
			}
			else
			{
				// keep lint happy
			}
		}
	}

_I2C_WR_STOP:
	// Generate Stop Command
	bTemp = I2cGetRegB(I2C_CSBASE, I2CCON);
	bTemp |= (I2CCON_STO_ + I2CCON_AA_);
	bTemp &= (~I2CCON_SI_);
	I2cSetRegB(I2C_CSBASE, I2CCON, bTemp);
	if (!IsI2cStopped())
	{
		SMSC_ERROR("ERROR: IsI2cStopped() failed after STOP command in I2cMasterWrite()\n");
		return FALSE;
	}
	// udelay udelay(1);
	//bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
	//if (bI2cState != (BYTE)0xF8) // Default state
	//{
	//	SMSC_ERROR("ERROR: Unexpected I2C state after STOP command in I2cMasterWrite(), I2CSTA=0x%02X\n", bI2cState);
	//	fResult = FALSE;
	//}

    retry = 0;
    do {
          bI2cState = I2cGetRegB(I2C_CSBASE, I2CSTA);
       } while  ( (bI2cState != (BYTE)0xF8) && (retry++ < I2C_SPI_MAX_RETRY ) );


    if  (retry  >= I2C_SPI_MAX_RETRY )
    {
        SMSC_ERROR("ERROR: Unexpected I2C state after STOP command in I2cMasterWrite(), I2CSTA=0x%02X\n", bI2cState);
        fResult = FALSE;
    }

    if (i2cMaxRetry < retry)
    {
        i2cMaxRetry = retry;
    }

	return fResult;
}






void HbiLanSetRegDW(const DWORD dwLanBase, const DWORD dwOffset, const DWORD dwVal)
{

	PlatformSetRegDW( dwLanBase, dwOffset, dwVal);
}

DWORD HbiLanGetRegDW(const DWORD dwLanBase, const DWORD dwOffset)
{
	return PlatformGetRegDW(dwLanBase,dwOffset);
}

/*
 *******************************************************************************
 *	MiiSetReg
 *******************************************************************************
 *
 *	Writes the passed-in data to specified MII PHY register.
 *
 */
void MiiSetRegW(const DWORD dwLanBase,
		const DWORD dwPhyAddress,
		const DWORD dwMiiIndex, const WORD wVal)
{
	DWORD dwAddr;
	int i;

	// confirm MII not busy
	if ((PlatformGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
	{
		SMSC_ERROR("ERROR:  MII is busy in MiiGetReg???\n");
		return;
	}

	// put the data to write in the MAC
	PlatformSetRegDW( dwLanBase, MII_DATA, (DWORD)wVal);

	// set the address, index & direction (write to PHY)
	dwAddr = ((dwPhyAddress & 0x1FUL)<<11) | ((dwMiiIndex & 0x1FUL)<<6) | MII_ACC_MII_WRITE_ | MII_ACC_MII_BUSY_;
	PlatformSetRegDW( dwLanBase, MII_ACC, dwAddr);

	// wait for write to complete w/ timeout
	for(i=0;i<100;i++) {
		// see if MII is finished yet
		if ((PlatformGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
		{
			goto DONE;
		}

	}
	SMSC_WARNING("timeout waiting for MiiSetReg to finish");
DONE:
	return;
}


/**
 *******************************************************************************
 *	ACCESS THROUGH MII/MAC
 *******************************************************************************
 *
 *	Reads from the specified MII PHY register & returns the value.
 *
 */
/* Reads the PHY Register over MII */
WORD MiiGetRegW(const DWORD dwLanBase, const DWORD dwPhyAddress,
		const DWORD dwMiiIndex)
{
	DWORD dwAddr;
	int i;
	// confirm MII not busy
	if ((PlatformGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
	{
		SMSC_ERROR("ERROR:  MII is busy in MiiGetReg???\n");
		return((WORD)0xFFFFU);;
	}

	// set the address, index & direction (read from PHY)
	dwAddr = ((dwPhyAddress & 0x1FUL)<<11) | ((dwMiiIndex & 0x1FUL)<<6) | MII_ACC_MII_BUSY_;
	PlatformSetRegDW( dwLanBase, MII_ACC, dwAddr);


	// wait for read to complete w/ timeout
	for(i=0;i<100;i++) {
		// see if MII is finished yet
		if ((PlatformGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
		{
			// get the read data from the MAC & return i
			return ((WORD)PlatformGetRegDW(dwLanBase, MII_DATA));
		}


	}
	return ((WORD)0xFFFFU);
}


static BOOLEAN isMacNotBusy(const DWORD dwLanBase)
{
        int i=0;
        // wait for MAC not busy, w/ timeout
        for(i=0;i<40;i++)
        {
                if( (PlatformGetRegDW(dwLanBase, MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_ )==0UL ) {
                        return TRUE;
                }
        }
        SMSC_WARNING("timeout waiting for MAC not BUSY. MAC_CSR_CMD = 0x%08lX",
                PlatformGetRegDW(dwLanBase,MAC_CSR_CMD) );
        return FALSE;
}


DWORD MacGetRegDW(const DWORD dwLanBase,DWORD dwRegOffset)
{
    DWORD result=0xFFFFFFFFUL;
    DWORD dwTemp=0;

    // wait until not busy
    if (PlatformGetRegDW(dwLanBase,MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_  )
    {
        SMSC_WARNING("MacGetRegDW() failed, MAC already busy at entry");
        goto DONE;
    }

    // send the MAC Cmd w/ offset
    PlatformSetRegDW( dwLanBase,MAC_CSR_CMD,
        ((dwRegOffset & 0x000000FFUL) |
        MAC_CSR_CMD_CSR_BUSY_ | MAC_CSR_CMD_R_NOT_W_));
    dwTemp=PlatformGetRegDW(dwLanBase,BYTE_TEST);//to flush previous write
    dwTemp=dwTemp;

    // wait for the read to happen, w/ timeout
    if (!isMacNotBusy(dwLanBase))
    {
        SMSC_WARNING("===> failed, waiting for MAC not busy after read");
        goto DONE;
    } else {
        // finally, return the read data
        result= PlatformGetRegDW(dwLanBase,MAC_CSR_DATA);
    }
DONE:
    return result;
}

void MacSetRegDW(const DWORD dwLanBase,DWORD dwRegOffset,DWORD dwVal)
{
    DWORD dwTemp=0;

    if ((PlatformGetRegDW(dwLanBase,MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_))
    {
        SMSC_WARNING("====> failed, MAC already busy at entry");
        goto DONE;
    }

    // send the data to write
    PlatformSetRegDW( dwLanBase,MAC_CSR_DATA,dwVal);

    // do the actual write
    PlatformSetRegDW( dwLanBase,MAC_CSR_CMD,((dwRegOffset & 0x000000FFUL) | MAC_CSR_CMD_CSR_BUSY_));

    dwTemp= PlatformGetRegDW(dwLanBase,BYTE_TEST);//force flush of previous write
    dwTemp=dwTemp;

    // wait for the write to complete, w/ timeout
    if (!isMacNotBusy(dwLanBase))
    {
        SMSC_WARNING("======>failed, waiting for MAC not busy after write");
    }
DONE:
    return;
}

#define BIT_10_HIGH    0x0200
#define LS_WORD_ACCESS 0x0000
#define MS_WORD_ACCESS 0x0001

WORD SmiGetRegMsw( const DWORD dwLanBase,
                      DWORD regAddress
                      )
{
    DWORD dwAddr=0;
    int i=0;
    WORD result=0xFFFFU;


#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" :: 1 lanBase=0x%lx, regAddress = 0x%lx \n", dwLanBase,regAddress);
      }
#endif
        // confirm MII not busy
    if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
    {
        SMSC_WARNING("MII is busy in Phy_GetRegW???");
        result=0;
        goto DONE;
    }
        // ---------------------------------------------------
        //      Take bit 9-2    and clear bit 0             L/M
        // --------------------------------------------------

    regAddress =  (regAddress >> 2);
        // Clear bit 1 and set
        // set the address, index & direction (read from PHY)
    dwAddr =   regAddress << 1 ;
    dwAddr &=   ~((WORD) (0x1) );   // clear LSB
    dwAddr |=  ( BIT_10_HIGH |MS_WORD_ACCESS  );  // Set Bit 10 High/LSB low
    dwAddr  = (dwAddr << 6 ) | MII_ACC_MII_BUSY_;

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" ::2 regAddress=0x%lx, Value to MIIC  = 0x%lx", regAddress, dwAddr);
      }
#endif

    MacSetRegDW(dwLanBase, MII_ACC ,dwAddr);

        // wait for read to complete w/ timeout
    for(i=0;i<1000;i++) {
                // see if MII is finished yet
        if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
        {
            // get the read data from the MAC & return i
            result=((WORD)MacGetRegDW(dwLanBase, MII_DATA));
            goto DONE;
        }
    }
    SMSC_WARNING("timeout waiting for MII write to finish");

DONE:
#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!!!::3 regAddress=0x%lx,  Value= 0x%x", regAddress, result);
      }
#endif
        return result;

}


WORD SmiGetRegLsw( const DWORD dwLanBase,
                      DWORD regAddress
                )
{
    DWORD dwAddr=0;
    int i=0;
    WORD result=0xFFFFU;

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" :: 1 lanBase=0x%lx, regAddress = 0x%lx \n", dwLanBase,regAddress);
      }
#endif

        // confirm MII not busy
    if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
    {
        SMSC_WARNING("MII is busy in Phy_GetRegW???");
        result=0;
        goto DONE;
    }
        // ---------------------------------------------------
        //      Take bit 9-2    and clear bit 0             L/M
        // --------------------------------------------------


    regAddress =  (regAddress >> 2)   ;
        // Clear bit 1 and set
        // set the address, index & direction (read from PHY)
    dwAddr =   regAddress << 1 ;
    dwAddr &=   ~((WORD) (0x1) );                 // clear LSB
    dwAddr |=  ( BIT_10_HIGH |LS_WORD_ACCESS  );  // Set Bit 10 High/LSB low
    dwAddr  = (dwAddr << 6 ) | MII_ACC_MII_BUSY_;


#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" ::2 regAddress=0x%lx, Value to MIIC  = 0x%lx", regAddress, dwAddr);
      }
#endif

    MacSetRegDW(dwLanBase,MII_ACC, dwAddr );

        // wait for read to complete w/ timeout
    for(i=0;i<1000;i++) {
                // see if MII is finished yet
        if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
        {
            result=((WORD)MacGetRegDW(dwLanBase, MII_DATA));
            goto DONE;
        }
    }
    SMSC_WARNING("timeout waiting for MII write to finish");

DONE:
#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!!!::3 regAddress=0x%lux,  Value= 0x%x", regAddress, result);
      }
#endif

        return result;

}

void SmiSetRegLsw(const DWORD dwLanBase,
                        DWORD regAddress,
                        DWORD wVal)
{
        DWORD   dwAddr = 0, i=0;

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!!:: 1 lanBase=0x%lx, regAddress = 0x%lx, value =0x%lx \n", dwLanBase,regAddress, wVal);
      }
#endif

        if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
        {
            SMSC_WARNING("MII is busy in Phy_SetRegW???");
            goto DONE;
        }


        MacSetRegDW(dwLanBase, MII_DATA, (WORD) (wVal & 0xFFFF));

        regAddress =  (regAddress >> 1);
        // Clear bit 1 and set
        // set the address, index & direction (read from PHY)
        dwAddr =  ((  regAddress  |  BIT_10_HIGH) & ( ~((WORD) (0x1)) ) )  |  LS_WORD_ACCESS;
        dwAddr = dwAddr << 6;
        dwAddr |= (MII_ACC_MII_WRITE_ |MII_ACC_MII_BUSY_);

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!::2 regAddress=0x%lx, Value to MIIC  = 0x%lx", regAddress, dwAddr);
      }
#endif


        MacSetRegDW(dwLanBase, MII_ACC, dwAddr );

        // wait for write to complete w/ timeout
        for(i=0;i<100;i++) {
                // see if MII is finished yet
        if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
        {
            goto DONE;
        }
    }

    SMSC_WARNING("timeout waiting for MII write to finish");
DONE:
        return;
}


void SmiSetRegMsw(const DWORD dwLanBase,
                    DWORD regAddress,
                    DWORD wVal)
{
    DWORD   dwAddr = 0, i=0;

#ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!!:: 1 lanBase=0x%lx, regAddress = 0x%lx, value =0x%lx\n", dwLanBase,regAddress, wVal);
      }
#endif

    if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) != 0UL)
    {
        SMSC_WARNING("MII is busy in Phy_SetRegW???");
        goto DONE;
    }

        // put the data to write in the MAC

    MacSetRegDW(dwLanBase, MII_DATA, (WORD) ((wVal >> 16) & 0xFFFF));

        // set the address, index & direction (write to PHY)
    regAddress =  (regAddress >> 1);
        // Clear bit 1 and set
        // set the address, index & direction (read from PHY)
    dwAddr =  ((  regAddress  |  BIT_10_HIGH) & ( ~((WORD) (0x1)) ) )  |  MS_WORD_ACCESS;
    dwAddr = dwAddr << 6;
    dwAddr |= (MII_ACC_MII_WRITE_ |MII_ACC_MII_BUSY_);


 #ifdef DEBUG_TRACE_ENABLE
    if ( debugSmi) {
      SMSC_TRACE(" !!!!::2 regAddress=0x%lx, Value to MIIC  = 0x%lx", regAddress, dwAddr);
      }
#endif
    MacSetRegDW(dwLanBase, MII_ACC, dwAddr );


        // wait for write to complete w/ timeout
    for(i=0;i<100;i++) {
                // see if MII is finished yet
        if ((MacGetRegDW(dwLanBase, MII_ACC) & MII_ACC_MII_BUSY_) == 0UL)
        {
            goto DONE;
        }
    }

    SMSC_WARNING("timeout waiting for MII write to finish");
DONE:
        return;
}

