#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <net/if_arp.h>
#include <arpa/inet.h>

#define DRIVER_VERSION	(0x00000119UL)

#define DRIVER_SIGNATURE	(0x82745BACUL)
#define APP_SIGNATURE		(0x987BEF28UL)

#define SMSC9118_IOCTL				(SIOCDEVPRIVATE + 0xB)

#define COMMAND_BASE				(0x974FB832UL)
#define COMMAND_LOAD_MAC_ADDRESS	(COMMAND_BASE+13)
#define COMMAND_SAVE_MAC_ADDRESS	(COMMAND_BASE+14)

int				sock;
unsigned long	DrvVersion = DRIVER_VERSION;
struct ifreq 	IfReq;
struct {
	unsigned long dwSignature;
	unsigned long dwCommand;
	unsigned long Data[0x60];
	char Strng1[30];
	char Strng2[10];
} IoctlData;

typedef	union {
	unsigned long	dword[2];
	unsigned char 	byte[8];
} MACADDR;

int LoadMacAddress(MACADDR *MacAddr)
{
	memset(&IoctlData, 0, sizeof(IoctlData));
	IfReq.ifr_data = (void *)&IoctlData;
	strcpy(IfReq.ifr_name, "eth0");
	IoctlData.dwSignature = APP_SIGNATURE + DrvVersion;
	IoctlData.dwCommand = COMMAND_LOAD_MAC_ADDRESS;
	ioctl(sock, SMSC9118_IOCTL, &IfReq);
	if(IoctlData.dwSignature == (DRIVER_SIGNATURE+DrvVersion)) {
		MacAddr->dword[1] = IoctlData.Data[0];		// ADDRH
		MacAddr->dword[0] = IoctlData.Data[1];		// ADDRL
		return 0;
	} else 
		return -1;
}

int SaveMacAddress(MACADDR MacAddr)
{
	memset(&IoctlData, 0, sizeof(IoctlData));
	IfReq.ifr_data = (void *)&IoctlData;
	strcpy(IfReq.ifr_name, "eth0");
	IoctlData.dwSignature = APP_SIGNATURE + DrvVersion;
	IoctlData.dwCommand = COMMAND_SAVE_MAC_ADDRESS;
	IoctlData.Data[0] = MacAddr.dword[1];		// ADDRH
	IoctlData.Data[1] = MacAddr.dword[0];		// ADDRL
	ioctl(sock, SMSC9118_IOCTL, &(IfReq));
	if(IoctlData.dwSignature != (DRIVER_SIGNATURE + DrvVersion)) {
		return -1;
	}
	return 0;
}

void usage(char *filename)
{
	fprintf(stderr, "Usage: %s [OPTION] [MACADDR]\n", filename);
	fprintf(stderr, "Set MAC address to EEPROM or dump MAC address in EEPROM\n");
	fprintf(stderr, "MAC address should be xx:xx:xx:xx:xx:xx format\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "Available Options\n");
	fprintf(stderr, " -v drv_ver              specify different driver version\n");
	fprintf(stderr, " -d                      dump current MAC address\n");
}

int DumpCurrentMac()
{
	MACADDR	MacAddr;

	if (LoadMacAddress(&MacAddr) < 0) {
		return -2;
	}
	else {
		fprintf(stdout, "Current MAC Addr: %02x:%02x:%02x:%02x:%02x:%02x\n", 
				MacAddr.byte[0], MacAddr.byte[1], MacAddr.byte[2], 
				MacAddr.byte[3], MacAddr.byte[4], MacAddr.byte[5]);
	}
}

int main(int argc, char *argv[])
{
	int	c;
	extern char *optarg;
	extern int optind, opterr, optopt;
	char *ptr, mac[6];
	MACADDR	MacAddr;

	if (argc == 1) {
		usage(argv[0]);
		return -1;
	}

	sock = socket(AF_INET,SOCK_DGRAM,0);
	if((sock) < 0) {
		perror("Failed to create socket !! ->");
		close(sock);
		return -2;
   	}

    while ((c = getopt(argc,argv,"hv:d"))!=-1) {
        switch(c) {
			case 'h':
				usage(argv[0]);
				return -1;
			case 'v':
				DrvVersion = strtol(optarg, NULL, 16);
				break;
			case 'd':
				if (DumpCurrentMac() < 0) {
					fprintf(stderr, "EEPROM may be blank or driver version mismatch! Try with -v option\n");
				}
				close(sock);
				return 0;
			default:
				usage(argv[0]);
				close(sock);
				return -1;
				break;
		}
	}

	MacAddr.dword[0] = MacAddr.dword[1] = 0;

	c = 0;
	if (argv[optind] == NULL)
	{
		usage(argv[0]);
		close(sock);
		return -1;
	}

	ptr = strtok(argv[optind], ":");
	do {
		if (ptr) {
			MacAddr.byte[c++] = strtol(ptr, NULL, 16);
			if (c > 6) {
				usage(argv[0]);
				close(sock);
				return -1;
			}
		}
		else {
			usage(argv[0]);
			close(sock);
		}
	} while (ptr = strtok(NULL, ":"));
	if (c != 6) {
		usage(argv[0]);
		close(sock);
		return -1;
	}

	if (SaveMacAddress(MacAddr) < 0) {
		fprintf(stderr, "Failed to save MAC address.\n");
		fprintf(stderr, "It might be driver version problem. Try with -v option.\n");
		close(sock);
		return -2;
	}

	close(sock);

	fprintf(stderr, "Note. network device should be restarted to use new MAC addrss\n");
}
