/***************************************************************************
 *
 * Copyright (C) 2004-2005  SMSC
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ***************************************************************************
 * File: sh3_polaris.c
 ***************************************************************************
 * 04/11/2005 Bryan Whitehead, rev 2
 *    updated platform code to support version 1.14 platform changes
 * 04/15/2005 Bryan Whitehead, rev 3
 *    migrated to perforce
 *    merged .c and .h file
 * 08/29/2005 Phong Le, rev4
 *    Updated to version 1.20
 * 10/31/2005 Phong Le, rev5
 *    Updated to version 1.21
 * 11/02/2005 Phong Le, still rev5
 *    Fixed bugs to version 1.21 (common code smsc911x.c)
 * 11/07/2005 Phong Le, still rev5
 *    Fixed bugs to version 1.21 (common code smsc911x.c)
 ***************************************************************************
 * NOTE: When making changes to platform specific code please remember to
 *   update the revision number in the PLATFORM_NAME macro. This is a
 *   platform specific version number and independent of the
 *   common code version number. The PLATFORM_NAME macro should be found in
 *   your platforms ".h" file.
 */

#include "sh3_polaris.h"
#define DUMP_PLATFORM_REGISTER   // for debugging
#define SETUP_IRQ           // set up IRQ to low level
extern DWORD wait_state;    // defined as module param
extern DWORD irq ;          // defined as module param
extern DWORD cache_line;
//////////////////////////////
//
//   THis should be Overwritten by Module param
//   lan_base
//
/////////////////////////////

#define LAN_CS	(6UL)
#define LAN_CSBASE (0xB8000000UL)


static const char date_code[] = "032306";

//for a description of these MACROs see readme.txt

#define PLATFORM_CACHE_LINE_BYTES (16UL)

#define PLATFORM_IRQ_POL	(0UL)
#define PLATFORM_IRQ_TYPE	(0UL)

#define PLATFORM_HYDRA_IRQ	(33UL)
#define PLATFORM_1588_IRQ   (36UL)
#define PLATFORM_RX_DMA		(0UL)
#define PLATFORM_TX_DMA		(1UL)
#define PLATFORM_NAME		"SH3 POLARIS r5"


//the dma threshold has not been thoroughly tuned but it is
//  slightly better than using zero
#define PLATFORM_DMA_THRESHOLD (200)
#define DEFAULT_WAIT_STATE	10UL;    //initially set to slowest setting
#define WAIT_STATE_NUMBER	11

#ifdef WITHOUT_HEADER_FILE
typedef struct _PLATFORM_DATA {
	DWORD dwBitWidth;
	DWORD dwIdRev;
	DWORD dwIrq;
	void * dev_id;
} PLATFORM_DATA, *PPLATFORM_DATA;

#endif

inline void Platform_SetRegDW(
		DWORD dwLanBase,
		DWORD dwOffset,
		DWORD dwVal)
{
	(*(volatile DWORD *)(dwLanBase+dwOffset))=dwVal;
}

inline DWORD Platform_GetRegDW(
	DWORD dwLanBase,
	DWORD dwOffset)
{
	return (*(volatile DWORD *)(dwLanBase+dwOffset));
}

//See readme.txt for a description of how these
//functions must be implemented
DWORD Platform_Initialize(
	PPLATFORM_DATA platformData,
	DWORD dwLanBase,
	DWORD dwBusWidth);
void Platform_CleanUp(
	PPLATFORM_DATA platformData);
BOOLEAN Platform_Is16BitMode(
	PPLATFORM_DATA platformData);
BOOLEAN Platform_RequestIRQ(
	PPLATFORM_DATA platformData,
	DWORD dwIrq,
	irqreturn_t (*pIsr)(int irq,void *dev_id,struct pt_regs *regs),
	void *dev_id);
DWORD Platform_CurrentIRQ(
	PPLATFORM_DATA platformData);
void Platform_FreeIRQ(
	PPLATFORM_DATA platformData);
BOOLEAN Platform_IsValidDmaChannel(DWORD dwDmaCh);
BOOLEAN Platform_DmaInitialize(
	PPLATFORM_DATA platformData,
	DWORD dwDmaCh);
BOOLEAN Platform_DmaDisable(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh);
void Platform_CacheInvalidate(
	PPLATFORM_DATA platformData,
	const void * const pStartAddress,
	const DWORD dwLengthInBytes);
void Platform_CachePurge(
	PPLATFORM_DATA platformData,
	const void * const pStartAddress,
	const DWORD dwLengthInBytes);
DWORD Platform_RequestDmaChannel(
	PPLATFORM_DATA platformData);
void Platform_ReleaseDmaChannel(
	PPLATFORM_DATA platformData,
	DWORD dwDmaChannel);
BOOLEAN Platform_DmaStartXfer(
	PPLATFORM_DATA platformData,
	const DMA_XFER * const pDmaXfer);
DWORD Platform_DmaGetDwCnt(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh);
void Platform_DmaComplete(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh);
void Platform_GetFlowControlParameters(
	PPLATFORM_DATA platformData,
	PFLOW_CONTROL_PARAMETERS flowControlParameters,
	BOOLEAN useDma);
void Platform_WriteFifo(
	DWORD dwLanBase,
	DWORD *pdwBuf,
	DWORD dwDwordCount);
void Platform_ReadFifo(
	DWORD dwLanBase,
	DWORD *pdwBuf,
	DWORD dwDwordCount);
DWORD    platformGetCacheLineSize(void);
void GpioOut (const DWORD dwGpioBase, const int iBitCombo, const BOOLEAN fBitValue);

#ifdef DUMP_PLATFORM_REGISTER
 void platformDumpRegisters(void);
#endif



/************************************************************************/
/*	Set Bus wait states for specific memory area			    		*/
/************************************************************************/
static const WORD wWaitArea[7] = {AREA0_WAIT_CTRL_, 0, AREA2_WAIT_CTRL_,
	 	AREA3_WAIT_CTRL_, AREA4_WAIT_CTRL_, AREA5_WAIT_CTRL_, AREA6_WAIT_CTRL_};

static const WORD wWaitState[WAIT_STATE_NUMBER] = {WAIT_STATES_0, WAIT_STATES_1, WAIT_STATES_2,
	 	WAIT_STATES_3, WAIT_STATES_4, 0, WAIT_STATES_6, 0,
		WAIT_STATES_8, 0, WAIT_STATES_10};



#define   PLATFORM_CACHE_LINE_BYTES_DEFAULT    (16UL)
DWORD 	Platform_cache_line_bytes  =  PLATFORM_CACHE_LINE_BYTES_DEFAULT;

DWORD   platformGetCacheLineSize()
{
	return (Platform_cache_line_bytes);
}


WORD  platformGetWaitState(void)
{

	WORD ret = 0;

	if (wait_state > WAIT_STATE_NUMBER) {
		ret = DEFAULT_WAIT_STATE;
		SMSC_WARNING("--> wait state passed %lu, wait_state_used= %ul\n",wait_state, ret);
	}
	else {
		ret = wait_state;
		SMSC_TRACE("--> wait state passed %lu, wait_state_used= %ul,\n",wait_state, ret);
	}


	return ret;

}
// volatile unsigned short *picr1 = (volatile unsigned short *)0xA4000010UL;
enum IRQ_LEVEL_ {
	ICR1_FALLING_EDGE_LEVEL_,
	ICR1_RISING_EDGE_LEVEL_,
	ICR1_LOW_LEVEL_,
	ICR1_RESERVED_
};


void platformSetIrq(unsigned int irq,enum IRQ_LEVEL_ irq_Level )
{
    // volatile unsigned short *picr1 = (volatile unsigned short *)0xA4000010UL;
	DWORD dwIntFlags=0;
	spinlock_t IntEnableLock;

	if ( (irq > 37 )   || (irq < 32))
	{
		SMSC_WARNING(" Invalid IRQ (%d) Should be in range [32..37] ", irq);
		return;
	}

	spin_lock_irqsave(&(IntEnableLock),dwIntFlags);
	{
		volatile WORD icr1 = ICR1;
		WORD hwIrq = (WORD) ( irq - 32);
		WORD irqLevel = (WORD) (irq_Level);
		WORD irqMask =  (WORD) ( (WORD) (ICR1_IRQ_MASK) << (2*hwIrq)  );
		SMSC_TRACE(" ICR1 before: 0x%x", ICR1);

		// Mask All Interrupt

		icr1 &=  (WORD) (~irqMask);
		icr1 |= (WORD) ( irqLevel << (2*hwIrq)  );
		ICR1 = icr1;
        }
	spin_unlock_irqrestore(&(IntEnableLock),dwIntFlags);
	SMSC_TRACE(" ICR1 after : 0x%x", ICR1);

}
DWORD Platform_Initialize(
	PPLATFORM_DATA platformData,
	DWORD dwLanBase,DWORD dwBusWidth)
{
	int iAreaNum=0;
	WORD wWaitCtrlReg, wTemp;

	WORD wBcrMask=1;
	int iWaitStNum = DEFAULT_WAIT_STATE; //initially set to slowest setting incase the chip is 115/116
	SMSC_ASSERT(platformData!=NULL);
	platformData->dwBitWidth=0;
	platformData->dwIdRev=0;

	SMSC_TRACE("--> Platform_Initialize()");


	if(dwLanBase==0x0UL) {
		dwLanBase=LAN_CSBASE;
	}

	if((0xA8000000UL<=dwLanBase)&&(dwLanBase<=(0xAC000000UL-LAN_REGISTER_EXTENT)))
	{
		iAreaNum=2;
	}
	if((0xAC000000UL<=dwLanBase)&&(dwLanBase<=(0xB0000000UL-LAN_REGISTER_EXTENT)))
	{
		iAreaNum=3;
	}
	if((0xB0000000UL<=dwLanBase)&&(dwLanBase<=(0xB4000000UL-LAN_REGISTER_EXTENT)))
	{
        iAreaNum=4;  /** 9217 **/
	}
	if((0xB4000000UL<=dwLanBase)&&(dwLanBase<=(0xB8000000UL-LAN_REGISTER_EXTENT)))
	{
        iAreaNum=5; /** 9218 also used by bootloader **/
	}
	if((0xB8000000UL<=dwLanBase)&&(dwLanBase<=(0xBC000000UL-LAN_REGISTER_EXTENT)))
	{
        iAreaNum=6;  /** Hydra memory map**/
	}
	if(iAreaNum==0) {
		SMSC_WARNING("Invalid Lan Base Address = 0x%08lX",dwLanBase);
		SMSC_WARNING("  must be in Area 2,3,4,5, or 6 and uncached");

		return 0;
	}
	SMSC_TRACE(" Lan Base at 0x%08lX in Area %d",dwLanBase,iAreaNum);
	SMSC_TRACE("BCR1 = 0x%04X, will not change it",BCR1);

	// Init BSC registers for 11x
	SMSC_TRACE(" Before: WCR2 = 0x%04X", WCR2);
	wWaitCtrlReg = WCR2;
	wWaitCtrlReg &= (WORD)(~(wWaitArea[iAreaNum]));	// Clear specified Area

	iWaitStNum = platformGetWaitState();
	wTemp = wWaitState[iWaitStNum];
	SMSC_TRACE(" WAIT state  Used= %d", wTemp);

	if ( (cache_line == (16UL)  ) ||(cache_line == (32UL)))
	{
		Platform_cache_line_bytes =  cache_line;
	}
	SMSC_TRACE ("Cache line Size = 0x%lu", Platform_cache_line_bytes);

	switch (iAreaNum)
	{
		case AREA_2:
			wTemp <<= 3;
			wBcrMask <<=4;
			break;

		case AREA_3:
			wTemp <<= 5;
			wBcrMask <<=6;
			break;

		case AREA_4:
			wTemp <<= 7;
			wBcrMask <<=8;
			break;

		case AREA_5:
			wTemp <<= 10;
			wBcrMask <<=10;
			break;

		case AREA_6:
			wTemp <<= 13;
			wBcrMask <<= 12;
			break;

		default:
			break;
	}

	wWaitCtrlReg |= wTemp;		// Inserted 2 wait states for Area 5 and 2.
	WCR2 = wWaitCtrlReg;
	SMSC_TRACE(" After:  WCR2 = 0x%04X", WCR2);

	wTemp=BCR2;
	SMSC_TRACE(" Before: BCR2 = 0x%04X", wTemp);
	if((dwBusWidth!=16)&&(dwBusWidth!=32)) {
		DWORD test=0;
		WORD dummy=0;
		BCR2|=wBcrMask;
#define BYTE_TEST_OFFSET	(0x64)
		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));
		dummy=(*(volatile unsigned short *)(dwLanBase+BYTE_TEST_OFFSET+2));
		if(test==0x87654321UL) {
			SMSC_TRACE(" 32 bit mode detected");
			platformData->dwBitWidth=32;
		} else {
                    SMSC_WARNING(" 32 bit mode not detected, test=0x%08lX",test);
			BCR2&=(~(wBcrMask));
			test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));
			if(test==0x87654321UL) {
				SMSC_TRACE(" 16 bit mode detected");
				platformData->dwBitWidth=16;
			} else {
				SMSC_WARNING(" neither 16 nor 32 bit mode detected, test=0x%08lX",test);
				BCR2=wTemp;
				dwLanBase=0;
				dummy=dummy;//make lint happy
			}
		}
	} else if(dwBusWidth==16) {
		DWORD test=0;
		BCR2&=(~(wBcrMask));
		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));
		if(test==0x87654321UL) {
			SMSC_TRACE(" 16 bit mode assigned and verified");
			platformData->dwBitWidth=16;
		} else {
			SMSC_WARNING(" 16 bit mode not verified");
			dwLanBase=0;
		}
	} else if(dwBusWidth==32) {
		DWORD test=0;
		BCR2|=wBcrMask;
		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));
		if(test==0x87654321UL) {
			SMSC_TRACE(" 32 bit mode assigned and verified");
			platformData->dwBitWidth=32;
		} else {
			SMSC_WARNING(" 32 bit mode not verified");
			dwLanBase=0;
		}
	} else {
		SMSC_ASSERT(FALSE);
	}
	SMSC_TRACE(" After: BCR2 = 0x%04X", BCR2);

	if(dwLanBase!=0)
	{
		DWORD idRev=0;
#define ID_REV_OFFSET	(0x50)
		idRev=(*(volatile unsigned long *)(dwLanBase+ID_REV_OFFSET));
		platformData->dwIdRev=idRev;
		SMSC_TRACE (" LAN VESRION: 0x%08lx, WCR2 Before = 0x%04x", idRev, WCR2)
		switch(idRev&0xFFFF0000UL) {
		case 0x01180000UL:
		case 0x118A0000UL:
		case 0x01170000UL:
		case 0x01120000UL:
		case 0x117A0000UL:
		case 0x93120000UL:

			iWaitStNum   = platformGetWaitState();
			wWaitCtrlReg = WCR2;
			// Now we have programmed Wait State

			wWaitCtrlReg &= (WORD)(~(wWaitArea[iAreaNum]));	// Clear specified Area
			wTemp = wWaitState[iWaitStNum];
			SMSC_TRACE (" Wait State passed =%u, Wait_state_get = %lu, Wait State value =%u",
					iWaitStNum, wait_state, wTemp)
			switch (iAreaNum)
			{
				case AREA_2:
					wTemp <<= 3;
					wBcrMask <<=4;
					break;

				case AREA_3:
					wTemp <<= 5;
					wBcrMask <<=6;
					break;

				case AREA_4:
					wTemp <<= 7;
					wBcrMask <<=8;
					break;

				case AREA_5:
					wTemp <<= 10;
					wBcrMask <<=10;
					break;

				case AREA_6:
					wTemp <<= 13;
					wBcrMask <<= 12;
					break;

				default:break;//make lint happy
			}

			wWaitCtrlReg |= wTemp;		// Inserted 2 wait states for Area 5 and 2.
			WCR2 = wWaitCtrlReg;
			break;
		default:break;//make lint happy
		}
		SMSC_TRACE (" LAN VESRION: 0x%08lx, WCR2 After = 0x%04x", idRev, WCR2)

	}
#ifdef SETUP_IRQ
	platformSetIrq(irq, ICR1_LOW_LEVEL_ );
#endif

#ifdef DUMP_PLATFORM_REG
       platformDumpRegisters();
#endif
      /* The function execution I2cHwInit and SpiHwInit moved to checktheAccessMode in smsc931x.c */
	SMSC_TRACE("<-- Platform_Initialize()");
	return dwLanBase;
}

void Platform_CleanUp(
	PPLATFORM_DATA platformData)
{
}

BOOLEAN Platform_Is16BitMode(
	PPLATFORM_DATA platformData)
{
	SMSC_ASSERT(platformData!=NULL);
	if(platformData->dwBitWidth==16) {
		return TRUE;
	}
	return FALSE;
}

void Platform_GetFlowControlParameters(
	PPLATFORM_DATA platformData,
	PFLOW_CONTROL_PARAMETERS flowControlParameters,
	BOOLEAN useDma)
{
	memset(flowControlParameters,0,sizeof(FLOW_CONTROL_PARAMETERS));
	flowControlParameters->BurstPeriod=100;
	if(useDma) {
		if(Platform_Is16BitMode(platformData)) {
			switch(platformData->dwIdRev&0xFFFF0000) {
			case 0x01180000UL:
			case 0x01170000UL:
			case 0x01120000UL:
				//117/118,16 bit,DMA
				flowControlParameters->MaxThroughput=(0xD7524UL);
				flowControlParameters->MaxPacketCount=(0x24AUL);
				flowControlParameters->PacketCost=(0x183UL);
				flowControlParameters->BurstPeriod=(0x1BUL);
				flowControlParameters->IntDeas=(0x2DUL);
				break;
			case 0x118A0000UL:
			case 0x117A0000UL:
			case 0x93120000UL:     // Hydra Note tested
				//9217/9218 16 bit,DMA
				flowControlParameters->MaxThroughput    = (0x000AB236UL);
				flowControlParameters->MaxPacketCount   = (0x000001CFUL);
				flowControlParameters->PacketCost       = (0x0000013AUL);
				flowControlParameters->BurstPeriod      = (0x00000076UL);
				flowControlParameters->IntDeas          = (0x0000004EUL);
				break;

			case 0x01160000UL:
			case 0x01150000UL:
				//115/116,16 bit,DMA
				flowControlParameters->MaxThroughput=(0x8D916UL);
				flowControlParameters->MaxPacketCount=(0x17FUL);
				flowControlParameters->PacketCost=(0x108UL);
				flowControlParameters->BurstPeriod=(0x3BUL);
				flowControlParameters->IntDeas=(0x02UL);
				break;
			default:break;//make lint happy
			}
		} else {
			switch(platformData->dwIdRev&0xFFFF0000) {
			case 0x01180000UL:
			case 0x01170000UL:
			case 0x01120000UL:

				//117/118,32 bit,DMA
				flowControlParameters->MaxThroughput=(0xF1304UL);
				flowControlParameters->MaxPacketCount=(0x292UL);
				flowControlParameters->PacketCost=(0x1D0UL);
				flowControlParameters->BurstPeriod=(0x6BUL);
				flowControlParameters->IntDeas=(0x39UL);
				break;
			case 0x118A0000UL:
			case 0x117A0000UL:
				// 9217- 9218 on linux 2.6 kernel with kernel pre-emption feature
				flowControlParameters->MaxThroughput   = 0x000B98DC;
				flowControlParameters->MaxPacketCount  = 0x000001F6;
				flowControlParameters->PacketCost      = 0x0000013A;
				flowControlParameters->BurstPeriod     = 0x0000000D;
				flowControlParameters->IntDeas         = 0x0000003E;
				break;
			case 0x93120000UL:
		                  //Hydra 32  bit,DMA
				flowControlParameters->MaxThroughput = (0x000C3E74UL);
				flowControlParameters->MaxPacketCount= (0x00000212UL);
				flowControlParameters->PacketCost    = (0x0000017DUL);
				flowControlParameters->BurstPeriod   = (0x0000000AUL);
				flowControlParameters->IntDeas       = (0x00000036UL);
				break;

			case 0x01160000UL:
			case 0x01150000UL:
				//115/116,32 bit,DMA
				flowControlParameters->MaxThroughput=(0xBD60EUL);
				flowControlParameters->MaxPacketCount=(0x203UL);
				flowControlParameters->PacketCost=(0x165UL);
				flowControlParameters->BurstPeriod=(0x27UL);
				flowControlParameters->IntDeas=(0x21UL);
				break;
			default:break;//make lint happy
			}
		}
	} else {
		if(Platform_Is16BitMode(platformData)) {
			SMSC_TRACE (" 16 Bits PIO, dev ID = 0x%luX", platformData->dwIdRev&0xFFFF0000);
			switch(platformData->dwIdRev&0xFFFF0000) {
				case 0x01180000UL:
				case 0x01170000UL:
				case 0x01120000UL:
					//117/118,16 bit,PIO
					flowControlParameters->MaxThroughput=(0xC0A2AUL);
					flowControlParameters->MaxPacketCount=(0x211UL);
					flowControlParameters->PacketCost=(0x14EUL);
					flowControlParameters->BurstPeriod=(0x47UL);
					flowControlParameters->IntDeas=(0x1BUL);
					break;
				case 0x118A0000UL:
				case 0x117A0000UL:
				case 0x93120000UL: // Hydra 16 bits, not tested
					// 92127/9218, 16 Bit PIO using 2.6 pre-emptive kernel.
					//  Tested on 92127 only
					flowControlParameters->MaxThroughput          = (0x0009EF0CUL);
					flowControlParameters->MaxPacketCount         = (0x000001AEUL);
					flowControlParameters->PacketCost             = (0x00000105UL) ;
					flowControlParameters->BurstPeriod            = (0x00000077UL) ;
					flowControlParameters->IntDeas                = (0x0000004CUL) ;
					break;
				case 0x01160000UL:
				case 0x01150000UL:
					//115/116,16 bit,PIO
					flowControlParameters->MaxThroughput=(0x7F270UL);
					flowControlParameters->MaxPacketCount=(0x158UL);
					flowControlParameters->PacketCost=(0xCDUL);
					flowControlParameters->BurstPeriod=(0x3AUL);
					flowControlParameters->IntDeas=(0x36UL);
					break;
				default:break;//make lint happy
			}
		} else {
			switch(platformData->dwIdRev&0xFFFF0000) {
				SMSC_TRACE (" 32 Bits PIO, dev ID = 0x%luX", platformData->dwIdRev&0xFFFF0000);
				case 0x01180000UL:
				case 0x01170000UL:
				case 0x01120000UL:

					//117/118,32 bit,PIO
					flowControlParameters->MaxThroughput=(0xD3838UL);
					flowControlParameters->MaxPacketCount=(0x240UL);
					flowControlParameters->PacketCost=(0x0EUL);
					flowControlParameters->BurstPeriod=(0x19UL);
					flowControlParameters->IntDeas=(0x39UL);
					break;
				case 0x118A0000UL:
				case 0x117A0000UL:
					// 92127/9218, 32 Bit PIO using 2.6 pre-emptive kernel.
					flowControlParameters->MaxThroughput          = 0x000A7CFC ;
					flowControlParameters->MaxPacketCount         = 0x000001C6 ;
					flowControlParameters->PacketCost             = 0x00000115 ;
					flowControlParameters->BurstPeriod            = 0x0000006F ;
					flowControlParameters->IntDeas                = 0x0000004F ;
					break;
				case 0x93120000UL:
					// Hydra, 32 Bit PIO using 2.6 pre-emptive kernel.
					flowControlParameters->MaxThroughput          = 0x000AE770 ;
					flowControlParameters->MaxPacketCount         = 0x000001D8 ;
					flowControlParameters->PacketCost             = 0x000000F1 ;
					flowControlParameters->BurstPeriod            = 0x00000063 ;
					flowControlParameters->IntDeas                = 0x00000036 ;
					break;

				case 0x01160000UL:
				case 0x01150000UL:
					//115/116,32 bit,PIO
					flowControlParameters->MaxThroughput=(0xB2E7AUL);
					flowControlParameters->MaxPacketCount=(0x1E6UL);
					flowControlParameters->PacketCost=(0x137UL);
					flowControlParameters->BurstPeriod=(0x57UL);
					flowControlParameters->IntDeas=(0x14UL);
					break;
				default:break;//make lint happy
			}
		}
		SMSC_TRACE ("FLOW Setting:: Max Thput=0x%08lX,maxPkt=0x%08lX, Cost=0x%08lx, burst=0x%08lX, intDeas=0x%08lX",
			    flowControlParameters->MaxThroughput,
			    flowControlParameters->MaxPacketCount,
			    flowControlParameters->PacketCost,
			    flowControlParameters->BurstPeriod,
			    flowControlParameters->IntDeas)
	}
}

BOOLEAN Platform_RequestIRQ(
	PPLATFORM_DATA platformData,
	DWORD dwIrq,
	irqreturn_t (*pIsr)(int a,void *b,struct pt_regs *c),
	void * dev_id)
{
	BOOLEAN result=FALSE;
	SMSC_ASSERT(platformData!=NULL);
	SMSC_ASSERT(platformData->dev_id==NULL);
	if(request_irq(
		dwIrq,
		pIsr,
		SA_INTERRUPT,
		"SMSC_LAN911x_ISR",
		dev_id)!=0)
	{
		SMSC_WARNING("Unable to use IRQ = %ld",dwIrq);
		goto DONE;
	}
	platformData->dwIrq=dwIrq;
	platformData->dev_id=dev_id;
	result=TRUE;
DONE:
	return result;
}

DWORD Platform_CurrentIRQ(
	PPLATFORM_DATA platformData)
{
	SMSC_ASSERT(platformData!=NULL);
	return platformData->dwIrq;
}

void Platform_FreeIRQ(
	PPLATFORM_DATA platformData)
{
	if(platformData==NULL) {
		SMSC_ASSERT(FALSE);
		return;
	}
	if(platformData->dev_id==NULL) {
		SMSC_ASSERT(FALSE);
		return;
	}

	free_irq(platformData->dwIrq,platformData->dev_id);

	platformData->dwIrq=0;
	platformData->dev_id=NULL;
}

BOOLEAN Platform_IsValidDmaChannel(DWORD dwDmaCh)
{
	if(dwDmaCh<=3) {
		return TRUE;
	}
	return FALSE;
}

#define GetRegDW(dwBase,dwOffset) \
	((DWORD)(*(volatile DWORD *)(dwBase + dwOffset)))

#define SetRegDW(dwBase,dwOffset,dwVal) \
	((*(volatile DWORD *)(dwBase + dwOffset)) = dwVal)


BOOLEAN Platform_DmaDisable(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh)
{
	const DWORD	dwDmaCh0CtrlReg = 0xA400002CUL;
	DWORD		dwTemp;

	platformData=platformData;//make lint happy

	if (dwDmaCh > 3UL)
	{
		SMSC_WARNING("Platform_DmaDisable(dwDmaCh=0x%lX), Invalid DMA Channel!\n",dwDmaCh);
		return FALSE;
	}

	dwTemp = GetRegDW(dwDmaCh0CtrlReg,(dwDmaCh * 0x10UL));
	dwTemp &= ~DMAC_ENBL_;	// Disable DMA controller

	SetRegDW(dwDmaCh0CtrlReg, (dwDmaCh * 0x10UL), dwTemp);

	return TRUE;
}

void Platform_CacheInvalidate(
	PPLATFORM_DATA platformData,
	const void * const pStartAddress,
	const DWORD dwLengthInBytes)
{
	Platform_CachePurge(
		platformData,
		pStartAddress,
		dwLengthInBytes);
}

void Platform_CachePurge(
	PPLATFORM_DATA platformData,
	const void * const pStartAddress,
	const DWORD dwLengthInBytes)
{
	DWORD	dwAddrIndex;
	DWORD	begin, end;

	const DWORD	dwStartAddr = CpuToPhysicalAddr(pStartAddress);

	platformData=platformData;//make lint happy

	begin = dwStartAddr & ~(Platform_cache_line_bytes - 1UL);
	end = (dwStartAddr + dwLengthInBytes + Platform_cache_line_bytes - 1UL) &
			~(Platform_cache_line_bytes - 1UL);

	for (dwAddrIndex = begin;
		dwAddrIndex < end;
		dwAddrIndex += Platform_cache_line_bytes)
	{
		DWORD	dwData, dwAddr;

		dwData = (dwAddrIndex & CACHE_PHYSADDR_MASK);	 /* _Virtual_ address, ~U, ~V */
		dwAddr = CACHE_OC_ADDRESS_ARRAY | (dwAddrIndex & CACHE_OC_ENTRY_MASK) |
				CACHE_OC_ASSOC_BIT_;
		(*(volatile unsigned long *)dwAddr) = dwData;
	}
}

DWORD Platform_RequestDmaChannel(
	PPLATFORM_DATA platformData)
{
	platformData=platformData;//make lint happy
	return TRANSFER_PIO;
}

void Platform_ReleaseDmaChannel(
	PPLATFORM_DATA platformData,
	DWORD dwDmaChannel)
{
	platformData=platformData;//make lint happy
	dwDmaChannel=dwDmaChannel;//make lint happy
	//since Platform_RequestDmaChannel
	//  never returns a dma channel
	//  then this function should never be called
	SMSC_ASSERT(FALSE);
}

void Platform_DmaComplete(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh)
{
	DWORD dwTimeOut=1000000;
	SMSC_ASSERT(dwDmaCh<4);
	while((Platform_DmaGetDwCnt(platformData,dwDmaCh))&&(dwTimeOut))
	{
		udelay(1);
		dwTimeOut--;
	}
	if(!Platform_DmaDisable(platformData,dwDmaCh)) {
		SMSC_WARNING("Failed Platform_DmaDisable");
	}
	if(dwTimeOut==0) {
		SMSC_WARNING("Platform_DmaComplete: Timed out");
	}
}

DWORD Platform_DmaGetDwCnt(
	PPLATFORM_DATA platformData,
	const DWORD dwDmaCh)
{
	const DWORD	dwDmaCh0XferCntReg = 0xA4000028UL;

	platformData=platformData;//make lint happy

	if (dwDmaCh > 3UL)
	{
		SMSC_WARNING("DmaGetDwCnt(dwDmaCh=0x%lX), Invalid DMA Channel!\n",dwDmaCh);
		return 0UL;
	}

	return (GetRegDW(dwDmaCh0XferCntReg,(dwDmaCh * 0x10UL)));
}

BOOLEAN Platform_DmaInitialize(
	PPLATFORM_DATA platformData,
	DWORD dwDmaCh)
{
	platformData=platformData;//make lint happy
	dwDmaCh=dwDmaCh;//make lint happy
	DMATCR0=0;
	DMATCR1=0;
	DMATCR2=0;
	DMATCR3=0;
	DMAOR|=DMA_MSTR_EN_;
	SMSC_TRACE(" Platform cache line bytes size = %lu\n",Platform_cache_line_bytes)
	return TRUE;
}

BOOLEAN Platform_DmaStartXfer(
	PPLATFORM_DATA platformData,
	const DMA_XFER * const pDmaXfer)
{
	const DWORD	dwDmaSrcBaseReg = (0xA4000020UL + (pDmaXfer->dwDmaCh * 0x10UL));
	const DWORD	dwDmaDstBaseReg = dwDmaSrcBaseReg + 0x04UL;
	const DWORD	dwDmaXferCntReg = dwDmaSrcBaseReg + 0x08UL;
	const DWORD	dwDmaCtrlReg    = dwDmaSrcBaseReg + 0x0CUL;

	DWORD dwTemp, dwSrcAddr, dwDestAddr;
	DWORD dwAlignMask;
	DWORD dwLanPhysAddr, dwMemPhysAddr;

	platformData=platformData;//make lint happy

	// 1. validate the requested channel #
	if (pDmaXfer->dwDmaCh > 3UL)
	{
		SMSC_WARNING("Platform_DmaStartXfer -- bad dwDmaCh=%ld", pDmaXfer->dwDmaCh);
		return FALSE;
	}

	// 2. make sure the channel's not already running
	if (GetRegDW(dwDmaXferCntReg,0) != 0UL)
	{
		SMSC_WARNING("Platform_DmaStartXfer -- requested channel (%ld) is still running", pDmaXfer->dwDmaCh);
		return FALSE;
	}

	// 3. calculate the physical transfer addresses
	dwLanPhysAddr = CpuToPhysicalAddr((void *)pDmaXfer->dwLanReg);
	dwMemPhysAddr = CpuToPhysicalAddr((void *)pDmaXfer->pdwBuf);

	// 4. validate the address alignments
	// need CL alignment for CL bursts
	dwAlignMask = (Platform_cache_line_bytes - 1UL);

	if ((dwLanPhysAddr & dwAlignMask) != 0UL)
	{
		SMSC_WARNING("Platform_DmaStartXfer -- bad dwLanPhysAddr (0x%08lX) alignment",
			dwLanPhysAddr);
		return FALSE;
	}


	if ((dwMemPhysAddr & dwAlignMask) != 0UL)
	{
		SMSC_WARNING("Platform_DmaStartXfer -- bad dwMemPhysAddr (0x%08lX) alignment",
			dwMemPhysAddr);
		return FALSE;
	}

	// 5. validate the transfer size & alignment
	if (pDmaXfer->dwDwCnt >= (1UL<<24))
	{
		SMSC_WARNING("Platform_DmaStartXfer -- dwDwCnt =%ld is too big (16777216 counts max on SH3)",
			pDmaXfer->dwDwCnt);
		return FALSE;
	}

	if ((pDmaXfer->dwDwCnt & 3UL) != 0UL)
	{
		SMSC_WARNING("Platform_DmaStartXfer -- bad dwDwCnt=%ld alignment",
			pDmaXfer->dwDwCnt);
		return FALSE;
	}

	// 6. Config Control reg
	dwTemp = GetRegDW(dwDmaCtrlReg,0);

	// Disable the selected channel first
	dwTemp &= ~DMAC_ENBL_;
	SetRegDW((dwDmaCtrlReg),0, dwTemp);

	// Select correct ch and set SRC, DST and counter
	if (pDmaXfer->fMemWr == TRUE)
	{
		dwSrcAddr = dwLanPhysAddr;
		dwDestAddr = dwMemPhysAddr;

		dwTemp &= ~DST_ADDR_MODE_;
		dwTemp |= INC_DST_ADDR;		// Always to increment mem buffer

		dwTemp &= ~SRC_ADDR_MODE_;
	}
	else
	{
		dwSrcAddr = dwMemPhysAddr;
		dwDestAddr = dwLanPhysAddr;

		dwTemp &= ~SRC_ADDR_MODE_;
		dwTemp |= INC_SRC_ADDR;		// Always increment mem buffer

		dwTemp &= ~DST_ADDR_MODE_;
	}


	// Set Source and destination addresses
	SetRegDW(dwDmaSrcBaseReg,0, dwSrcAddr);
	SetRegDW(dwDmaDstBaseReg,0, dwDestAddr);

	// Set the transmit size in terms of the xfer mode
	dwTemp &= ~XMIT_SIZE_;

	dwTemp &= ~XMIT_MODE_;	// Clear to 0:  Cycle-steal mode

	dwTemp |= XMIT_QDWORD;	// Set 16 bytes (QDWORD) xfer
	SetRegDW(dwDmaXferCntReg,0, (pDmaXfer->dwDwCnt/4UL));

	// bugbug (nw)  4/23/2004 -- these somehow were down below the section that sets the counts???
	dwTemp &= ~XMIT_RESRC_;
	dwTemp |= AUTO_REQ;		// Auto request
	dwTemp &= ~XFER_END_;
	SetRegDW(dwDmaCtrlReg,0, dwTemp);
	// bugbug (nw)  4/23/2004 end of bugfix

	// Clrea NMIF and AE bits and enable DMA master
	DMAOR &= ~(NMI_FLAG_ | ADDR_ERR_FLAG_);

	// Enable DMA controller ch x
	dwTemp = GetRegDW(dwDmaCtrlReg,0);
	dwTemp |= DMAC_ENBL_;
	SetRegDW(dwDmaCtrlReg,0, dwTemp);

	// DMA Transfering....

	return TRUE;
}

void Platform_WriteFifo(
	DWORD dwLanBase,
	DWORD *pdwBuf,
	DWORD dwDwordCount)
{
	volatile DWORD *pdwReg;
	pdwReg = (volatile DWORD *)(dwLanBase+TX_DATA_FIFO);
	while(dwDwordCount)
	{
		*pdwReg = *pdwBuf++;
		dwDwordCount--;
	}
}
void Platform_ReadFifo(
	DWORD dwLanBase,
	DWORD *pdwBuf,
	DWORD dwDwordCount)
{
	const volatile DWORD * const pdwReg =
		(const volatile DWORD * const)(dwLanBase+RX_DATA_FIFO);

	while (dwDwordCount)
	{
		*pdwBuf++ = *pdwReg;
		dwDwordCount--;
	}
}


#ifdef DUMP_PLATFORM_REGISTER

void platformDumpRegisters()
{
   SMSC_TRACE("\n");
   SMSC_TRACE(" ICR0   = 0x%x ", ICR0);
   SMSC_TRACE(" ICR1   = 0x%x ", ICR1);
   SMSC_TRACE(" ICR2   = 0x%x ", ICR2);
   SMSC_TRACE(" ICR3   = 0x%x ", ICR3);
   SMSC_TRACE(" PINTER = 0x%x ", PINTER);
   SMSC_TRACE(" IPRA   = 0x%x ", IPRA);
   SMSC_TRACE(" IPRB   = 0x%x ", IPRB);
   SMSC_TRACE(" IPRC   = 0x%x ", IPRC);
   SMSC_TRACE(" IPRD   = 0x%x ", IPRD);
   SMSC_TRACE(" IPRE   = 0x%x ", IPRE);
   SMSC_TRACE(" IPRF   = 0x%x ", IPRF);
   SMSC_TRACE(" IPRG   = 0x%x ", IPRG);
   SMSC_TRACE(" IRR0   = 0x%x ", IRR0);
   SMSC_TRACE(" IRR1   = 0x%x ", IRR1);
   SMSC_TRACE(" IRR2   = 0x%x ", IRR2);
   SMSC_TRACE(" IRR3   = 0x%x ", IRR3);
   SMSC_TRACE(" IRR4   = 0x%x ", IRR4);

   SMSC_TRACE (" BCR1   = 0x%04X ", BCR1  );
   SMSC_TRACE (" WCR1   = 0x%04X ", WCR1  );
   SMSC_TRACE (" WCR2   = 0x%04X ", WCR2  );
   SMSC_TRACE (" MCR    = 0x%04X ", MCR   );
   SMSC_TRACE (" PCR    = 0x%04X ", PCR   );
   SMSC_TRACE (" RTCSR  = 0x%04X ", RTCSR );
   SMSC_TRACE (" RTCNT  = 0x%04X ", RTCNT );
   SMSC_TRACE (" RTCOR  = 0x%04X ", RTCOR );
   SMSC_TRACE (" RFCR   = 0x%04X ", RFCR  );
}




void I2cHwInit(void)
{
    DWORD dwTemp;

    // PCA9564 I2C 8-bit bus width
    dwTemp = BCR2;
    dwTemp &= ~0x00000020UL;    // clear bit 5 for CS2
    BCR2 = (WORD)dwTemp;
    // AH adjust wait state in Rd/Wr Bus Cycles
    dwTemp = WCR2;
    dwTemp &= (DWORD)(~AREA2_WAIT_CTRL_);
    dwTemp |= (DWORD)((unsigned int)WAIT_STATES_2 << 3);
    WCR2 = (WORD)dwTemp;
    // AH adjust wait state between Rd/Wr Bus Cycles
    dwTemp = WCR1;
    dwTemp &= (~0x00000030UL); // 1 idle state between bus cycles
    WCR1 = (WORD)dwTemp;

    I2cSetRegB(I2C_CSBASE, I2CTO, (BYTE)0xFF); // Time out Enable (Bit7), Timeout period = ([Bit6:Bit0] + 1) * 113.7usec = 14.44 msec
    I2cSetRegB(I2C_CSBASE, I2CADR, (BYTE)0x0);
    I2cSetRegB(I2C_CSBASE, I2CCON, (BYTE)(I2CCON_AA_ + I2CCON_ENSIO_ + I2CCON_330KHZ_SCLK_));
}


void SpiHwInit(void)
{
    // External clock has only been tested in LAN9303 FPGA board. To use external 5MHz clock
    // in Aldebaran platform, the CPLD jedec file should be updated to CPLD_6441_rev6.jed which is
    // located at U:\NPG\NPG_VALIDATION\Hydra_Leo\Aldeberan_CPLD. To achieve minimum access time,
    // Delay1uS() in SpiTxW() has been replaced with for() loop delay which makes it vulnerable to
    // any changes in SH3 frequency. In the current version, SH3 internal frequency 200MHz and peripheral
    // frequency 33MHz are assumed.

#ifdef _SPI_EXTERNAL_CLK_MODE
    SCSCR  = 0x03;  // CKE=11, External clock, SCK pin used for synchronous clock input
#else
    SCSCR  = 0x00;  // Clear everything and pick default mode for CKE=00
#endif
    SCPCR  &= 0xFFF0;   // Enable the SC0
    SCSMR  = 0x80;      // Select synchronous mode

    //SCBRR  = 0xE;       // Set the Bit Rate Register for 500KHz (Peripheral frequency is 30 MHz)
    //SCBRR = 0x1D;     // Set the Bit Rate Register for 250KHz (Peripheral frequency is 30 MHz)
    SCBRR = 0x4A;       // Set the Bit Rate Register for 100KHz (Peripheral frequency is 30 MHz)
    SCSSR = 0x00;       // Clear the error flags
    //Delay1uS(10UL);   // Delay 10us to guarntee at least on bit time has passed
    //SCSCR = 0x30;         // Enable the transmitter and receiver

    EnableGPIO();       // Enable GPIOs of SH3


    SetSpiCs();         // Set GPIO0 to High
	udelay(5UL);
#ifdef INIT_SPI_AT_STARTUP


	ClearSpiCs();
	udelay(5); //Give 5uS of setup for Chip select

// Enable SPI Tx
	EnableSpiTx();


	EnableSpiRx();
#endif

}

/*
 ****************************************************************************
 *  EnableGPIO
 ****************************************************************************
 */

/*
 ****************************************************************************
 *  GpioOut
 ****************************************************************************
 */
void GpioOut (const DWORD dwGpioBase, const int iBitCombo, const BOOLEAN fBitValue)
{
    BYTE    bTemp;

    bTemp = (BYTE)dwGpioBase;   // dwGpioBase is not used in SH3. Keep lint happy
    bTemp = (BYTE)iBitCombo;

    if (fBitValue == GPIO_HI)
    {
        PDDR |= bTemp;
    }
    else
    {
        PDDR &= ~bTemp;
    }
}

void EnableGPIO(void)
{
    // Init them
    GpioOut(0UL, GPIO0, GPIO_LO);
    GpioOut(0UL, GPIO1 | GPIO2 | GPIO3 | GPIO4 | GPIO7, GPIO_HI);

    // Configure Port D as output
    PDCR = 0x5555;
}
#endif

