/***************************************************************************
*
* Copyright (C) 2006-200v  SMSC
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have resceived a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
***************************************************************************
* File: mibStats.h
*/

#ifndef LAN_931X_HEADER_
#define LAN_931X_HEADER_


#include <linux/netdevice.h>
#include <linux/timer.h>
#include <asm/semaphore.h>
#include <linux/rwsem.h>
#include <linux/skbuff.h>

#include <linux/kthread.h>
#include <linux/netlink.h>

#include <net/sock.h>
#include <linux/completion.h>

#include "931xTypes.h"
#include "switchReg.h"
#include "mibStats.h"
#include "931xReg.h"

#define DEBUG_TRACE_ENABLE

#ifdef DEBUG_TRACE_ENABLE
extern BOOLEAN   debugSmi;
extern BOOLEAN   debugI2C;
extern BOOLEAN   debugSpi;

#endif

/*******************************************************
* Macro: SMSC_TRACE
* Description:
*    This macro is used like printk.
*    It can be used anywhere you want to display information
*    For any release version it should not be left in
*      performance sensitive Tx and Rx code paths.
*    To use this macro define USE_TRACE and set bit 0 of debug_mode
*******************************************************/

#ifdef USE_DEBUG
//select debug modes
#define USE_WARNING
#define USE_TRACE
#define USE_ASSERT
#define SMSC_TRACE_STP
//#define USE_ENTER_EXIT
#define USE_SMSC_TRACE_DMA
#endif

#ifdef USING_LINT
    extern void SMSC_TRACE(const char * a, ...);
#else //not USING_LINT

#ifdef USE_TRACE
    extern DWORD debug_mode;
    #ifdef USE_ENTER_EXIT
        #define SMSC_ENTER  printk("----> entering %s\n",__func__);
        #define SMSC_EXIT   printk("<---- Exit %s\n",__func__);
    #else
        #define SMSC_ENTER
        #define SMSC_EXIT
    #endif

    #ifndef USE_WARNING
        #define USE_WARNING
    #endif   // USE_WARNING

    #ifndef USE_SMSC_TRACE_DMA
        #define  SMSC_TRACE_DMA(msg,args...)
    #else
        #define SMSC_TRACE_DMA(msg,args...)			\
        if(debug_mode & 0x01UL) {					\
            printk("SMSC: Function %s:: " msg "\n",__func__, ## args);	\
            }
    #endif   // USE_SMSC_TRACE_DMA

    #define SMSC_TRACE(msg,args...)			\
        if(debug_mode & 0x01UL) {					\
             printk("SMSC931x::%s:: " msg "\n",__func__, ## args);	\
        }
#else
    #define SMSC_TRACE(msg,args...)
//    #define SMSC_ENTER
//    #define SMSC_EXIT
    #define SMSC_TRACE_DMA(msg, args...)
#endif  // USE_TRACE

#define SMSC_TRACE_NONE(msg,args...)
#endif //not USING_LINT

#ifndef SMSC_ERROR
    #define SMSC_ERROR(msg,args...)	\
    printk("SMSC931x::!!!! ERROR !!!:: %s:: " msg "\n",__func__, ## args);

#endif   // USE_WARNING
/*******************************************************
* Macro: SMSC_WARNING
* Description:
*    This macro is used like printk.
*    It can be used anywhere you want to display warning information
*    For any release version it should not be left in
*      performance sensitive Tx and Rx code paths.
*    To use this macro define USE_TRACE or
*      USE_WARNING and set bit 1 of debug_mode
*******************************************************/
#ifdef USING_LINT
extern void SMSC_WARNING(const char * a, ...);
#else //not USING_LINT
#ifdef USE_WARNING
extern DWORD debug_mode;
#ifndef USE_ASSERT
#define USE_ASSERT
#endif
#define SMSC_WARNING(msg, args...)				\
    if(debug_mode) {							\
            printk("SMSC931x:!!!!! WARNING!!!!!: " msg "\n",## args);	\
}
#else
#	define SMSC_WARNING(msg, args...)
#endif
#endif //not USING_LINT

#define LINK_OFF                        (0x00UL)
#define LINK_SPEED_10HD                 (0x01UL)
#define LINK_SPEED_10FD                 (0x02UL)
#define LINK_SPEED_100HD                (0x04UL)
#define LINK_SPEED_100FD                (0x08UL)
#define LINK_SYMMETRIC_PAUSE            (0x10UL)
#define LINK_ASYMMETRIC_PAUSE           (0x20UL)
#define LINK_AUTO_NEGOTIATE             (0x40UL)
#define MAX_PROTO_LEN                   1500
typedef enum  CONTROL_ACCESS_E{
UNKNOWN_ACCESS,
SMI_ACCESS,
SPI_ACCESS,
I2C_ACCESS,
HBI_ACCESS,
} CNTROL_ACCESS;



#ifndef CHIP_IS_LAN9303
const char *switch_port_state_str[4] =
{
"FORWARDING", //0
"BLOCKING",   //1
"LEARNING",   //2
"LISTENING",   //3
} ;

typedef enum SWITCH_PORT_STATE_S {
// registe Port State register 0x1843
SWITCH_PORT_FORWARDING, //0
SWITCH_PORT_BLOCKING,   //1
SWITCH_PORT_LEARNING,   //2
SWITCH_PORT_LISTENING,   //3
SWITCH_PORT_INVALID_STATE   //3

} SWITCH_PORT_STATE;
#else

const char *switch_port_state_str[4] =
{
"FORWARDING", //0
"BLOCKING",   //1 /* LISTENING */
"LEARNING",   //2
"DISABLED"    //3
} ;

typedef enum SWITCH_PORT_STATE_S {
// registe Port State register 0x1843
SWITCH_PORT_FORWARDING, //0
SWITCH_PORT_BLOCKING,   //1
SWITCH_PORT_LEARNING,   //2
SWITCH_PORT_DISABLED,   //3
SWITCH_PORT_INVALID_STATE   //5
} SWITCH_PORT_STATE;
#define SWITCH_PORT_LISTENING SWITCH_PORT_BLOCKING;
#endif

typedef enum  PORT_STATE_E{
    STP_BLOCKING = 0x8001,   // start from non zero number
    STP_LISTENING,
    STP_LEARNING,
    STP_FORWARDING,
    STP_DISABLED,
    STP_ENABLED
} PORT_STATE;
typedef enum  PACKET_TYPE_S{
    TYPE_UKNOWN,
    STP_TYPE_PACKET,
    PTP_TYPE_PACKET,
    IGMP_TYPE_PACKET,
    MLD_TYPE_PACKET,
    OTHER_TYPE_PACKET
} PACKET_TYPE;
#define PKT_TYPE_IS_IGMP IGMP_TYPE_PACKET
#define PKT_TYPE_IS_MLD  MLD_TYPE_PACKET


const unsigned char *ctrlInfo[5]=
{
    "HYDRA HBI  ",
    "LEO   SMI ",
    "LEO   SPI ",
    "LEO   I2C ",
    "UNKNOWN "
};
typedef struct _DMA_XFER
{
    DWORD dwLanReg;
    DWORD *pdwBuf;
    DWORD dwDmaCh;
    DWORD dwDwCnt;
    BOOLEAN fMemWr;
} DMA_XFER;


typedef struct _FLOW_CONTROL_PARAMETERS
{
    DWORD MaxThroughput;
    DWORD MaxPacketCount;
    DWORD PacketCost;
    DWORD BurstPeriod;
    DWORD IntDeas;
} FLOW_CONTROL_PARAMETERS, *PFLOW_CONTROL_PARAMETERS;

typedef struct _PLATFORM_DATA {
    DWORD dwBitWidth;
    DWORD dwIdRev;
    DWORD dwIrq;
    void * dev_id;
} PLATFORM_DATA, *PPLATFORM_DATA;

typedef unsigned long VL_KEY;
typedef struct _VERIFIABLE_LOCK {
    spinlock_t Lock;
    VL_KEY KeyCode;
} VERIFIABLE_LOCK, * PVERIFIABLE_LOCK;


/**
* The following are for Tagged packets
which is used for FULL managed switchReg

All packets coming from the switch are tagged with special VLAN tag
All packets sent to he switch from this driver must be tagged

**/

// 2 bytes+ 12 MAC + 4 CPU + 2 VLAN
#define TAGGED_INFO_SIZE  (16 + 4)
#define TAG_SIZE          4



typedef struct TAG_FORMAT_S {
    unsigned char dummy_1[2];
    unsigned char dstMac[6];
    unsigned char srcMAc[6];
    WORD     vlanTag;
    WORD     PortId;
    WORD     Type;
} TAG_FORMAT, *PTAG_FORMAT;

typedef struct TAGGED_DATA_INFO_S {
    unsigned char tagged_data[TAGGED_INFO_SIZE];  // 20 bytes
    struct sk_buff *tagged_skb;
    BOOLEAN valid_tag;
    BOOLEAN isMulticast;
} TAGGED_DATA_INFO, *PTAGGED_DATA_INFO;


typedef struct _PRIVATE_DATA {
    DWORD dwLanBase;
    BOOLEAN isSwitchPort;
    DWORD   PortNumber;
    /** For RSTP support **/
    BOOLEAN rstpEnabled;
    PORT_STATE rstpState;
    BOOLEAN    stpAlrSet;
#ifdef FULLY_MANAGED
    struct sk_buff_head StpQueue;
    spinlock_t stpQueueLock;
    DWORD      StpQueueDebt;
    PACKET_TYPE packetType;
    /** PTP  Support **/
    BOOLEAN   ptpStarted;
    struct task_struct *ptp_thread;
    pid_t    ptp_thread_pid;
    pid_t    ptp_deamon_pid;
    struct sock *nls;
    struct completion ptp_ready;
    struct sk_buff_head PtpQueue;
    spinlock_t ptpQueueLock;
    /** IGMP Support */
    BOOLEAN   SnoopingStarted;
    struct task_struct *igmp_thread;
    pid_t    igmp_thread_pid;
    pid_t    igmp_deamon_pid;
    struct sock *igmp_nls;
    struct completion igmp_ready;
    struct sk_buff_head IgmpQueue;
    spinlock_t igmpQueueLock;
    int      SnoopedPktCnt;
#endif
    BOOLEAN   VLANEnable;
    BOOLEAN DAHighPriorityEnbl;
    DWORD dwIdRev;
    DWORD dwFpgaRev;
    void  (*PlatformSetRegDW)(
                    DWORD dwLanBase,
                    DWORD dwOffset,
                    DWORD dwVal);
    DWORD (*PlatformGetRegDW)(
                        DWORD dwLanBase,
                        DWORD dwOffset);
    CNTROL_ACCESS access_mode;
    struct net_device *dev;

#ifndef UNMANAGED
    struct net_device *switchDev[PHY_NUMBER];
#endif

    DWORD dwGeneration;//used to decide which workarounds apply

    spinlock_t IntEnableLock;
    BOOLEAN LanInitialized;
    VERIFIABLE_LOCK MacPhyLock;
    VERIFIABLE_LOCK PtpAccessLock;
    spinlock_t SwitchRegLock;
    int      nbLock;
    VERIFIABLE_LOCK SwitchAccessLock;

    DWORD dwTxDmaCh;
    BOOLEAN TxDmaChReserved;
    DMA_XFER TxDmaXfer;
    DWORD dwTxDmaThreshold;
    DWORD dwTxQueueDisableMask;
    struct sk_buff *TxSkb;
    spinlock_t TxSkbLock;
    spinlock_t TxQueueLock;
    spinlock_t TxCounterLock;
    BOOLEAN TxInitialized;

    DWORD dwRxDmaCh;
    struct sk_buff *RxSkb;
    DWORD rxStatus;
    // Lock access to tagged data received
    // This is used mainly in FULL managed switch
    spinlock_t taggedInfoLock;
    TAGGED_DATA_INFO   taggedInfo;

    BOOLEAN RxDmaChReserved;
    DWORD dwRxDmaThreshold;
    BOOLEAN RxCongested;
    DWORD dwRxOffCount;
    BOOLEAN RxOverrun;
    DWORD RxOverrunCount;
    DWORD RxStatusDWReadCount;
    DWORD RxDataDWReadCount;
    DWORD RxPacketReadCount;
    DWORD RxFastForwardCount;
    DWORD RxPioReadCount;
    DWORD RxDmaReadCount;
    DWORD RxCongestedCount;
    DWORD RxDumpCount;
    DWORD LastReasonForReleasingCPU;
    DWORD LastRxStatus1;
    DWORD LastRxStatus2;
    DWORD LastRxStatus3;
    DWORD LastIntStatus1;
    DWORD LastIntStatus2;
    DWORD LastIntStatus3;
    DWORD RxUnloadProgress;
    DWORD RxUnloadPacketProgress;
    DWORD RxMaxDataFifoSize;

    DWORD RxFlowCurrentThroughput;
    DWORD RxFlowCurrentPacketCount;
    DWORD RxFlowCurrentWorkLoad;
    BOOLEAN MeasuringRxThroughput;
    DWORD RxFlowMeasuredMaxThroughput;
    DWORD RxFlowMeasuredMaxPacketCount;

//RX_FLOW_ACTIVATION specifies the percentage that RxFlowCurrentWorkLoad must exceed
//     RxFlowMaxWorkLoad in order to activate flow control
#define RX_FLOW_ACTIVATION	(4UL)

//RX_FLOW_DEACTIVATION specifies the percentage that RxFlowCurrentWorkLoad must reduce
//     from RxFlowMaxWorkLoad in order to deactivate flow control
#define RX_FLOW_DEACTIVATION (25UL)
    DWORD RxFlowMaxWorkLoad;

    FLOW_CONTROL_PARAMETERS RxFlowParameters;

    DWORD RxFlowBurstWorkLoad;
    DWORD RxFlowBurstMaxWorkLoad;
    BOOLEAN RxFlowControlActive;
    BOOLEAN RxFlowBurstActive;
    DWORD RxInterrupts;

#define GPT_SCHEDULE_DEPTH	(3)
    void *GptFunction[GPT_SCHEDULE_DEPTH];
    DWORD GptCallTime[GPT_SCHEDULE_DEPTH];
    DWORD Gpt_scheduled_slot_index;
    spinlock_t GpTimerLock;

    BOOLEAN Running;
    struct net_device_stats stats;
        // Phy Registers need to be changed...mobbad we have three PHYs.
    DWORD dwPhyAddress;
    DWORD dwPhyId;

    BYTE bPhyModel;
    BYTE bPhyRev;
    DWORD dwLinkSpeed;
    DWORD dwLinkSettings;
    DWORD dwRemoteFaultCount;
    struct timer_list LinkPollingTimer;
    BOOLEAN StopLinkPolling;
#ifdef FULLY_MANAGED
    struct timer_list SwitchLinkPollingTimer;
    BOOLEAN StopSwitchLinkPolling;
    DWORD   SwitchPort1LinkSpeed;
    DWORD   SwitchPort2LinkSpeed;
    BOOLEAN SwitchPort1StateChanged;
    BOOLEAN SwitchPort2StateChanged;
#endif

#ifdef SNMP_SUPPORT
        // Mib Stuff
        struct timer_list MibPollingTimer;
        BOOLEAN StopMibPolling;
        spinlock_t mibPollTimerLock;
#endif
    /** ETHER-LIKE stats **/
    SwitchMibCounters       	switchMibCounter[PHY_NUMBER];
    Dot3StatsTable_data     	dot3StatsTable_data_array[PHY_NUMBER];
    Dot3CollTable_data      	dot3CollTable_data_array[PHY_NUMBER];
    Dot3ControlTable_data   	dot3ControlTable_data_array[PHY_NUMBER];
    Dot3PauseTable_data     	dot3PauseTable_data_array[PHY_NUMBER];
    Dot3HCStatsTable_data   	dot3HCStatsTable_data_array[PHY_NUMBER];
    /** RMON stats **/
    EtherStatsTable_data    	etherStatsTable_data_array[PHY_NUMBER];
    /** IEEE802_3 Stats Table Data**/
    IEEE802_3StatsTable_data 	MACStatsTable_data_array[PHY_NUMBER];
    Bridge_Stats_data			BridgeStatsTable_data_array[PHY_NUMBER];
    /**** Time ***/
    struct timespec startUpTime;
    // Time sine the interface is in actual state */
    struct timespec interfaceChangeTime[PHY_NUMBER];
    /** IF_MIB**/
    IfTable_data  IfTable_data_array[PHY_NUMBER];
    IfXTable_Data IfXTable_Data_array[PHY_NUMBER];
// #endif
    WORD wLastADV;
    WORD wLastADVatRestart;


    BOOLEAN SoftwareInterruptSignal;

    PLATFORM_DATA PlatformData;

    //NAPI
    DWORD RxProcessingMode;
    int RxWorkLimit;
    int RxPacketsReceived;
    BOOLEAN RxDone;

#define SMSC_IF_NAME_SIZE	(10)
    char ifName[SMSC_IF_NAME_SIZE];

    /* for Rx Multicast work around */
    volatile DWORD HashLo;
    volatile DWORD HashHi;
    volatile BOOLEAN MulticastUpdatePending;
    volatile DWORD set_bits_mask;
    volatile DWORD clear_bits_mask;


} PRIVATE_DATA, *PPRIVATE_DATA;


/**
**/
struct smc931x_ethhdr {
        unsigned char   h_dest[ETH_ALEN];       /* destination eth addr */
        unsigned char   h_source[ETH_ALEN];     /* source ether addr    */
        unsigned short  vlan[2];
        unsigned short  h_proto;                /* packet type ID field */
} __attribute__((packed));


typedef struct _SWITCH_PORT_PRIVATE_DATA {
    DWORD    dwLanBase;
    BOOLEAN  isSwitchPort;
    DWORD    PortNumber;
    /** For RSTP support **/
    BOOLEAN rstpEnabled;
    PORT_STATE rstpState;
    BOOLEAN  Disabled ;
	/** PTP Support **/
	BOOLEAN ptpEnabled;

    spinlock_t deviceLock;
    void  (*PlatformSetRegDW)(
            DWORD dwLanBase,
            DWORD dwOffset,
            DWORD dwVal);
    DWORD (*PlatformGetRegDW)(
            DWORD dwLanBase,
            DWORD dwOffset);
    struct net_device *dev;
    BOOLEAN LanInitialized;
    VERIFIABLE_LOCK MacPhyLock;
    struct rw_semaphore SwitchRegLock;

    struct sk_buff *TxSkb;
    spinlock_t TxSkbLock;
    spinlock_t TxQueueLock;
    struct sk_buff *RxSkb;
    DWORD dwPhyAddress;
    DWORD dwPhyId;
    char ifName[SMSC_IF_NAME_SIZE];
    PPRIVATE_DATA pMaster;
    struct net_device_stats stats;


} SWITCH_PRIVATE_DATA, *PSWITCH_PPRIVATE_DATA;

#define LanGetRegDW(dwOffset)	\
    privateData->PlatformGetRegDW(privateData->dwLanBase,dwOffset)

#define LanSetRegDW(dwOffset,dwVal) \
    privateData->PlatformSetRegDW(privateData->dwLanBase,dwOffset,dwVal)



/**   Read direct register, memory mapped */
#define LanGetDirectRegDW(dwOffset) \
    Platform_GetRegDW(privateData->dwLanBase,dwOffset)
#define LanSetDirectRegDW(dwOffset,dwVal) \
    Platform_SetRegDW(privateData->dwLanBase,dwOffset,dwVal)


#define Lan_ClrBitsDW(dwOffset,dwBits)    \
    LanSetDirectRegDW(dwOffset,LanGetDirectRegDW(dwOffset) & (~dwBits) )

#define Lan_SetBitsDW(dwOffset,dwBits)                      \
    LanSetDirectRegDW(dwOffset,LanGetDirectRegDW(dwOffset)|dwBits);

extern void  smsc931xEnableIRQ(void);
extern void  smsc931xDisableIRQ(void);

#define ETH_ADDR_SIZE 6

#endif
