/***************************************************************************
*
* Copyright (C) 2004-2005  SMSC
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
***************************************************************************
* File: smsc931x.c
*   see readme.txt for programmers guide
*
*
*  2/17/06  Mobbad  modified skb reserve to 16 if FULLY MANAGED
*/

#ifndef __KERNEL__
#	define __KERNEL__
#endif

#ifdef USING_LINT

#include "lint.h"

#else //not USING_LINT

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/errno.h>


#include <linux/etherdevice.h>
#include <linux/delay.h>
#include <linux/mii.h>
//#include <linux/timer.h>
#include <asm/irq.h>
#include <asm/dma.h>
#include <asm/bitops.h>
#include <linux/version.h>


#include <linux/time.h>
#ifdef FULLY_MANAGED
/*#undef _HAVE_ARCH_IPV6_CSUM*/
/*#include <net/ip6_checksum.h>*/
#include <linux/ipv6.h>
#include <linux/netlink.h>
#include <linux/if_vlan.h>
#include "snoop_msg.h"
#endif

#ifdef HYDRA_MODE
    #include "lan9312Config.h"
#else
    #include "lan9313Config.h"
#endif
#include "switchReg.h"
#include "mibStats.h"
#include "931xReg.h"
#include "smsc931x.h"
#include "931xControlInterface.h"
#include "ioctl_118.h"

#endif //not USING_LINT


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#undef NAPI_PROCESS
#else
#define LINUX_2_6_OR_NEWER
#define NAPI_PROCESS
#endif
#define TOTAX_MTHDS 3
#define SWITCH_PORT_MASK 0x7
/**
*********************************************************
*   DEBUG Variables
*       Can be removed
*
*********************************************************
**/
#ifdef DEBUG_TRACE_ENABLE
BOOLEAN   debugSmi = 0;
BOOLEAN   debugI2C = 0;
BOOLEAN   debugSpi = 0;

#endif

const unsigned char RSTP_MAC_ADDRESS[6] = { 0x01, 0x80, 0xc2, 0x00, 0x00, 0x00 };
const unsigned char BROADCAST_ADDRESS[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };


const unsigned char RSVD_MCAST_ADDRESS[16][6]={ {0x01,0x80,0xC2,0x00,0x00,0x00},
												{0x01,0x80,0xC2,0x00,0x00,0x01},
												{0x01,0x80,0xC2,0x00,0x00,0x02},
												{0x01,0x80,0xC2,0x00,0x00,0x03},
												{0x01,0x80,0xC2,0x00,0x00,0x04},
												{0x01,0x80,0xC2,0x00,0x00,0x05},
												{0x01,0x80,0xC2,0x00,0x00,0x06},
												{0x01,0x80,0xC2,0x00,0x00,0x07},
												{0x01,0x80,0xC2,0x00,0x00,0x08},
												{0x01,0x80,0xC2,0x00,0x00,0x09},
												{0x01,0x80,0xC2,0x00,0x00,0x0A},
												{0x01,0x80,0xC2,0x00,0x00,0x0B},
												{0x01,0x80,0xC2,0x00,0x00,0x0C},
												{0x01,0x80,0xC2,0x00,0x00,0x0D},
												{0x01,0x80,0xC2,0x00,0x00,0x0E},
												{0x01,0x80,0xC2,0x00,0x00,0x0F},};
const DWORD   RSTP_MAC_ADDRESS_HI = 0x0000UL;
const DWORD   RSTP_MAC_ADDRESS_LO = 0x0180c200;

#define IP_PROTO_TYPE   0x0800
#define VLAN_TYPE       0x8100
#define IGMP_PROTOCOL   0x2
#define IPV6_PROTO_TYPE 0x86DD

static PPRIVATE_DATA thisModulePrivate = NULL;

#define TRANSFER_PIO			(256UL)
#define TRANSFER_REQUEST_DMA	        (255UL)
#define PHY_MAP	           	7
//these are values that can be assigned to
//PLATFORM_RX_DMA
//PLATFORM_TX_DMA
// in addition to any specific dma channel




/*******************************************************
* Macro: SMSC_ASSERT
* Description:
*    This macro is used to test assumptions made when coding.
*    It can be used anywhere, but is intended only for situations
*      where a failure is fatal.
*    If code execution where allowed to continue it is assumed that
*      only further unrecoverable errors would occur and so this macro
*      includes an infinite loop to prevent further corruption.
*    Assertions are only intended for use during developement to
*      insure consistency of logic through out the driver.
*    A driver should not be released if assertion failures are
*      still occuring.
*    To use this macro define USE_TRACE or USE_WARNING or
*      USE_ASSERT
*******************************************************/
#ifdef USING_LINT
extern void SMSC_ASSERT(BOOLEAN condition);
#else //not USING_LINT
#ifdef USE_ASSERT
#define SMSC_ASSERT(condition)													\
if(!(condition)) { \
    printk("SMSC_ASSERTION_FAILURE: File=" __FILE__ ",Function=%s Line=%d\n",__func__,__LINE__);	\
        while(TRUE); \
}
#else
#	define SMSC_ASSERT(condition)
#endif
#endif //not USING_LINT

//Below are the register offsets and bit definitions
//  of the Lan9118 memory space


#define LAN_REGISTER_EXTENT		(0x00002000UL)

/** Default MAC Address **/
// DWORD DW_HIGH_MAC16 = 0x00000070UL;
DWORD DW_HIGH_MAC16 = 0x12345670UL;
DWORD DW_LOW_MAC    = 0x110F8000UL;

#include PLATFORM_SOURCE
#include PLATFORM_ACCESS
#ifdef CHIP_IS_LAN9303
#include <linux/ipv6.h>
#include LAN9303_PERI_ACCESS
#endif

//PLATFORM_SOURCE is defined from the command line
//  with the -D option
//  example: -D"PLATFORM_SOURCE=\"platform.c\""

void Vl_InitLock(PVERIFIABLE_LOCK pVl);
BOOLEAN Vl_CheckLock(PVERIFIABLE_LOCK pVl,VL_KEY keyCode);
VL_KEY Vl_WaitForLock(PVERIFIABLE_LOCK pVl,DWORD *pdwIntFlags);
void Vl_ReleaseLock(PVERIFIABLE_LOCK pVl,VL_KEY keyCode,DWORD *pdwIntFlags);
void  getSwitchReadLock(PPRIVATE_DATA priv);
void  releaseSwitchReadLock(PPRIVATE_DATA priv);
void  getSwitchWriteLock(PPRIVATE_DATA priv);
void  releaseSwitchWriteLock(PPRIVATE_DATA priv);

/* Check the Access mode (HBI, SPI, I2C, SMI) */
void checktheAccessMode(PPRIVATE_DATA privateData);


#ifdef TEST_DMA
DWORD   nb_interrupt   = 0;
DWORD   nb_rxInterrupt = 0;
DWORD   nb_txInterrupt = 0;
DWORD   nb_rxPorcessInterrupt = 0;

#endif

extern void I2cHwInit(void);
extern void SpiHwInit(void);
/** *********************************************
*
*    MAC  BLOCK
*
************************************************
*/

BOOLEAN 	Mac_Initialize(PPRIVATE_DATA privateData);
static 		BOOLEAN MacNotBusy(PPRIVATE_DATA privateData,VL_KEY keyCode);
DWORD 		Mac_GetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,VL_KEY keyCode);
void 		Mac_SetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,DWORD dwVal,VL_KEY keyCode);



/** *********************************************
*
*    PHY BLOCK
*
************************************************
*/

/** *************************************************************
*      ACCESS To PHY APIS
*      Access to PHY can be direct using MII or indirect using PMI
*
*   **************************************************************
**/


BOOLEAN Phy_Initialize(
                    PPRIVATE_DATA privateData,
                    DWORD dwPhyIndex,
                    DWORD dwPhyAddress,
                    DWORD dwLinkMode);
void Phy_SetLink(PPRIVATE_DATA privateData,
                    DWORD dwPhyIndex,
                    DWORD dwLinkRequest);

/** =================================================
*  General APIs to access PHY
** =================================================
**/

WORD Phy_GetRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            VL_KEY keyCode);

void Phy_SetRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            WORD wVal,
            VL_KEY keyCode);

/** **************************
*  Platform dependent PHY access :
PMI for SPI and I2c access
MII or Direct  for SMI access
** *******************************
**/
static void PhySetDirectRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            WORD wVal,
            VL_KEY keyCode);

static WORD PhyGetDirectRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            VL_KEY keyCode);


static WORD PhyGetPmiRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            VL_KEY keyCode);

static void PhySetPmiRegW(
            PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwRegIndex,
            WORD wVal,
            VL_KEY keyCode);



void PmiSetRegW( PPRIVATE_DATA privateData,
            const DWORD dwPhyAddress,
            const DWORD dwMiiIndex,
            const WORD wVal);
WORD PmiGetRegW( PPRIVATE_DATA privateData,
            const DWORD dwPhyAddress,
            const DWORD dwMiiIndex);

/** ******************************************
*   PHY LINK
*********************************************
**/

void Phy_UpdateLinkMode(
                PPRIVATE_DATA privateData);
DWORD  Phy_GetLinkMode(
                PPRIVATE_DATA privateData,
                unsigned long PhyIndex,
                VL_KEY keyCode);
void Phy_CheckLink(unsigned long ptr);
#ifdef FULLY_MANAGED
void Phy_CheckSwitchLink(unsigned long ptr);
#endif

/** *********************************************
*
*    GPT BLOCK
*
************************************************
*/
TIME_SPAN Gpt_FreeRunCompare(DWORD time1,DWORD time2);
void Gpt_ScheduleInterrupt(PPRIVATE_DATA privateData,TIME_SPAN timeSpan);
void Gpt_CancelInterrupt(PPRIVATE_DATA privateData);
void Gpt_CancelCallBack(
            PPRIVATE_DATA privateData,
            void (*callBackFunction)(PPRIVATE_DATA privateData));
void Gpt_ScheduleCallBack(
            PPRIVATE_DATA privateData,
            void (*callBackFunction)(PPRIVATE_DATA privateData),
            DWORD callBackTime);//100uS units relative to now
            BOOLEAN Gpt_HandleInterrupt(
            PPRIVATE_DATA privateData,DWORD dwIntSts);
void GptCB_RxCompleteMulticast(PPRIVATE_DATA privateData);
void GptCB_RestartBurst(PPRIVATE_DATA privateData);
void GptCB_MeasureRxThroughput(PPRIVATE_DATA privateData);


/** *********************************************
*
*    TX BLOCK
*
************************************************
*/

void Tx_Initialize(
                PPRIVATE_DATA privateData,
                DWORD dwTxDmaCh,
                DWORD dwTxDmaThreshold);

#ifdef FULLY_MANAGED
void Switch_SendSkb(
            PPRIVATE_DATA privateData,
            struct sk_buff *skb);
#endif

void Tx_SendSkb(
                PPRIVATE_DATA privateData,
                struct sk_buff *skb);
BOOLEAN Tx_HandleInterrupt(
                PPRIVATE_DATA privateData,
                DWORD dwIntSts);
void Tx_StopQueue(
                PPRIVATE_DATA privateData,
                DWORD dwSource);
void Tx_WakeQueue(
                PPRIVATE_DATA privateData,
                DWORD dwSource);
static DWORD Tx_GetTxStatusCount(
                        PPRIVATE_DATA privateData);
static DWORD Tx_CompleteTx(
                    PPRIVATE_DATA privateData);
void Tx_UpdateTxCounters(
                    PPRIVATE_DATA privateData);

void Tx_CompleteDma(
                PPRIVATE_DATA privateData);


/** *********************************************
*
*    RX BLOCK
*
************************************************
*/
void Rx_Initialize(
                    PPRIVATE_DATA privateData,
                    DWORD dwRxDmaCh,
                    DWORD dwDmaThreshold);

void Rx_CompleteMulticastUpdate (PPRIVATE_DATA privateData);
static void Rx_HandleOverrun(PPRIVATE_DATA privateData);
static void Rx_HandOffSkb(
                        PPRIVATE_DATA privateData,
                        struct sk_buff *skb);
static DWORD Rx_PopRxStatus(
                    PPRIVATE_DATA privateData);
BOOLEAN Rx_CountErrors(PPRIVATE_DATA privateData,DWORD dwRxStatus);
void Rx_FastForward(PPRIVATE_DATA privateData,DWORD dwDwordCount);
void Rx_ProcessPackets(PPRIVATE_DATA privateData);
void Rx_BeginMulticastUpdate (PPRIVATE_DATA privateData);

unsigned long Rx_TaskletParameter=0;

void Rx_ProcessPacketsTasklet(unsigned long data);

DECLARE_TASKLET(Rx_Tasklet,Rx_ProcessPacketsTasklet,0);

#ifdef FULLY_MANAGED
void Rx_ProcessStpPacket(unsigned long data);
DECLARE_TASKLET(STP_Tasklet,Rx_ProcessStpPacket,0);

void SwitchProcessLinkState(unsigned long data);
DECLARE_TASKLET(STP_LINK_Tasklet,SwitchProcessLinkState,0);

void Rx_ProcessIgmpPacket(unsigned long data);
DECLARE_TASKLET(IGMP_Tasklet,Rx_ProcessIgmpPacket,0);

#endif

struct sk_buff *readQueue(struct sk_buff_head *queue) ;
void WriteQueue(struct sk_buff_head *queue, struct sk_buff *skb) ;


BOOLEAN RxStop_HandleInterrupt(
PPRIVATE_DATA privateData,
DWORD dwIntSts);
BOOLEAN Rx_HandleInterrupt(
PPRIVATE_DATA privateData,
DWORD dwIntSts);
static DWORD Rx_Hash(BYTE addr[6]);

void Rx_SetMulticastList(
struct net_device *dev);
void Rx_ReceiverOff(
PPRIVATE_DATA privateData);
void Rx_ReceiverOn(
PPRIVATE_DATA privateData, VL_KEY callerKeyCode);



#ifdef FULLY_MANAGED
static int  Rx_RestoreTaggedData(PPRIVATE_DATA privateData,struct sk_buff *skb) ;
#endif

void   dumpSkb(struct sk_buff *skb);
/** *********************************************
*
*    EEPROM  BLOCK
*
************************************************
*/
void Eeprom_EnableAccess(PPRIVATE_DATA privateData);
void Eeprom_DisableAccess(PPRIVATE_DATA privateData);

BOOLEAN Eeprom_IsMacAddressLoaded(PPRIVATE_DATA privateData);
BOOLEAN Eeprom_IsBusy(PPRIVATE_DATA privateData);
BOOLEAN Eeprom_Timeout(PPRIVATE_DATA privateData);

BOOLEAN Eeprom_ReadLocation(PPRIVATE_DATA privateData,WORD address, BYTE * data);
BOOLEAN Eeprom_EnableEraseAndWrite(
PPRIVATE_DATA privateData);
BOOLEAN Eeprom_DisableEraseAndWrite(
PPRIVATE_DATA privateData);
BOOLEAN Eeprom_WriteLocation(
PPRIVATE_DATA privateData,WORD address,BYTE data);
BOOLEAN Eeprom_EraseAll(
PPRIVATE_DATA privateData);
BOOLEAN Eeprom_Reload(
PPRIVATE_DATA privateData);

BOOLEAN Eeprom_SaveMacAddress(
PPRIVATE_DATA privateData,
DWORD dwHi16,DWORD dwLo32);


/** *********************************************
*
*    SWITCH REGISTER   BLOCK
*
************************************************
*/
#ifndef CHIP_IS_LAN9303 /* LAN9312 , LAN9313 */
typedef union ALR_C {
struct alr_data1 {
    DWORD MAcHigh:16;
    DWORD PortId:3;
    DWORD Priority:2;
    DWORD Filter  :1;
    DWORD Static:1;
    DWORD age:1;
    DWORD valid:1 ;
    DWORD res:7;
}alr_data1;
DWORD DATA1;
} ALR_UNION;
typedef struct __ALRFieldProp{
	 DWORD port;
	 DWORD pri;
	 DWORD filter;
	 DWORD statc;
	 DWORD age_alias_oride;
}ALREntryProp;
#else /* LEO LITE */
typedef union ALR_C {
struct alr_data1 {
    DWORD MAcHigh:16;
    DWORD PortId:3;
    DWORD Priority:3;
    DWORD PriorityEnbl:1;
    DWORD Filter  :1;
    DWORD Static:1;
    DWORD age:1;
    DWORD valid:1 ;
    DWORD res:5;
}alr_data1;
DWORD DATA1;
} ALR_UNION;
typedef struct __ALRFieldProp{
	 DWORD port;
	 DWORD pri_alia_tag;
	 DWORD prienbl;
	 DWORD filter;
	 DWORD statc;
	 DWORD age_alias_oride;
}ALREntryProp;

const DWORD CPU_TAG_STP_ID = 0x5;
const DWORD CPU_TAG_IGMP_ID = 0x4;
const DWORD CPU_TAG_PTP_ID = 0x3; /* PTP is not supported in Leo Lite, Just for completion*/
/* CPU TAG Declarations for LeoLite */
#define CPU_TAG_PORT_MASK  			0x0003 /* for Leo Lite */
#define CPU_TAG_SNOOP_MSK  			0x0008
#define CPU_TAG_STP_ORIDE  			0x0010
#define CPU_TAG_MAC_STATIC 			0x0020
#define CPU_TAG_ALR_PRIOENBL 		0x0040
#define CPU_TAG_PRIO_ALIAS_ID_FIELD 0x0380
#define CPU_TAG_STPORIDE_AND_STATIC (CPU_TAG_STP_ORIDE | CPU_TAG_MAC_STATIC)

#define EGRS_CPU_TAG_VLAN_RULE_SET 0x0040
#define EGRS_CPU_TAG_VLAN_PRI_SET  0x0020
#define EGRS_CPU_TAG_STP_ORIDE_SET 0x0010
#define EGRS_CPU_TAG_PRI_SET       EGRS_CPU_TAG_VLAN_PRI_SET

#define DIFF_PAUSE_MACADDR_EN_  (0x400000)
#define PORT2_MAC_ADDR   (0x200000)
#define PORT1_MAC_ADDR   (0x40000)
void EnblTurboMode(PPRIVATE_DATA privateData, int portNo);
void DisblTurboMode(PPRIVATE_DATA privateData, int portNo);
int isIPV6Pkt(char* pkt, unsigned int len);
#endif

DWORD Switch_GetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset);
void  Switch_SetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,DWORD dwVal);
void  Switch_Add_Entry_in_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, ALREntryProp *entryProp);
void  Switch_Add_Static_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, ALREntryProp *entryProp);
void  Switch_remove_Static_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, BYTE portMap);
void  Switch_Enbl_Fltr_on_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc);
BOOLEAN  Switch_Rad_First_ALR(PPRIVATE_DATA privateData, DWORD *data0, DWORD *data1);
BOOLEAN  Switch_Rad_Next_ALR(PPRIVATE_DATA privateData, DWORD *data0, DWORD *data1);

void Switch_Enbl_disbl_shortAgeTimer(PPRIVATE_DATA privateData, DWORD regValue);
void  Switch_Set_port_state(PPRIVATE_DATA privateData, DWORD PortNumber, SWITCH_PORT_STATE portState);
SWITCH_PORT_STATE  Switch_Get_port_state(PPRIVATE_DATA privateData, DWORD PortNumber);

#ifdef FULLY_MANAGED
void EnblPermanentDB(PPRIVATE_DATA privateData);
int  setPortsToCPUTag(PPRIVATE_DATA privateData);
void  InitializeSkbQueues(PPRIVATE_DATA privateData);
#endif
#ifndef UNMANAGED
void  mib_start_timer(PPRIVATE_DATA privateData);
void  MibUpdateMibCounters(unsigned long ptr);
void  mibUpdateMibPortCounters( PPRIVATE_DATA privateData );

#endif

#ifdef FULLY_MANAGED

/** **********************************************
*
*   Spanning Tree Block
*
************************************************
*/

void EnableSpanningTree(PSWITCH_PPRIVATE_DATA priv);
void DisableSpanningTree(PSWITCH_PPRIVATE_DATA priv);
void STP_Set_Port_State(PSWITCH_PPRIVATE_DATA priv ,PORT_STATE state);
void stpAddSkpToRxQueue(PPRIVATE_DATA privateData , struct sk_buff *skb);

/** **********************************************
 *
 *   IGMP  Block
 *
 ************************************************
 */
void SwitchDisableIGMP(PPRIVATE_DATA privateData);
void SwitchEnableIGMP(PPRIVATE_DATA privateData);
#ifdef CHIP_IS_LAN9303
void SwitchDisableMLD(PPRIVATE_DATA privateData);
void SwitchEnableMLD(PPRIVATE_DATA privateData);
#endif
int  igmpProcessMsg(void *data);
void igmp_rx_msg_ready (struct sock *sk, int len);
static void sendSnoopedMsgToManager( PPRIVATE_DATA privateData, const char *data, DWORD datalen, PACKET_TYPE pktType, int portNumber);
void igmpAddSkpToRxQueue(PPRIVATE_DATA privateData, struct sk_buff *skb);
int isIGMPPacket( char *data ,int len );
int igmpPaktype=0;
int isVLANPacket(char *data ,int len, int *proto );
#endif   /** FULLY_MANAGED */

/* not sure where to add VLAN stuff fully managed or un managed so currently avilable in all configration---Muthu  */

void SwitchDisableVLAN(PPRIVATE_DATA privateData);

void SwitchEnableVLAN(PPRIVATE_DATA privateData);
void SwitchEnableDAPriority(PPRIVATE_DATA privateData);
void SwitchDisableDAPriority(PPRIVATE_DATA privateData);

#define OLD_REGISTERS(privData) (((privData->dwIdRev)==0x01180000UL)&& \
                            ((privData->dwFpgaRev)>=0x01)&& \
                            ((privData->dwFpgaRev)<=0x25))

extern volatile DWORD g_GpioSetting;
extern DWORD debug_mode;
#define GP_0	(0x01UL)
#define GP_1	(0x02UL)
#define GP_2	(0x04UL)
#define GP_3	(0x08UL)
#define GP_4	(0x10UL)
#define GP_OFF  (0x00UL)
#define GP_ISR	GP_OFF
#define GP_RX	GP_OFF
#define GP_TX	GP_OFF
#define GP_BEGIN_MULTICAST_UPDATE		GP_OFF
#define GP_COMPLETE_MULTICAST_UPDATE	GP_OFF

#define SET_GPIO(gpioBit)					\
if(debug_mode&0x04UL) {						\
 g_GpioSetting|=gpioBit;					\
 LanSetDirectRegDW(GPIO_CFG,g_GpioSetting);	\
}

#define CLEAR_GPIO(gpioBit)					\
if(debug_mode&0x04UL) {						\
 g_GpioSetting&=(~gpioBit);				    \
 LanSetDirectRegDW(GPIO_CFG,g_GpioSetting); \
}

#define PULSE_GPIO(gpioBit,count)	\
if(debug_mode&0x04UL) {				\
DWORD pulseNum=0;				\
/*make first pulse longer */	\
SET_GPIO(gpioBit);				\
while(pulseNum<count) {			\
    SET_GPIO(gpioBit);			\
    CLEAR_GPIO(gpioBit);		\
    pulseNum++;					\
}								\
}


BOOLEAN Lan_Initialize(
PPRIVATE_DATA privateData,DWORD dwIntCfg,
DWORD dwTxFifSz,DWORD dwAfcCfg);
void Lan_EnableInterrupt(PPRIVATE_DATA privateData,DWORD dwIntEnMask);
void Lan_DisableInterrupt(PPRIVATE_DATA privateData,DWORD dwIntEnMask);
void Lan_EnableIRQ(PPRIVATE_DATA privateData);
void Lan_DisableIRQ(PPRIVATE_DATA privateData);
void Lan_SetIntDeas(PPRIVATE_DATA privateData,DWORD dwIntDeas);
void Lan_SetTDFL(PPRIVATE_DATA privateData,BYTE level);
void Lan_SetTSFL(PPRIVATE_DATA privateData,BYTE level);
void Lan_SetRDFL(PPRIVATE_DATA privateData,BYTE level);
void Lan_SetRSFL(PPRIVATE_DATA privateData,BYTE level);
void Lan_ClearIntDeasIntvl(PPRIVATE_DATA privateData);

void Lan_SignalSoftwareInterrupt(PPRIVATE_DATA privateData);
BOOLEAN Lan_HandleSoftwareInterrupt(PPRIVATE_DATA privateData,DWORD dwIntSts);

void Lan_ShowRegs(PPRIVATE_DATA privateData);


/* Finding the Access Method */
typedef struct t_acessMode{
	CNTROL_ACCESS PPheralAccess;
	char * periPheralName;
	void (*PPheralHWInit)(void);
	DWORD  (*LANGETREGW) (const DWORD, const DWORD);
	void (*LANSETREGW) (const DWORD, const DWORD, const DWORD);
}tPeriAccMthods, *ptPeriAccMthods;

tPeriAccMthods periAccessMethods[4]={{SMI_ACCESS,"SMI_ACCESS",NULL,SmiLanGetRegDW,SmiLanSetRegDW},
									  {I2C_ACCESS,"I2C_ACCESS",I2cHwInit ,I2cLanGetRegDW,I2cLanSetRegDW},
											{SPI_ACCESS,"SPI_ACCESS",SpiHwInit,SpiLanGetRegDW,SpiLanSetRegDW},
											{HBI_ACCESS,"HBI_ACCESS",NULL,HbiLanGetRegDW,HbiLanSetRegDW} };

#ifdef CHIP_IS_LAN9303
/* This option configures the  STRP Register in the FPGA, to have LEO_MAC_SMI, LEO_MAC_SPI,
 * LEO_MAC_I2C, LEO_PHY_SMI, LEO_MAC_SPI, LEO_PHY_SMI */
DWORD strap_cfg = 0x5;
MODULE_PARM(strap_cfg,"i");
MODULE_PARM_DESC(strap_cfg,"strap Configuraiton for LAN9303 LeoLite family of chips: Applicable only for FPGA Boards: Default is LEO PHY I2C");

DWORD pauseMacAddrEnbl =0x1;
MODULE_PARM(pauseMacAddrEnbl,"i");
MODULE_PARM_DESC(pauseMacAddrEnbl,"Enable the seperate Mac addresses for Pause frames, so the pause frame will have different MAC address on each Port ");

DWORD enblDAPriority =0x0;
MODULE_PARM(enblDAPriority, "i");
MODULE_PARM_DESC(enblDAPriority,"Enable the Destination Mac addresses Priority  ");

DWORD icmpv6CksumValidate=0x0;
MODULE_PARM(icmpv6CksumValidate, "i");
MODULE_PARM_DESC(icmpv6CksumValidate,"Enable the ICMPv6 check sum validation on MLD packets ");

DWORD ipv6HoplimitValidate=0x0;
MODULE_PARM(ipv6HoplimitValidate, "i");
MODULE_PARM_DESC(ipv6HoplimitValidate,"Enable the IPV6 Hop Limit validation for incoming MLD packets ");

#endif

DWORD fp_lan930x_access_if = 0x0;
MODULE_PARM(fp_lan930x_access_if,"i");
MODULE_PARM_DESC(fp_lan930x_access_if,"Access IF for FPGA LAN9303 family of chips: work only with Aldebaran boards");

DWORD access_if = 0x0;
MODULE_PARM(access_if,"i");
MODULE_PARM_DESC(access_if,"Access IF for LAN931x family of chips: work only for Aldebaran boards");

DWORD access_debug=0x0;
MODULE_PARM(access_debug,"i");
MODULE_PARM_DESC(access_debug,"access_debug reads the byte value registers multiple times and check the errors ");

DWORD lan_base=0x0UL;
MODULE_PARM(lan_base,"i");
MODULE_PARM_DESC(lan_base,"Base Address of LAN9118, (default: choosen by platform code)");

DWORD bus_width=0UL;  // Self detected
MODULE_PARM(bus_width,"i");
MODULE_PARM_DESC(bus_width,"Force bus width of 16 or 32 bits, default: autodetect");

DWORD link_mode=0x7FUL;
MODULE_PARM(link_mode,"i");
MODULE_PARM_DESC(link_mode,"Set Link speed and Duplex, 1=10HD,2=10FD,4=100HD,8=100FD,default=0xF");

DWORD irq=PLATFORM_HYDRA_IRQ;
MODULE_PARM(irq,"i");
MODULE_PARM_DESC(irq,"Force use of specific IRQ, (default: choosen by platform code)");

DWORD irq_1588=PLATFORM_1588_IRQ;
MODULE_PARM(irq_1588,"i");
MODULE_PARM_DESC(irq_1588,"Force use of specific IRQ, (default: choosen by platform code)");


DWORD int_deas=0xFFFFFFFFUL;
MODULE_PARM(int_deas,"i");
MODULE_PARM_DESC(int_deas,"Interrupt Deassertion Interval in 10uS units");

DWORD irq_pol=PLATFORM_IRQ_POL;
MODULE_PARM(irq_pol,"i");
MODULE_PARM_DESC(irq_pol,"IRQ Polarity bit, see definition of INT_CFG register");

DWORD irq_type=PLATFORM_IRQ_TYPE;
MODULE_PARM(irq_type,"i");
MODULE_PARM_DESC(irq_type,"IRQ Buffer Type bit, see definition of INT_CFG register");

DWORD rx_dma=TRANSFER_PIO;
MODULE_PARM(rx_dma,"i");
MODULE_PARM_DESC(rx_dma,"Receiver DMA Channel, 255=find available channel, 256=use PIO");

#ifdef NAPI_PROCESS
#define PROCESSING_MODE_IDLE    (0UL)
#define PROCESSING_MODE_TASKLET (1UL)
#define PROCESSING_MODE_NAPI    (2UL)

DWORD rx_mode=PROCESSING_MODE_TASKLET;
MODULE_PARM(rx_mode,"i");
MODULE_PARM_DESC(rx_mode,"0==use ISR, 1==use Rx Tasklet, 2==use NAPI");

DWORD napi_weight=4UL;
MODULE_PARM(napi_weight,"i");
MODULE_PARM_DESC(napi_weight,"The weight value to use if NAPI is used");

DWORD tasklets=1UL;
MODULE_PARM(tasklets,"i");
MODULE_PARM_DESC(tasklets,"non-zero== use tasklets for receiving packets, zero==receive packets in ISR");

#else
DWORD tasklets=1UL;
MODULE_PARM(tasklets,"i");
MODULE_PARM_DESC(tasklets,"non-zero== use tasklets for receiving packets, zero==receive packets in ISR");
#endif

DWORD tx_dma=TRANSFER_PIO;
MODULE_PARM(tx_dma,"i");
MODULE_PARM_DESC(tx_dma,"Transmitter DMA Channel, 255=find available channel, 256=use PIO");

DWORD dma_threshold=PLATFORM_DMA_THRESHOLD;
MODULE_PARM(dma_threshold,"i");
MODULE_PARM_DESC(dma_threshold,"Specifies the minimum packet size for DMA to be used.");

DWORD mac_addr_hi16=0xFFFFFFFFUL;
MODULE_PARM(mac_addr_hi16,"i");
MODULE_PARM_DESC(mac_addr_hi16,"Specifies the high 16 bits of the mac address");

DWORD mac_addr_lo32=0xFFFFFFFFUL;
MODULE_PARM(mac_addr_lo32,"i");
MODULE_PARM_DESC(mac_addr_lo32,"Specifies the low 32 bits of the mac address");

DWORD mibTimer = 0UL;
MODULE_PARM(mibTimer,"i");
MODULE_PARM_DESC(mibTimer,"Specifies The timer for Polling MIB counters");

DWORD linkSpeed=LINK_OFF;
#if 0
#ifdef HYDRA_MODE
DWORD  wait_state=6;
#else
// We asume it's leo
DWORD  wait_state=2;
#endif
#endif
DWORD  wait_state=2;
MODULE_PARM(wait_state,"i");
MODULE_PARM_DESC(wait_state,"Specifies twait state for LAN chip Select");


DWORD  cache_line = 16UL;
MODULE_PARM(cache_line,"i");
MODULE_PARM_DESC(cache_line,"Specifies Cache Line  Size, default = 16");

DWORD VirtualPhyIndex = 0UL;
MODULE_PARM(VirtualPhyIndex,"i");
MODULE_PARM_DESC(VirtualPhyIndex,"Specifies Virtual Phy Index, 0 or 1 ");

#ifdef USE_DEBUG
DWORD debug_mode=0x7UL;
#else
DWORD debug_mode=0x0UL;
#endif
MODULE_PARM(debug_mode,"i");
MODULE_PARM_DESC(debug_mode,"bit 0 enables trace points, bit 1 enables warning points, bit 2 enables gpios");

DWORD switch_tracing=0UL;
MODULE_PARM(switch_tracing,"i");
MODULE_PARM_DESC(switch_tracing,"setting to non zero will enable tracing for switch register accesses");

int mac_sniffing_on=0;
unsigned char mac_sniffing[6]={0,0,0,0,0,0};
MODULE_PARM(mac_sniffing,"6b");
MODULE_PARM_DESC(mac_sniffing,"default =OFF, All FFs means dump all, else sniff only matching addresses");

DWORD tx_fif_sz=0x00050000UL;
MODULE_PARM(tx_fif_sz,"i");
MODULE_PARM_DESC(tx_fif_sz,"Specifies TX_FIF_SZ of the HW_CFG register");

DWORD afc_cfg=0xFFFFFFFFUL;
MODULE_PARM(afc_cfg,"i");
MODULE_PARM_DESC(afc_cfg,"Specifies the setting for the AFC_CFG register");

DWORD phy_addr= 0x0UL;

MODULE_PARM(phy_addr,"i");
MODULE_PARM_DESC(phy_addr,"phy_addr, 0xFFFFFFFF=use interal phy, 0-31=use external phy with specified address, else autodetect external phy addr");

DWORD max_throughput=0xFFFFFFFFUL;
MODULE_PARM(max_throughput,"i");
MODULE_PARM_DESC(max_throughput,"See readme.txt");

DWORD max_packet_count=0xFFFFFFFFUL;
MODULE_PARM(max_packet_count,"i");
MODULE_PARM_DESC(max_packet_count,"See Readme.txt");

DWORD packet_cost=0xFFFFFFFFUL;
MODULE_PARM(packet_cost,"i");
MODULE_PARM_DESC(packet_cost,"See Readme.txt");

DWORD burst_period=0xFFFFFFFFUL;
MODULE_PARM(burst_period,"i");
MODULE_PARM_DESC(burst_period,"See Readme.txt");

DWORD max_work_load=0xFFFFFFFFUL;
MODULE_PARM(max_work_load,"i");
MODULE_PARM_DESC(max_work_load,"See Readme.txt");

MODULE_LICENSE("GPL");

int Smsc9312_init_module(void);
void Smsc9312_cleanup_module(void);
int Smsc9312_init(struct net_device *dev);
int Smsc9312_open(struct net_device *dev);
int Smsc9312_stop(struct net_device *dev);
int Smsc9312_hard_start_xmit(struct sk_buff *skb, struct net_device *dev);
struct net_device_stats * Smsc9312_get_stats(struct net_device *dev);
void Smsc9312_set_multicast_list(struct net_device *dev);
int Smsc9312_do_ioctl(struct net_device *dev, struct ifreq *ifr,int cmd);
irqreturn_t Smsc9312_ISR(int irq,void *dev_id,struct pt_regs *regs);

//#ifdef FULLY_MANAGED
#ifndef UNMANAGED

int SwitchPort_init(struct net_device *dev);
int SwitchPort_open(struct net_device *dev);
int SwitchPort_stop(struct net_device *dev);
int SwitchPort_hard_start_xmit(
struct sk_buff *skb, struct net_device * const dev);
struct net_device_stats * SwitchPort_get_stats(struct net_device *dev);
void SwitchPort_set_multicast_list(struct net_device *dev);
int SwitchPort_do_ioctl(struct net_device *dev, struct ifreq *ifr,int cmd);
int Switch_create_Interfaces(PPRIVATE_DATA pMaster, const char *devName,
                        int portNumber, struct net_device **newDev );
#endif
#ifdef NAPI_PROCESS
static int Smsc9132_rx_poll(struct net_device *dev,int * budget);
#endif
#ifdef USING_LINT
struct net_device SMSC931X;
#else //not USING_LINT
struct net_device SMSC931X = {init: Smsc9312_init,};

#endif //not USING_LINT
#ifdef FULLY_MANAGED
#ifndef CHIP_IS_LAN9303
#include PTP_DRV_SOURCE
#endif
#endif
static const unsigned char MAC_00_00_00_00_00_00[6]={0,0,0,0,0,0};
static const unsigned char MAC_FF_FF_FF_FF_FF_FF[6]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

int Smsc9312_init_module(void)
{
    int result=0;
    int device_present=0;
    struct net_device *dev = NULL;

    SMSC_TRACE("--> init_module()");
    SMSC_TRACE("Driver Version = %lX.%02lX",
        (DRIVER_VERSION>>8),(DRIVER_VERSION&0xFFUL));
    SMSC_TRACE("Compiled: %s, %s",__DATE__,__TIME__);
    SMSC_TRACE("Platform: %s",PLATFORM_NAME);
    SMSC_TRACE("Date Code: %s",date_code);
    SMSC_TRACE("Driver Parameters");
    if(lan_base==0UL) {
        SMSC_TRACE("  lan_base         = 0x%08lX, driver will decide",lan_base);
    } else {
        SMSC_TRACE("  lan_base         = 0x%08lX",lan_base);
    }
    if((bus_width==16UL)||(bus_width==32UL)) {
        SMSC_TRACE("  bus_width        = %ld",bus_width);
    } else {
        SMSC_TRACE("  bus_width        = %ld, driver will autodetect",bus_width);
    }
    if(link_mode>0x7FUL) {
        SMSC_WARNING("  link_mode     = %ld, Unknown",link_mode);
        link_mode=0x7FUL;
        SMSC_WARNING("    resetting link_mode to %ld, 100FD,100HD,10FD,10HD,ASYMP,SYMP,ANEG",link_mode);
    } else if(link_mode==0UL) {
        SMSC_TRACE("  link_mode        = %ld, LINK_OFF",link_mode);
    } else {
        SMSC_TRACE("  link_mode        = 0x%lX, %s,%s,%s,%s,%s,%s,%s",
            link_mode,
            (link_mode&LINK_SPEED_10HD)?"10HD":"",
            (link_mode&LINK_SPEED_10FD)?"10FD":"",
            (link_mode&LINK_SPEED_100HD)?"100HD":"",
            (link_mode&LINK_SPEED_100FD)?"100FD":"",
            (link_mode&LINK_ASYMMETRIC_PAUSE)?"ASYMP":"",
            (link_mode&LINK_SYMMETRIC_PAUSE)?"SYMP":"",
            (link_mode&LINK_AUTO_NEGOTIATE)?"ANEG":"");
    }
    SMSC_TRACE(    "  irq              = %ld",irq);
    if(int_deas!=0xFFFFFFFFUL) {
        if(int_deas>0xFFUL) {
            SMSC_WARNING("  int_deas     = %ld, too high",int_deas);
            int_deas=0xFFFFFFFFUL;
            SMSC_WARNING("    resetting int_deas to %ld",int_deas);
        }
    }
    if(int_deas==0xFFFFFFFFUL) {
        SMSC_TRACE(    "  int_deas         = 0x%08lX, use platform default",int_deas);
    } else {
        SMSC_TRACE(    "  int_deas         = %ld, %lduS",int_deas,10UL*int_deas);
    }
    if(irq_pol) {
        SMSC_TRACE("  irq_pol          = %ld, IRQ output is active high",irq_pol);
    } else {
        SMSC_TRACE("  irq_pol          = %ld, IRQ output is active low",irq_pol);
    }
    if(irq_type) {
        SMSC_TRACE("  irq_type         = %ld, IRQ output is Push-Pull driver",irq_type);
    } else {
        SMSC_TRACE("  irq_type         = %ld, IRQ output is Open-Drain buffer",irq_type);
    }

#ifdef HYDRA_FORCE_PIO
    rx_dma = TRANSFER_PIO
    tx_dma = TRANSFER_PIO;
#else
    if(rx_dma<TRANSFER_REQUEST_DMA) {
        if(Platform_IsValidDmaChannel(rx_dma)) {
            SMSC_TRACE(
                "  rx_dma           = %ld, DMA Channel %ld",rx_dma,rx_dma);
        } else {
            SMSC_WARNING("  rx_dma        = %ld, Invalid Dma Channel",rx_dma);
            rx_dma=TRANSFER_PIO;
            SMSC_WARNING("    resetting rx_dma to %ld, RX will use PIO",rx_dma);
        }
    } else if(rx_dma==TRANSFER_REQUEST_DMA) {
        SMSC_TRACE("  rx_dma           = %ld, RX will try to find available channel",rx_dma);
    } else {
        SMSC_TRACE("  rx_dma           = %ld, RX will use PIO",rx_dma);
    }
    if(tx_dma<TRANSFER_REQUEST_DMA) {
        if(Platform_IsValidDmaChannel(tx_dma)) {
            if(tx_dma!=rx_dma) {
                SMSC_TRACE(
                "  tx_dma           = %ld, DMA Channel %ld",tx_dma,tx_dma);
            } else {
                SMSC_WARNING("  tx_dma == rx_dma");
                tx_dma=TRANSFER_PIO;
                SMSC_WARNING("    resetting tx_dma to %ld, TX will use PIO",tx_dma);
            }
        } else {
            SMSC_WARNING("  tx_dma        = %ld, Invalid Dma Channel",tx_dma);
            tx_dma=TRANSFER_PIO;
            SMSC_WARNING("    resetting tx_dma to %ld, TX will use PIO",tx_dma);
        }
    } else if(tx_dma==TRANSFER_REQUEST_DMA) {
        SMSC_TRACE("  tx_dma           = %ld, TX will try to find available channel",tx_dma);
    } else {
        SMSC_TRACE("  tx_dma           = %ld, TX will use PIO",tx_dma);
    }
#endif

    SMSC_TRACE(    "  dma_threshold+    = %ld",dma_threshold);

    if(mac_addr_hi16==0xFFFFFFFFUL) {
        SMSC_TRACE("  mac_addr_hi16    = 0x%08lX, will attempt to read from LAN9118",mac_addr_hi16);
        SMSC_TRACE("  mac_addr_lo32    = 0x%08lX, will attempt to read from LAN9118",mac_addr_lo32);
    } else {
        if(mac_addr_hi16&0xFFFF0000UL) {
            //The high word is reserved
            SMSC_WARNING("  mac_addr_hi16 = 0x%08lX, reserved bits are high.",mac_addr_hi16);
            mac_addr_hi16&=0x0000FFFFUL;
            SMSC_WARNING("    reseting to mac_addr_hi16 = 0x%08lX",mac_addr_hi16);
        }
        if(mac_addr_lo32&0x00000001UL) {
            //bit 0 is the I/G bit
            SMSC_WARNING("  mac_addr_lo32 = 0x%08lX, I/G bit is set.",mac_addr_lo32);
            mac_addr_lo32&=0xFFFFFFFEUL;
            SMSC_WARNING("    reseting to mac_addr_lo32 = 0x%08lX",mac_addr_lo32);
        }
        SMSC_TRACE("  mac_addr_hi16    = 0x%08lX",mac_addr_hi16);
        SMSC_TRACE("  mac_addr_lo32    = 0x%08lX",mac_addr_lo32);
    }
    SMSC_TRACE(    "  debug_mode       = 0x%08lX",debug_mode);
    SMSC_TRACE(    "  switch_tracing   = %lu",switch_tracing);
    SMSC_TRACE(    "  mac_sniffing = %02X:%02X:%02X:%02X:%02X:%02X",
    		mac_sniffing[0],mac_sniffing[1],mac_sniffing[2],
    		mac_sniffing[3],mac_sniffing[4],mac_sniffing[5]);
    if(memcmp(mac_sniffing,MAC_00_00_00_00_00_00,6)!=0) {
    	mac_sniffing_on=1;
    }

    if(tx_fif_sz&(~HW_CFG_TX_FIF_SZ_)) {
        SMSC_WARNING("tx_fif_sz = 0x%08lX is invalid",tx_fif_sz);
        tx_fif_sz&=HW_CFG_TX_FIF_SZ_;
        SMSC_WARNING("  resetting tx_fif_sz to 0x%08lX",tx_fif_sz);
    }
    if(tx_fif_sz>0x000E0000UL) {
        SMSC_WARNING("tx_fif_sz = 0x%08lX is too high",tx_fif_sz);
        tx_fif_sz=0x000E0000UL;
        SMSC_WARNING(" resetting tx_fif_sz to 0x%08lX",tx_fif_sz);
    }
    if(tx_fif_sz<0x00020000UL) {
        SMSC_WARNING("tx_fif_sz = 0x%08lX is too low",tx_fif_sz);
        tx_fif_sz=0x00020000UL;
        SMSC_WARNING(" resetting tx_fif_sz to 0x%08lX",tx_fif_sz);
    }
    SMSC_TRACE(    "  tx_fif_sz        = 0x%08lX",tx_fif_sz);

    if(afc_cfg==0xFFFFFFFFUL) {
        SMSC_TRACE("  afc_cfg          = 0x%08lX, driver will decide",afc_cfg);
    } else {
        if(afc_cfg&0xFF000000UL) {
            SMSC_WARNING("afc_cfg = 0x%08lX is invalid",afc_cfg);
            afc_cfg&=0xFFFFFFFFUL;
            SMSC_WARNING(" resetting to afc_cfg = 0x%08lX, driver will decide",afc_cfg);
        } else {
            SMSC_TRACE(
                "  afc_cfg          = 0x%08lX",afc_cfg);
        }
    }
#ifdef NAPI_PROCESS
    if(rx_mode>2) {
            SMSC_WARNING(" rx_mode==%lu, invalid, reseting to 1",rx_mode);
            rx_mode=1;
    }
    if(rx_mode==2) {
            SMSC_TRACE("  rx_mode          = 2, use NAPI");
    } else if(rx_mode==1) {
            SMSC_TRACE("  rx_mode          = 1, use Rx Tasklet");
    } else {
            SMSC_TRACE("  rx_mode          = 0, use ISR");
    }
#endif

    if(phy_addr==0xFFFFFFFFUL) {
        SMSC_TRACE("  phy_addr         = 0xFFFFFFFF, Use internal phy");
    } else if(phy_addr<=31UL) {
        SMSC_TRACE("  phy_addr         = 0x%08lX, use this address for external phy",phy_addr);
    } else {
        SMSC_TRACE("  phy_addr         = 0x%08lX, auto detect external phy",phy_addr);
    }
    if(max_throughput) {
        SMSC_TRACE("  max_throughput   = 0x%08lX, Use platform default",max_throughput);
    } else {
        SMSC_TRACE("  max_throughput   = 0x%08lX",max_throughput);
    }
    if(max_packet_count) {
        SMSC_TRACE("  max_packet_count = 0x%08lX, Use platform default",max_packet_count);
    } else {
        SMSC_TRACE("  max_packet_count = 0x%08lX",max_packet_count);
    }
    if(packet_cost) {
        SMSC_TRACE("  packet_cost      = 0x%08lX, Use platform default",packet_cost);
    } else {
        SMSC_TRACE("  packet_cost      = 0x%08lX",packet_cost);
    }
    if(burst_period) {
        SMSC_TRACE("  burst_period     = 0x%08lX, Use platform default",burst_period);
    } else {
        SMSC_TRACE("  burst_period     = 0x%08lX",burst_period);
    }
    if(max_work_load) {
        SMSC_TRACE("  max_work_load    = 0x%08lX, Use platform default",max_work_load);
    } else {
        SMSC_TRACE("  max_work_load    = 0x%08lX",max_work_load);
    }
#ifdef DUMP_PLATFORM_REGISTER
    platformDumpRegisters();
#endif

    dev = &SMSC931X;

    strcpy(dev->name,"eth%d");
    SMSC_TRACE("Calling register_netdev for device:%s", dev->name );
    result=register_netdev(dev );

    if(result) {
        SMSC_WARNING("error %i registering device %s",result, SMSC931X.name );
    } else {
        device_present=1;
        SMSC_TRACE("  Interface Name = \"%s\"",SMSC931X.name);
    }


#ifndef UNMANAGED
    {
        PPRIVATE_DATA pMaster = dev->priv;
        struct net_device *newDev = NULL;
        int portNumber= 0;
        const char *switchPortName[PHY_NUMBER] ={
            "switch0",
            "switch1",
            "switch2"
        };
        for (portNumber = 1; portNumber < PHY_NUMBER ; portNumber++)
        {
            udelay(1000);
            result = Switch_create_Interfaces(pMaster, switchPortName[portNumber],
            portNumber, &newDev );
            if(result != 0) {
                    SMSC_WARNING("error %i registering device %s",result, switchPortName[portNumber] );
            } else {
                if (newDev != NULL) {
                pMaster->switchDev[portNumber] = newDev;
                SwitchPort_open(newDev);
                SMSC_TRACE("  Interface Name = \"%s\"",newDev->name);
                }
            }
        }
    }
#endif
    result=result;//make lint happy
    SMSC_TRACE("<-- init_module()");
    return device_present ? 0 : -ENODEV;
}
/** end of  int Smsc9312_init_module(void) **/



void Smsc9312_cleanup_module(void)
{


    PPRIVATE_DATA privateData=(PPRIVATE_DATA)SMSC931X.priv;

    PPLATFORM_DATA platformData=(PPLATFORM_DATA)&(privateData->PlatformData);

    SMSC_TRACE("--> cleanup_module()");

#ifndef UNMANAGED
    {
        int portId =0;
        struct net_device *switchDev;
        PSWITCH_PPRIVATE_DATA switchPriv;

        for (portId=1; portId < PHY_NUMBER ;   portId++) {
            if( privateData->switchDev[portId])  {
                udelay(50);
                switchDev = privateData->switchDev[portId];
                switchPriv=  switchDev->priv;
                SMSC_TRACE("--> Unregister Device %s ", switchDev->name);
                unregister_netdev( switchDev);
                kfree(switchDev);
                privateData->switchDev[portId] = NULL;
            } else {
            	SMSC_WARNING("privateData->switchDev[%d]==NULL",portId);
            }
        }

#ifdef FULLY_MANAGED
        if(SMSC931X.priv!=NULL) {
#ifndef CHIP_IS_LAN9303
            if (privateData->ptp_thread_pid)  {
                if (privateData->nls)
                    sock_release (privateData->nls->sk_socket);
                kill_proc(privateData->ptp_thread_pid, SIGTERM, 1);
                udelay(50);
                privateData->ptp_thread_pid  = 0;
                privateData->ptp_thread = NULL;
                privateData->nls = NULL;
            }
#endif
        }
#endif
    }
#endif

    udelay(50);
    unregister_netdev(&SMSC931X);
    udelay(100);


        SMSC_TRACE("--> cleanup_module() kfree(SMSC931X.priv)");

        Platform_CleanUp(platformData);
        kfree(SMSC931X.priv);
        SMSC931X.priv=NULL;
        thisModulePrivate = NULL;

    SMSC_TRACE("<-- cleanup_module()");
}




int smc931x_eth_header( struct sk_buff *skb,
                        struct net_device *dev,
                        unsigned short type,
                        void *daddr,
                        void *saddr,
                        unsigned len)
{


    struct smc931x_ethhdr *eth;

    /*
    *  Set the protocol type. For a packet of type ETH_P_802_3 we put the length
    *  in here instead. It is up to the 802.2 layer to carry protocol information.
    */

    eth = (struct smc931x_ethhdr *)skb_push(skb,ETH_HLEN+4);


    if(type!=ETH_P_802_3)
        eth->h_proto = htons(type);
    else
        eth->h_proto = htons(len);

    /*
    *  Set the source hardware address.
    */

    if(!saddr)
        saddr = dev->dev_addr;
    memcpy(eth->h_source,saddr,dev->addr_len);

    if(daddr)
    {
        memcpy(eth->h_dest,daddr,dev->addr_len);
        return ETH_HLEN;
    }

    /*
    *  Anyway, the loopback-device should never use this function...
    */

    if (dev->flags & (IFF_LOOPBACK|IFF_NOARP))
    {
        memset(eth->h_dest, 0, dev->addr_len);
        return ETH_HLEN;
    }

    return -ETH_HLEN;
}

/**
**********************************************************
*        SWITCH PORT
*
*
*********************************************************
**/
#ifndef UNMANAGED

int Switch_create_Interfaces(PPRIVATE_DATA pMaster, const char *devName,
    int portNumber,struct net_device **newDev )

{
    PSWITCH_PPRIVATE_DATA priv;
    int result=-ENODEV;
    struct net_device *dev = NULL;

    dev =  alloc_etherdev(sizeof (SWITCH_PRIVATE_DATA));
    if (dev)
    {
        *newDev = dev;
        dev->init = SwitchPort_init;
        priv = (PSWITCH_PPRIVATE_DATA) (dev->priv);

        priv->isSwitchPort = TRUE;
        priv->PortNumber = portNumber;
        priv->pMaster = pMaster;

        strcpy(dev->name,devName);

        SMSC_TRACE("Calling register_netdev for device:%s", dev->name );
        result=register_netdev(dev );

        if(result) {
            SMSC_WARNING("error %i registering device %s",result, dev->name );
            kfree(dev);
        } else {
            SMSC_TRACE("  Interface Name = \"%s\"",dev->name);
        }
    }

    return result;
}


int SwitchPort_init(struct net_device *dev)
{
    int result=-ENODEV;
    SMSC_TRACE("-->(dev=0x%08lX)",(DWORD)dev);



    dev->init               =   SwitchPort_init;
    dev->open               =   SwitchPort_open;
    dev->stop               =   SwitchPort_stop;
    dev->hard_start_xmit    =   SwitchPort_hard_start_xmit;
    dev->get_stats          =   SwitchPort_get_stats;
    dev->do_ioctl           =   SwitchPort_do_ioctl;
    dev->set_multicast_list =   SwitchPort_set_multicast_list;
    // dev->flags              |=    (IFF_UP |IFF_NOARP|IFF_RUNNING) ;
    result=0;

    SMSC_TRACE("<--, result=%d",result);
    return result;
}

void  setDefaultAddress(struct net_device *dev)
{

    dev->dev_addr[0]=LOBYTE(LOWORD(DW_LOW_MAC));
    dev->dev_addr[1]=HIBYTE(LOWORD(DW_LOW_MAC));
    dev->dev_addr[2]=LOBYTE(HIWORD(DW_LOW_MAC));
    dev->dev_addr[3]=HIBYTE(HIWORD(DW_LOW_MAC));
    dev->dev_addr[4]=LOBYTE(LOWORD(DW_HIGH_MAC16));
    dev->dev_addr[5]=HIBYTE(LOWORD(DW_HIGH_MAC16));
}

int SwitchPort_open(struct net_device *dev)
{
    int result=-ENODEV;
    PSWITCH_PPRIVATE_DATA privateData=NULL;
    //    PPRIVATE_DATA msaterPrivate=NULL;

    SMSC_TRACE("-->Smsc9312_open(dev=0x%08lX), device Name=%s ",(DWORD)dev, dev->name);
    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_open(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    privateData=(PSWITCH_PPRIVATE_DATA)(dev->priv);
    if(privateData==NULL) {
        SMSC_WARNING("(privateData==NULL)");
        result=-EFAULT;
        goto DONE;
    }

    if (privateData->pMaster) {
            setDefaultAddress(dev);
    }

#ifdef FULLY_MANAGED
    netif_start_queue(dev);     // fixme mobba
#endif

    return ( 0 );
DONE:

    return result;
}


int SwitchPort_stop(struct net_device *dev)
{
    int result=0;
    PSWITCH_PPRIVATE_DATA privateData=NULL;
    SMSC_TRACE("-->(dev=0x%08lX)",(DWORD)dev);
    if(dev==NULL) {
        SMSC_WARNING("(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    privateData=(PSWITCH_PPRIVATE_DATA)(dev->priv);
    if(privateData==NULL) {
        SMSC_WARNING("(privateData==NULL)");
        result=-EFAULT;
        goto DONE;
    }


    netif_stop_queue(dev);

    privateData->rstpEnabled = FALSE;
    privateData->rstpState   = STP_FORWARDING ;
    memset(&(privateData->stats),0, sizeof(struct net_device_stats));


DONE:
    SMSC_TRACE("<-- result=%d",result);
    return result;
}



int SwitchPort_hard_start_xmit(
        struct sk_buff *skb,
        struct net_device * const dev)
{
    int result=0;

#ifdef FULLY_MANAGED
    unsigned char *pDstMAc ;
    PSWITCH_PPRIVATE_DATA privateData=NULL;
    unsigned long protocol = 0;

    if(skb==NULL) {
        SMSC_WARNING("(skb==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    if(dev==NULL) {
        SMSC_WARNING("(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    if(dev->priv==NULL) {
        SMSC_WARNING("(dev->priv==NULL)");
        result=-EFAULT;
        goto DONE;
    }

    privateData=(PSWITCH_PPRIVATE_DATA)(dev->priv);

    ///
    /// Check if it a Command or mesage
    ///
        protocol =  atomic_read(&(skb->users));

        if ( (skb->protocol == SKB_TYPE_IS_STP) && (protocol == SKB_TYPE_IS_STP))
        {

            DeviceProtocol *pProto;
            pProto = (DeviceProtocol *)(skb->data);


            if (pProto->signature == SMSC931X_DRIVER_SIGNATURE ) {
                switch (pProto->command)  {
                    case COMMAND_ENABLE_STP:
                        SMSC_TRACE("EnableSpanningTree");
                        EnableSpanningTree(privateData);
                        break;
                    case COMMAND_DISABLE_STP:
                        SMSC_TRACE("DisableSpanningTree");
                        DisableSpanningTree(privateData);
                        break;
                    case COMMAND_ENBL_DISBL_SHRTAGE_TIMR:
                    	SMSC_TRACE("Setting the SHORT AGE Timer");
                    	Switch_Enbl_disbl_shortAgeTimer(privateData->pMaster,(DWORD)pProto->Data[0]);
                    	break;
                    case COMMAND_PORT_STATE_BLOCKING:
                    case COMMAND_PORT_STATE_LISTENING:
                    case COMMAND_PORT_STATE_LEARNING:
                    case COMMAND_PORT_STATE_FORWARDING:
                    case COMMAND_PORT_STATE_DISABLE:
                    case COMMAND_PORT_STATE_ENABLE:
                        STP_Set_Port_State(privateData,(PORT_STATE)
                            ((pProto->command - COMMAND_PORT_STATE_BLOCKING)) + STP_BLOCKING);
                        break;
                    default:
                        SMSC_WARNING("Unkown Command =%lu", pProto->command);
                        break;
                    }
            }
            dev_kfree_skb(skb);

            return 0;
        }




    //  SET_GPIO(GP_TX);
    {
        struct smc931x_ethhdr *eth;
        int proto=0;
        int VLANPri;
        VLANPri=isVLANPacket(skb->data,skb->len, &proto);
        memmove(skb->data -4 , skb->data , 12); // Move Dst/SRC with 4
        skb->data -=4;
        skb->len+= 4;
        eth = (struct smc931x_ethhdr *) (skb->data);
        eth->vlan[0] = htons(0x8100);
#ifndef CHIP_IS_LAN9303
        eth->vlan[1] = (VLANPri|(htons(privateData->PortNumber)));
#else
        /* By default enable VLAN TAG priority and VLAN LOOKUP Rule to be set */
        eth->vlan[1]= htons(EGRS_CPU_TAG_VLAN_RULE_SET | EGRS_CPU_TAG_PRI_SET| EGRS_CPU_TAG_STP_ORIDE_SET | privateData->PortNumber);
#endif
    }

    pDstMAc = (unsigned char *) (skb->data);

    if (privateData->pMaster)
    {
    if (!(privateData->rstpEnabled)) {
//           SMSC_TRACE("Sending SKB --- Switch_SendSkb\n");
            Switch_SendSkb(privateData->pMaster,skb);
        }
        else
        {

            switch(privateData->rstpState) {
#ifndef CHIP_IS_LAN9303
                case STP_BLOCKING:
#endif
                case STP_DISABLED:
                    dev_kfree_skb(skb);
                    break;
#ifdef CHIP_IS_LAN9303
                case STP_BLOCKING:
#endif
                case STP_LEARNING:
                case STP_LISTENING:
                   if (memcmp(pDstMAc, RSTP_MAC_ADDRESS,6) == 0)  {
                        Switch_SendSkb(privateData->pMaster,skb);
                        }
                    else {
                            dev_kfree_skb(skb);
                            SMSC_TRACE("!!!!!!! NOT STP FRAME  Discarding it ---> port %s\n", skb->dev->name );
                        //SMSC_TRACE("DMAC = %02x %02x %02x %02x %02x %02x\n",
                        //*pDstMAc++,pDstMAc++,*pDstMAc++,*pDstMAc++,*pDstMAc++,*pDstMAc++ );
                        }
                        break;
                case STP_FORWARDING:
                	//printk("port is in Forwarding State \n");
                    Switch_SendSkb(privateData->pMaster,skb);
                    break;
                default:
                    Switch_SendSkb(privateData->pMaster,skb);
                    break;
            }; //switch


        }
    }
    else
    {
        SMSC_TRACE("Deleting SKB, Invalid master");
        dev_kfree_skb(skb);
    }
    DONE:
#else
    dev_kfree_skb(skb);
#endif
    //  CLEAR_GPIO(GP_TX);

    //  SMSC_TRACE("<--SwitchPort_hard_start_xmit, result=%d",result);
    return result;
}


struct net_device_stats * SwitchPort_get_stats(struct net_device *dev)
{

    PSWITCH_PPRIVATE_DATA privateData = (PSWITCH_PPRIVATE_DATA) (dev->priv);
    return &(privateData->stats);
}

void SwitchPort_set_multicast_list(struct net_device *dev)
{
    SMSC_TRACE("!!!!!!!!");
    return;
}

int SwitchPort_do_ioctl(struct net_device *dev, struct ifreq *ifr,int cmd)
{
    PSWITCH_PPRIVATE_DATA SprivateData=NULL;
    PSMSC931X_IOCTL_DATA ioctlData=NULL;
    BOOLEAN success=FALSE;
    PPRIVATE_DATA pMasterrivateData;

    SMSC_ASSERT(dev!=NULL);
    SMSC_ASSERT(dev->priv!=NULL);
    SprivateData=((PSWITCH_PPRIVATE_DATA) dev->priv);
    pMasterrivateData = SprivateData->pMaster;
    SMSC_ASSERT(pMasterrivateData!=NULL);
    if(ifr==NULL) {
        SMSC_WARNING("(ifr==NULL)");
        goto DONE;
    }


    if(ifr->ifr_data==NULL) {
//		SMSC_WARNING("Smsc9312_do_ioctl(ifr->ifr_data==NULL)");
        goto DONE;
    }

    if(cmd!=((int)SMSC931X_IOCTL))
    {
        SMSC_ERROR(" Command Unknown %d ( expecting %d)\n", cmd, (int)SMSC931X_IOCTL);
        goto DONE;
    }

    ioctlData=(PSMSC931X_IOCTL_DATA)(ifr->ifr_data);



    if(ioctlData->dwSignature!=SMSC931X_APP_SIGNATURE) {
        goto DONE;
    }



    switch(ioctlData->dwCommand) {
        case COMMAND_GET_SIGNATURE:
            success=TRUE;
            break;
#ifdef FULLY_MANAGED
        case COMMAND_ENABLE_STP:
            SMSC_TRACE("EnableSpanningTree");
            EnableSpanningTree(SprivateData);
            break;
        case COMMAND_DISABLE_STP:
            SMSC_TRACE("DisableSpanningTree");
            DisableSpanningTree(SprivateData);
            break;
        case COMMAND_ENBL_DISBL_SHRTAGE_TIMR:
        	SMSC_TRACE("Setting the SHORT AGE Timer");
        	Switch_Enbl_disbl_shortAgeTimer(SprivateData->pMaster,ioctlData->Data[0]);
        	break;
        case COMMAND_PORT_STATE_BLOCKING:
        case COMMAND_PORT_STATE_LISTENING:
        case COMMAND_PORT_STATE_LEARNING:
        case COMMAND_PORT_STATE_FORWARDING:
        case COMMAND_PORT_STATE_DISABLE:
        case COMMAND_PORT_STATE_ENABLE:
            STP_Set_Port_State(SprivateData,(PORT_STATE)
                ((ioctlData->dwCommand - COMMAND_PORT_STATE_BLOCKING)) + STP_BLOCKING);
            break;
#endif
        /** ********************************
        *    MIB stuff here
        *******************************/
        case COMMAND_GET_DOT3_STAT_TBL:
            if( (pMasterrivateData->LanInitialized) &&
                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                DWORD index = 0;
                Dot3StatsTable_data *pDot3StatsTable;
                DWORD *tblMap;
                pDot3StatsTable = &(pMasterrivateData->dot3StatsTable_data_array[ ioctlData->Data[0]]);
                tblMap= (DWORD *) pDot3StatsTable;
                for (index = 0; index < sizeof(Dot3StatsTable_data); index++, tblMap++)
                    ioctlData->Data[index] = *tblMap;

                success=TRUE;
                            } else {
                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                SMSC_WARNING( "  Use ifconfig to bring interface UP");
                            }
                            break;
        case COMMAND_GET_DOT3_CTRL_TBL:
        	if( (pMasterrivateData->LanInitialized) &&
        	                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
        	                DWORD index = 0;
        	                Dot3ControlTable_data *pDot3CntrlTable;
        	                DWORD *tblMap;
         	                pDot3CntrlTable = &(pMasterrivateData->dot3ControlTable_data_array[ ioctlData->Data[0]]);
        	                tblMap= (DWORD *) pDot3CntrlTable;
        	                for (index = 0; index < sizeof(Dot3ControlTable_data); index++, tblMap++)
        	                    ioctlData->Data[index] = *tblMap;

        	                success=TRUE;
        	                            } else {
        	                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
        	                                SMSC_WARNING( "  Use ifconfig to bring interface UP");
        	                            }
        	break;
        case COMMAND_GET_DOT3_PAUSE_TBL:
              	if( (pMasterrivateData->LanInitialized) &&
              	                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
              	                DWORD index = 0;
              	                Dot3PauseTable_data *pDot3PauseTable;
              	                DWORD *tblMap;
              	                pDot3PauseTable = &(pMasterrivateData->dot3PauseTable_data_array[ ioctlData->Data[0]]);
              	                tblMap= (DWORD *) pDot3PauseTable;
              	                for (index = 0; index < sizeof(Dot3PauseTable_data); index++, tblMap++)
              	                    ioctlData->Data[index] = *tblMap;

              	                success=TRUE;
              	                            } else {
              	                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
              	                                SMSC_WARNING( "  Use ifconfig to bring interface UP");
              	                            }
              	break;
        case COMMAND_GET_RMON_STAT_TBL:
            if( (pMasterrivateData->LanInitialized) &&
                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                DWORD index = 0;
                EtherStatsTable_data *pRmonStatsTable;
                DWORD *tblMap;
                mibUpdateMibPortCounters( pMasterrivateData );


                // Get rmonetherStats data pointer  for the index.

                pRmonStatsTable = &(pMasterrivateData->etherStatsTable_data_array[ ioctlData->Data[0]]);
                tblMap= (DWORD *) pRmonStatsTable;
                for (index = 0; index < sizeof(EtherStatsTable_data); index++, tblMap++)
                    ioctlData->Data[index] = *tblMap;

                success=TRUE;
                            } else {
                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                SMSC_WARNING("  Use ifconfig to bring interface UP");
                            }
                            break;
        case COMMAND_GET_IEE8023_STAT_TBL:
                   if( (pMasterrivateData->LanInitialized) &&
                                   ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                       DWORD index = 0;
                       IEEE802_3StatsTable_data *pMACStatsTable;
                       DWORD *tblMap;
                       mibUpdateMibPortCounters( pMasterrivateData );

                       // Get rmonetherStats data pointer  for the index.

                       pMACStatsTable = &(pMasterrivateData->MACStatsTable_data_array[ ioctlData->Data[0]]);
                       tblMap= (DWORD *) pMACStatsTable;
                       for (index = 0; index < sizeof(IEEE802_3StatsTable_data); index++, tblMap++)
                           ioctlData->Data[index] = *tblMap;

                       success=TRUE;
                                   } else {
                                       SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                       SMSC_WARNING("  Use ifconfig to bring interface UP");
                                   }
                                   break;

        case COMMAND_GET_BRIDGE_STAT_TBL:
                   if( (pMasterrivateData->LanInitialized) &&
                                   ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                       DWORD index = 0;
                       Bridge_Stats_data *pBridgeStatsTable;
                       DWORD *tblMap;
                       mibUpdateMibPortCounters( pMasterrivateData );

                       // Get rmonetherStats data pointer  for the index.

                       pBridgeStatsTable = &(pMasterrivateData->BridgeStatsTable_data_array[ ioctlData->Data[0]]);
                       tblMap= (DWORD *) pBridgeStatsTable;
                       for (index = 0; index < sizeof(Bridge_Stats_data); index++, tblMap++)
                           ioctlData->Data[index] = *tblMap;

                       success=TRUE;
                                   } else {
                                       SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                       SMSC_WARNING("  Use ifconfig to bring interface UP");
                                   }
                                   break;

        case COMMAND_GET_IF_TBL:
            if( (pMasterrivateData->LanInitialized) &&
                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                DWORD index = 0;
                IfTable_data *pIfTable_data;
                DWORD *tblMap;

                mibUpdateMibPortCounters( pMasterrivateData );
               // Get rmonetherStats data pointer  for the index.
                pIfTable_data = &(pMasterrivateData->IfTable_data_array[ioctlData->Data[0]]);
                tblMap= (DWORD *) pIfTable_data;
                for (index = 0; index < sizeof(IfTable_data); index++, tblMap++)
                    ioctlData->Data[index] = *tblMap;

                success=TRUE;
                            } else {
                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                SMSC_WARNING("  Use ifconfig to bring interface UP");
                            }
                            break;

        case COMMAND_GET_IF_XTBL:
            if( (pMasterrivateData->LanInitialized) &&
                            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                DWORD index = 0;
                IfXTable_Data *pIfXTable_Data;
                DWORD *tblMap;

                mibUpdateMibPortCounters( pMasterrivateData );
                // Get rmonetherStats data pointer  for the index.
                pIfXTable_Data = &(pMasterrivateData->IfXTable_Data_array[ioctlData->Data[0]]);
                tblMap= (DWORD *) pIfXTable_Data;
                for (index = 0; index < sizeof(IfXTable_Data); index++, tblMap++)
                    ioctlData->Data[index] = *tblMap;

                success=TRUE;
                            } else {
                                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                SMSC_WARNING("  Use ifconfig to bring interface UP");
                            }
                            break;
        default:
            SMSC_TRACE("Unknown Command %lu ( see ioctl.h)\n", ioctlData->dwCommand);
            break;//make lint happy
    }

DONE:
    if((success)&&(ioctlData!=NULL)) {
        ioctlData->dwSignature=SMSC931X_DRIVER_SIGNATURE;
        return 0;
    }
//	SMSC_TRACE("<--Smsc9312_do_ioctl");
    return -1;
}
#endif



/**
*   For CPU port
*
**/
int Smsc9312_init(struct net_device *dev)
{
    DWORD dwLanBase=0UL;
    DWORD dwIdRev=0UL;
    DWORD dwFpgaRev=0UL;
    PPRIVATE_DATA privateData=NULL;
    PPLATFORM_DATA platformData=NULL;
    BOOLEAN platformInitialized=FALSE;
    int result=-ENODEV;
    DWORD dwByteTestVal=0UL;
    int errCounter=0,i=0;
    SMSC_TRACE("-->Smsc9312_init(dev=0x%08lX)",(DWORD)dev);

    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_init(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }

    if(dev->priv!=NULL) {
        SMSC_WARNING("dev->priv!=NULL, Value=0x%lx \n", (DWORD) (dev->priv) );
        return 0;
    }

    dev->priv=kmalloc(sizeof(PRIVATE_DATA),GFP_KERNEL);

    if(dev->priv==NULL) {
        SMSC_WARNING("Unable to allocate PRIVATE_DATA");
        result=-ENOMEM;
        goto DONE;
    }
    thisModulePrivate = dev->priv;

    memset(dev->priv,0,sizeof(PRIVATE_DATA));
    privateData=(PPRIVATE_DATA)(dev->priv);
    platformData=&(privateData->PlatformData);
   	if(access_if == 4)
    	wait_state =6;

    dwLanBase=Platform_Initialize(
        platformData,
        lan_base,bus_width);

    if(dwLanBase==0UL) {
            SMSC_WARNING("dwLanBase==0x00000000");
            result=-ENODEV;
            goto DONE;
    }

    if(check_mem_region(dwLanBase,LAN_REGISTER_EXTENT)!=0) {
        SMSC_WARNING("  Memory Region specified (0x%08lX to 0x%08lX) is not available.",
            dwLanBase,dwLanBase+LAN_REGISTER_EXTENT-1UL);
        result=-ENOMEM;
        goto DONE;
    }

    SMSC_TRACE("dwLanBase=0x%08lX",dwLanBase);
    privateData->dwLanBase=dwLanBase;
#ifdef CHIP_IS_LAN9303
   if(fp_lan930x_access_if) {
    	SMSC_TRACE("Configuring the STRAP Registers\n");
    	ACCESS_STRAP_REGISTER =TRUE;
    	InitLeoJmprs(strap_cfg);
    	ACCESS_STRAP_REGISTER =FALSE;
    	SMSC_TRACE("Done with the Configuration of the STRAP Registers \n");
    	mdelay(100);
    	access_if = ((strap_cfg%3))?(strap_cfg%3):3;
   }
#endif
    if(!access_if)
    {
    	/* This is an auto detection mode */
	  //  privateData->dwLanBase=dwLanBase;
	    privateData->access_mode = UNKNOWN_ACCESS;
	    checktheAccessMode(privateData);
	    /* This is a patch-up work for time being for the FPGA Board, to avoid intermittent failiures for SMI*/ //fix me
	    if(privateData->access_mode == UNKNOWN_ACCESS)
	    {
	    			//printk(" The PeripHeral Access Mode =%s \n\n\n",periAccessMethods[0].periPheralName);
	    			privateData->PlatformGetRegDW = periAccessMethods[0].LANGETREGW;
	    			privateData->PlatformSetRegDW = periAccessMethods[0].LANSETREGW;
	    			privateData->access_mode = periAccessMethods[0].PPheralAccess;
	    			SMSC_TRACE("Peripheral Access Method is %s\n",periAccessMethods[0].periPheralName )
	    }
    }
    else
    {
		if(periAccessMethods[(access_if-1)].PPheralHWInit != NULL)
			periAccessMethods[(access_if-1)].PPheralHWInit();

    	/* access_if(1)is SMI, access_if(2) is I2C, access_if(3) is SPI, access_if(4) is HYDRA */
		privateData->PlatformGetRegDW = periAccessMethods[(access_if-1)].LANGETREGW;
		privateData->PlatformSetRegDW = periAccessMethods[(access_if-1)].LANSETREGW;
		privateData->access_mode = periAccessMethods[(access_if-1)].PPheralAccess;

		mdelay(1000);
		if(access_debug)
		{
			for(i=0; i<10; i++)
			{
				dwByteTestVal= LanGetRegDW(BYTE_TEST);
				if(dwByteTestVal !=(DWORD)0x87654321)
				{
					SMSC_WARNING("dwByteTestVal is not equal to 0x87654321, Trying multiple times \n");
					errCounter++;
				}
			}

			if (errCounter==9)
			{
				SMSC_WARNING("ByteTestVal is not readable on peripherial Access =%s",periAccessMethods[(access_if-1)].periPheralName);
			}
			else if(errCounter>0)
			{
				SMSC_WARNING("Intermittent Noise Error on the Access %s Bus, errCounter is =%d",periAccessMethods[(access_if-1)].periPheralName, errCounter);
			}
			else
			{
				SMSC_TRACE("PeriPherial access Method is initilaized to %s\n", periAccessMethods[(access_if-1)].periPheralName)
			}
		}
		else
		{
			SMSC_TRACE("PeriPherial access Method is initilaized to %s\n", periAccessMethods[(access_if-1)].periPheralName)
		}
    }
    //printk("The PeripHeral Access Mode =%d \n\n\n",privateData->access_mode);

    platformInitialized=TRUE;
    dwIdRev=LanGetDirectRegDW(ID_REV);
    SMSC_TRACE("dwIdRev=0x%08lX",dwIdRev);
    if(HIWORD(dwIdRev)==LOWORD(dwIdRev)) {
        //this may mean the chip is set for 32 bit
        //  while the bus is reading as 16 bit
    UNKNOWN_CHIP:
        SMSC_WARNING("  LAN9118 Family NOT Identified, dwIdRev==0x%08lX",dwIdRev);
        result=-ENODEV;
        goto DONE;
    }
    switch(dwIdRev&0xFFFF0000UL) {
    case 0x01180000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            SMSC_TRACE("  LAN9118 Beacon identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=0;
            break;
        case 1UL:
            SMSC_TRACE("  LAN9118 Concord A0 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=1;
            break;
        case 2UL:
            SMSC_TRACE("  LAN9118 Concord A1 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        default:
            SMSC_TRACE("  LAN9118 Concord A1 identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        };break;
    case 0x01170000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            SMSC_TRACE("  LAN9117 Beacon identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=0;
            break;
        case 1UL:
            SMSC_TRACE("  LAN9117 Concord A0 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=1;
            break;
        case 2UL:
            SMSC_TRACE("  LAN9117 Concord A1 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        default:
            SMSC_TRACE("  LAN9117 Concord A1 identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        };break;
    case 0x01160000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            goto UNKNOWN_CHIP;
        case 1UL:
            SMSC_TRACE("  LAN9116 Concord A0 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=1;
            break;
        case 2UL:
            SMSC_TRACE("  LAN9116 Concord A1 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        default:
            SMSC_TRACE("  LAN9116 Concord A1 identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        };break;
    case 0x01150000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            goto UNKNOWN_CHIP;
        case 1UL:
            SMSC_TRACE("  LAN9115 Concord A0 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=1;
            break;
        case 2UL:
            SMSC_TRACE("  LAN9115 Concord A1 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        default:
            SMSC_TRACE("  LAN9115 Concord A1 identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        };break;
    case 0x01120000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            goto UNKNOWN_CHIP;
        case 1UL:
            SMSC_TRACE("  LAN9112 Concord A0 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=1;
            break;
        case 2UL:
            SMSC_TRACE("  LAN9112 Concord A1 identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        default:
            SMSC_TRACE("  LAN9112 Concord A1 identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=2;
            break;
        };break;
    case 0x118A0000UL:
    case 0x93120000UL:
    case 0x93130000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            SMSC_TRACE("  LAN9218 Boylston identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        default:
            SMSC_TRACE("  LAN9218 Boylston identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        };break;
    case 0x117A0000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            SMSC_TRACE("  LAN9217 Boylston identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        default:
            SMSC_TRACE("  LAN9217 Boylston identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        };break;
    case 0x116A0000UL:
        switch(dwIdRev&0x0000FFFFUL) {
        case 0UL:
            SMSC_TRACE("  LAN9216 Boylston identified, dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        default:
            SMSC_TRACE("  LAN9216 Boylston identified (NEW), dwIdRev==0x%08lX",dwIdRev);
            privateData->dwGeneration=3;
            break;
        };break;
    case 0x115A0000UL:
        switch(dwIdRev&0x0000FFFFUL) {
            case 0UL:
                SMSC_TRACE("  LAN9215 Boylston identified, dwIdRev==0x%08lX",dwIdRev);
                privateData->dwGeneration=3;
                break;
            default:
                SMSC_TRACE("  LAN9215 Boylston identified (NEW), dwIdRev==0x%08lX",dwIdRev);
                privateData->dwGeneration=3;
                break;
        };
        break;
    }
#if 0
    dwFpgaRev=LanGetRegDW(FPGA_REV);
    SMSC_TRACE("  FPGA_REV == 0x%08lX",dwFpgaRev);
#endif
    ether_setup(dev);

    dev->open=		Smsc9312_open;
    dev->stop=		Smsc9312_stop;
    dev->hard_start_xmit=	Smsc9312_hard_start_xmit;
    dev->get_stats=		Smsc9312_get_stats;
    dev->do_ioctl=		Smsc9312_do_ioctl;
    dev->set_multicast_list=Smsc9312_set_multicast_list;
   // dev->flags|= IFF_PROMISC;    // mobbad ---last
    dev->flags |= (IFF_MULTICAST | IFF_ALLMULTI);
#ifdef NAPI_PROCESS
    if(rx_mode==PROCESSING_MODE_NAPI) {
            dev->poll=Smsc9132_rx_poll;
            dev->weight=napi_weight;
    }
#endif
    SET_MODULE_OWNER(dev);

    privateData->dwIdRev=dwIdRev;
    privateData->dwFpgaRev=dwFpgaRev&(0x000000FFUL);
    privateData->dev=dev;

 //   printk(privateData->ifName,"%s",dev->name);

    result=0;

    DONE:
    if(result!=0) {
        if(dev!=NULL) {
            if(dev->priv!=NULL) {
                if(platformInitialized) {
                    Platform_CleanUp(platformData);
                }
                kfree(dev->priv);
                dev->priv=NULL;
                thisModulePrivate = NULL;

            }
        }
    }
    SMSC_TRACE("<--Smsc9312_init(), result=%d",result);
    return result;
}


void checktheAccessMode(PPRIVATE_DATA privateData)
{
    DWORD dwIdRev=0UL;
    DWORD dwByteTestVal=0UL;
    int i,j;
    int errCounter;

    /* TRY SPI, SMI, I2C */
	for (i=0; i< TOTAX_MTHDS;i ++)
	{
		if(periAccessMethods[i].PPheralHWInit != NULL)
			periAccessMethods[i].PPheralHWInit();

		privateData->PlatformGetRegDW = periAccessMethods[i].LANGETREGW;
		privateData->PlatformSetRegDW = periAccessMethods[i].LANSETREGW;
		privateData->access_mode = periAccessMethods[i].PPheralAccess;
		mdelay(1000);
		if(access_debug)
		{
			errCounter=0;
			for(j=0; j< 10; j++)
			{
				dwByteTestVal= LanGetRegDW(BYTE_TEST);
				if(dwByteTestVal !=(DWORD)0x87654321)
				{
					SMSC_WARNING("dwByteTestVal is not equal to 0x87654321, incrementing the error Counter \n");
					errCounter++;
				}
			}
			if (errCounter==10)
			{
				SMSC_WARNING("ByteTestVal is not readable on peripherial Access =%s",periAccessMethods[i].periPheralName);
			}
			else if(errCounter>0)
			{
				SMSC_WARNING("Intermittent Noise Error on the Access %s Bus, errCounter is =%d",periAccessMethods[i].periPheralName, errCounter);
			}
			if(dwByteTestVal !=(DWORD)0x87654321)
			{
				SMSC_TRACE("dwByteTestVal is not equal to 0x87654321 , Trying the next access method \n");
				continue;
			}
			else
			{
				SMSC_TRACE("Peripheral Access Method is %s\n",periAccessMethods[i].periPheralName )
			}
		}
		else
		{
			dwByteTestVal= LanGetRegDW(BYTE_TEST);
			if(dwByteTestVal !=(DWORD)0x87654321)
			{
				SMSC_TRACE("dwByteTestVal is not equal to 0x87654321, Trying the next access method \n");
				continue;
			}
			else
			{
				SMSC_TRACE("Peripheral Access Method is %s\n",periAccessMethods[i].periPheralName )
			}
		}


		dwIdRev=LanGetRegDW(ID_REV);
		SMSC_TRACE("dwIdRev=0x%08lX",dwIdRev);

	    switch(dwIdRev&0xFFFF0000UL) {
	    case 0x93110000UL:
	    	switch(dwIdRev&0x0000FFFFUL) {
	    	case 0UL:
	    	SMSC_TRACE("  HYDRA-16bit & HOST MAC Boylston revision A0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	case 1UL:
	    	SMSC_TRACE("  HYDRA-16bit & HOST MAC Boylston revision B0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	}
	    case 0x93120000UL:
	    	switch(dwIdRev&0x0000FFFFUL) {
	    	case 0UL:
	    	SMSC_TRACE("  HYDRA-32bit & HOST MAC Boylston revision A0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	case 1UL:
	    	SMSC_TRACE("  HYDRA-32bit & HOST MAC Boylston revision B0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	}
	    case 0x93130000UL:
	    	switch(dwIdRev&0x0000FFFFUL) {
	    	case 0UL:
	    	SMSC_TRACE("  LEO revision A0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	case 1UL:
	    	SMSC_TRACE("  LEO revision B0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	}
	    case 0x93030000UL:
	    	switch(dwIdRev&0x0000FFFFUL) {
	    	case 0UL:
	    	SMSC_TRACE("  LEO LITE revision A0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	case 1UL:
	    	SMSC_TRACE("  LEO LITE revision B0 identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    	}
	    case 0x9E040000UL:
	    	SMSC_TRACE("  LEO LITE AUTO identified, dwIdRev==0x%08lX",dwIdRev);
	    	goto DONE;
	    default:
	    	continue;
	    }
	}
DONE:
	if(i==TOTAX_MTHDS)
		privateData->access_mode= UNKNOWN_ACCESS;
//	printk("The PeripHeral Access Mode =%s \n\n\n",periAccessMethods[i].periPheralName);
}

int Smsc9312_open(struct net_device *dev)
{
    int i;
    int result=-ENODEV;
    PPRIVATE_DATA privateData=NULL;
    PPLATFORM_DATA platformData=NULL;
    BOOLEAN acquired_mem_region=FALSE;
    BOOLEAN acquired_isr=FALSE;
    DWORD reg_value;
    SMSC_TRACE("-->Smsc9312_open(dev=0x%08lX)",(DWORD)dev);
    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_open(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    privateData=(PPRIVATE_DATA)(dev->priv);
    if(privateData==NULL) {
        SMSC_WARNING("Smsc9312_open(privateData==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    platformData=&(privateData->PlatformData);

    for (i = 0; i < GPT_SCHEDULE_DEPTH; i++) {
        privateData->GptFunction [i] = NULL;
    }
    privateData->Gpt_scheduled_slot_index = GPT_SCHEDULE_DEPTH;

    //get memory region
    if(check_mem_region(privateData->dwLanBase,LAN_REGISTER_EXTENT)!=0)
    {
        SMSC_WARNING("Device memory is already in use.");
        result=-ENOMEM;
        goto DONE;
    }
    request_mem_region(privateData->dwLanBase,LAN_REGISTER_EXTENT,"SMSC_LAN9118");
    acquired_mem_region=TRUE;

    //initialize the LAN9118
    {
        DWORD dwIntCfg=0;
        if(irq_pol) {
            dwIntCfg|=INT_CFG_IRQ_POL_;
        }
        if(irq_type) {
            dwIntCfg|=INT_CFG_IRQ_TYPE_;
        }
        if(!Lan_Initialize(privateData,dwIntCfg,tx_fif_sz,afc_cfg))
        {
            SMSC_WARNING("Failed Lan_Initialize");
            result=-ENODEV;
            goto DONE;
        }
    }

    if(!Platform_RequestIRQ(platformData,irq,Smsc9312_ISR,privateData)) {
        result=-ENODEV;
        goto DONE;
    }
    acquired_isr=TRUE;

    //must now test the IRQ connection to the ISR
    SMSC_TRACE("Testing ISR using IRQ %ld",Platform_CurrentIRQ(platformData));
    {
        DWORD dwTimeOut=100000;
        Lan_SignalSoftwareInterrupt(privateData);
        do {
            udelay(10);
            dwTimeOut--;
        } while((dwTimeOut)&&(!(privateData->SoftwareInterruptSignal)));
        if(!(privateData->SoftwareInterruptSignal)) {
            SMSC_WARNING("ISR failed signaling test");
            result=-ENODEV;
            goto DONE;
        }
    }
    SMSC_TRACE("ISR passed test using IRQ %ld",Platform_CurrentIRQ(platformData));

    if(!Mac_Initialize(privateData)) {
        SMSC_WARNING("Failed Mac_Initialize");
        result=-ENODEV;
        goto DONE;
    }
    {//get mac address
        DWORD dwHigh16=0;
        DWORD dwLow32=0;
        DWORD dwIntFlags=0;
        VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
        if(mac_addr_hi16==0xFFFFFFFF) {
            dwHigh16=Mac_GetRegDW(privateData,ADDRH,keyCode);
            dwLow32=Mac_GetRegDW(privateData,ADDRL,keyCode);
            if((dwHigh16==0x0000FFFFUL)&&(dwLow32==0xFFFFFFFF))
            {
                dwHigh16=DW_HIGH_MAC16;
                dwLow32=DW_LOW_MAC;
                Mac_SetRegDW(privateData,ADDRH,dwHigh16,keyCode);
                Mac_SetRegDW(privateData,ADDRL,dwLow32,keyCode);
                SMSC_TRACE("Mac Address is set by default to 0x%04lX%08lX",
                    dwHigh16,dwLow32);
            } else {
                SMSC_TRACE("Mac Address is read from LAN9118 as 0x%04lX%08lX",
                    dwHigh16,dwLow32);
            }
        } else {
            //SMSC_ASSERT((mac_addr_hi16&0xFFFF8000UL)==0);
            dwHigh16=mac_addr_hi16;
            dwLow32=mac_addr_lo32;
            Mac_SetRegDW(privateData,ADDRH,dwHigh16,keyCode);
            Mac_SetRegDW(privateData,ADDRL,dwLow32,keyCode);
            SMSC_TRACE("Mac Address is set by parameter to 0x%04lX%08lX",
                dwHigh16,dwLow32);
        }
#ifdef CHIP_IS_LAN9303
        if(pauseMacAddrEnbl)
        {
        		dwHigh16 = dwHigh16&0xFFFF;
        		dwHigh16 = (dwHigh16 | (DIFF_PAUSE_MACADDR_EN_|PORT2_MAC_ADDR|PORT1_MAC_ADDR));
        		LanSetRegDW(SWITCH_MAC_ADDRL,dwLow32);
        		LanSetRegDW(SWITCH_MAC_ADDRH,dwHigh16);

        }
        if(enblDAPriority)
        {
        	SwitchEnableDAPriority(privateData);
        }
#endif
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
        dev->dev_addr[0]=LOBYTE(LOWORD(dwLow32));
        dev->dev_addr[1]=HIBYTE(LOWORD(dwLow32));
        dev->dev_addr[2]=LOBYTE(HIWORD(dwLow32));
        dev->dev_addr[3]=HIBYTE(HIWORD(dwLow32));
        dev->dev_addr[4]=LOBYTE(LOWORD(dwHigh16));
        dev->dev_addr[5]=HIBYTE(LOWORD(dwHigh16));
    }

    privateData->MulticastUpdatePending = FALSE;



    {
        DWORD dwRxDmaCh=rx_dma;
        DWORD dwTxDmaCh=tx_dma;
        privateData->RxDmaChReserved=FALSE;


        if(rx_dma==TRANSFER_REQUEST_DMA) {
            dwRxDmaCh=Platform_RequestDmaChannel(&(privateData->PlatformData));
            SMSC_ASSERT(dwRxDmaCh!=TRANSFER_REQUEST_DMA);
            if(dwRxDmaCh<TRANSFER_REQUEST_DMA) {
                privateData->RxDmaChReserved=TRUE;
            }
        }
        privateData->TxDmaChReserved=FALSE;
        if(tx_dma==TRANSFER_REQUEST_DMA) {
            dwTxDmaCh=Platform_RequestDmaChannel(&(privateData->PlatformData));
            SMSC_ASSERT(dwTxDmaCh!=TRANSFER_REQUEST_DMA);
            if(dwTxDmaCh<TRANSFER_REQUEST_DMA) {
                privateData->TxDmaChReserved=TRUE;
            }
        }
        Tx_Initialize(privateData,dwTxDmaCh,dma_threshold);
        Rx_Initialize(privateData,dwRxDmaCh,dma_threshold);
    }

#ifndef LINUX_2_6_OR_NEWER
    MOD_INC_USE_COUNT;
#endif
    privateData->Running=TRUE;
    netif_start_queue(dev);


    spin_lock_init(&(privateData->GpTimerLock));
    Lan_EnableInterrupt(privateData,INT_EN_GPT_INT_EN_);

#ifdef FULLY_MANAGED
    spin_lock_init(&(privateData->stpQueueLock));
#ifndef CHIP_IS_LAN9303
    spin_lock_init(&(privateData->ptpQueueLock));
#endif
    spin_lock_init(&(privateData->igmpQueueLock));

#endif

    if(privateData->access_mode==HBI_ACCESS)
    {
        DWORD phyIndex =0;
        for (phyIndex = 0; phyIndex <1 ; phyIndex++)
        {
                if(!Phy_Initialize(
                        privateData,
                        phyIndex,
                        phy_addr,
                        link_mode))
                {
                        SMSC_WARNING("Failed to initialize Phy");
                        result=-ENODEV;
                        goto DONE;
                }
        }
    }
    else
    {
	    if(!Phy_Initialize(
	                    privateData,
	                    privateData->dwPhyAddress,
	                    phy_addr,
	                    link_mode))
	    {
	        SMSC_WARNING("Failed to initialize Phy");
	        result=-ENODEV;
	        goto DONE;
	    }
    }

#ifdef SNMP_SUPPORT
    mib_start_timer(privateData);
 //   privateData->startUpTime = current_kernel_time();
#endif

#ifdef FULLY_MANAGED
    if(setPortsToCPUTag(privateData)==0) {
    	SMSC_WARNING("Failed to detect Switch");
    	result=-ENODEV;
    	goto DONE;
    }
    InitializeSkbQueues	(privateData);
#endif
#ifdef FULLY_MANAGED
    //  Added a PermenantDB of Multicast address to the ALR, these are static and never aged */
    EnblPermanentDB(privateData);
#endif
    /* This is a bug fix, Earlier revisions of A0 does not have the default value of one for the field Reject
     * MAC Type in the register MAC_RX_CFG[MII,1,2],  */
   /*  The code below se the value one, for the Register, the field 1 hs to set for Reject Control Pause Frames */

    reg_value = Switch_GetRegDW(privateData, MII_RX_CFG);
    reg_value |= MII_RX_CFG_REJ_MAC_TYP_;
    Switch_SetRegDW(privateData,(MII_RX_CFG),reg_value);

    reg_value = Switch_GetRegDW(privateData, (MII_RX_CFG+0x400) );
    reg_value |= MII_RX_CFG_REJ_MAC_TYP_;
    Switch_SetRegDW(privateData,(MII_RX_CFG + 0x400),reg_value);

    reg_value = Switch_GetRegDW(privateData, (MII_RX_CFG+0x800) );
    reg_value |= MII_RX_CFG_REJ_MAC_TYP_;
    Switch_SetRegDW(privateData,(MII_RX_CFG + 0x800),reg_value);

    result=0;
DONE:
    if(result!=0) {

#ifndef LINUX_2_6_OR_NEWER
        MOD_DEC_USE_COUNT;
#endif
        if(privateData!=NULL) {
            if(privateData->TxDmaChReserved) {
                Platform_ReleaseDmaChannel(platformData,
                    privateData->dwTxDmaCh);
                privateData->TxDmaChReserved=FALSE;
            }
            if(privateData->RxDmaChReserved) {
                Platform_ReleaseDmaChannel(platformData,
                    privateData->dwRxDmaCh);
                privateData->RxDmaChReserved=FALSE;
            }
            if(acquired_isr) {
                Platform_FreeIRQ(platformData);
            }
            if(acquired_mem_region) {
                release_mem_region(
                    privateData->dwLanBase,
                    LAN_REGISTER_EXTENT);
            }
        }
    }

    //NOTE: DO NOT put any initializations here
    //   all initialization should be done before DONE:
    //   and before result=0;
    //   If any initializations fail then be sure to free
    //   resources in the block if(result!=0) {...}
    //   which is after DONE:
    if(privateData->access_mode==HBI_ACCESS)
    {
    	   Phy_SetLink(privateData,1, link_mode);
    	   Phy_SetLink(privateData,2, link_mode);
    }

    SMSC_TRACE("<--Smsc9312_open, result=%d",result);
    SMSC_TRACE (" sizeof LWORD = %u, sixeof DWORD=%u \n", sizeof (LWORD),sizeof (DWORD) );
    SMSC_TRACE("Private data =%lx", (DWORD) (privateData))
    return result;
}
/** end of int Smsc9312_open(struct net_device *dev) **/


int Smsc9312_stop(struct net_device *dev)
{
    int result=0;
    PPRIVATE_DATA privateData=NULL;
    SMSC_TRACE("-->Smsc9312_stop(dev=0x%08lX)",(DWORD)dev);
    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_stop(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    privateData=(PPRIVATE_DATA)(dev->priv);
    if(privateData==NULL) {
        SMSC_WARNING("Smsc9312_stop(privateData==NULL)");
        result=-EFAULT;
        goto DONE;
    }

    privateData->StopLinkPolling=TRUE;
    del_timer_sync(&(privateData->LinkPollingTimer));

#ifdef SNMP_SUPPORT
    privateData->StopMibPolling=TRUE;
    del_timer_sync(&(privateData->MibPollingTimer));
#endif

    Lan_DisableInterrupt(privateData,INT_EN_GPT_INT_EN_);

    Tx_UpdateTxCounters(privateData);
    privateData->Running=FALSE;
    Lan_DisableIRQ(privateData);

    Tx_CompleteDma(privateData);

    Tx_StopQueue(privateData,0x01UL);

#ifndef LINUX_2_6_OR_NEWER
    MOD_DEC_USE_COUNT;
#endif

    if(privateData->TxDmaChReserved) {
        Platform_ReleaseDmaChannel(
            &(privateData->PlatformData),
            privateData->dwTxDmaCh);
        privateData->TxDmaChReserved=FALSE;
    }
    if(privateData->RxDmaChReserved) {
        Platform_ReleaseDmaChannel(
            &(privateData->PlatformData),
            privateData->dwRxDmaCh);
        privateData->RxDmaChReserved=FALSE;
    }

    Platform_FreeIRQ(&(privateData->PlatformData));
    release_mem_region(privateData->dwLanBase,LAN_REGISTER_EXTENT);

    {
        const DWORD dwLanBase=privateData->dwLanBase;
        const DWORD dwIdRev=privateData->dwIdRev;
        const DWORD dwFpgaRev=privateData->dwFpgaRev;
        struct net_device * const tempDev=privateData->dev;
        DWORD dwGeneration=privateData->dwGeneration;
        void * platformSetRegDW=privateData->PlatformSetRegDW;
        void * platformGetRegDW=privateData->PlatformGetRegDW;
        CNTROL_ACCESS accessMode=privateData->access_mode;
        char ifName[SMSC_IF_NAME_SIZE];
        PLATFORM_DATA platformDataBackup;
#ifndef UNMANAGED
        struct net_device *switchDev[PHY_NUMBER];
        memcpy(switchDev,privateData->switchDev,sizeof(struct net_device *)*PHY_NUMBER);
#endif
        memcpy(ifName,privateData->ifName,SMSC_IF_NAME_SIZE);
        memcpy(&platformDataBackup,&(privateData->PlatformData),sizeof(PLATFORM_DATA));

        memset(privateData,0,sizeof(PRIVATE_DATA));

#ifndef UNMANAGED
        memcpy(privateData->switchDev,switchDev,sizeof(struct net_device *)*PHY_NUMBER);
#endif
        privateData->dwLanBase=dwLanBase;
        privateData->dwIdRev=dwIdRev;
        privateData->dwFpgaRev=dwFpgaRev;
        privateData->dev=tempDev;
        privateData->PlatformSetRegDW=platformSetRegDW;
        privateData->PlatformGetRegDW=platformGetRegDW;
        privateData->access_mode=accessMode;
        privateData->dwGeneration=dwGeneration;
        privateData->StopLinkPolling=TRUE;
#ifdef SNMP_SUPPORT
        privateData->StopMibPolling=TRUE;
#endif
        memcpy(privateData->ifName,ifName,SMSC_IF_NAME_SIZE);
        memcpy(&(privateData->PlatformData),&platformDataBackup,sizeof(PLATFORM_DATA));
    }

DONE:
    SMSC_TRACE("<--Smsc9312_stop, result=%d",result);
    return result;
}

int Smsc9312_hard_start_xmit(
struct sk_buff *skb, struct net_device * const dev)
{
    int result=0;
#ifdef FULLY_MANAGED
    int VLANTAG=0x0008;	 // let the switch decide.
    int proto=0;
#endif //FULLY_MANAGED
    PPRIVATE_DATA privateData=NULL;
    //	SMSC_TRACE("-->Smsc9312_hard_start_xmit(skb=0x%08lX,dev=0x%08lX)",(DWORD)skb,(DWORD)dev);
    if(skb==NULL) {
        SMSC_WARNING("Smsc9312_hard_start_xmit(skb==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_hard_start_xmit(dev==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    if(dev->priv==NULL) {
        SMSC_WARNING("Smsc9312_hard_start_xmit(dev->priv==NULL)");
        result=-EFAULT;
        goto DONE;
    }
    privateData=(PPRIVATE_DATA)(dev->priv);
    //	SET_GPIO(GP_TX);

#ifdef DUMP_TX_PACKET
    SMSC_TRACE("\nDUMP SKB before adding Tag: skb head 0x%lx, data=0x%lx\n", (DWORD)(skb->head), (DWORD)(skb->data));

#endif


#ifdef FULLY_MANAGED
    {
    struct smc931x_ethhdr *eth;
#ifndef CHIP_IS_LAN9303
     if(privateData->SnoopingStarted)
	{
	igmpPaktype=isIGMPPacket(skb->data,skb->len );
	switch (igmpPaktype)
		{
		case 1:
		SMSC_TRACE("\nIGMP packet from MII port ");
		privateData->packetType = PKT_TYPE_IS_IGMP;
		atomic_set(&skb->users,((PKT_TYPE_IS_IGMP<<4)| (0x0 & 0xF)));
		sendSnoopedMsgToManager(privateData, skb->data,skb->len,PKT_TYPE_IS_IGMP, 4);
	        privateData->SnoopedPktCnt++;
		igmpPaktype=0;
		break;
		case 2:
		VLANTAG=0x0003;//forward to both port
		igmpPaktype=0;
		goto FORWARD;
		break;
		case FALSE:
		goto FORWARD;
		break;
		default:
		VLANTAG |= (igmpPaktype&0x0000E000);
		SMSC_TRACE("\nVLAN packet from MII port %x %x",VLANTAG,igmpPaktype);
		goto FORWARD;
		break;
		}

	}
	else
#else
	goto FORWARD; /* just to avoid the warning while compiling the driver for LAN9303*/
#endif
	{
	FORWARD:
		isVLANPacket(skb->data,skb->len, &proto);
	    memmove(skb->data -4 , skb->data , 12); // Move Dst/SRC with 4
	    skb->data -=4;
	    skb->len+= 4;
	    eth = (struct smc931x_ethhdr *) (skb->data);
	    eth->vlan[0] = htons(0x8100);
#ifndef CHIP_IS_LAN9303
	    eth->vlan[1] = htons(VLANTAG);
#else
	    /* By default Enable Priority lookup, and enable VLAN Rules to be on */
	    eth->vlan[1]= htons(EGRS_CPU_TAG_VLAN_RULE_SET | EGRS_CPU_TAG_PRI_SET | VLANTAG);
#endif
	    Switch_SendSkb(privateData,skb);
	}
    }
#else
    Tx_SendSkb(privateData,skb);
#endif

    //	CLEAR_GPIO(GP_TX);
    DONE:
    //	SMSC_TRACE("<--Smsc9312_hard_start_xmit, result=%d",result);
    return result;
}

struct net_device_stats * Smsc9312_get_stats(struct net_device * const dev)
{
    PPRIVATE_DATA privateData=NULL;
    if(dev==NULL) {
        SMSC_WARNING("Smsc9312_get_stats(dev==NULL)");
        return NULL;
    }
    if(dev->priv==NULL) {
    //	SMSC_WARNING("Smsc9312_get_stats(dev->priv==NULL)");
        return NULL;
    }

    privateData=(PPRIVATE_DATA)(dev->priv);
    if(privateData->Running) {
        privateData->stats.rx_dropped+=LanGetDirectRegDW(RX_DROP);
        Tx_UpdateTxCounters(privateData);
    }
    return &(privateData->stats);
}

void Smsc9312_set_multicast_list(struct net_device *dev)
{
    SMSC_ASSERT(dev!=NULL);
    SMSC_TRACE("-->Smsc9312_set_multicast_list");
    Rx_SetMulticastList(dev);
}

int Smsc9312_do_ioctl(
struct net_device *dev,
struct ifreq *ifr,
int cmd)
{
    PPRIVATE_DATA privateData=NULL;
    PSMSC931X_IOCTL_DATA ioctlData=NULL;
    BOOLEAN success=FALSE;
    //SMSC_TRACE("-->Smsc9312_do_ioctl with switch register");
    //SMSC_TRACE("cmd=%d,SIOCGMIIPHY=%d,SIOCDEVPRIVATE=%d",
    //	cmd,SIOCGMIIPHY,SIOCDEVPRIVATE);
    SMSC_ASSERT(dev!=NULL);
    SMSC_ASSERT(dev->priv!=NULL);
    privateData=((PPRIVATE_DATA)dev->priv);
    if(ifr==NULL) {
        SMSC_WARNING("Smsc9312_do_ioctl(ifr==NULL)");
        goto DONE;
    }

    if(privateData->LanInitialized) {
        // standard MII IOC's
        struct mii_ioctl_data * const data=
            (struct mii_ioctl_data *) & ifr->ifr_data;
        switch(cmd) {
        case SIOCGMIIPHY:

        case SIOCDEVPRIVATE:
            data->phy_id=1;
    //			SMSC_TRACE("SIOCGMIIPHY: phy_id set to 0x%04X",data->phy_id);
            return 0;
        case SIOCGMIIREG:
        case SIOCDEVPRIVATE+1:
            {
                DWORD dwIntFlags=0;
                VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
                                data->val_out=Phy_GetRegW(privateData,
                                        privateData->dwPhyAddress,data->reg_num,keyCode);
                Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            }
            SMSC_TRACE("SIOCGMIIREG: phy_id=0x%04X, reg_num=0x%04X, val_out set to 0x%04X",
                data->phy_id,data->reg_num,data->val_out);
            return 0;
        case SIOCSMIIREG:
        case SIOCDEVPRIVATE+2:
            SMSC_TRACE("SIOCSMIIREG: phy_id=0x%04X, reg_num=0x%04X, val_in=0x%04X",
                data->phy_id,data->reg_num,data->val_in);
            {
                DWORD dwIntFlags=0;
                VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
                                Phy_SetRegW(privateData,
                                        privateData->dwPhyAddress,data->reg_num,
                                        ((WORD)(data->val_in)),keyCode);
                Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            }
            return 0;
        default:break;//make lint happy
        }
    }
    if(ifr->ifr_data==NULL) {
    //		SMSC_WARNING("Smsc9312_do_ioctl(ifr->ifr_data==NULL)");
        goto DONE;
    }
    if(cmd!=((int)SMSC931X_IOCTL))
        {
            SMSC_TRACE(" Command Unknown %d ( expecting %d)\n", cmd, (int)SMSC931X_IOCTL);
            goto DONE;
        }
    ioctlData=(PSMSC931X_IOCTL_DATA)(ifr->ifr_data);
    if(ioctlData->dwSignature!=SMSC931X_APP_SIGNATURE) {
        goto DONE;
    }

    switch(ioctlData->dwCommand) {
    case COMMAND_GET_SIGNATURE:
        success=TRUE;
        break;
    case COMMAND_GET_FLOW_PARAMS:
        ioctlData->Data[0]=privateData->RxFlowMeasuredMaxThroughput;
        ioctlData->Data[1]=privateData->RxFlowMeasuredMaxPacketCount;
        ioctlData->Data[2]=privateData->RxFlowParameters.MaxThroughput;
        ioctlData->Data[3]=privateData->RxFlowParameters.MaxPacketCount;
        ioctlData->Data[4]=privateData->RxFlowParameters.PacketCost;
        ioctlData->Data[5]=privateData->RxFlowParameters.BurstPeriod;
        ioctlData->Data[6]=privateData->RxFlowMaxWorkLoad;
        ioctlData->Data[7]=LanGetDirectRegDW(INT_CFG)>>24;
        privateData->RxFlowMeasuredMaxThroughput=0;
        privateData->RxFlowMeasuredMaxPacketCount=0;
        success=TRUE;
        break;
    case COMMAND_SET_FLOW_PARAMS:
        if(!(privateData->RxFlowControlActive)) {
            privateData->RxFlowParameters.MaxThroughput=ioctlData->Data[2];
            privateData->RxFlowParameters.MaxPacketCount=ioctlData->Data[3];
            privateData->RxFlowParameters.PacketCost=ioctlData->Data[4];
            privateData->RxFlowParameters.BurstPeriod=ioctlData->Data[5];
            if(ioctlData->Data[6]==0xFFFFFFFFUL) {
                privateData->RxFlowMaxWorkLoad=
                    privateData->RxFlowParameters.MaxThroughput+
                    (privateData->RxFlowParameters.MaxPacketCount*
                    privateData->RxFlowParameters.PacketCost);
            } else {
                privateData->RxFlowMaxWorkLoad=ioctlData->Data[6];
            }
            Lan_SetIntDeas(privateData,ioctlData->Data[7]);
            privateData->RxFlowBurstMaxWorkLoad=
                (privateData->RxFlowMaxWorkLoad*
                privateData->RxFlowParameters.BurstPeriod)/1000;
            success=TRUE;
        };break;
    case COMMAND_GET_CONFIGURATION:
        ioctlData->Data[0]=DRIVER_VERSION;
        ioctlData->Data[1]=lan_base;
        ioctlData->Data[2]=bus_width;
        ioctlData->Data[3]=link_mode;
        ioctlData->Data[4]=irq;
        ioctlData->Data[5]=int_deas;
        ioctlData->Data[6]=irq_pol;
        ioctlData->Data[7]=irq_type;
        ioctlData->Data[8]=rx_dma;
        ioctlData->Data[9]=tx_dma;
        ioctlData->Data[10]=dma_threshold;
        ioctlData->Data[11]=mac_addr_hi16;
        ioctlData->Data[12]=mac_addr_lo32;
        ioctlData->Data[13]=debug_mode;
        ioctlData->Data[14]=tx_fif_sz;
        ioctlData->Data[15]=afc_cfg;
#ifndef NAPI_PROCESS
        ioctlData->Data[16]=tasklets;
#else
        ioctlData->Data[16]=rx_mode;
#endif
        ioctlData->Data[17]=max_throughput;
        ioctlData->Data[18]=max_packet_count;
        ioctlData->Data[19]=packet_cost;
        ioctlData->Data[20]=burst_period;
        ioctlData->Data[21]=max_work_load;
        ioctlData->Data[22]=privateData->dwIdRev;
        ioctlData->Data[23]=privateData->dwFpgaRev;
        ioctlData->Data[24]=1;
        ioctlData->Data[25]=privateData->dwPhyId;
        ioctlData->Data[26]=privateData->bPhyModel;
        ioctlData->Data[27]=privateData->bPhyRev;
        ioctlData->Data[28]=privateData->dwLinkSpeed;
        ioctlData->Data[29]=privateData->RxFlowMeasuredMaxThroughput;
        ioctlData->Data[30]=privateData->RxFlowMeasuredMaxPacketCount;
        ioctlData->Data[31]=privateData->RxFlowParameters.MaxThroughput;
        ioctlData->Data[32]=privateData->RxFlowParameters.MaxPacketCount;
        ioctlData->Data[33]=privateData->RxFlowParameters.PacketCost;
        ioctlData->Data[34]=privateData->RxFlowParameters.BurstPeriod;
        ioctlData->Data[35]=privateData->RxFlowMaxWorkLoad;
//        printk(ioctlData->Strng1,"%s, %s",__DATE__,__TIME__);
//        printk(ioctlData->Strng2,"%s",privateData->ifName);
        privateData->RxFlowMeasuredMaxThroughput=0;
        privateData->RxFlowMeasuredMaxPacketCount=0;
        success=TRUE;
        break;
    case COMMAND_LAN_GET_REG:
        if((ioctlData->Data[0]<LAN_REGISTER_EXTENT)&&
            ((ioctlData->Data[0]&0x3UL)==0))
        {
            ioctlData->Data[1]=
                (*((volatile DWORD *)(privateData->dwLanBase+
                        ioctlData->Data[0])));
            success=TRUE;
        } else {
            SMSC_WARNING("Reading LAN9118 Mem Map Failed");
            goto MEM_MAP_ACCESS_FAILED;
        }
        break;
    case COMMAND_LAN_SET_REG:
        if((ioctlData->Data[0]<LAN_REGISTER_EXTENT)&&
            ((ioctlData->Data[0]&0x3UL)==0))
        {
            (*((volatile DWORD *)(privateData->dwLanBase+
                        ioctlData->Data[0])))=ioctlData->Data[1];
            success=TRUE;
        } else {
            SMSC_WARNING("Reading LAN9118 Mem Map Failed");
MEM_MAP_ACCESS_FAILED:
            SMSC_WARNING("  Invalid offset == 0x%08lX",ioctlData->Data[0]);
            if(ioctlData->Data[0]>=LAN_REGISTER_EXTENT) {
                SMSC_WARNING("    Out of range");
            }
            if(ioctlData->Data[0]&0x3UL) {
                SMSC_WARNING("    Not DWORD aligned");
            }
        }
        break;
    case COMMAND_MAC_GET_REG:
        if((ioctlData->Data[0]<=0xC)&&(privateData->LanInitialized)) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            ioctlData->Data[1]=
                Mac_GetRegDW(privateData,ioctlData->Data[0],keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING("Reading Mac Register Failed");
            goto MAC_ACCESS_FAILURE;
        }
        break;
    case COMMAND_MAC_SET_REG:
        if((ioctlData->Data[0]<=0xC)&&(privateData->LanInitialized)) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            Mac_SetRegDW(
                privateData,
                ioctlData->Data[0],
                ioctlData->Data[1],
                keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING("Writing Mac Register Failed");
    MAC_ACCESS_FAILURE:
            if(!(privateData->LanInitialized)) {

                SMSC_WARNING("  LAN Not Initialized,");
                SMSC_WARNING("    Use ifconfig to bring interface UP");
            }
            if(!(ioctlData->Data[0]<=0xC)) {
                SMSC_WARNING("  Invalid index == 0x%08lX",ioctlData->Data[0]);
            }
        }
        break;
    case COMMAND_PHY_GET_REG:
                if((ioctlData->Data[0]<32)&&(privateData->LanInitialized)) {
                    DWORD dwIntFlags=0;
                    VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
                    ioctlData->Data[2]=((DWORD)
                            Phy_GetRegW(privateData,ioctlData->Data[0],ioctlData->Data[1],keyCode));
                    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
                    success=TRUE;
                } else {
                    SMSC_WARNING("Reading Phy Register Failed");
                    goto PHY_ACCESS_FAILURE;
                }

        break;
    case COMMAND_PHY_SET_REG:

        if((ioctlData->Data[0]<32)&&(privateData->LanInitialized)) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            Phy_SetRegW(
                    privateData,
            ioctlData->Data[0],
            ioctlData->Data[1],
            ((WORD)(ioctlData->Data[2])),
            keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
                SMSC_WARNING("Writing Phy Register Failed");
PHY_ACCESS_FAILURE:
                if(!(privateData->LanInitialized)) {
                    SMSC_WARNING("  Lan Not Initialized,");
                    SMSC_WARNING("    Use ifconfig to bring interface UP");
                }
                if(!(ioctlData->Data[0]<32)) {
                    SMSC_WARNING("  Invalid index == 0x%ld",ioctlData->Data[0]);
                }
    }

        break;
    case COMMAND_DUMP_LAN_REGS:
        ioctlData->Data[LAN_REG_ID_REV]=LanGetDirectRegDW(ID_REV);
        ioctlData->Data[LAN_REG_INT_CFG]=LanGetDirectRegDW(INT_CFG);
        ioctlData->Data[LAN_REG_INT_STS]=LanGetDirectRegDW(INT_STS);
        ioctlData->Data[LAN_REG_INT_EN]=LanGetDirectRegDW(INT_EN);
        ioctlData->Data[LAN_REG_BYTE_TEST]=LanGetDirectRegDW(BYTE_TEST);
        ioctlData->Data[LAN_REG_FIFO_INT]=LanGetDirectRegDW(FIFO_INT);
        ioctlData->Data[LAN_REG_RX_CFG]=LanGetDirectRegDW(RX_CFG);
        ioctlData->Data[LAN_REG_TX_CFG]=LanGetDirectRegDW(TX_CFG);
        ioctlData->Data[LAN_REG_HW_CFG]=LanGetDirectRegDW(HW_CFG);
        ioctlData->Data[LAN_REG_RX_DP_CTRL]=LanGetDirectRegDW(RX_DP_CTL);
        ioctlData->Data[LAN_REG_RX_FIFO_INF]=LanGetDirectRegDW(RX_FIFO_INF);
        ioctlData->Data[LAN_REG_TX_FIFO_INF]=LanGetDirectRegDW(TX_FIFO_INF);
        ioctlData->Data[LAN_REG_PMT_CTRL]=LanGetDirectRegDW(PMT_CTRL);
        ioctlData->Data[LAN_REG_GPIO_CFG]=LanGetDirectRegDW(GPIO_CFG);
        ioctlData->Data[LAN_REG_GPT_CFG]=LanGetDirectRegDW(GPT_CFG);
        ioctlData->Data[LAN_REG_GPT_CNT]=LanGetDirectRegDW(GPT_CNT);
        ioctlData->Data[LAN_REG_FPGA_REV]=LanGetRegDW(FPGA_REV);
        ioctlData->Data[LAN_REG_ENDIAN]=LanGetDirectRegDW(ENDIAN);
        ioctlData->Data[LAN_REG_FREE_RUN]=LanGetDirectRegDW(FREE_RUN);
        ioctlData->Data[LAN_REG_RX_DROP]=LanGetDirectRegDW(RX_DROP);
        if(privateData->LanInitialized) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            ioctlData->Data[LAN_REG_MAC_CSR_CMD]=LanGetDirectRegDW(MAC_CSR_CMD);
            ioctlData->Data[LAN_REG_MAC_CSR_DATA]=LanGetDirectRegDW(MAC_CSR_DATA);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
        } else {
            ioctlData->Data[LAN_REG_MAC_CSR_CMD]=LanGetDirectRegDW(MAC_CSR_CMD);
            ioctlData->Data[LAN_REG_MAC_CSR_DATA]=LanGetDirectRegDW(MAC_CSR_DATA);
        }
        ioctlData->Data[LAN_REG_AFC_CFG]=LanGetDirectRegDW(AFC_CFG);
        ioctlData->Data[LAN_REG_E2P_CMD]=LanGetRegDW(E2P_CMD);
        ioctlData->Data[LAN_REG_E2P_DATA]=LanGetRegDW(E2P_DATA);
        success=TRUE;
        break;

    case COMMAND_DUMP_MAC_REGS:
        if(privateData->LanInitialized) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            ioctlData->Data[MAC_REG_MAC_CR]=Mac_GetRegDW(privateData,MAC_CR,keyCode);
            ioctlData->Data[MAC_REG_ADDRH]=Mac_GetRegDW(privateData,ADDRH,keyCode);
            ioctlData->Data[MAC_REG_ADDRL]=Mac_GetRegDW(privateData,ADDRL,keyCode);
            ioctlData->Data[MAC_REG_HASHH]=Mac_GetRegDW(privateData,HASHH,keyCode);
            ioctlData->Data[MAC_REG_HASHL]=Mac_GetRegDW(privateData,HASHL,keyCode);
            ioctlData->Data[MAC_REG_MII_ACC]=Mac_GetRegDW(privateData,MII_ACC,keyCode);
            ioctlData->Data[MAC_REG_MII_DATA]=Mac_GetRegDW(privateData,MII_DATA,keyCode);
            ioctlData->Data[MAC_REG_FLOW]=Mac_GetRegDW(privateData,FLOW,keyCode);
            ioctlData->Data[MAC_REG_VLAN1]=Mac_GetRegDW(privateData,VLAN1,keyCode);
            ioctlData->Data[MAC_REG_VLAN2]=Mac_GetRegDW(privateData,VLAN2,keyCode);
            ioctlData->Data[MAC_REG_WUFF]=Mac_GetRegDW(privateData,WUFF,keyCode);
            ioctlData->Data[MAC_REG_WUCSR]=Mac_GetRegDW(privateData,WUCSR,keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING("Mac Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_DUMP_SWITCH_REGS:
        if(privateData->LanInitialized) {
                DWORD regIndex = 0;
                getSwitchReadLock(privateData);
                for (regIndex =0; regIndex <  SWITCH_MAX_REG_INDEX ; regIndex++) {
                    ioctlData->Data[regIndex]=Switch_GetRegDW(privateData,SwitchRegDef[regIndex].index);
                }
                releaseSwitchReadLock(privateData);
                success=TRUE;
        } else {
                SMSC_WARNING("MAC Not Initialized,");
                SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_SWITCH_GET_REG:
        if(privateData->LanInitialized) {

            getSwitchReadLock(privateData);
            ioctlData->Data[1]=Switch_GetRegDW(privateData,ioctlData->Data[0]);
            releaseSwitchReadLock(privateData);
//            printk("Kernel:the Register Value is 0x%x \n", ioctlData->Data[1]);
            success=TRUE;
        } else {
            SMSC_WARNING("Mac Not Initialized,");
            SMSC_WARNING(" Use ifconfig to bring interface UP");
        }

        break;
    case COMMAND_SWITCH_SET_REG:
        if(privateData->LanInitialized) {
            getSwitchWriteLock(privateData);
	    //printk("Got the Command SWITCH SET REG \n");
            Switch_SetRegDW(privateData,ioctlData->Data[0],ioctlData->Data[1]);
            releaseSwitchWriteLock(privateData);
            success=TRUE;
        } else {
            SMSC_WARNING("Mac Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;

    case COMMAND_DUMP_PHY_REGS:
        if( (privateData->LanInitialized) &&
            (ioctlData->Data[0] < PHY_NUMBER) ) {
            DWORD dwIntFlags=0;
            WORD phyIndex = ioctlData->Data[0];

            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            ioctlData->Data[PHY_REG_0]=Phy_GetRegW(privateData,phyIndex,0,keyCode);
            ioctlData->Data[PHY_REG_1]=Phy_GetRegW(privateData,phyIndex,1,keyCode);
            ioctlData->Data[PHY_REG_2]=Phy_GetRegW(privateData,phyIndex,2,keyCode);
            ioctlData->Data[PHY_REG_3]=Phy_GetRegW(privateData,phyIndex,3,keyCode);
            ioctlData->Data[PHY_REG_4]=Phy_GetRegW(privateData,phyIndex,4,keyCode);
            ioctlData->Data[PHY_REG_5]=Phy_GetRegW(privateData,phyIndex,5,keyCode);
            ioctlData->Data[PHY_REG_6]=Phy_GetRegW(privateData,phyIndex,6,keyCode);
            ioctlData->Data[PHY_REG_16]=Phy_GetRegW(privateData,phyIndex,16,keyCode);
            ioctlData->Data[PHY_REG_17]=Phy_GetRegW(privateData,phyIndex,17,keyCode);
            ioctlData->Data[PHY_REG_18]=Phy_GetRegW(privateData,phyIndex,18,keyCode);
            ioctlData->Data[PHY_REG_20]=Phy_GetRegW(privateData,phyIndex,20,keyCode);
            ioctlData->Data[PHY_REG_21]=Phy_GetRegW(privateData,phyIndex,21,keyCode);
            ioctlData->Data[PHY_REG_22]=Phy_GetRegW(privateData,phyIndex,22,keyCode);
            ioctlData->Data[PHY_REG_23]=Phy_GetRegW(privateData,phyIndex,23,keyCode);
            ioctlData->Data[PHY_REG_27]=Phy_GetRegW(privateData,phyIndex,27,keyCode);
            ioctlData->Data[PHY_REG_28]=Phy_GetRegW(privateData,phyIndex,28,keyCode);
            ioctlData->Data[PHY_REG_29]=Phy_GetRegW(privateData,phyIndex,29,keyCode);
            ioctlData->Data[PHY_REG_30]=Phy_GetRegW(privateData,phyIndex,30,keyCode);
            ioctlData->Data[PHY_REG_31]=Phy_GetRegW(privateData,phyIndex,31,keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING(" LAN Not Initialized or Invalid Phy Index=%lu", ioctlData->Data[0]);
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_DUMP_EEPROM:
        {
            BYTE data=0;
            WORD index=0;
            Eeprom_EnableAccess(privateData);
            success=TRUE;
            for(index=0;index<8;index++) {
                if(Eeprom_ReadLocation(privateData,index,&data)) {
                    ioctlData->Data[index]=(DWORD)data;
                } else {
                    success=FALSE;
                    break;
                }
            }
            Eeprom_DisableAccess(privateData);
        };break;
	case COMMAND_READ_EEPROM:
        {
            BYTE data=0;
            Eeprom_EnableAccess(privateData);
            success=TRUE;
            if(Eeprom_ReadLocation(privateData,(WORD)ioctlData->Data[0],&data)) {
                ioctlData->Data[1]=(DWORD)data;
                } else {
                    success=FALSE;
                    break;
                }
           Eeprom_DisableAccess(privateData);
        };break;
	case COMMAND_WRITE_EEPROM:
        {
            Eeprom_EnableAccess(privateData);
            success=TRUE;
           if(Eeprom_WriteLocation(privateData,(WORD)ioctlData->Data[0],(BYTE)ioctlData->Data[1])) {
              } else {
                    success=FALSE;
                    break;
                }
            Eeprom_DisableAccess(privateData);
        };break;
    case COMMAND_GET_MAC_ADDRESS:
        if(privateData->LanInitialized) {
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            ioctlData->Data[0]=Mac_GetRegDW(privateData,ADDRH,keyCode);
            ioctlData->Data[1]=Mac_GetRegDW(privateData,ADDRL,keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING("Lan Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;

    case COMMAND_SET_MAC_ADDRESS:
        if(privateData->LanInitialized)
        {
            DWORD dwLow32=ioctlData->Data[1];
            DWORD dwHigh16=ioctlData->Data[0];
            DWORD dwIntFlags=0;
            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
            Mac_SetRegDW(privateData,ADDRH,dwHigh16,keyCode);
            Mac_SetRegDW(privateData,ADDRL,dwLow32,keyCode);
            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
            success=TRUE;
        } else {
            SMSC_WARNING("Lan Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        };break;
    case COMMAND_LOAD_MAC_ADDRESS:
        if(privateData->LanInitialized) {
            Eeprom_EnableAccess(privateData);
            if(Eeprom_Reload(privateData)) {
                if(Eeprom_IsMacAddressLoaded(privateData)) {
                    DWORD dwIntFlags=0;
                    VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
                    ioctlData->Data[0]=Mac_GetRegDW(privateData,ADDRH,keyCode);
                    ioctlData->Data[1]=Mac_GetRegDW(privateData,ADDRL,keyCode);
                    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
                    success=TRUE;
                } else {
                    SMSC_WARNING("Failed to Load Mac Address(1)");
                }
            } else {
                SMSC_WARNING("Failed to Load Mac Address(2)");
            }
            Eeprom_DisableAccess(privateData);
        } else {
            SMSC_WARNING("Lan Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        };break;
    case COMMAND_SAVE_MAC_ADDRESS:
        if(privateData->LanInitialized) {
            if(Eeprom_SaveMacAddress(privateData,
                ioctlData->Data[0],ioctlData->Data[1])) {
                success=TRUE;
            }
        } else {
            SMSC_WARNING("Lan Not Initialized,");
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        };break;
    case COMMAND_SET_DEBUG_MODE:
        debug_mode=ioctlData->Data[0];
        if(debug_mode&0x04UL) {
            if(OLD_REGISTERS(privateData))
            {
                g_GpioSetting=0x00270700UL;
            } else {
                g_GpioSetting=0x00670700UL;
            }
            LanSetDirectRegDW(GPIO_CFG,g_GpioSetting);
        } else {
            LanSetDirectRegDW(GPIO_CFG,0x70070000);
        }
        success=TRUE;
        break;
    case COMMAND_SET_LINK_MODE:
        link_mode=(ioctlData->Data[0]&0x7FUL);
//        printk("link_mode = %d\n", link_mode);
//        printk("dwPhyAddres=%d\n", privateData->dwPhyAddress);
        if(privateData->LanInitialized) {
                        Phy_SetLink(privateData, privateData->dwPhyAddress,link_mode);
                        //Phy_SetLink(privateData, 1,link_mode);
                        // Phy_SetLink(privateData, 2,link_mode);
        }
        success=TRUE;
        break;
    case COMMAND_GET_LINK_MODE:
        ioctlData->Data[0]=link_mode;
        success=TRUE;
        break;
    case COMMAND_CHECK_LINK:
        Phy_UpdateLinkMode(privateData);
        success=TRUE;
        break;
    case COMMAND_GET_LINK_SPEED:
    	ioctlData->Data[0]=linkSpeed;
    	success=TRUE;
    	break;
    case COMMAND_READ_BYTE:
        ioctlData->Data[1]=(*((volatile BYTE *)(ioctlData->Data[0])));
        success=TRUE;
        break;
    case COMMAND_READ_WORD:
        ioctlData->Data[1]=(*((volatile WORD *)(ioctlData->Data[0])));
        success=TRUE;
        break;
    case COMMAND_READ_DWORD:
        ioctlData->Data[1]=(*((volatile DWORD *)(ioctlData->Data[0])));
        success=TRUE;
        break;
    case COMMAND_WRITE_BYTE:
        (*((volatile BYTE *)(ioctlData->Data[0])))=
            ((BYTE)(ioctlData->Data[1]));
        success=TRUE;
        break;
    case COMMAND_WRITE_WORD:
        (*((volatile WORD *)(ioctlData->Data[0])))=
            ((WORD)(ioctlData->Data[1]));
        success=TRUE;
        break;
    case COMMAND_WRITE_DWORD:
        (*((volatile DWORD *)(ioctlData->Data[0])))=
            ((DWORD)(ioctlData->Data[1]));
        success=TRUE;
        break;

    case COMMAND_GET_LEO_REG:
        if (privateData->LanInitialized)
        {
            ioctlData->Data[1] = LanGetRegDW(ioctlData->Data[0]);
            success=TRUE;
        } else {
            SMSC_WARNING( " MAC Not Initialized ");
            SMSC_WARNING( " Operation failed ");
        }
        break;
    case COMMAND_SET_LEO_REG:
    if (privateData->LanInitialized)
        {
            LanSetRegDW(ioctlData->Data[0], ioctlData->Data[1]);
            success=TRUE;
        } else {
            SMSC_WARNING( " MAC Not Initialized ");
            SMSC_WARNING( " Operation failed ");
        }
        break;
    case COMMAND_DUMP_ALR:
        {
        struct ALRDATA {
                unsigned long data0;
                unsigned long data1;
            };

            struct ALRDATA *pALRData = (struct ALRDATA *)(&(ioctlData->Data[0]));
            SMSC_WARNING( " DUMPING ALR RX ");
            ioctlData->Data[0]  = 0;
            ioctlData->Data[1]  = 0;
            pALRData++;
            if (privateData->LanInitialized)
            {

                if (Switch_Rad_First_ALR(privateData, &(pALRData->data0), &(pALRData->data1)) )
                {
                    ioctlData->Data[0]++;
                    pALRData++;
                    while (Switch_Rad_Next_ALR(privateData, &(pALRData->data0), &(pALRData->data1)) )
                    {
                        ioctlData->Data[0]++;
                        pALRData++;
                    }

                }

                success=TRUE;
            }
        break;
        }
    case COMMAND_SET_ALR:
        if( (privateData->LanInitialized) && ((ioctlData->Data[0]) < PHY_MAP) )
        {
            ALREntryProp entryProp={0};
            entryProp.port = (ioctlData->Data[0]&SWITCH_PORT_MASK);
            Switch_Add_Static_ALR( privateData, ioctlData->Data[1],ioctlData->Data[2], &entryProp);
            success=TRUE;
        }
	else
	{
        	SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_DEL_ALR:
       if( (privateData->LanInitialized) && ((ioctlData->Data[0]) < PHY_MAP) )
	{
           Switch_remove_Static_ALR( privateData, ioctlData->Data[1],ioctlData->Data[2], ioctlData->Data[0]);
           success=TRUE;
        }
	else
	{
               SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
               SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
  	break;
    case COMMAND_ADD_ENTRY_IN_ALR:
        if( (privateData->LanInitialized) )
        {
            ALREntryProp entryProp={0};
            entryProp.port = (ioctlData->Data[2]&SWITCH_PORT_MASK);
            if(ioctlData->Data[3]>0)
            {
#ifdef CHIP_IS_LAN9303
            	entryProp.prienbl =1;
                entryProp.pri_alia_tag = ioctlData->Data[3];
#else
                entryProp.pri = ioctlData->Data[3];
#endif
            }
            Switch_Add_Entry_in_ALR( privateData, ioctlData->Data[0],ioctlData->Data[1], &entryProp);
            success=TRUE;
         }
        else
        {
                 SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                 SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
    break;
#ifdef FULLY_MANAGED
#ifndef CHIP_IS_LAN9303
    case COMMAND_DUMP_1588_REGS:
    case COMMAND_PTP_ENABLE:
    case COMMAND_PTP_DISABLE:
    case COMMAND_PTP_SET_MAC_ADDRESS:
    case COMMAND_PTP_GET_RX_SYNC_STAMP:
    case COMMAND_PTP_GET_TX_DELAY_REQ_STAMP:
    case COMMAND_PTP_GET_RX_SEQID_UUID:
    case COMMAND_PTP_GET_CLK_PREFERENCE:
    case COMMAND_PTP_SET_CLK_PREFERENCE:
    case COMMAND_PTP_SET_CLK_ADDENDUM:
    case COMMAND_PTP_GET_CLK_ADDENDUM:
    case COMMAND_PTP_SET_CLOCK_TARGET:
    case COMMAND_PTP_SET_CLOCK_TARGET_RELOAD:
    case COMMAND_PTP_SET_PORT_MODE:
    case COMMAND_PTP_SET_PORT_MAC:
    case COMMAND_PTP_RELOAD_TARGET_CLOCK:
    case COMMAND_PTP_SEND_MSG:
    	success=execPTPCommand(privateData,ioctlData);
    break;
#endif /* CHIP_IS_LAN9303 */
            /** IGMP Config **/
        case  COMMAND_IGMP_ENABLE:

            SwitchEnableIGMP(privateData);
            success=TRUE;
            break;
        case  COMMAND_IGMP_DISABLE:
             SwitchDisableIGMP(privateData);
            success=TRUE;
            break;
        case COMMAND_MLD_ENABLE:
            SwitchEnableIGMP(privateData);
             success=TRUE;
             break;
#ifdef CHIP_IS_LAN9303
         case  COMMAND_MLD_DISABLE:
              SwitchDisableMLD(privateData);
             success=TRUE;
             break;
#endif
        case COMMAND_IGMP_SEND_MSG:
            if(privateData->LanInitialized) {
                int portNumber = 1; /* Hard code port Number*/
                sendSnoopedMsgToManager( privateData, (const char *) (&ioctlData->Data[1]),ioctlData->Data[0],PKT_TYPE_IS_IGMP, portNumber);
                success=TRUE;
            } else {
                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                SMSC_WARNING("  Use ifconfig to bring interface UP");
            }
            break;
#endif
		 case  COMMAND_VLAN_DISABLE:
					 SwitchDisableVLAN(privateData);
					 success=TRUE;
					 break;
		 case  COMMAND_VLAN_ENABLE:
			        SwitchEnableVLAN(privateData);
					 success=TRUE;
					 break;
		 case COMMAND_DAPRIO_ENABLE:
			 	    SwitchEnableDAPriority(privateData);
			 	   success=TRUE;
			 		break;
		 case COMMAND_DAPRIO_DISABLE:
			 		SwitchDisableDAPriority(privateData);
			 		success=TRUE;
		 			break;
#ifdef CHIP_IS_LAN9303
		 case COMMAND_TURBOMODE_ENBL:
			        EnblTurboMode(privateData, ioctlData->Data[0]);
			        success=TRUE;
			        break;
		 case COMMAND_TURBOMODE_DSBL:
			 		DisblTurboMode(privateData, ioctlData->Data[0]);
			 		success=TRUE;
			 		break;
#endif
 /** FOLLOWING ARE FOR MIBS AND STATISTICS **/
#ifndef UNMANAGED
        /** ********************************
        *    MIB stuff here
        *******************************/
    case COMMAND_GET_DOT3_STAT_TBL:
        if( (privateData->LanInitialized) &&
                ((ioctlData->Data[0]) < PHY_NUMBER) ) {
            DWORD index = 0;
            Dot3StatsTable_data *pDot3StatsTable;
            DWORD *tblMap;
            pDot3StatsTable = &(privateData->dot3StatsTable_data_array[ ioctlData->Data[0]]);
            tblMap= (DWORD *) pDot3StatsTable;
            for (index = 0; index < sizeof(Dot3StatsTable_data); index++, tblMap++)
                ioctlData->Data[index] = *tblMap;

            success=TRUE;
        } else {
            SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
            SMSC_WARNING( "  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_GET_RMON_STAT_TBL:
        if( (privateData->LanInitialized) &&
            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
            DWORD index = 0;
            EtherStatsTable_data *pRmonStatsTable;
            DWORD *tblMap;

            mibUpdateMibPortCounters( privateData );

            // Get rmonetherStats data pointer  for the index.

            pRmonStatsTable = &(privateData->etherStatsTable_data_array[ ioctlData->Data[0]]);
            tblMap= (DWORD *) pRmonStatsTable;
            for (index = 0; index < sizeof(EtherStatsTable_data); index++, tblMap++)
                ioctlData->Data[index] = *tblMap;
            success=TRUE;
        } else {
            SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
            SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
    case COMMAND_GET_IEE8023_STAT_TBL:
                      if( (privateData->LanInitialized) &&
                                      ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                          DWORD index = 0;
                          IEEE802_3StatsTable_data *pMACStatsTable;
                          DWORD *tblMap;
                          mibUpdateMibPortCounters( privateData );

                          // Get rmonetherStats data pointer  for the index.

                          pMACStatsTable = &(privateData->MACStatsTable_data_array[ ioctlData->Data[0]]);
                          tblMap= (DWORD *) pMACStatsTable;
                          for (index = 0; index < sizeof(IEEE802_3StatsTable_data); index++, tblMap++)
                              ioctlData->Data[index] = *tblMap;

                          success=TRUE;
                                      } else {
                                          SMSC_WARNING( "MAC is Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                          SMSC_WARNING("  Use ifconfig to bring the interface UP");
                                      }
                                      break;

    case COMMAND_GET_BRIDGE_STAT_TBL:
               if( (privateData->LanInitialized) &&
                               ((ioctlData->Data[0]) < PHY_NUMBER) ) {
                   DWORD index = 0;
                   Bridge_Stats_data *pBridgeStatsTable;
                   DWORD *tblMap;
                   mibUpdateMibPortCounters( privateData );

                   // Get rmonetherStats data pointer  for the index.

                   pBridgeStatsTable = &(privateData->BridgeStatsTable_data_array[ ioctlData->Data[0]]);
                   tblMap= (DWORD *) pBridgeStatsTable;
                   for (index = 0; index < sizeof(Bridge_Stats_data); index++, tblMap++)
                       ioctlData->Data[index] = *tblMap;

                   success=TRUE;
                               } else {
                                   SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                                   SMSC_WARNING("  Use ifconfig to bring interface UP");
                               }
                               break;

    case COMMAND_GET_IF_TBL:
        if( (privateData->LanInitialized) &&
            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
            DWORD index = 0;
            IfTable_data *pIfTable_data;
            DWORD *tblMap;

            mibUpdateMibPortCounters( privateData );
        // Get rmonetherStats data pointer  for the index.
            pIfTable_data = &(privateData->IfTable_data_array[ioctlData->Data[0]]);
            tblMap= (DWORD *) pIfTable_data;
            for (index = 0; index < sizeof(IfTable_data); index++, tblMap++)
                ioctlData->Data[index] = *tblMap;

            success=TRUE;
        } else {
                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                SMSC_WARNING("  Use ifconfig to bring interface UP");
            }
        break;

    case COMMAND_GET_IF_XTBL:
        if( (privateData->LanInitialized) &&
            ((ioctlData->Data[0]) < PHY_NUMBER) ) {
            DWORD index = 0;
            IfXTable_Data *pIfXTable_Data;
            DWORD *tblMap;

            mibUpdateMibPortCounters( privateData );
            // Get rmonetherStats data pointer  for the index.
            pIfXTable_Data = &(privateData->IfXTable_Data_array[ioctlData->Data[0]]);
            tblMap= (DWORD *) pIfXTable_Data;
            for (index = 0; index < sizeof(IfXTable_Data); index++, tblMap++)
                ioctlData->Data[index] = *tblMap;

            success=TRUE;
        } else {
                SMSC_WARNING( "MAC Not Initialized or Invalid port index (%lu)",ioctlData->Data[0]);
                SMSC_WARNING("  Use ifconfig to bring interface UP");
        }
        break;
#endif

    default:
        SMSC_TRACE("Unknown Command %lu ( see ioctl.h)\n", ioctlData->dwCommand);
        break;//make lint happy
    }

DONE:
    if((success)&&(ioctlData!=NULL)) {
        ioctlData->dwSignature=SMSC931X_DRIVER_SIGNATURE;
        return 0;
    }
    //	SMSC_TRACE("<--Smsc9312_do_ioctl");
    return -1;
}

/// end of int Smsc9312_do_ioctl

//returns time1-time2;
TIME_SPAN Gpt_FreeRunCompare(DWORD time1,DWORD time2)
{
    return ((TIME_SPAN)(time1-time2));
}


void Gpt_ScheduleInterrupt(PPRIVATE_DATA privateData,TIME_SPAN timeSpan)
{
    DWORD timerValue=0;
    if(timeSpan<0) timeSpan=0;
    timerValue=(DWORD)timeSpan;
    if((timerValue%2500)>=1250) {
        timerValue=(timerValue/2500)+1;
    } else {
        timerValue=(timerValue/2500);
    }
    if(timerValue>0x0000FFFFUL) {
        timerValue=0x0000FFFF;
    }
    LanSetDirectRegDW(GPT_CFG,(timerValue|GPT_CFG_TIMER_EN_));
    LanSetDirectRegDW(INT_STS,INT_STS_GPT_INT_);
}

void Gpt_CancelInterrupt(PPRIVATE_DATA privateData)
{
    LanSetDirectRegDW(GPT_CFG,0UL);
    LanSetDirectRegDW(INT_STS,INT_STS_GPT_INT_);
}

void Gpt_ScheduleCallBack(
                PPRIVATE_DATA privateData,
                void (*callBackFunction)(PPRIVATE_DATA privateData),
                DWORD callBackTime
                )
{
    DWORD slot_index=GPT_SCHEDULE_DEPTH;
    BOOLEAN result=FALSE;
    if((callBackFunction!=NULL)&&(callBackTime!=0)) {
        DWORD dwIntFlags=0;
        SMSC_ASSERT(privateData!=NULL);
        spin_lock_irqsave(&(privateData->GpTimerLock),dwIntFlags);
        {
            DWORD index=0;
            DWORD currentTime=LanGetDirectRegDW(FREE_RUN);
            TIME_SPAN nextCallTime=MAX_TIME_SPAN;
            TIME_SPAN timeSpan=MAX_TIME_SPAN;
            BOOLEAN rescheduleRequired=FALSE;
            for(index=0;index<GPT_SCHEDULE_DEPTH;index++) {
                if(privateData->GptFunction[index]==NULL) {
                    if(!result) {
                        result=TRUE;
                        //lint -save
                        //lint -e611 //suspicious cast
                        privateData->GptFunction[index]=(void *)callBackFunction;
                        //lint -restore_
                        privateData->GptCallTime[index]=currentTime+(2500*callBackTime);
                        timeSpan=Gpt_FreeRunCompare(privateData->GptCallTime[index],currentTime);
                        if(nextCallTime>timeSpan) {
                            nextCallTime=timeSpan;
                            rescheduleRequired=TRUE;
                            slot_index = index;
                        }
                    }
                } else {
                    timeSpan=Gpt_FreeRunCompare(privateData->GptCallTime[index],currentTime);
                    if(nextCallTime>=timeSpan) {
                        nextCallTime=timeSpan;
                        rescheduleRequired=FALSE;
                    }
                }
            }
            if(rescheduleRequired) {
                privateData->Gpt_scheduled_slot_index = slot_index;
                Gpt_ScheduleInterrupt(privateData,nextCallTime);
            }
        }
        spin_unlock_irqrestore(&(privateData->GpTimerLock),dwIntFlags);
    }
    if(!result) {
        SMSC_WARNING("Gpt_ScheduleCallBack: Failed");
    }
}

void Gpt_CancelCallBack(
                        PPRIVATE_DATA privateData,
                        void (*callBackFunction)(PPRIVATE_DATA privateData)
                       )
{
    BOOLEAN result=FALSE;
    if(callBackFunction!=NULL) {
        DWORD dwIntFlags=0;
        SMSC_ASSERT(privateData!=NULL);
        spin_lock_irqsave(&(privateData->GpTimerLock),dwIntFlags);
        {
            DWORD index=0;
            DWORD currentTime=LanGetDirectRegDW(FREE_RUN);
            TIME_SPAN nextCallTime=MAX_TIME_SPAN;
            TIME_SPAN timeSpan=MAX_TIME_SPAN;
            BOOLEAN rescheduleRequired=FALSE;
            for(index=0;index<GPT_SCHEDULE_DEPTH;index++) {
                if(privateData->GptFunction[index]==callBackFunction) {
                    result=TRUE;
                    //lint -save
                    //lint -e611 //suspicious cast
                    privateData->GptFunction[index]=(void *)NULL;
                    // cancelled time will not need a
                    // re-scheduled

                    // re-scheduled is done at other
                    // non-null slots
                }
                else if(privateData->GptFunction[index]!=NULL) {
                    timeSpan=Gpt_FreeRunCompare(privateData->GptCallTime[index],currentTime);
                    // if this scheduled time is earlier
                    // than current scheduled time
                    // AND not a duplicated one
                    if(nextCallTime>=timeSpan && privateData->Gpt_scheduled_slot_index != index) {
                        nextCallTime=timeSpan;
                        rescheduleRequired=TRUE;
                        privateData->Gpt_scheduled_slot_index = index;
                    }
                }
            }
            if(rescheduleRequired) {
                Gpt_ScheduleInterrupt(privateData,nextCallTime);
            }
            else if (privateData->Gpt_scheduled_slot_index==GPT_SCHEDULE_DEPTH) {
                Gpt_CancelInterrupt(privateData);
            }
        }
        spin_unlock_irqrestore(&(privateData->GpTimerLock),dwIntFlags);
    }
    if(!result) {
        SMSC_WARNING("Gpt_CancelCallBack: Failed");
    }
}

BOOLEAN Gpt_HandleInterrupt( PPRIVATE_DATA privateData,
                             DWORD dwIntSts
                           )
{
    SMSC_ASSERT(privateData!=NULL);
    if(dwIntSts&INT_STS_GPT_INT_)
    {
        DWORD dwIntFlags=0;
        LanSetDirectRegDW(INT_STS,INT_STS_GPT_INT_);
        spin_lock_irqsave(&(privateData->GpTimerLock),dwIntFlags);
        {
            DWORD index=0;
            DWORD currentTime=LanGetDirectRegDW(FREE_RUN);
            TIME_SPAN timeSpan=MAX_TIME_SPAN;
            TIME_SPAN nextCallTime=MAX_TIME_SPAN;
            BOOLEAN rescheduleRequired=FALSE;
            for(index=0;index<GPT_SCHEDULE_DEPTH;index++) {
                if(privateData->GptFunction[index]!=NULL) {
                    timeSpan=Gpt_FreeRunCompare(privateData->GptCallTime[index],currentTime);
                    if(timeSpan<1250) {
                        void (*callBackFunction)(PPRIVATE_DATA privateData);
                        callBackFunction=privateData->GptFunction[index];
                        privateData->GptFunction[index]=NULL;
                        spin_unlock_irqrestore(&(privateData->GpTimerLock),dwIntFlags);
                        privateData->Gpt_scheduled_slot_index = GPT_SCHEDULE_DEPTH;
                        callBackFunction(privateData);
                        spin_lock_irqsave(&(privateData->GpTimerLock),dwIntFlags);
                    }
                }
            }
            for(index=0;index<GPT_SCHEDULE_DEPTH;index++) {
                if(privateData->GptFunction[index]!=NULL) {
                    rescheduleRequired=TRUE;
                    timeSpan=Gpt_FreeRunCompare(privateData->GptCallTime[index],currentTime);
                    if(nextCallTime>timeSpan) {
                        nextCallTime=timeSpan;
                        privateData->Gpt_scheduled_slot_index = index;
                    }
                }
            }
            if(rescheduleRequired) {
                Gpt_ScheduleInterrupt(privateData,nextCallTime);
            }
        }
        spin_unlock_irqrestore(&(privateData->GpTimerLock),dwIntFlags);
        return TRUE;
    }
    return FALSE;
}

void GptCB_RxCompleteMulticast(PPRIVATE_DATA privateData)
{
    Rx_CompleteMulticastUpdate (privateData);
}

void GptCB_RestartBurst(PPRIVATE_DATA privateData)
{
    if(privateData->RxFlowControlActive) {
        privateData->RxFlowBurstActive=TRUE;
        if(privateData->RxFlowBurstWorkLoad>privateData->RxFlowBurstMaxWorkLoad) {
            privateData->RxFlowBurstWorkLoad-=privateData->RxFlowBurstMaxWorkLoad;
        } else {
            privateData->RxFlowBurstWorkLoad=0;
        }
        Gpt_ScheduleCallBack(privateData,GptCB_RestartBurst,
                privateData->RxFlowParameters.BurstPeriod);
    }
    Lan_EnableInterrupt(privateData,privateData->RxInterrupts);
}

void GptCB_MeasureRxThroughput(PPRIVATE_DATA privateData)
{
    if(privateData->RxFlowMeasuredMaxThroughput<privateData->RxFlowCurrentThroughput) {
        privateData->RxFlowMeasuredMaxThroughput=privateData->RxFlowCurrentThroughput;
    }
    if(privateData->RxFlowMeasuredMaxPacketCount<privateData->RxFlowCurrentPacketCount) {
        privateData->RxFlowMeasuredMaxPacketCount=privateData->RxFlowCurrentPacketCount;
    }
    if(privateData->RxFlowCurrentThroughput!=0) {
        if(privateData->RxFlowMaxWorkLoad!=0) {
            if(!(privateData->RxFlowControlActive)) {
                DWORD activationLevel=
                    (privateData->RxFlowMaxWorkLoad*(100+RX_FLOW_ACTIVATION))/100;
                if(privateData->RxFlowCurrentWorkLoad>=activationLevel) {
                    privateData->RxFlowControlActive=TRUE;
                    privateData->RxFlowBurstActive=TRUE;
                    privateData->RxFlowBurstWorkLoad=0;
                    Gpt_ScheduleCallBack(privateData,GptCB_RestartBurst,
                            privateData->RxFlowParameters.BurstPeriod);
                    //SET_GPIO(GP_TX);
                }
            } else {
                DWORD deactivationLevel=
                    (privateData->RxFlowMaxWorkLoad*(100-RX_FLOW_DEACTIVATION))/100;
                if(privateData->RxFlowCurrentWorkLoad<=deactivationLevel) {
                    privateData->RxFlowControlActive=FALSE;
                    //CLEAR_GPIO(GP_TX);
                }
            }
        }
        privateData->RxFlowCurrentThroughput=0;
        privateData->RxFlowCurrentPacketCount=0;
        privateData->RxFlowCurrentWorkLoad=0;
        Gpt_ScheduleCallBack(privateData,GptCB_MeasureRxThroughput,1000);
    } else {
        if(privateData->RxFlowMaxWorkLoad!=0) {
            if(privateData->RxFlowControlActive) {
                privateData->RxFlowControlActive=FALSE;
                //CLEAR_GPIO(GP_TX);
            }
        }
        privateData->MeasuringRxThroughput=FALSE;
    }
}

irqreturn_t Smsc9312_ISR(int Irq, void *dev_id, struct pt_regs *regs)
{
    DWORD dwIntCfg=0;
    DWORD dwIntSts=0;
    DWORD dwIntEn=0;
    DWORD dwIntBits=0;
    PPRIVATE_DATA privateData=(PPRIVATE_DATA)dev_id;
    BOOLEAN serviced=FALSE;

    regs=regs;//make lint happy
    Irq=Irq;//make lint happy

#ifdef TEST_DMA
    nb_interrupt ++;
#endif
    if(privateData==NULL) {
        SMSC_WARNING("Smsc9312_ISR(privateData==NULL)");
        goto DONE;
    }
    if(privateData->dwLanBase==0) {
        SMSC_WARNING("Smsc9312_ISR(dwLanBase==0)");
        goto DONE;
    }
    //SET_GPIO(GP_ISR);
    dwIntCfg=LanGetDirectRegDW(INT_CFG);
    if((dwIntCfg&0x00001100)!=0x00001100) {
        SMSC_TRACE("In ISR, not my interrupt, dwIntCfg=0x%08lX",
            dwIntCfg);
        goto ALMOST_DONE;
    }

    {
        DWORD reservedBits;
        if(OLD_REGISTERS(privateData)) {
            reservedBits=0x00FFEEEEUL;
        } else {
            reservedBits=0x00FFCEEEUL;
        }
        if(dwIntCfg&reservedBits) {
            SMSC_WARNING("In ISR, reserved bits are high.");
            //this could mean surprise removal
            goto ALMOST_DONE;
        }
    }

    dwIntSts=LanGetDirectRegDW(INT_STS);
    dwIntEn=LanGetDirectRegDW(INT_EN);
    dwIntBits=dwIntSts&dwIntEn;
    privateData->LastIntStatus3=privateData->LastIntStatus2;
    privateData->LastIntStatus2=privateData->LastIntStatus1;
    privateData->LastIntStatus1=dwIntBits;
    if(Lan_HandleSoftwareInterrupt(privateData,dwIntBits)) {
        serviced=TRUE;
    }
    if(Gpt_HandleInterrupt(privateData,dwIntBits)) {
        serviced=TRUE;
    }
    if(Tx_HandleInterrupt(privateData,dwIntBits)) {
        serviced=TRUE;
    }
    if(RxStop_HandleInterrupt(privateData,dwIntBits)) {
        serviced=TRUE;
    }
    if(Rx_HandleInterrupt(privateData,dwIntBits)) {
        serviced=TRUE;
    }

    if(!serviced) {
        SMSC_WARNING("unserviced interrupt dwIntCfg=0x%08lX,dwIntSts=0x%08lX,dwIntEn=0x%08lX,dwIntBits=0x%08lX",
            dwIntCfg,dwIntSts,dwIntEn,dwIntBits);
    }

    ALMOST_DONE:
    //CLEAR_GPIO(GP_ISR);
    DONE:

    return IRQ_RETVAL(serviced);
}

BOOLEAN Phy_Reset(PPRIVATE_DATA privateData,DWORD dwPhyIndex,VL_KEY keyCode)
{
    BOOLEAN result=FALSE;
    WORD wTemp=0;
    DWORD dwLoopCount=100000;
    SMSC_TRACE("Performing PHY BCR Reset");
        Phy_SetRegW(privateData,dwPhyIndex,PHY_BCR,PHY_BCR_RESET_,keyCode);
    do {
        udelay(10);
                wTemp=Phy_GetRegW(privateData,dwPhyIndex,PHY_BCR,keyCode);
        dwLoopCount--;
    } while((dwLoopCount>0)&&(wTemp&PHY_BCR_RESET_));
    if(wTemp&PHY_BCR_RESET_) {
        SMSC_WARNING("Phy Reset failed to complete.");
        goto DONE;
    }
    //extra delay required because the phy may not be completed with its reset
    //  when PHY_BCR_RESET_ is cleared.
    //  They say 256 uS is enough delay but I'm using 500 here to be safe
    udelay(500);
    result=TRUE;
    DONE:
    return result;
}

void Phy_SetLink(PPRIVATE_DATA privateData,
            DWORD dwPhyIndex,
            DWORD dwLinkRequest)
{
    DWORD dwIntFlags=0;
    VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
    if(dwLinkRequest&LINK_AUTO_NEGOTIATE) {
        WORD wTemp;
                wTemp=Phy_GetRegW(privateData, dwPhyIndex,
            PHY_ANEG_ADV,keyCode);
        wTemp&=~PHY_ANEG_ADV_PAUSE_;
        if(dwLinkRequest&LINK_ASYMMETRIC_PAUSE) {
            wTemp|=PHY_ANEG_ADV_ASYMP_;
        }
        if(dwLinkRequest&LINK_SYMMETRIC_PAUSE) {
            wTemp|=PHY_ANEG_ADV_SYMP_;
        }
        wTemp&=~PHY_ANEG_ADV_SPEED_;
        if(dwLinkRequest&LINK_SPEED_10HD) {
            wTemp|=PHY_ANEG_ADV_10H_;
        }
        if(dwLinkRequest&LINK_SPEED_10FD) {
            wTemp|=PHY_ANEG_ADV_10F_;
        }
        if(dwLinkRequest&LINK_SPEED_100HD) {
            wTemp|=PHY_ANEG_ADV_100H_;
        }
        if(dwLinkRequest&LINK_SPEED_100FD) {
            wTemp|=PHY_ANEG_ADV_100F_;
        }

         Phy_SetRegW(privateData,dwPhyIndex,PHY_ANEG_ADV,wTemp,keyCode);

        // begin to establish link
        // mobbad privateData->dwRemoteFaultCount=0;
           Phy_SetRegW(privateData,dwPhyIndex,
                PHY_BCR,
                PHY_BCR_AUTO_NEG_ENABLE_|
                PHY_BCR_RESTART_AUTO_NEG_,
                keyCode);
    } else {
        DWORD wTemp=0;
        if(dwLinkRequest&(LINK_SPEED_100FD)) {
            dwLinkRequest=LINK_SPEED_100FD;
        } else if(dwLinkRequest&(LINK_SPEED_100HD)) {
            dwLinkRequest=LINK_SPEED_100HD;
        } else if(dwLinkRequest&(LINK_SPEED_10FD)) {
            dwLinkRequest=LINK_SPEED_10FD;
        } else if(dwLinkRequest&(LINK_SPEED_10HD)) {
            dwLinkRequest=LINK_SPEED_10HD;
        }
        if(dwLinkRequest&(LINK_SPEED_10FD|LINK_SPEED_100FD)) {
            wTemp|=PHY_BCR_DUPLEX_MODE_;
        }
        if(dwLinkRequest&(LINK_SPEED_100HD|LINK_SPEED_100FD)) {
            wTemp|=PHY_BCR_SPEED_SELECT_;
        }
                Phy_SetRegW(privateData,dwPhyIndex,PHY_BCR,wTemp,keyCode);
    }
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);

    {
        //DWORD wTemp=Phy_GetRegW(privateData, dwPhyIndex,PHY_BCR,keyCode);
        SMSC_TRACE("--> Phy Index= %lu, PHY_BCR = %u\n", dwPhyIndex, Phy_GetRegW(privateData, dwPhyIndex,PHY_BCR,keyCode));
    }
}

BOOLEAN Phy_Initialize(
                PPRIVATE_DATA privateData,
                DWORD dwPhyIndex,
                DWORD dwPhyAddr,
                DWORD dwLinkRequest
                )
{
    BOOLEAN result=FALSE;
    DWORD dwTemp=0;
    WORD wTemp=0;
    DWORD dwLoopCount=0;

    SMSC_TRACE("-->Phy_Initialize, dwPhyAddr = %lu", dwPhyAddr);
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    SMSC_ASSERT(dwLinkRequest<=0x7FUL);

    if(dwPhyAddr!=0xFFFFFFFFUL) {
        switch(privateData->dwIdRev&0xFFFF0000) {
            case 0x117A0000UL:
            case 0x115A0000UL:
                goto EXTERNAL_PHY_SUPPORTED;
            case 0x01170000UL:
            case 0x01150000UL:
                if(privateData->dwIdRev&0x0000FFFF) {
                    DWORD dwHwCfg=0;
EXTERNAL_PHY_SUPPORTED:
                    dwHwCfg=LanGetDirectRegDW(HW_CFG);
                    SMSC_TRACE(" is External PHY (%d) dwHwCfg = %lx",(int) (dwHwCfg & HW_CFG_EXT_PHY_DET_),dwHwCfg );
                    if(dwHwCfg & HW_CFG_EXT_PHY_DET_) {
                            //External phy is requested, supported, and detected
                            //Attempt to switch
                            //NOTE: Assuming Rx and Tx are stopped
                            //  because Phy_Initialize is called before
                            //  Rx_Initialize and Tx_Initialize

                        WORD wPhyId1=0;
                        WORD wPhyId2=0;
                        SMSC_TRACE("!!!!1!! External PHY Detected !!!!!!")

                        //Disable phy clocks to the mac
                        dwHwCfg&= (~HW_CFG_PHY_CLK_SEL_);
                        dwHwCfg|= HW_CFG_PHY_CLK_SEL_DISABLED_;
                        LanSetDirectRegDW(HW_CFG,dwHwCfg);
                        udelay(10);//wait for clocks to acutally stop

                        dwHwCfg|=HW_CFG_EXT_PHY_EN_;
                        LanSetDirectRegDW(HW_CFG,dwHwCfg);

                        dwHwCfg&= (~HW_CFG_PHY_CLK_SEL_);
                        dwHwCfg|= HW_CFG_PHY_CLK_SEL_EXT_PHY_;
                        LanSetDirectRegDW(HW_CFG,dwHwCfg);
                        udelay(10);//wait for clocks to actually start

                        dwHwCfg|=HW_CFG_SMI_SEL_;
                        LanSetDirectRegDW(HW_CFG,dwHwCfg);

                        {
                            DWORD dwIntFlags=0;
                            VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
                            if(dwPhyAddr<=31) {
                                            //only check the phy address specified
                                privateData->dwPhyAddress=dwPhyAddr;
                                wPhyId1=Phy_GetRegW(privateData,dwPhyIndex,PHY_ID_1,keyCode);
                                wPhyId2=Phy_GetRegW(privateData,dwPhyIndex,PHY_ID_2,keyCode);
                            } else {
                                            //auto detect phy
                                DWORD address=0;
                                for(address=0;address<=31;address++) {
                                    privateData->dwPhyAddress=address;
                                    wPhyId1=Phy_GetRegW(privateData,dwPhyIndex,PHY_ID_1,keyCode);
                                    wPhyId2=Phy_GetRegW(privateData,dwPhyIndex,PHY_ID_2,keyCode);
                                    if((wPhyId1!=0xFFFFU)||(wPhyId2!=0xFFFFU)) {
                                        SMSC_TRACE("!!!!!! Detected Phy at address = 0x%02lX = %ld",
                                                address,address);
                                        break;
                                    }
                                }
                                if(address>=32) {
                                    SMSC_WARNING("Failed to auto detect external phy");
                                }
                                else
                                {
                                    phy_addr = address;
                                }
                            }
                            Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
                        }

                        if((wPhyId1==0xFFFFU)&&(wPhyId2==0xFFFFU)) {
                            SMSC_WARNING("External Phy is not accessable");
                            SMSC_WARNING("  using internal phy instead");
                                    //revert back to interal phy settings.

                                    //Disable phy clocks to the mac
                            dwHwCfg&= (~HW_CFG_PHY_CLK_SEL_);
                            dwHwCfg|= HW_CFG_PHY_CLK_SEL_DISABLED_;
                            LanSetDirectRegDW(HW_CFG,dwHwCfg);
                            udelay(10);//wait for clocks to actually stop

                            dwHwCfg&=(~HW_CFG_EXT_PHY_EN_);
                            LanSetDirectRegDW(HW_CFG,dwHwCfg);

                            dwHwCfg&= (~HW_CFG_PHY_CLK_SEL_);
                            dwHwCfg|= HW_CFG_PHY_CLK_SEL_INT_PHY_;
                            LanSetDirectRegDW(HW_CFG,dwHwCfg);
                            udelay(10);//wait for clocks to actually start

                            dwHwCfg&=(~HW_CFG_SMI_SEL_);
                            LanSetDirectRegDW(HW_CFG,dwHwCfg);
                            goto USE_INTERNAL_PHY;
                        } else {

                            SMSC_TRACE("!!!!!!!!Successfully switched to external phy!!!!!!");

                        }

                    } else {
                        SMSC_WARNING("No External Phy Detected");
                        SMSC_WARNING("  using internal phy instead");
                        goto USE_INTERNAL_PHY;
                    }
                } else {
                    SMSC_WARNING("External Phy is not supported");
                    SMSC_WARNING("  using internal phy instead");
                    goto USE_INTERNAL_PHY;
                };break;
            default:
                SMSC_WARNING("External Phy is not supported");
                SMSC_WARNING("  using internal phy instead");
                goto USE_INTERNAL_PHY;
        }
    } else {
USE_INTERNAL_PHY:

        privateData->dwPhyAddress= phy_addr;
    }

    {
        DWORD dwIntFlags=0;
        VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
        dwTemp=Phy_GetRegW(privateData,dwPhyIndex,PHY_ID_2,keyCode);
        privateData->bPhyRev=((BYTE)(dwTemp&(0x0FUL)));
        privateData->bPhyModel=((BYTE)((dwTemp>>4)&(0x3FUL)));
        privateData->dwPhyId=((dwTemp&(0xFC00UL))<<8);
        dwTemp=Phy_GetRegW(privateData,dwPhyIndex, PHY_ID_1,keyCode);
        privateData->dwPhyId|=((dwTemp&(0x0000FFFFUL))<<2);

        privateData->dwLinkSpeed=LINK_OFF;
        privateData->dwLinkSettings=LINK_OFF;

#ifdef FULLY_MANAGED
        privateData->SwitchPort1LinkSpeed = LINK_OFF;
        privateData->SwitchPort2LinkSpeed = LINK_OFF;
#endif
        //reset the PHY
        Phy_SetRegW(privateData,dwPhyIndex,PHY_BCR,PHY_BCR_RESET_,keyCode);
        dwLoopCount=100000;
        do {

            udelay(10);
            wTemp=Phy_GetRegW(privateData,dwPhyIndex,PHY_BCR,keyCode);
            dwLoopCount--;
        } while((dwLoopCount>0) && (wTemp&PHY_BCR_RESET_));
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    }

    if(wTemp&PHY_BCR_RESET_) {
        SMSC_WARNING("PHY reset failed to complete.");
        goto DONE;
    }


    Phy_SetLink(privateData,dwPhyIndex, dwLinkRequest);

    init_timer(&(privateData->LinkPollingTimer));
    privateData->LinkPollingTimer.function=Phy_CheckLink;
    privateData->LinkPollingTimer.data=(unsigned long)privateData;
    privateData->LinkPollingTimer.expires=jiffies+2*HZ;
    add_timer(&(privateData->LinkPollingTimer));

    result=TRUE;
DONE:
    SMSC_TRACE("<--Phy_Initialize, result=%s",result?"TRUE":"FALSE");
    return result;
}


WORD PhyGetPmiRegW(
    PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
    DWORD dwRegIndex,
    VL_KEY keyCode)
{
WORD result=0xFFFFU;

SMSC_ASSERT(privateData!=NULL);
SMSC_ASSERT(privateData->LanInitialized==TRUE);
SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));

SMSC_TRACE("PhyIndex=%lu, Register Index=%lu\n", dwPhyIndex,dwRegIndex);
// Read PhY register using PMI access
result =  PmiGetRegW(privateData, dwPhyIndex, dwRegIndex);

return result;
}

WORD PhyGetDirectRegW(
    PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
DWORD dwRegIndex,
VL_KEY keyCode)
{
DWORD dwAddr=0;
int i=0;
WORD result=0xFFFFU;

SMSC_ASSERT(privateData!=NULL);
SMSC_ASSERT(privateData->LanInitialized==TRUE);
SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));

// confirm MII not busy
if ((Mac_GetRegDW(privateData, MII_ACC,keyCode) & MII_ACC_MII_BUSY_) != 0UL)
{
    SMSC_WARNING("MII is busy in Phy_GetRegW???");
    result=0;
    goto DONE;
}

// set the address, index & direction (read from PHY)
dwAddr = ((dwPhyIndex &0x1FUL)<<11) | ((dwRegIndex & 0x1FUL)<<6);
Mac_SetRegDW(privateData, MII_ACC, dwAddr,keyCode);

// wait for read to complete w/ timeout
for(i=0;i<100;i++) {
    // see if MII is finished yet
    if ((Mac_GetRegDW(privateData, MII_ACC,keyCode) & MII_ACC_MII_BUSY_) == 0UL)
    {
        // get the read data from the MAC & return i
        result=((WORD)Mac_GetRegDW(privateData, MII_DATA,keyCode));
        goto DONE;
    }
}
SMSC_WARNING("timeout waiting for MII write to finish");

DONE:
    return result;
}


WORD Phy_GetRegW(
    PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
    DWORD dwRegIndex,
    VL_KEY keyCode)
{
    WORD retValue = 0xFFFF;
	 switch(privateData->access_mode)
	 {
		 case SMI_ACCESS:
			 retValue = PhyGetDirectRegW(
	                privateData,
	                dwPhyIndex,
	                dwRegIndex,
	                keyCode);
		 break;
		 case HBI_ACCESS:
			 retValue = PhyGetDirectRegW(
	                privateData,
	                dwPhyIndex,
	                dwRegIndex,
	                keyCode);
	    break;
		 case SPI_ACCESS:
		 {
		    if (  dwPhyIndex == VirtualPhyIndex) {
		        retValue = PhyGetDirectRegW(
		                    privateData,
		                    dwPhyIndex,
		                    dwRegIndex,
		                    keyCode);
		    }
		    else
		    {
		        retValue = PhyGetPmiRegW(
		                    privateData,
		                    dwPhyIndex,
		                    dwRegIndex,
		                    keyCode);
		    }
		 }
		 break;
		 case I2C_ACCESS:
		 {
		    if (  dwPhyIndex == VirtualPhyIndex) {
		        retValue = PhyGetDirectRegW(
		                    privateData,
		                    dwPhyIndex,
		                    dwRegIndex,
		                    keyCode);
		    }
		    else
		    {
		        retValue = PhyGetPmiRegW(
		                    privateData,
		                    dwPhyIndex,
		                    dwRegIndex,
		                    keyCode);
		    }
		 }
		 break;
		 default:
		 {
		    // unmanaged Mode
		    retValue = PhyGetDirectRegW(
		                    privateData,
		                    dwPhyIndex,
		                    dwRegIndex,
		                    keyCode);
		 }
		 break;
	 }
    return retValue;
}


void PhySetPmiRegW(
    PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
    DWORD dwRegIndex,
    WORD wVal,
    VL_KEY keyCode)
{

    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);

    SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));

    SMSC_TRACE("PhyIndex=%lu, Register Index=%lu, New Value=0x%08x\n", dwPhyIndex,dwRegIndex, wVal);
    //Call PMI API
    PmiSetRegW(privateData,dwPhyIndex, dwRegIndex, wVal);
    return;

}

void PhySetDirectRegW(
    PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
DWORD dwRegIndex,
WORD wVal,
VL_KEY keyCode)
{
    DWORD dwAddr=0;
    int i=0;

    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);

    SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));

    if(dwRegIndex==0) {
        if((wVal&0x1200)==0x1200) {
            privateData->wLastADVatRestart=privateData->wLastADV;
        }
    }
    if(dwRegIndex==4) {
        privateData->wLastADV=wVal;
    }

    // confirm MII not busy
    if ((Mac_GetRegDW(privateData, MII_ACC,keyCode) & MII_ACC_MII_BUSY_) != 0UL)
    {
        SMSC_WARNING("MII is busy in Phy_SetRegW???");
        goto DONE;
    }

    // put the data to write in the MAC
    Mac_SetRegDW(privateData, MII_DATA, (DWORD)wVal,keyCode);

    // set the address, index & direction (write to PHY)
    dwAddr = ((dwPhyIndex & 0x1FUL)<<11) | ((dwRegIndex & 0x1FUL)<<6) | MII_ACC_MII_WRITE_;
    Mac_SetRegDW(privateData, MII_ACC, dwAddr,keyCode);

    // wait for write to complete w/ timeout
    for(i=0;i<100;i++) {
        // see if MII is finished yet
        if ((Mac_GetRegDW(privateData, MII_ACC,keyCode) & MII_ACC_MII_BUSY_) == 0UL)
        {
            goto DONE;
        }
    }
    SMSC_WARNING("timeout waiting for MII write to finish");
    DONE:
#ifdef DEBUG_PHY_SETTING
    {
    WORD regVal= 0xffff;
    regVal = Phy_GetRegW(
            privateData,
    dwPhyIndex,
    dwRegIndex,
    keyCode);
    printk("Function:%s, PhyIndex= %lu, Phyreg=%lu, value =%u, Value Read Back=%u\n",
        __func__, dwPhyIndex,dwRegIndex,wVal, regVal);
    }
#endif
    return;
}


void Phy_SetRegW(
PPRIVATE_DATA privateData,
    DWORD dwPhyIndex,
    DWORD dwRegIndex,
    WORD wVal,
    VL_KEY keyCode)
{
	 switch(privateData->access_mode)
	 {
		 case SMI_ACCESS:
			 PhySetDirectRegW(
					 privateData,
					 dwPhyIndex,
					 dwRegIndex,
					 wVal,
					 keyCode);
		break;
		 case HBI_ACCESS:
			 PhySetDirectRegW(
					 privateData,
					 dwPhyIndex,
					 dwRegIndex,
					 wVal,
					 keyCode);
		break;
		case SPI_ACCESS:
			 if (  dwPhyIndex == VirtualPhyIndex) {
				 PhySetDirectRegW(
						 privateData,
						 dwPhyIndex,
						 dwRegIndex,
						 wVal,
						 keyCode);
			 }
			 else
			 {
					 PhySetPmiRegW(
	                privateData,
	                dwPhyIndex,
	                dwRegIndex,
	                wVal,
	                keyCode);
			 }
		 break;
		 case I2C_ACCESS:
			 if (  dwPhyIndex == VirtualPhyIndex) {
	        PhySetDirectRegW(
	        		privateData,
	            dwPhyIndex,
	            dwRegIndex,
	            wVal,
	            keyCode);
	    }
		else
		{
	        PhySetPmiRegW(
	        privateData,
	        dwPhyIndex,
	        dwRegIndex,
	        wVal,
	        keyCode);
		}
		break;
		 default:
    // unmanaged Mode
	    PhySetDirectRegW(
	            privateData,
	            dwPhyIndex,
	            dwRegIndex,
	            wVal,
	            keyCode);
	    break;
	 }
    return;
}

void Phy_UpdateLinkMode(PPRIVATE_DATA privateData)
{
    DWORD dwOldLinkSpeed=privateData->dwLinkSpeed;
    DWORD dwIntFlags=0;
    VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

    Phy_GetLinkMode(privateData,privateData->dwPhyAddress, keyCode);
//    printk("inside Phy_UpdateLinkMode\n");
//    printk("privateData->dwPhyAddresss=%d\n", privateData->dwPhyAddress);
    if(dwOldLinkSpeed != (privateData->dwLinkSpeed)) {
        if(privateData->dwLinkSpeed !=LINK_OFF) {
            DWORD dwRegVal=0;
            linkSpeed = privateData->dwLinkSpeed;
            switch(privateData->dwLinkSpeed) {
            case LINK_SPEED_10HD:
                SMSC_TRACE("Link is now UP at 10Mbps HD");
                break;
            case LINK_SPEED_10FD:
                SMSC_TRACE("Link is now UP at 10Mbps FD");
                break;
            case LINK_SPEED_100HD:
                SMSC_TRACE("Link is now UP at 100Mbps HD");
                break;
            case LINK_SPEED_100FD:
                SMSC_TRACE("Link is now UP at 100Mbps FD");
                break;
            default:
                SMSC_WARNING("Link is now UP at Unknown Link Speed, dwLinkSpeed=0x%08lX",
                    privateData->dwLinkSpeed);
                break;
            }

            dwRegVal=Mac_GetRegDW(privateData,MAC_CR,keyCode);
            dwRegVal&=~(MAC_CR_FDPX_|MAC_CR_RCVOWN_);
            switch(privateData->dwLinkSpeed) {
            case LINK_SPEED_10HD:
            case LINK_SPEED_100HD:
                dwRegVal|=MAC_CR_RCVOWN_;
                break;
            case LINK_SPEED_10FD:
            case LINK_SPEED_100FD:
                dwRegVal|=MAC_CR_FDPX_;
                break;
            default:break;//make lint happy
            }

            Mac_SetRegDW(privateData,
                MAC_CR,dwRegVal,keyCode);

            if(privateData->dwLinkSettings&LINK_AUTO_NEGOTIATE) {
                WORD linkPartner=0;
                WORD localLink=0;
                                localLink=Phy_GetRegW(privateData,privateData->dwPhyAddress, 4,keyCode);
                                linkPartner=Phy_GetRegW(privateData,privateData->dwPhyAddress,5,keyCode);
                switch(privateData->dwLinkSpeed) {
                case LINK_SPEED_10FD:
                case LINK_SPEED_100FD:
                    if(((localLink&linkPartner)&((WORD)0x0400U)) != ((WORD)0U)) {
                        //Enable PAUSE receive and transmit
                        Mac_SetRegDW(privateData,FLOW,0xFFFF0002UL,keyCode);
                        Lan_SetBitsDW(AFC_CFG,(afc_cfg&0x0000000FUL));
                    } else if(((localLink&((WORD)0x0C00U))==((WORD)0x0C00U)) &&
                            ((linkPartner&((WORD)0x0C00U))==((WORD)0x0800U)))
                    {
                        //Enable PAUSE receive, disable PAUSE transmit
                        Mac_SetRegDW(privateData,FLOW,0xFFFF0002UL,keyCode);
                        Lan_ClrBitsDW(AFC_CFG,0x0000000FUL);
                    } else {
                        //Disable PAUSE receive and transmit
                        Mac_SetRegDW(privateData,FLOW,0UL,keyCode);
                        Lan_ClrBitsDW(AFC_CFG,0x0000000FUL);
                    };break;
                case LINK_SPEED_10HD:
                case LINK_SPEED_100HD:
                    Mac_SetRegDW(privateData,FLOW,0UL,keyCode);
                    Lan_SetBitsDW(AFC_CFG,0x0000000FUL);
                    break;
                default:break;//make lint happy
                }
                SMSC_TRACE("LAN9118: %s,%s,%s,%s,%s,%s",
                    (localLink&PHY_ANEG_ADV_ASYMP_)?"ASYMP":"     ",
                    (localLink&PHY_ANEG_ADV_SYMP_)?"SYMP ":"     ",
                    (localLink&PHY_ANEG_ADV_100F_)?"100FD":"     ",
                    (localLink&PHY_ANEG_ADV_100H_)?"100HD":"     ",
                    (localLink&PHY_ANEG_ADV_10F_)?"10FD ":"     ",
                    (localLink&PHY_ANEG_ADV_10H_)?"10HD ":"     ");

                SMSC_TRACE("Partner: %s,%s,%s,%s,%s,%s",
                    (linkPartner&PHY_ANEG_LPA_ASYMP_)?"ASYMP":"     ",
                    (linkPartner&PHY_ANEG_LPA_SYMP_)?"SYMP ":"     ",
                    (linkPartner&PHY_ANEG_LPA_100FDX_)?"100FD":"     ",
                    (linkPartner&PHY_ANEG_LPA_100HDX_)?"100HD":"     ",
                    (linkPartner&PHY_ANEG_LPA_10FDX_)?"10FD ":"     ",
                    (linkPartner&PHY_ANEG_LPA_10HDX_)?"10HD ":"     ");
            } else {
                switch(privateData->dwLinkSpeed) {
                case LINK_SPEED_10HD:
                case LINK_SPEED_100HD:
                    Mac_SetRegDW(privateData,FLOW,0x0UL,keyCode);
                    Lan_SetBitsDW(AFC_CFG,0x0000000FUL);
                    break;
                default:
                    Mac_SetRegDW(privateData,FLOW,0x0UL,keyCode);
                    Lan_ClrBitsDW(AFC_CFG,0x0000000FUL);
                    break;
                }
            }
            netif_carrier_on(privateData->dev);
            Tx_WakeQueue(privateData,0x01);

        } else {
    #if 0
            SMSC_TRACE("Link is now DOWN");
            Tx_StopQueue(privateData,0x01);
            netif_carrier_off(privateData->dev);
            Mac_SetRegDW(privateData,FLOW,0UL,keyCode);
            Lan_ClrBitsDW(AFC_CFG,0x0000000FUL);
    #endif
        }
    }
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
}

void Phy_CheckLink(unsigned long ptr)
{
    PPRIVATE_DATA privateData=(PPRIVATE_DATA)ptr;
    if(privateData==NULL) {
        SMSC_WARNING("Phy_CheckLink(ptr==0)");
        return;
    }

    //must call this twice
    Phy_UpdateLinkMode(privateData);
    Phy_UpdateLinkMode(privateData);


    if(!(privateData->StopLinkPolling)) {
        privateData->LinkPollingTimer.expires=jiffies+ 2*HZ;
        add_timer(&(privateData->LinkPollingTimer));
    }
}

DWORD  Phy_GetLinkMode(
            PPRIVATE_DATA privateData,
            DWORD PhyIndex,
            VL_KEY keyCode)
{
    DWORD result=LINK_OFF;
    WORD wRegVal=0;
    WORD wRegBSR=Phy_GetRegW(
                            privateData,
                            PhyIndex,
                            PHY_BSR,
                            keyCode
                            );

 #if 0
 {
     if ( PhyIndex != privateData->dwPhyAddress)
     {
         if( wRegBSR & PHY_BSR_LINK_STATUS_) {
            SMSC_TRACE("Phy Index =%d, Link is UP\n", PhyIndex);
         }
          else {
            SMSC_TRACE("Phy Index =%d, Link is DOWN\n", PhyIndex);
         }
      }
 }
#endif

     if (PhyIndex == privateData->dwPhyAddress)
                 privateData->dwLinkSettings=LINK_OFF;

    if(wRegBSR & PHY_BSR_LINK_STATUS_) {
        wRegVal=Phy_GetRegW(
                            privateData,PhyIndex,
                            PHY_BCR,keyCode
                            );
        if(wRegVal & PHY_BCR_AUTO_NEG_ENABLE_) {
                DWORD linkSettings=LINK_AUTO_NEGOTIATE;
                WORD wRegADV=privateData->wLastADVatRestart;
        //					Phy_GetRegW(
        //						privateData,
        //						PHY_ANEG_ADV,keyCode);
                WORD wRegLPA=Phy_GetRegW(
                                        privateData,PhyIndex,
                                        PHY_ANEG_LPA,keyCode
                                        );
                if(wRegADV & PHY_ANEG_ADV_ASYMP_) {
                    linkSettings|=LINK_ASYMMETRIC_PAUSE;
                }
                if(wRegADV & PHY_ANEG_ADV_SYMP_) {
                    linkSettings|=LINK_SYMMETRIC_PAUSE;
                }
                if(wRegADV & PHY_ANEG_LPA_100FDX_) {
                    linkSettings|=LINK_SPEED_100FD;
                }
                if(wRegADV & PHY_ANEG_LPA_100HDX_) {
                    linkSettings|=LINK_SPEED_100HD;
                }
                if(wRegADV & PHY_ANEG_LPA_10FDX_) {
                    linkSettings|=LINK_SPEED_10FD;
                }
                if(wRegADV & PHY_ANEG_LPA_10HDX_) {
                    linkSettings|=LINK_SPEED_10HD;
                }

                if (PhyIndex == privateData->dwPhyAddress)
                    privateData->dwLinkSettings=linkSettings;

                wRegLPA &= wRegADV;

                if(wRegLPA & PHY_ANEG_LPA_100FDX_) {
                    result=LINK_SPEED_100FD;
                } else if(wRegLPA & PHY_ANEG_LPA_100HDX_) {
                    result=LINK_SPEED_100HD;
                } else if(wRegLPA & PHY_ANEG_LPA_10FDX_) {
                    result=LINK_SPEED_10FD;
                } else if(wRegLPA & PHY_ANEG_LPA_10HDX_) {
                    result=LINK_SPEED_10HD;
                }
        } else {
            if(wRegVal&PHY_BCR_SPEED_SELECT_) {
                if(wRegVal & PHY_BCR_DUPLEX_MODE_) {

                if (PhyIndex == privateData->dwPhyAddress)
                    privateData->dwLinkSettings=result=LINK_SPEED_100FD;

                } else {
                        if (PhyIndex == privateData->dwPhyAddress)
                            privateData->dwLinkSettings=result=LINK_SPEED_100HD;
                }
            } else {
                if(wRegVal & PHY_BCR_DUPLEX_MODE_) {

                        if (PhyIndex == privateData->dwPhyAddress)
                            privateData->dwLinkSettings=result=LINK_SPEED_10FD;
                } else {
                        if (PhyIndex == privateData->dwPhyAddress)
                                privateData->dwLinkSettings=result=LINK_SPEED_10HD;
                }
            }
        }
    }

    if (PhyIndex == privateData->dwPhyAddress)
        privateData->dwLinkSpeed = result;

    return result;
}



BOOLEAN Mac_Initialize(PPRIVATE_DATA privateData)
{
    SMSC_ASSERT(privateData!=NULL);
    //This function is kept only as a place holder

    return TRUE;
}

static BOOLEAN MacNotBusy(PPRIVATE_DATA privateData, VL_KEY keyCode)
{
    int i=0;
    SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));
    // wait for MAC not busy, w/ timeout
    for(i=0;i<40;i++)
    {
        if((LanGetDirectRegDW(MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_)==(0UL)) {
            return TRUE;
        }
    }
    SMSC_WARNING("timeout waiting for MAC not BUSY. MAC_CSR_CMD = 0x%08lX",
            LanGetDirectRegDW(MAC_CSR_CMD));
    return FALSE;
}

DWORD Mac_GetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,VL_KEY keyCode)
{
    DWORD result=0xFFFFFFFFUL;
    DWORD dwTemp=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));
    SMSC_ASSERT(privateData->dwLanBase!=0);

    // wait until not busy
    if (LanGetDirectRegDW(MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_)
    {
        SMSC_WARNING("Mac_GetRegDW() failed, MAC already busy at entry");
        goto DONE;
    }

    // send the MAC Cmd w/ offset
    LanSetDirectRegDW(MAC_CSR_CMD,
        ((dwRegOffset & 0x000000FFUL) |
        MAC_CSR_CMD_CSR_BUSY_ | MAC_CSR_CMD_R_NOT_W_));
    dwTemp=LanGetDirectRegDW(BYTE_TEST);//to flush previous write
    dwTemp=dwTemp;

    // wait for the read to happen, w/ timeout
    if (!MacNotBusy(privateData,keyCode))
    {
        SMSC_WARNING("Mac_GetRegDW() failed, waiting for MAC not busy after read");
        goto DONE;
    } else {
        // finally, return the read data
        result=LanGetDirectRegDW(MAC_CSR_DATA);
    }
DONE:
    return result;
}

void Mac_SetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,DWORD dwVal,VL_KEY keyCode)
{
    DWORD dwTemp=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),keyCode));
    SMSC_ASSERT(privateData->dwLanBase!=0);

    if (LanGetDirectRegDW(MAC_CSR_CMD) & MAC_CSR_CMD_CSR_BUSY_)
    {
        SMSC_WARNING("Mac_SetRegDW() failed, MAC already busy at entry");
        goto DONE;
    }

    // send the data to write
    LanSetDirectRegDW(MAC_CSR_DATA,dwVal);

    // do the actual write
    LanSetDirectRegDW(MAC_CSR_CMD,((dwRegOffset & 0x000000FFUL) | MAC_CSR_CMD_CSR_BUSY_));

    dwTemp=LanGetDirectRegDW(BYTE_TEST);//force flush of previous write
    dwTemp=dwTemp;

    // wait for the write to complete, w/ timeout
    if (!MacNotBusy(privateData,keyCode))
    {
        SMSC_WARNING("Mac_SetRegDW() failed, waiting for MAC not busy after write");
    }
DONE:
    return;
}


static BOOLEAN SwitchNotBusy(PPRIVATE_DATA privateData)
{
    int i=0;
    // wait for MAC not busy, w/ timeout
    for(i=0;i<40;i++)
    {
        if((LanGetRegDW(SWITCH_CSR_CMD) & SWITCH_CSR_CMD_CSR_BUSY_)==(0UL)) {
            return TRUE;
        }
    }
    SMSC_WARNING("timeout waiting for SWITCH not BUSY. MAC_CSR_CMD = 0x%08lX",
    LanGetRegDW(SWITCH_CSR_CMD));
    return FALSE;
}

/**
///////////////////////////////////////
//
//       ALR Block
//
//////////////////////////////////
**/


static BOOLEAN SwitchALRNotBusy(PPRIVATE_DATA privateData)
{
    int i=0;
    BOOLEAN swNobusy  = FALSE;
    BOOLEAN alrbusy = TRUE;
    DWORD regValue   = 0;
    // wait for MAC not busy, w/ timeout
    for(i=0;i<40;i++)
    {
        if((LanGetRegDW(SWITCH_CSR_CMD) & SWITCH_CSR_CMD_CSR_BUSY_)==(0UL)) {
            swNobusy = TRUE;
            break;
        }
    }
    if ( swNobusy)
    {
        regValue = Switch_GetRegDW(privateData, SE_ALR_STS);
        if  ( ( regValue & SE_ALR_STS_ALR_STS_) ||
                    (regValue & SE_ALR_STS_INI_DONE_) )
        {
            alrbusy = FALSE;
        }
    }



    return (!alrbusy);
}
void  Switch_Add_Static_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, ALREntryProp *entryProp)
{
    ALR_UNION data1;

    // Take sem for reading
    getSwitchWriteLock(privateData);
//    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Add ALR entry:HighMac=%lx LowMac=%lx",
            highMAc, LowMAC );
    goto DONE;
    ALR_READY:
  //  SMSC_TRACE("--------------->, ------------");
    //Set Low MAC
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_0,htonl(LowMAC));
    //Set High MAC with all other entries.
    data1.alr_data1.res = 0;
    data1.alr_data1.valid = 1 ;
    data1.alr_data1.age = entryProp->age_alias_oride;
    data1.alr_data1.Static = 1; /* Entries are always static */
    data1.alr_data1.Filter =  0;
#ifdef CHIP_IS_LAN9303
    data1.alr_data1.PriorityEnbl = entryProp->prienbl;
    data1.alr_data1.Priority = entryProp->pri_alia_tag;
#else
    data1.alr_data1.Priority = entryProp->pri;
#endif
    data1.alr_data1.PortId=entryProp->port;
    data1.alr_data1.MAcHigh = htons ((WORD) (highMAc & 0xffff));
    SMSC_TRACE("ALR DATA = DATA1=0x%lx", data1.DATA1);
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_1,data1.DATA1);
    // Set ALR command to Write
    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_RST_WR_);


    DONE:
    releaseSwitchWriteLock(privateData);
  //  SMSC_TRACE("<----------------------");
    return;
}

void  Switch_Add_Entry_in_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, ALREntryProp *entryProp)
{
    ALR_UNION data1;

    // Take sem for reading
    getSwitchWriteLock(privateData);
//    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Add ALR entry:HighMac=%lx LowMac=%lx",
            highMAc, LowMAC );
    goto DONE;
    ALR_READY:
  //  SMSC_TRACE("--------------->, ------------");
    //Set Low MAC
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_0,htonl(LowMAC));
    //Set High MAC with all other entries.
    data1.alr_data1.res = 0;
    data1.alr_data1.valid = 1 ;
    data1.alr_data1.age = entryProp->age_alias_oride;
    data1.alr_data1.Static = entryProp->statc; /* Entries are always static */
    data1.alr_data1.Filter =  entryProp->filter;
#ifdef CHIP_IS_LAN9303
    data1.alr_data1.PriorityEnbl = entryProp->prienbl;
    data1.alr_data1.Priority = entryProp->pri_alia_tag;
#else
    data1.alr_data1.Priority = entryProp->pri;
#endif
    data1.alr_data1.PortId=entryProp->port;
    data1.alr_data1.MAcHigh = htons ((WORD) (highMAc & 0xffff));
    SMSC_TRACE("ALR DATA = DATA1=0x%lx", data1.DATA1);
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_1,data1.DATA1);
    // Set ALR command to Write
    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_RST_WR_);


    DONE:
    releaseSwitchWriteLock(privateData);
  //  SMSC_TRACE("<----------------------");
    return;
}


void  Switch_Enbl_Fltr_on_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc)
{
    ALR_UNION data1;

    // Take sem for reading
    getSwitchWriteLock(privateData);
//    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Add ALR entry:HighMac=%lx LowMac=%lx",
            highMAc, LowMAC );
    goto DONE;
    ALR_READY:
  //  SMSC_TRACE("--------------->, ------------");
    //Set Low MAC
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_0,htonl(LowMAC));
    //Set High MAC with all other entries.
    data1.alr_data1.res = 0;
    data1.alr_data1.valid = 1 ;
    data1.alr_data1.age = 0;
    data1.alr_data1.Static = 1;
    data1.alr_data1.Filter =  1;
    data1.alr_data1.Priority = 0;
    data1.alr_data1.PortId=0;
    data1.alr_data1.MAcHigh = htons ((WORD) (highMAc & 0xffff));
    SMSC_TRACE("ALR DATA = DATA1=0x%lx", data1.DATA1);
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_1,data1.DATA1);
    // Set ALR command to Write
    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_RST_WR_);


    DONE:
    releaseSwitchWriteLock(privateData);
  //  SMSC_TRACE("<----------------------");
    return;
}



void  Switch_remove_Static_ALR(PPRIVATE_DATA privateData, DWORD LowMAC, DWORD highMAc, BYTE portMap)
{

  ALR_UNION data1;
    // Take sem for reading
    getSwitchWriteLock(privateData);
//    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;

	SMSC_ERROR("ALR Busy, Unable to delete ALR entry:HighMac=%lx LowMac=%lx",
            highMAc, LowMAC );
    goto DONE;
    ALR_READY:
//    SMSC_TRACE("--------------->, ------------");
    //Set Low MAC
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_0,htonl(LowMAC));
    //Set High MAC with all other entries.
    data1.alr_data1.res = 0;
    data1.alr_data1.valid = 0 ;
    data1.alr_data1.age = 0;
    data1.alr_data1.Static =0;
    data1.alr_data1.Filter =  0;
    data1.alr_data1.Priority = 0;
    data1.alr_data1.PortId = portMap;
    data1.alr_data1.MAcHigh = htons ((WORD) (highMAc & 0xffff));
    SMSC_TRACE("ALR DATA = DATA1=0x%lx", data1.DATA1);
    Switch_SetRegDW(privateData,SE_ALR_WR_DAT_1,data1.DATA1);
    // Set ALR command to Write
    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_RST_WR_);


    DONE:
    releaseSwitchWriteLock(privateData);
//    SMSC_TRACE("<----------------------");
    return;
}


/////////////

BOOLEAN  Switch_Rad_First_ALR(PPRIVATE_DATA privateData, DWORD *data0, DWORD *data1)
{
    BOOLEAN done = FALSE;
    int  count = 0;
    *data0 = *data1 = 0xFFFFFFFF;
  //  SMSC_TRACE("ALR ENTER");

    getSwitchReadLock(privateData);

    if ( SwitchALRNotBusy(privateData))
            goto ALR_READY;


    SMSC_ERROR("ALR Busy");
    goto DONE;

    ALR_READY:
    // Set ALR command to Read first bit 2
    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_RST_RD_);
    /** Read register5 data 1. This register is valid if:
            -  SE_ALR_RD_DAT_1_VLD_ is set
            -  SE_ALR_RD_DAT_1_OVF_ is set
    **/

    do {
        *data1 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_1);
        if ( (*data1 & SE_ALR_RD_DAT_1_VLD_)  ||(*data1 & SE_ALR_RD_DAT_1_OVF_))
        {
            done = TRUE;
        }
        else
        {
            count++;
        }

    } while ( (!done) || (count > 100 )) ;

    if (done)
    {
            *data0 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_0);
            *data1 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_1);
            SMSC_TRACE("data0=0x%lx, data1=0x%lx", *data0,*data1);
            if (*data1 &  SE_ALR_RD_DAT_1_OVF_)
            {
                // last entry.
                done = FALSE;
            }
    }
    else
    {
            *data0 = *data1 = 0xFFFFFFFF;
    }
    DONE:
    releaseSwitchReadLock(privateData);

 //   SMSC_TRACE("ALR EXIT");
    return done;

}
BOOLEAN  Switch_Rad_Next_ALR(PPRIVATE_DATA privateData, DWORD *data0, DWORD *data1)
{
    BOOLEAN done = FALSE;
    int i,count = 0;

  //  SMSC_TRACE("ALR ENTER  ");
    getSwitchReadLock(privateData);

    for(i=0;i<40;i++)
    {
        if ( SwitchALRNotBusy(privateData))
            goto ALR_READY;

    }

    SMSC_ERROR("ALR Busy");
    releaseSwitchReadLock(privateData);
    return FALSE;
    ALR_READY:
    // Set ALR command to Read Next

    Switch_SetRegDW(privateData,SE_ALR_CMD,SE_ALR_CMD_GET_NXT_);

    /** Read register5 data 1. This register is valid if:
            -  SE_ALR_RD_DAT_1_VLD_ is set
            -  SE_ALR_RD_DAT_1_OVF_ is set
    **/

    do {
        *data1 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_1);
        if ( (*data1 & SE_ALR_RD_DAT_1_VLD_)  ||(*data1 & SE_ALR_RD_DAT_1_OVF_))
        {
            done = TRUE;
        }
        else
        {
            count++;
        }

    } while ( (!done) || (count > 100 )) ;

    if (done)
    {
            *data0 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_0);
            *data1 =  Switch_GetRegDW(privateData, SE_ALR_RD_DAT_1);
            SMSC_TRACE("0x%lx%lx  data1=%lx", *data0,(*data1&0xffff),(*data1&0xffff0000));
            if (*data1 &  SE_ALR_RD_DAT_1_OVF_)
            {
                // last entry.
                done = FALSE;
            }
    }
    else
    {    SMSC_ERROR("ALR Invalid Entry");
            *data0 = *data1 = 0xFFFFFFFF;
    }
    releaseSwitchReadLock(privateData);

    //SMSC_TRACE("ALR EXIT");
    return done;

}

/**
///////////////////////////////////////
//
//       END OF ALR Block
//
//////////////////////////////////
**/

void  Switch_Set_port_state(PPRIVATE_DATA privateData, DWORD PortNumber, SWITCH_PORT_STATE portState)
{
    DWORD regValue = 0;
    DWORD olDregValue = 0;
    DWORD portMask = 0x3UL << PortNumber*2;

    getSwitchWriteLock(privateData);
    regValue = Switch_GetRegDW(privateData, SE_PORT_STATE);
    olDregValue = regValue;
    regValue &= ~portMask;
    regValue |= portState << PortNumber*2 ;
    Switch_SetRegDW(privateData,SE_PORT_STATE, regValue);

    SMSC_TRACE("Port Id =%lu, new State=%s, oldValue=0x%lx, New Val=0x%lx",
    PortNumber, switch_port_state_str[portState], olDregValue, regValue);
    releaseSwitchWriteLock(privateData);
}

void Switch_Enbl_disbl_shortAgeTimer(PPRIVATE_DATA privateData, DWORD regValue)
{
    getSwitchWriteLock(privateData);
    SMSC_TRACE("Setting the ALR Aging Timer (1) is 50 ms and (0) is 5 sec")
    SMSC_TRACE("Setting the ALR Aging Timer Val=0x%lx",regValue);
    Switch_SetRegDW(privateData,SE_ALR_CFG, regValue);
    releaseSwitchWriteLock(privateData);
}


SWITCH_PORT_STATE  Switch_Get_port_state(PPRIVATE_DATA privateData, DWORD PortNumber)
{
    DWORD regValue = 0;
    DWORD portMask = 0x3UL;
    SWITCH_PORT_STATE portState;

    getSwitchReadLock(privateData);
    regValue = Switch_GetRegDW(privateData, SE_PORT_STATE);
    portState = (regValue >> PortNumber*2) & portMask;
    releaseSwitchReadLock(privateData);
    if (portState >= SWITCH_PORT_INVALID_STATE)
    {
        SMSC_ERROR("INVALID SWITCH PORT STATE: port=%lu, state=%d",PortNumber, portState);
    }
    else
    {
        SMSC_TRACE("Port Id =%lu,  State=%s",PortNumber, switch_port_state_str[portState]);
    }

    return portState;
}
/**
*  Use semaphore to synchronize access to Switch_GetRegDW
*  Don't call this API from Interrupt Context
**/
DWORD Switch_GetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset)
{

    DWORD result=0xFFFFFFFFUL;
    DWORD dwTemp=0, i =0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    SMSC_ENTER
    SMSC_ASSERT(privateData->PlatformGetRegDW!=NULL);
    // wait until not busy
    if (LanGetRegDW(SWITCH_CSR_CMD) & SWITCH_CSR_CMD_CSR_BUSY_)
    {
        SMSC_WARNING("%s: failed, SWITCH already busy at entry", __func__);
        goto DONE;
    }

    // send the MAC Cmd w/ offset
    LanSetRegDW(SWITCH_CSR_CMD,
        ((dwRegOffset & 0x0000FFFFUL) |
                        MAC_CSR_CMD_CSR_BUSY_ | MAC_CSR_CMD_R_NOT_W_));
  for ( i=0; i < 40; i++)
    {
        dwTemp=LanGetRegDW(BYTE_TEST);//to flush previous write
        dwTemp=dwTemp;
    }

    // wait for the read to happen, w/ timeout

    if (!SwitchNotBusy(privateData))
    {
        SMSC_WARNING("%s failed, waiting for SWITCH not busy after read",__func__);
        goto DONE;
    } else {
        // finally, return the read data
        result=LanGetRegDW(SWITCH_CSR_DATA);
    // SMSC_TRACE("--->LanGetRegDW: reg = 0x%lx, value = 0x%lx",SWITCH_CSR_DATA,result );
    }

DONE:
	if(switch_tracing) {
		SMSC_TRACE("offset=0x%lX,value=0x%08lX",dwRegOffset,result);
	}
    SMSC_EXIT
    return result;
}


/**
*  Use semaphore to synchronize access to Switch_SetRegDW
*  Don't call this API from Interrupt Context
**/
void Switch_SetRegDW(PPRIVATE_DATA privateData,DWORD dwRegOffset,DWORD dwVal)
{
    DWORD dwTemp=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    SMSC_ASSERT(privateData->PlatformGetRegDW!=NULL);

    if(switch_tracing) {
    	SMSC_TRACE("offset=0x%lX,value=0x%08lX",dwRegOffset,dwVal);
    }

    if (LanGetRegDW(SWITCH_CSR_CMD) & SWITCH_CSR_CMD_CSR_BUSY_)
    {
        SMSC_WARNING("Failed, SWITCH already busy at entry");
        goto DONE;
    }

    //printk("Inside Switch_SetRegDW dwVal = 0x%x dwRegOffSet=0x%x\n", dwVal, dwRegOffset);

    // send the data to write
    LanSetRegDW(SWITCH_CSR_DATA,dwVal );

    // do the actual write
    LanSetRegDW(SWITCH_CSR_CMD,((dwRegOffset & 0xFFFFUL) | SWITCH_CSR_CMD_CSR_BUSY_ | SWITCH_CSR_BYTE_EN_));
    dwTemp=LanGetRegDW(BYTE_TEST);//force flush of previous write
    dwTemp=dwTemp;

    // wait for the write to complete, w/ timeout
    if (!SwitchNotBusy(privateData))
    {
        SMSC_WARNING("Switch_SetRegDW() failed, waiting for MAC not busy after write");
    }
    DONE:
    return;
}



/**
*******************************************************************************
*	PmiSetReg
*******************************************************************************
*
*	Writes the passed-in data to specified MII PHY register.
*
*/
void PmiSetRegW(PPRIVATE_DATA privateData,
            const DWORD dwPhyAddress,
            const DWORD dwMiiIndex, const WORD wVal)
{
    DWORD dwAddr;
    DWORD dwStartTick = 0;
    // confirm MII not busy
    if ((LanGetRegDW(PMI_ACCESS) & PMI_ACCESS_MIIBZY_) != 0UL)
    {
        SMSC_ERROR("ERROR:  PMI is busy in PmiGetReg???\n");
        return;
    }

    // put the data to write in the PMI
    LanSetRegDW(PMI_ACCESS, PMI_ACCESS_MIIWnR_);
    LanSetRegDW(PMI_DATA, (DWORD)wVal);

    // set the address, index & direction (write to PHY)
    dwAddr = ((dwPhyAddress & 0x1FUL)<<11) | ((dwMiiIndex & 0x1FUL)<<6) | PMI_ACCESS_MIIWnR_;
    LanSetRegDW(PMI_ACCESS, dwAddr);

    // wait for write to complete w/ timeout
    for(dwStartTick =0; dwStartTick < I2C_SPI_MAX_RETRY ; dwStartTick++) {
        // see if MII is finished yet
        if ((LanGetRegDW(PMI_ACCESS) & PMI_ACCESS_MIIBZY_) == 0UL)
        {
        goto DONE;
        }
    }

        SMSC_ERROR("timeout waiting for PMI write to finish\n");

    DONE:
    return;
}


/**
*******************************************************************************
*	PmiGetReg
*******************************************************************************
*
*	Reads from the specified PMI PHY register & returns the value.
*
*/
/* Reads the PHY Register over MII */
WORD PmiGetRegW(PPRIVATE_DATA privateData,
            const DWORD dwPhyAddress,
            const DWORD dwMiiIndex)
{
    DWORD dwAddr, dwStartTick;
    WORD  PhyRegValue = 0xFFFF;

    // confirm PMI not busy
    if ((LanGetRegDW(PMI_ACCESS) & PMI_ACCESS_MIIBZY_) != 0UL)
    {
        SMSC_ERROR("  !!! PMI is busy in PmiGetReg???\n");
        return (PhyRegValue);
    }

    // set the address, index & direction (read from PHY)
    dwAddr = ((dwPhyAddress & 0x1FUL)<<11) | ((dwMiiIndex & 0x1FUL)<<6);
    LanSetRegDW(PMI_ACCESS, dwAddr);

    // wait for read to complete w/ timeout
    for(dwStartTick =0; dwStartTick < I2C_SPI_MAX_RETRY ; dwStartTick++) {
        // see if PMI is finished yet
        if ((LanGetRegDW(PMI_ACCESS) & PMI_ACCESS_MIIBZY_) == 0UL)
        {
            // get the read data from the MAC & return i
            PhyRegValue =  ((WORD)LanGetRegDW(PMI_DATA));
            goto DONE;
        }
    }

    SMSC_ERROR("ERROR:  timeout waiting for PMI write to finish\n");
    DONE:
    return PhyRegValue;
}

////////////////////////////////////
//  End of Switch Registers
//////////////////////////////////
#define TX_FIFO_LOW_THRESHOLD	(1600)

void Tx_Initialize(
PPRIVATE_DATA privateData,
DWORD dwTxDmaCh,
DWORD dwDmaThreshold)
{
    DWORD dwRegVal=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);

    dwRegVal=LanGetDirectRegDW(HW_CFG);
    dwRegVal&=(HW_CFG_TX_FIF_SZ_|0x00000FFFUL);
    dwRegVal|=HW_CFG_SF_;
    LanSetDirectRegDW(HW_CFG,dwRegVal);

    Lan_SetTDFL(privateData,0xFF);
    Lan_EnableInterrupt(privateData,INT_EN_TDFA_EN_);

    privateData->dwTxDmaThreshold=dwDmaThreshold;
    privateData->dwTxDmaCh=dwTxDmaCh;
    if(dwTxDmaCh>=TRANSFER_PIO) {
        SMSC_TRACE("Tx will use PIO");
    } else {
        SMSC_TRACE("Tx will use DMA channel %ld",dwTxDmaCh);
        SMSC_ASSERT(Platform_IsValidDmaChannel(dwTxDmaCh));
        if(!Platform_DmaInitialize(
            &(privateData->PlatformData),
            dwTxDmaCh))
        {
            SMSC_WARNING("Failed Platform_DmaInitialize, dwTxDmaCh=%lu",dwTxDmaCh);
        }
        privateData->TxDmaXfer.dwLanReg=privateData->dwLanBase+TX_DATA_FIFO;
        privateData->TxDmaXfer.pdwBuf=NULL;//this will be reset per dma request
        privateData->TxDmaXfer.dwDmaCh=privateData->dwTxDmaCh;
        privateData->TxDmaXfer.dwDwCnt=0;//this will be reset per dma request
        privateData->TxDmaXfer.fMemWr=FALSE;
    }

    {
        DWORD dwIntFlags=0;
        VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
        DWORD dwMacCr=Mac_GetRegDW(privateData,MAC_CR,keyCode);
        dwMacCr|=(MAC_CR_TXEN_|MAC_CR_HBDIS_ | MAC_CR_MCPAS_);
        Mac_SetRegDW(privateData,MAC_CR,dwMacCr,keyCode);
        LanSetDirectRegDW(TX_CFG,TX_CFG_TX_ON_);
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    }

    privateData->TxSkb=NULL;
    spin_lock_init(&(privateData->TxSkbLock));
    privateData->dwTxQueueDisableMask=0;
    spin_lock_init(&(privateData->TxQueueLock));
    spin_lock_init(&(privateData->TxCounterLock));

    privateData->TxInitialized=TRUE;
}

BOOLEAN Tx_HandleInterrupt(
PPRIVATE_DATA privateData,DWORD dwIntSts)
{
    SMSC_ASSERT(privateData!=NULL);
    if(dwIntSts&INT_STS_TDFA_)
    {
        Lan_SetTDFL(privateData,0xFF);
        LanSetDirectRegDW(INT_STS,INT_STS_TDFA_);
        Tx_WakeQueue(privateData,0x02UL);
        return TRUE;
    }
    return FALSE;
}

void Tx_StopQueue(
PPRIVATE_DATA privateData,DWORD dwSource)
{
    DWORD intFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dev!=NULL);
    SMSC_ASSERT(privateData->TxInitialized);
    spin_lock_irqsave(&(privateData->TxQueueLock),intFlags);
    if(privateData->dwTxQueueDisableMask==0) {
        netif_stop_queue(privateData->dev);
    }
    privateData->dwTxQueueDisableMask|=dwSource;
    spin_unlock_irqrestore(&(privateData->TxQueueLock),intFlags);
}

void Tx_WakeQueue(
PPRIVATE_DATA privateData,DWORD dwSource)
{
    DWORD intFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dev!=NULL);
    SMSC_ASSERT(privateData->TxInitialized);
    spin_lock_irqsave(&(privateData->TxQueueLock),intFlags);
    privateData->dwTxQueueDisableMask&=(~dwSource);
    if(privateData->dwTxQueueDisableMask==0) {
        netif_wake_queue(privateData->dev);
    }
    spin_unlock_irqrestore(&(privateData->TxQueueLock),intFlags);
}

static DWORD Tx_GetTxStatusCount(
PPRIVATE_DATA privateData)
{
    DWORD result=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    result=LanGetDirectRegDW(TX_FIFO_INF);
    if(OLD_REGISTERS(privateData)) {
        result&=TX_FIFO_INF_TSFREE_;
        result>>=16;
        if(result>0x80) {
            SMSC_WARNING("TX_FIFO_INF_TSFREE_>0x80");
            result=0x80;
        }
        result=0x80-result;
    } else {
        result&=TX_FIFO_INF_TXSUSED_;
        result>>=16;
    }
    return result;
}

#ifdef FULLY_MANAGED

void Switch_SendSkb(
    PPRIVATE_DATA privateData,
    struct sk_buff *skb)
{
    DWORD dwFreeSpace=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);


	//#if DUMP_TX_PACKET
		if(debug_mode&0x10)
		{
		SMSC_TRACE("Switch_sendskb:Packet Dump data:");
		dumpSkb(skb);
		}
	//#endif


    spin_lock(&(privateData->TxSkbLock));

    if(privateData->dwTxDmaCh>=TRANSFER_PIO)
    {
        //Use PIO only
        DWORD dwTxCmdA=0;
        DWORD dwTxCmdB=0;
        dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
        dwFreeSpace&=TX_FIFO_INF_TDFREE_;
        if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
            SMSC_WARNING("Tx Data Fifo Low, space available = %ld",dwFreeSpace);
        }
        dwTxCmdA=
                ((((DWORD)(skb->data))&0x03UL)<<16) | //DWORD alignment adjustment
                TX_CMD_A_INT_FIRST_SEG_ | TX_CMD_A_INT_LAST_SEG_ |
                ((DWORD)(skb->len));
        dwTxCmdB=
                (((DWORD)(skb->len))<<16) |
                ((DWORD)(skb->len));
        LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
        LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
        Platform_WriteFifo(
                privateData->dwLanBase,
        (DWORD *)(((DWORD)(skb->data))&0xFFFFFFFCUL),
        (((DWORD)(skb->len))+3+
                (((DWORD)(skb->data))&0x03UL))>>2);
        dwFreeSpace-=(skb->len+32);
        dev_kfree_skb(skb);
    }
    else
    {
        //Use DMA and PIO
        DWORD dwDmaCh=privateData->dwTxDmaCh;
        PPLATFORM_DATA platformData=&(privateData->PlatformData);
        SMSC_ASSERT(TX_FIFO_LOW_THRESHOLD>(skb->len+32));
        SMSC_TRACE("Using DMA");
        if(skb->len>=privateData->dwTxDmaThreshold)
        {
                //use DMA
            DWORD dwTxCmdA = 0;
            DWORD dwTxCmdB = 0;
            DWORD dwPlatformCacheLineSize = platformGetCacheLineSize();
                //prepare for 16 byte alignment

            if (dwPlatformCacheLineSize == 16)
                dwTxCmdA= (0x01UL<<24) ;//16 byte end alignment
            else if (dwPlatformCacheLineSize == 32)
                dwTxCmdA=  (0x02UL<<24); //32 byte end alignment
            else
            {       SMSC_WARNING("!!!! Cache Line size is = %ld, using default 32 !!!!! \n",dwPlatformCacheLineSize);
            dwPlatformCacheLineSize = 32;
            dwTxCmdA=  (0x02UL<<24); //32 byte end alignment
            }



            dwTxCmdA |= ((((DWORD)(skb->data))&(dwPlatformCacheLineSize-1))<<16) |   //16 Byte start alignment
                    TX_CMD_A_INT_FIRST_SEG_ |
                    TX_CMD_A_INT_LAST_SEG_ |
                    ((DWORD)(skb->len));//buffer length
            dwTxCmdB=
                    (((DWORD)(skb->len))<<16)|
                    ((DWORD)(skb->len)&0x7FFUL);
            privateData->TxDmaXfer.pdwBuf=
                    (DWORD *)(((DWORD)(skb->data))&
                    (~(dwPlatformCacheLineSize-1)));
            privateData->TxDmaXfer.dwDwCnt=
                    ((((DWORD)(skb->len))+
                    (dwPlatformCacheLineSize-1)+
                    (((DWORD)(skb->data))&
                    (dwPlatformCacheLineSize-1)))&
                    (~(dwPlatformCacheLineSize-1)))>>2;
            Platform_CachePurge(
                    platformData,
            privateData->TxDmaXfer.pdwBuf,
            (privateData->TxDmaXfer.dwDwCnt)<<2);
            SMSC_TRACE_DMA("Seting DMA count: Tx Bytes =%u, dma count =%lu, DMA Addr=0x%lx", skb->len,
                        privateData->TxDmaXfer.dwDwCnt,(DWORD ) (privateData->TxDmaXfer.pdwBuf));

            {

                if(privateData->TxSkb)
                    Platform_DmaComplete(platformData,dwDmaCh);

                dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
                dwFreeSpace&=TX_FIFO_INF_TDFREE_;
                if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
                    SMSC_WARNING("Tx DATA FIFO LOW, space available = %ld",dwFreeSpace);
                }

                LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
                LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
                if(!Platform_DmaStartXfer(platformData,&(privateData->TxDmaXfer)))
                {
                    SMSC_WARNING("Failed Platform_DmaStartXfer");
                }

                dwFreeSpace-=(skb->len+32);
                if(privateData->TxSkb)
                    dev_kfree_skb(privateData->TxSkb);

                privateData->TxSkb=skb;
            }


        }
        else
        {
                //use PIO
            DWORD dwTxCmdA=0;
            DWORD dwTxCmdB=0;
            spin_lock(&(privateData->TxSkbLock));
            SMSC_TRACE_DMA("-------->Using PIO ");
            if(privateData->TxSkb) {
                Platform_DmaComplete(platformData,dwDmaCh);
                dev_kfree_skb(privateData->TxSkb);
                privateData->TxSkb=NULL;
            }
            spin_unlock(&(privateData->TxSkbLock));
            dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
            dwFreeSpace&=TX_FIFO_INF_TDFREE_;
            if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
                SMSC_WARNING("Tx DATA FIFO LOW, space available = %ld",dwFreeSpace);
            }
            dwTxCmdA=
                    ((((DWORD)(skb->data))&0x03UL)<<16) | //DWORD alignment adjustment
                    TX_CMD_A_INT_FIRST_SEG_ | TX_CMD_A_INT_LAST_SEG_ |
                    ((DWORD)(skb->len));
            dwTxCmdB=
                    (((DWORD)(skb->len))<<16) |
                    ((DWORD)(skb->len));
            LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
            LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
            Platform_WriteFifo(
                    privateData->dwLanBase,
            (DWORD *)(((DWORD)(skb->data))&0xFFFFFFFCUL),
            (((DWORD)(skb->len))+3+
                    (((DWORD)(skb->data))&0x03UL))>>2);

            dwFreeSpace-=(skb->len+32);
            dev_kfree_skb(skb);
        }
    }

    if(Tx_GetTxStatusCount(privateData)>=30)
    {
        Tx_UpdateTxCounters(privateData);
    }
    if(dwFreeSpace < TX_FIFO_LOW_THRESHOLD) {
        SMSC_WARNING("Function: %s, dwFreeSpace space available = %ld",__func__, dwFreeSpace);
        Tx_StopQueue(privateData,0x02UL);
        Lan_SetTDFL(privateData,0x32);
    }

    spin_unlock(&(privateData->TxSkbLock));
}

#endif   /**  FULLY_MANAGED */

/// end of void Switch_SendSkb

void Tx_SendSkb(
PPRIVATE_DATA privateData,
struct sk_buff *skb)
{
    DWORD dwFreeSpace=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);



	//#ifdef DUMP_TX_PACKET
	if(debug_mode&0x10)
		{
		SMSC_TRACE("Tx_SendSkb:Packet Dump data: \n");
		dumpSkb(skb);
		}
	//#endif


    if(privateData->dwTxDmaCh>=TRANSFER_PIO)
    {
        //Use PIO only
        DWORD dwTxCmdA=0;
        DWORD dwTxCmdB=0;
        SMSC_TRACE_NONE("Using PIO");
        dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
        dwFreeSpace&=TX_FIFO_INF_TDFREE_;
        if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
            SMSC_WARNING("Tx Data Fifo Low, space available = %ld",dwFreeSpace);
        }
        dwTxCmdA=
            ((((DWORD)(skb->data))&0x03UL)<<16) | //DWORD alignment adjustment
            TX_CMD_A_INT_FIRST_SEG_ | TX_CMD_A_INT_LAST_SEG_ |
            ((DWORD)(skb->len));
        dwTxCmdB=
            (((DWORD)(skb->len))<<16) |
            ((DWORD)(skb->len));
        LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
        LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
        Platform_WriteFifo(
            privateData->dwLanBase,
            (DWORD *)(((DWORD)(skb->data))&0xFFFFFFFCUL),
            (((DWORD)(skb->len))+3+
            (((DWORD)(skb->data))&0x03UL))>>2);
        dwFreeSpace-=(skb->len+32);
        dev_kfree_skb(skb);
    }
    else
    {
        //Use DMA and PIO
        DWORD dwDmaCh=privateData->dwTxDmaCh;
        PPLATFORM_DATA platformData=&(privateData->PlatformData);
        SMSC_ASSERT(TX_FIFO_LOW_THRESHOLD>(skb->len+32));
        SMSC_TRACE("Using DMA");
        if(skb->len>=privateData->dwTxDmaThreshold)
        {
            //use DMA
            DWORD dwTxCmdA = 0;
            DWORD dwTxCmdB = 0;
            DWORD dwPlatformCacheLineSize = platformGetCacheLineSize();
            //prepare for 16 byte alignment

            if (dwPlatformCacheLineSize == 16)
                dwTxCmdA= (0x01UL<<24) ;//16 byte end alignment
            else if (dwPlatformCacheLineSize == 32)
                dwTxCmdA=  (0x02UL<<24); //32 byte end alignment
            else
            {       SMSC_WARNING("!!!! Cache Line size is = %ld, using default 32 !!!!! \n",dwPlatformCacheLineSize);
                dwPlatformCacheLineSize = 32;
                dwTxCmdA=  (0x02UL<<24); //32 byte end alignment
            }



            dwTxCmdA |= ((((DWORD)(skb->data))&(dwPlatformCacheLineSize-1))<<16) |   //16 Byte start alignment
                TX_CMD_A_INT_FIRST_SEG_ |
                TX_CMD_A_INT_LAST_SEG_ |
                ((DWORD)(skb->len));//buffer length
            dwTxCmdB=
                (((DWORD)(skb->len))<<16)|
                ((DWORD)(skb->len)&0x7FFUL);
            privateData->TxDmaXfer.pdwBuf=
                (DWORD *)(((DWORD)(skb->data))&
                    (~(dwPlatformCacheLineSize-1)));
            privateData->TxDmaXfer.dwDwCnt=
                ((((DWORD)(skb->len))+
                    (dwPlatformCacheLineSize-1)+
                (((DWORD)(skb->data))&
                    (dwPlatformCacheLineSize-1)))&
                    (~(dwPlatformCacheLineSize-1)))>>2;
            Platform_CachePurge(
                platformData,
                privateData->TxDmaXfer.pdwBuf,
                (privateData->TxDmaXfer.dwDwCnt)<<2);
            SMSC_TRACE_DMA("Seting DMA count: Tx Bytes =%u, dma count =%lu, DMA Addr=0x%lx", skb->len,
                privateData->TxDmaXfer.dwDwCnt,(DWORD ) (privateData->TxDmaXfer.pdwBuf));
            spin_lock(&(privateData->TxSkbLock));
            {

                if(privateData->TxSkb)
                    Platform_DmaComplete(platformData,dwDmaCh);

                dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
                dwFreeSpace&=TX_FIFO_INF_TDFREE_;
                if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
                    SMSC_WARNING("Tx DATA FIFO LOW, space available = %ld",dwFreeSpace);
                }

                LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
                LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
                if(!Platform_DmaStartXfer(platformData,&(privateData->TxDmaXfer)))
                {
                    SMSC_WARNING("Failed Platform_DmaStartXfer");
                }

                dwFreeSpace-=(skb->len+32);
                if(privateData->TxSkb)
                    dev_kfree_skb(privateData->TxSkb);

                privateData->TxSkb=skb;
            }
            spin_unlock(&(privateData->TxSkbLock));

        }
        else
        {
            //use PIO
            DWORD dwTxCmdA=0;
            DWORD dwTxCmdB=0;
            spin_lock(&(privateData->TxSkbLock));
            SMSC_TRACE_DMA("-------->Using PIO ");
            if(privateData->TxSkb) {
                Platform_DmaComplete(platformData,dwDmaCh);
                dev_kfree_skb(privateData->TxSkb);
                privateData->TxSkb=NULL;
            }
            spin_unlock(&(privateData->TxSkbLock));
            dwFreeSpace=LanGetDirectRegDW(TX_FIFO_INF);
            dwFreeSpace&=TX_FIFO_INF_TDFREE_;
            if(dwFreeSpace<TX_FIFO_LOW_THRESHOLD) {
                SMSC_WARNING("Tx DATA FIFO LOW, space available = %ld",dwFreeSpace);
            }
            dwTxCmdA=
                ((((DWORD)(skb->data))&0x03UL)<<16) | //DWORD alignment adjustment
                TX_CMD_A_INT_FIRST_SEG_ | TX_CMD_A_INT_LAST_SEG_ |
                ((DWORD)(skb->len));
            dwTxCmdB=
                (((DWORD)(skb->len))<<16) |
                ((DWORD)(skb->len));
            LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdA);
            LanSetDirectRegDW(TX_DATA_FIFO,dwTxCmdB);
            Platform_WriteFifo(
                privateData->dwLanBase,
                (DWORD *)(((DWORD)(skb->data))&0xFFFFFFFCUL),
                (((DWORD)(skb->len))+3+
                (((DWORD)(skb->data))&0x03UL))>>2);

            dwFreeSpace-=(skb->len+32);
            dev_kfree_skb(skb);
        }
    }

    if(Tx_GetTxStatusCount(privateData)>=30)
    {
        Tx_UpdateTxCounters(privateData);
    }

    if(dwFreeSpace < TX_FIFO_LOW_THRESHOLD) {
        SMSC_WARNING("Function: %s, dwFreeSpace space available = %ld",__func__, dwFreeSpace);
        Tx_StopQueue(privateData,0x02UL);
        Lan_SetTDFL(privateData,0x32);
    }
}

/// end of void Tx_SendSkb
static DWORD Tx_CompleteTx(
PPRIVATE_DATA privateData)
{
    DWORD result=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    SMSC_ASSERT(privateData->TxInitialized==TRUE);
    result=LanGetDirectRegDW(TX_FIFO_INF);
    if(OLD_REGISTERS(privateData)) {
        result&=TX_FIFO_INF_TSFREE_;
        if(result!=0x00800000UL) {
            result=LanGetDirectRegDW(TX_STATUS_FIFO_PORT);
        } else {
            result=0;
        }
    } else {
        result&=TX_FIFO_INF_TXSUSED_;
        if(result!=0x00000000UL) {
            result=LanGetDirectRegDW(TX_STATUS_FIFO_PORT);
        } else {
            result=0;
        }
    }
    return result;
}

void Tx_UpdateTxCounters(
PPRIVATE_DATA privateData)
{

    DWORD dwTxStatus=0;
    SMSC_ASSERT(privateData!=NULL);
    spin_lock(&(privateData->TxCounterLock));
    while((dwTxStatus=Tx_CompleteTx(privateData))!=0)
    {
        if(dwTxStatus&0x80000000UL) {
            SMSC_WARNING("Packet tag reserved bit is high");
            privateData->stats.tx_errors++;
        } else if(dwTxStatus&0x00007080UL) {
            SMSC_WARNING("Tx Status reserved bits are high");
            privateData->stats.tx_errors++;
            }
            // Check for errors now.
        if( (dwTxStatus&0x00008000UL) == 0x00000000UL) {
                privateData->stats.tx_packets++;
                privateData->stats.tx_bytes+=(dwTxStatus>>16);
                }
        else {
            privateData->stats.tx_errors++;

            if(dwTxStatus&0x00000100UL) {
                SMSC_WARNING("Tx Status Excessive collisions error");
            privateData->stats.collisions+=16;
            privateData->stats.tx_aborted_errors+=1;
            } else {
            privateData->stats.collisions+=
                ((dwTxStatus>>3)&0xFUL);
            }
            if(dwTxStatus&0x00000800UL) {
                SMSC_WARNING("Tx Status Loss of carrier error");
            privateData->stats.tx_carrier_errors+=1;
            }

    #if 0
            if(dwTxStatus&0x00000400UL) {
                SMSC_WARNING("Tx Status NO carrier error");

            }
    #endif
            if(dwTxStatus&0x00000200UL) {
                    SMSC_WARNING("Tx Status Late collision (after 64 bytes) error");
            privateData->stats.collisions++;
            privateData->stats.tx_aborted_errors++;
            }

            if(dwTxStatus&0x00000004UL) {
                SMSC_WARNING("Tx Status Excessive Deferral");
            }
            if(dwTxStatus&0x00000002UL) {
                SMSC_WARNING("Tx Status Underrun error ");
            }
            if(dwTxStatus&0x00000001UL) {
                SMSC_WARNING("Tx Status Packet deferred ");
            }

        }
    }


    spin_unlock(&(privateData->TxCounterLock));
}

void Tx_CompleteDma(PPRIVATE_DATA privateData)
{
    SMSC_ASSERT(privateData!=NULL);

    spin_lock(&(privateData->TxSkbLock));
    if(privateData->TxSkb) {
        Platform_DmaComplete(
            &(privateData->PlatformData),
            privateData->dwTxDmaCh);
        dev_kfree_skb(privateData->TxSkb);
        privateData->TxSkb=NULL;
    }
    spin_unlock(&(privateData->TxSkbLock));
}

void Rx_Initialize(
            PPRIVATE_DATA privateData,
            DWORD dwRxDmaCh,
            DWORD dwDmaThreshold)
{
    SMSC_ASSERT(privateData!=NULL);

    privateData->dwRxDmaCh=dwRxDmaCh;
    if(dwRxDmaCh>=TRANSFER_PIO) {
        SMSC_TRACE_DMA("Rx will use PIO");
        Platform_GetFlowControlParameters(
            &(privateData->PlatformData),
            &(privateData->RxFlowParameters),
            FALSE);
    } else {
        SMSC_TRACE_DMA("Rx will use DMA Channel %ld",dwRxDmaCh);
        SMSC_ASSERT(Platform_IsValidDmaChannel(dwRxDmaCh));
        if(!Platform_DmaInitialize(
            &(privateData->PlatformData),
            dwRxDmaCh))
        {
            SMSC_WARNING("Failed Platform_DmaInitialize, dwRxDmaCh=%lu",dwRxDmaCh);
        }
        Platform_GetFlowControlParameters(
            &(privateData->PlatformData),
            &(privateData->RxFlowParameters),
            TRUE);
    }
    if(max_throughput!=0xFFFFFFFFUL) {
        privateData->RxFlowParameters.MaxThroughput=max_throughput;
    }
    if(max_packet_count!=0xFFFFFFFFUL) {
        privateData->RxFlowParameters.MaxPacketCount=max_packet_count;
    }
    if(packet_cost!=0xFFFFFFFFUL) {
        privateData->RxFlowParameters.PacketCost=packet_cost;
    }
    if(burst_period!=0xFFFFFFFFUL) {
        privateData->RxFlowParameters.BurstPeriod=burst_period;
    }
    if(privateData->RxFlowParameters.BurstPeriod==0) {
        SMSC_WARNING("burst_period of 0 is not allowed");
        SMSC_WARNING(" resetting burst_period to 100");
        privateData->RxFlowParameters.BurstPeriod=100;
    }
    if(max_work_load!=0xFFFFFFFFUL) {
        privateData->RxFlowMaxWorkLoad=max_work_load;
    } else {
        privateData->RxFlowMaxWorkLoad=
            privateData->RxFlowParameters.MaxThroughput+
            (privateData->RxFlowParameters.MaxPacketCount*
            privateData->RxFlowParameters.PacketCost);
    }
    privateData->RxFlowBurstMaxWorkLoad=
            (privateData->RxFlowMaxWorkLoad*
            privateData->RxFlowParameters.BurstPeriod)/1000;
    if(int_deas!=0xFFFFFFFFUL) {
        Lan_SetIntDeas(privateData,int_deas);
    } else {
        Lan_SetIntDeas(privateData,privateData->RxFlowParameters.IntDeas);
    }

    //initially the receiver is off
    //  a following link up detection will turn the receiver on
    privateData->dwRxOffCount=1;
    LanSetDirectRegDW(RX_CFG,0x00000200UL);
    Rx_ReceiverOn(privateData, 0);

    privateData->dwRxDmaThreshold=dwDmaThreshold;
    Lan_SetRDFL(privateData,0x01);
    Lan_SetRSFL(privateData,0x00);
    privateData->RxInterrupts=INT_EN_RSFL_EN_;
    privateData->RxInterrupts|=INT_EN_RXE_EN_;
    if(privateData->dwGeneration==0) {
        privateData->RxInterrupts|=INT_EN_RDFL_EN_;
    } else {
        privateData->RxInterrupts|=INT_EN_RDFO_EN_;
    }
    privateData->RxInterrupts|=INT_EN_RXDFH_INT_EN_;
    spin_lock_init(&(privateData->taggedInfoLock));
    Lan_EnableInterrupt(privateData,privateData->RxInterrupts);
}

/// end of  void Rx_Initialize

static void Rx_HandleOverrun(PPRIVATE_DATA privateData)
{
    if(privateData->dwGeneration==0) {
        if(privateData->RxOverrun==FALSE) {
            Rx_ReceiverOff(privateData);
            privateData->RxUnloadProgress=
                    (((((privateData->LastRxStatus1)&0x3FFF0000UL)>>16)+2+3)&0xFFFFFFFCUL);
            if(privateData->dwRxDmaCh<TRANSFER_REQUEST_DMA) {
                privateData->RxUnloadProgress+=
                    (((((privateData->LastRxStatus2)&0x3FFF0000UL)>>16)+2+3)&0xFFFFFFFCUL);
            }
            privateData->RxUnloadPacketProgress=0;
            privateData->RxOverrun=TRUE;
            privateData->RxOverrunCount++;
        }
    } else {
        privateData->RxOverrunCount++;
    }
}


#ifdef FULLY_MANAGED
int Rx_RestoreTaggedData(PPRIVATE_DATA privateData,
                struct sk_buff *skb)
{
    PTAGGED_DATA_INFO pTaggedInfo;
    int portId = 0,i=0;
    PTAG_FORMAT pFormat;
    DWORD *skb_head, *tag_data;
#ifdef CHIP_IS_LAN9303
    BOOLEAN isRSTP=FALSE, isSnoopedPkt=FALSE;
    DWORD cputag;
    DWORD DApriority;
    MLDPKTTypes SnoopedPktType;
	struct in6_addr *saddr, *daddr;
#endif
    unsigned char *skbData;

    spin_lock(&(privateData->taggedInfoLock));


    privateData->packetType = TYPE_UKNOWN;

    pTaggedInfo = &(privateData->taggedInfo);
    pFormat = (PTAG_FORMAT)(&(pTaggedInfo->tagged_data));

     skb_head = (DWORD *)skb->head;
     tag_data = ((DWORD *)(pTaggedInfo->tagged_data));
     for(i=0; i<(TAGGED_INFO_SIZE/4); i++)
     {
    	 *(tag_data++)=*(skb_head++);
     }
     /* Skb_pull subtracts the length and adjust the data */
     skb_pull(skb, TAG_SIZE);
     skbData = (unsigned char *) skb->data;

     /*
      * Remove the CPU tag, and insert the Destination mac address, Source mac address, and
      * the Type field
      */
        // Copy Dmac
        memcpy(skbData,&(pFormat->dstMac), 12);
        skbData += 12;
        memcpy(skbData, &(pFormat->Type),2);
        pTaggedInfo->valid_tag = FALSE;
        pTaggedInfo->tagged_skb = NULL;

        if ( !pTaggedInfo->isMulticast) {
            //SMSC_TRACE("Not Multicast address  \n");
            spin_unlock(&(privateData->taggedInfoLock));
            return portId;
        }

    if ( memcmp(skb->data,BROADCAST_ADDRESS,6) == 0)  {
        //SMSC_TRACE("!!!!!!!!!! Broadcast Packet !!!!!!!!!!!!!!\n");
        goto DONE;
    }

    /**  it is a multicast packet **/
#ifndef CHIP_IS_LAN9303 /* for LAN9131, 9132,9133 */
    portId = htons(pFormat->PortId);
#else
    cputag = htons(pFormat->PortId);
    portId = (cputag & CPU_TAG_PORT_MASK);
    if(!(cputag&CPU_TAG_ALR_PRIOENBL ))
    {
    	if( (cputag & CPU_TAG_STPORIDE_AND_STATIC))
    	{
    		isRSTP = ((cputag & CPU_TAG_PRIO_ALIAS_ID_FIELD) ==  (CPU_TAG_STP_ID << 7));
    	}
    }
    else
    {
    	DApriority = ((cputag & CPU_TAG_PRIO_ALIAS_ID_FIELD)>>7);
    	skb->priority = DApriority;
    }
    isSnoopedPkt = ((cputag&CPU_TAG_SNOOP_MSK));
#endif
    //SMSC_TRACE("\n SKP is for  Multicast address: portID = %d  \n", portId);
    //dumpSkb(skb);

    // If we are here, that means packet needs special attention.
    if (privateData->rstpEnabled)  {
#ifndef CHIP_IS_LAN9303
    	if ( memcmp(skb->data,RSTP_MAC_ADDRESS,6) == 0) {
#else
    		if(isRSTP) {
#endif
    		SMSC_TRACE("\n SKP is STP packe: portID= %d\n", portId);
    		privateData->packetType = STP_TYPE_PACKET;
    		atomic_set(&skb->users,portId);
    		stpAddSkpToRxQueue(privateData, skb);
    		goto DONE;
    	}
    }
#ifndef CHIP_IS_LAN9303
    if (privateData->ptpStarted) {
    	if ( isPTPPacket(privateData, pTaggedInfo->tagged_data)) {
    		SMSC_TRACE("\n SKB is PTP packet: portID= %d\n", portId);
    		//dumpSkb(skb);

    		privateData->packetType = PTP_TYPE_PACKET;
    		atomic_set(&skb->users,portId);
    		ptpAddSkpToRxQueue(privateData, skb);
    		goto DONE;
    	}
    }
#endif
    if (privateData->SnoopingStarted ) {
#ifndef CHIP_IS_LAN9303
    	igmpPaktype=isIGMPPacket(skb->data,skb->len);
    	if (igmpPaktype) {
    		privateData->packetType = PKT_TYPE_IS_IGMP;
    		atomic_set(&skb->users,portId);
    		igmpAddSkpToRxQueue(privateData, skb);
    		/*	    dumpSkb(skb);*/
    		privateData->SnoopedPktCnt++;
    		goto DONE;
    	}
#else
    	if(isSnoopedPkt) {
    		 SnoopedPktType= isIPV6Pkt(skb->data, skb->len);
    		 switch(SnoopedPktType)
    		 {
    		 	case NOT_AN_IPV6PKT:
    		 		atomic_set(&skb->users,((PKT_TYPE_IS_IGMP<<4)| (portId&0xF)));
    		 		privateData->packetType = PKT_TYPE_IS_IGMP;
    		 		break;
    		 	case IPV6_MLD_PKT:
    		 		if(icmpv6CksumValidate)
    		 		{
    		 			saddr = &skb->nh.ipv6h->saddr;
    		 			daddr = &skb->nh.ipv6h->daddr;
    	                skb->csum = ~csum_ipv6_magic(saddr, daddr, skb->len,IPPROTO_ICMPV6, 0);

    	                if (__skb_checksum_complete(skb)) {
    	                        SMSC_ERROR("ICMPv6 checksum failed ");
    	                        atomic_set(&skb->users, portId);
    	                        privateData->packetType = IPV6_BUT_NOT_AN_MLD_PKT;
    	                        goto DONE;
    	                }
    		 		}
    		 		atomic_set(&skb->users,((PKT_TYPE_IS_MLD<<4)| (portId&0xF)));
    		 		privateData->packetType = PKT_TYPE_IS_MLD;
    		 		break;
    		 	case IPV6_NO_ICMP_BUT_RTR_ALRT_MLD_PKT:
    		 	case IPV6_BUT_NOT_AN_MLD_PKT:
    		 		atomic_set(&skb->users, portId);
    		 		privateData->packetType = IPV6_BUT_NOT_AN_MLD_PKT;
    		 		goto DONE;
    		 }
    		igmpAddSkpToRxQueue(privateData, skb);
    		/*	    dumpSkb(skb);*/
    		privateData->SnoopedPktCnt++;
    		goto DONE;
    	}
#endif
    }
    portId = 0;

DONE:
    spin_unlock(&(privateData->taggedInfoLock));
    return   portId;
}

#endif /** FULLY_MANAGED */

void  dumpSkb(struct sk_buff *skb)
{
    WORD len = 0;
    WORD  align =0;
    unsigned char *pData;

    if (!skb)
        return;

    pData = (unsigned char *) (skb->data);
    for (  len = 0; len < skb->len; len++)
    {
        printk("%02x ",(int) (*pData));
        if (align++ == 15)
        {
            printk("\n");
            align = 0;
        }
        pData++;
    }
    printk("\n");
}

#ifdef NAPI_PROCESS
static int Smsc9132_rx_poll(struct net_device *dev,int * budget)
{
        int result=0;
        PPRIVATE_DATA privateData=NULL;
        SMSC_ASSERT(dev!=NULL);
        SMSC_ASSERT(budget!=NULL);
        privateData=((PPRIVATE_DATA)(dev->priv));
        SMSC_ASSERT(privateData!=NULL);

//        SET_SYS_GPO(GP_RXT);

        privateData->RxWorkLimit=dev->quota;
        if((privateData->RxWorkLimit)>(*budget)) {
                privateData->RxWorkLimit=(*budget);
        }
        privateData->RxPacketsReceived=0;
        privateData->RxDone=FALSE;

        Rx_ProcessPackets(privateData);

        dev->quota-=privateData->RxPacketsReceived;
        (*budget)-=privateData->RxPacketsReceived;

        if(privateData->RxDone) {
                netif_rx_complete(dev);
                Lan_ClearIntDeasIntvl(privateData);
//                privateData->RxProcessingMode=PROCESSING_MODE_IDLE;
                Lan_EnableIRQ(privateData);
        } else {
                result=1;
        }
//        CLEAR_SYS_GPO(GP_RXT);
        return result;
}
#endif

static void Rx_HandOffSkb(
                        PPRIVATE_DATA privateData,
                        struct sk_buff *skb
                        )
{
    int result=0;

#ifdef FULLY_MANAGED
    struct sk_buff *tempskb=NULL;
    int portId = 0;
    /** Only in  fully  managed switch,
    switch port can send and receive data to/from host
    The MAC addreses are stored in Platform
    TAGGED_DATA_INFO  structure.
    skb->dev=privateData->switchDev[1];
    atomic_add(1,&skb->users);  // port Number
    send data to STP/1588 or to network stack
    for 1588 or RSTO, use AF_NET socket
    **/
    portId =  Rx_RestoreTaggedData(privateData, skb);
    if(mac_sniffing_on) {
    	if((memcmp(mac_sniffing,MAC_FF_FF_FF_FF_FF_FF,6)==0)||
    		(memcmp(mac_sniffing,skb->data,6)==0)||
    		(memcmp(mac_sniffing,&(((unsigned char *)(skb->data))[6]),6)==0))
    	{
    		SMSC_TRACE("DST=%02X:%02X:%02X:%02X:%02X:%02X,SRC=%02X:%02X:%02X:%02X:%02X:%02X",
    				((unsigned char *)(skb->data))[0],
    				((unsigned char *)(skb->data))[1],
    				((unsigned char *)(skb->data))[2],
    				((unsigned char *)(skb->data))[3],
    				((unsigned char *)(skb->data))[4],
    				((unsigned char *)(skb->data))[5],
    				((unsigned char *)(skb->data))[6],
    				((unsigned char *)(skb->data))[7],
    				((unsigned char *)(skb->data))[8],
    				((unsigned char *)(skb->data))[9],
    				((unsigned char *)(skb->data))[10],
    				((unsigned char *)(skb->data))[11]);
    	}
    }
	//#ifdef DUMP_RX_PACKET
	if(debug_mode&0x20)
	{
		SMSC_TRACE("Printing receive data Before:");
		dumpSkb(skb);
	}
	//#endif

    if (portId < 0 )
    {
        // The packet is one of those we are looking for
        SMSC_ERROR ("UNKOWN PORT ID " );
        dev_kfree_skb(skb);
        return;
    }
    else if  ( (portId > 0) && (portId < PHY_NUMBER ))
    {

        skb->dev = privateData->switchDev[portId];

        if (privateData->packetType == STP_TYPE_PACKET)
        {
	      skb->protocol= eth_type_trans(skb,skb->dev);
            // SMSC_TRACE ("!!!!!! Schedule STP !!!!" );
            tasklet_schedule(&STP_Tasklet);
        }
#ifndef CHIP_IS_LAN9303
        else if (privateData->packetType == PTP_TYPE_PACKET)
        {
            //SMSC_TRACE ("Schedule PTP " );
	        skb->protocol= eth_type_trans(skb,skb->dev);
            tasklet_schedule(&PTP_Tasklet);
        }
#endif
        else if( (privateData->packetType == PKT_TYPE_IS_IGMP) || (privateData->packetType == PKT_TYPE_IS_MLD) ||
        			(privateData->packetType == IPV6_BUT_NOT_AN_MLD_PKT)){

        	if(privateData->SnoopingStarted)
        	{
#ifdef CHIP_IS_LAN9303
        		if(privateData->packetType == IPV6_BUT_NOT_AN_MLD_PKT)
        		{
        			SMSC_TRACE("NOT an IPV6 MLD packet forwarding on the other Port\n");
        		   if(portId ==1)
        			   skb->dev=privateData->switchDev[2];
        		   else
        			  skb->dev= privateData->switchDev[1];

        			 dev_queue_xmit(skb);
        			 return;
        		}
#endif
        		/* if we are getting IGMP quries forward to Network stack also*/
        		if (igmpPaktype==2)
        		{
        			tempskb=skb_copy(skb,GFP_ATOMIC);
        			//BW: Fix for BUG 472
        			//must reassign dev to the root device interface.
        			// or else the packet will not go up the proper network stack.
        			tempskb->dev=privateData->dev;
        			//		SMSC_TRACE ("Schedule IGMP QUERY ");
        			tempskb->protocol= eth_type_trans(tempskb,tempskb->dev);
        			tempskb->ip_summed = CHECKSUM_NONE;
#ifdef NAPI_PROCESS
                    if(rx_mode==PROCESSING_MODE_NAPI) {
                            result=netif_receive_skb(tempskb);
                            privateData->RxWorkLimit--;
                            privateData->RxPacketsReceived++;
                            if(privateData->RxWorkLimit<=0) {
                                    privateData->RxCongested=TRUE;
                            }
                    }else
                    {
                    	result=netif_rx(tempskb);
                    }
#else
        			result=netif_rx(tempskb);
#endif
        			igmpPaktype=0;
        		}
        	}
        	tasklet_schedule(&IGMP_Tasklet);
        }
        else {
        	/* This case never should happen unless there is a corruption in the heap */
        	dev_kfree_skb(skb);
        	SMSC_ERROR ("UNKOWN SPECIAL TAG " );
        }

        return;
    }
    else
    {
    	skb->dev=privateData->dev;
    }
#else

    skb->dev=privateData->dev;

#endif  /** FULLY_MANAGED*/

#ifndef FULLY_MANAGED
  if(debug_mode&0x20)
	{
        SMSC_TRACE("Printing receive data Before:");
        dumpSkb(skb);
	}
#endif


    skb->protocol= eth_type_trans(skb,skb->dev);
    skb->ip_summed = CHECKSUM_NONE;


#ifdef DUMP_RX_PACKET
    SMSC_TRACE("\nRx_HandOffSkb: skb: === dev=%s ===,proto=0x%04x \n",skb->dev->name,htons(skb->protocol));

#endif
#ifdef NAPI_PROCESS
    if(rx_mode==PROCESSING_MODE_NAPI) {
            result=netif_receive_skb(skb);
            privateData->RxWorkLimit--;
            privateData->RxPacketsReceived++;
            if(privateData->RxWorkLimit<=0) {
                    privateData->RxCongested=TRUE;
            }
    }else
    {
    	result=netif_rx(skb);
    }
#else
    result=netif_rx(skb);
#endif
#ifdef FULLY_MANAGED
//FORWARD_RESULT:
#endif
    switch(result)
    {
        case NET_RX_SUCCESS:
            break;
        case NET_RX_CN_LOW:
        case NET_RX_CN_MOD:
        case NET_RX_CN_HIGH:
        case NET_RX_DROP:
            privateData->RxCongested=TRUE;
            privateData->RxCongestedCount++;
            SMSC_WARNING("!!!!!!Return value from netif_rx, result=%d!!!!!!",result);
            break;
        default:
            privateData->RxCongested=TRUE;
            privateData->RxCongestedCount++;
            SMSC_WARNING("Unknown return value from netif_rx, result=%d",result);
            break;
    }
}

void Rx_CompleteMulticastUpdate (PPRIVATE_DATA privateData)
{
    DWORD local_MACCR;
    VL_KEY keyCode=0;
    DWORD dwIntFlags=0;

    keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
    if (privateData->MulticastUpdatePending) {
        // mobbad? SET_GPIO(GP_COMPLETE_MULTICAST_UPDATE);
        Mac_SetRegDW(privateData,HASHH,privateData->HashHi,keyCode);
        Mac_SetRegDW(privateData,HASHL,privateData->HashLo,keyCode);
        local_MACCR = Mac_GetRegDW(privateData,MAC_CR,keyCode);
        local_MACCR |= privateData->set_bits_mask;
        local_MACCR &= ~(privateData->clear_bits_mask);
        Mac_SetRegDW(privateData,MAC_CR,local_MACCR,keyCode);
        Rx_ReceiverOn(privateData, keyCode);
        privateData->MulticastUpdatePending = FALSE;
        //CLEAR_GPIO(GP_COMPLETE_MULTICAST_UPDATE);
    }
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
}

void Rx_BeginMulticastUpdate (PPRIVATE_DATA privateData)
{
DWORD startTime, currentTime;
DWORD timeout;
DWORD flags;

// mobbad? SET_GPIO(GP_BEGIN_MULTICAST_UPDATE);

//NOTE: we can't rely on privateData->dwLinkSpeed because
// it updates only once per second and may be out dated.

local_irq_save(flags);
Rx_ReceiverOff(privateData);
if(privateData->dwGeneration>0) {
    //since this is concord or later there is no
    // overrun processing that might turn off the receiver.
    // there for we can rely on RxStop Int.

    //if the speed is 100Mb then lets poll rx stop to get the
    //  quickest response.
    timeout = 200UL;
    while ((timeout)&&(!(LanGetDirectRegDW(INT_STS)&(INT_STS_RXSTOP_INT_)))) {
        // wait 1 uSec
        startTime=LanGetDirectRegDW(FREE_RUN);
        while (1) {
            currentTime=LanGetDirectRegDW(FREE_RUN);
            if (currentTime-startTime >= 25UL)
                break;
        }
        timeout--;
    }
    if(timeout==0) {
        //this is probably a 10Mb link, therefore prepare
        // interrupt for update later.
        Lan_EnableInterrupt(privateData,INT_EN_RXSTOP_INT_EN_);

        // if this is a 10Mbps half duplex connection
        //  then Rx stop is only 99.6%  reliable
        //  Therefor we must schedule Gpt callback as
        //  back up

        // using 18*(100uS) because we already waited 200uS
        Gpt_ScheduleCallBack(privateData,GptCB_RxCompleteMulticast, 18UL);
    } else {
        //Rx is stopped
        LanSetDirectRegDW(INT_STS,INT_STS_RXSTOP_INT_);//clear interrupt signal
        Rx_CompleteMulticastUpdate(privateData);
    }
} else {
    // for generation 0 we can't rely on Rx stop because
    // the receiver may have already been stopped due to
    // overflow processing

    // for the same reason we can't just wait 200uS and
    // check stopped status there for we must rely on GP timer
    // and we must assume a worse case of 10Mb speed

    Gpt_ScheduleCallBack(privateData,GptCB_RxCompleteMulticast, 20UL);
}
local_irq_restore (flags);
//CLEAR_GPIO(GP_BEGIN_MULTICAST_UPDATE);
}

static DWORD Rx_PopRxStatus(
PPRIVATE_DATA privateData)
{
DWORD result=0;
result=LanGetDirectRegDW(RX_FIFO_INF);
if((privateData->RxCongested==FALSE)||
    ((privateData->RxCongested==TRUE)&&((result&0x00FF0000UL)==0UL)))
{
    if(result&0x00FF0000UL) {
        DWORD dwIntSts=LanGetDirectRegDW(INT_STS);
        if(privateData->dwGeneration==0) {
            if(dwIntSts&INT_STS_RDFL_) {
                LanSetDirectRegDW(INT_STS,INT_STS_RDFL_);
                Rx_HandleOverrun(privateData);
            }
        } else {
            if(dwIntSts&INT_STS_RDFO_) {
                LanSetDirectRegDW(INT_STS,INT_STS_RDFO_);
                Rx_HandleOverrun(privateData);
            }
        }
        if((privateData->RxFlowControlActive==FALSE)||
            ((privateData->RxFlowControlActive==TRUE)&&
            (privateData->RxFlowBurstActive==TRUE)))
        {
            //Rx status is available, read it
            result=LanGetDirectRegDW(RX_STATUS_FIFO_PORT);
            privateData->RxStatusDWReadCount++;
            privateData->LastRxStatus3=
                privateData->LastRxStatus2;
            privateData->LastRxStatus2=
                privateData->LastRxStatus1;
            privateData->LastRxStatus1=result;

            if(privateData->RxOverrun) {
                DWORD dwPacketLength=((result&0x3FFF0000UL)>>16);
                DWORD dwByteCount=((dwPacketLength+2+3)&0xFFFFFFFCUL);
                if((privateData->RxUnloadProgress+dwByteCount)>=
                    ((privateData->RxMaxDataFifoSize)-16))
                {
                    //This is the packet that crosses the corruption point
                    //  so just ignore it and complete the overrun processing.
                    result=0;
                    goto FINISH_OVERRUN_PROCESSING;
                }
                privateData->RxUnloadProgress+=dwByteCount;
                privateData->RxUnloadPacketProgress++;
            }

            privateData->RxFlowCurrentThroughput+=
                    ((((result&0x3FFF0000UL)>>16)-4UL));
            privateData->RxFlowCurrentPacketCount++;
            privateData->RxFlowCurrentWorkLoad+=
                    ((((result&0x3FFF0000UL)>>16)-4UL)+privateData->RxFlowParameters.PacketCost);
            if(privateData->RxFlowControlActive) {
                privateData->RxFlowBurstWorkLoad+=
                    ((((result&0x3FFF0000UL)>>16)-4UL)+privateData->RxFlowParameters.PacketCost);
                if(privateData->RxFlowBurstWorkLoad>=
                    privateData->RxFlowBurstMaxWorkLoad)
                {
                    privateData->RxFlowBurstActive=FALSE;
                    Lan_DisableInterrupt(privateData,privateData->RxInterrupts);
                }
            }
        } else {
            result=0;
        }
    }
    else
    {
        if(privateData->RxOverrun) {
            DWORD timeOut;
            DWORD temp;
FINISH_OVERRUN_PROCESSING:
            temp=0;
            {
                timeOut=2000;
                while((timeOut>0)&&(!(LanGetDirectRegDW(INT_STS)&(INT_STS_RXSTOP_INT_)))) {
                    udelay(1);
                    timeOut--;
                }
                if(timeOut==0) {
//						privateData->RxStopTimeOutCount++;
//						PULSE_GPIO(GP_TX,1);
                    SMSC_WARNING("Timed out waiting for Rx to Stop\n");
                }
                LanSetDirectRegDW(INT_STS,INT_STS_RXSTOP_INT_);
            }

            if(privateData->dwRxDmaCh<TRANSFER_REQUEST_DMA) {
                //make sure DMA has stopped before doing RX Dump
                if(privateData->RxSkb) {
                    Platform_DmaComplete(
                        &(privateData->PlatformData),
                        privateData->dwRxDmaCh);

                    Rx_HandOffSkb(privateData,privateData->RxSkb);
                    privateData->RxSkb=NULL;
                }
            }

            temp=LanGetDirectRegDW(RX_CFG);
            LanSetDirectRegDW(RX_CFG,(temp&0x3FFFFFFFUL));
            timeOut=10000000;
            Lan_SetBitsDW(RX_CFG,RX_CFG_RX_DUMP_);
            while((timeOut>0)&&(LanGetDirectRegDW(RX_CFG)&(RX_CFG_RX_DUMP_))) {
                udelay(1);
                timeOut--;
            }
            if(timeOut==0) {
                SMSC_WARNING("Timed out waiting for Rx Dump to complete\n");
            }
            LanSetDirectRegDW(RX_CFG,temp);

            privateData->RxDumpCount++;
            LanSetDirectRegDW(INT_STS,INT_STS_RDFL_);
            Rx_ReceiverOn(privateData, 0);
            privateData->RxOverrun=FALSE;
        }
        result=0;
        privateData->LastReasonForReleasingCPU=1;//Status FIFO Empty
    }
} else {
    //disable and reenable the INT_EN
    //  This will allow the deassertion interval to begin
    DWORD temp=LanGetDirectRegDW(INT_EN);
    LanSetDirectRegDW(INT_EN,0);
    LanSetDirectRegDW(INT_EN,temp);
    result=0;
    privateData->LastReasonForReleasingCPU=2;//High Congestion
}

return result;
}

#define RX_STATUS_ERROR           0x00008000UL    // bit 15
#define RX_STATUS_LENGTH_ERROR    0x00001000UL    // bit 12
#define RX_STATUS_RUNT_FRAME      0x00000800UL    // bit 11
#define RX_STATUS_OVERSIZE_ERROR  0x00000080UL    // bit 7
#define RX_STATUS_MII_ERROR       0x00000008UL    // bit 3
#define RX_STATUS_DRIBBLING_ERROR 0x00000004UL
#define RX_STATUS_CRC_ERROR       0x00000002UL


BOOLEAN Rx_CountErrors(PPRIVATE_DATA privateData,DWORD dwRxStatus)
{
    BOOLEAN crcError=FALSE;
    BOOLEAN ignorePkt=FALSE;
    if(dwRxStatus&0x00008000UL) {
        privateData->stats.rx_errors++;
        if(dwRxStatus&0x00000002UL) {
            privateData->stats.rx_crc_errors++;
            crcError=TRUE;
        }
    }
    if(!crcError) {
//    	printk("Checking the Error Status =0x%x\n", dwRxStatus);
        if((dwRxStatus&0x00000020UL)==0x00000000UL) {
        	if((dwRxStatus&0x00001000UL)==0x00001000UL) {
            //Frame type indicates length, and length error is set
            privateData->stats.rx_length_errors++;
            ignorePkt = TRUE;
        	}
        }
        if(dwRxStatus&RX_STS_MCAST_) {
            privateData->stats.multicast++;
        }
    }
#ifdef DEBUG_RX
    if(dwRxStatus&0x00008000UL) {
    if(dwRxStatus&RX_STATUS_LENGTH_ERROR)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_LENGTH_ERROR  Error \n");
    if(dwRxStatus&RX_STATUS_RUNT_FRAME)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_RUNT_FRAME Error \n");
    if(dwRxStatus&RX_STATUS_OVERSIZE_ERROR)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_OVERSIZE_ERROR Error \n");
    if(dwRxStatus&RX_STATUS_MII_ERROR)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_MII_ERROR Error \n");

    if(dwRxStatus&RX_STATUS_DRIBBLING_ERROR)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_DRIBBLING_ERROR Error \n");
    if(dwRxStatus&RX_STATUS_CRC_ERROR)
        SMSC_TRACE("RECEIVE ERROR : RX_STATUS_CRC_ERROR Error\n");
    if((dwRxStatus&0x00000020)==0x00000020UL)
    {
    	if((dwRxStatus&0x00001000)==0x000001000) {
    	SMSC_TRACE("RECEIVE ERROR: RX_STATUS_IEEE802_3_LENGTH_Error \n");
    	}
    }
    }
#endif
    return ignorePkt;
}

void Rx_FastForward(PPRIVATE_DATA privateData,DWORD dwDwordCount)
{
    BOOLEAN ignore_fast_forward = FALSE;
    privateData->RxFastForwardCount++;

#ifdef DUMP_PACKET
    ignore_fast_forward = TRUE;
#endif

    if ( (dwDwordCount>=4) && (!ignore_fast_forward))

    {
        DWORD dwTimeOut=500;
        SMSC_WARNING("!!! Rx_FastForward !!!! \n");
        LanSetDirectRegDW(RX_DP_CTL,(dwDwordCount|RX_DP_CTL_FFWD_BUSY_));
        while((dwTimeOut)&&(LanGetDirectRegDW(RX_DP_CTL)&
                RX_DP_CTL_FFWD_BUSY_))
        {
            udelay(1);
            dwTimeOut--;
        }
        if(dwTimeOut==0) {

            SMSC_WARNING("timed out waiting for RX FFWD to finish, RX_DP_CTRL=0x%08lX",
                LanGetDirectRegDW(RX_DP_CTL));
        }
    } else {
        unsigned char temp[2000];
        Platform_ReadFifo(privateData->dwLanBase,((DWORD *)(temp)),
                dwDwordCount);

#ifdef DUMP_PACKET
    SMSC_TRACE("Rx_FastForward:Printing received frame to be discarded: \n");
    {
        WORD len = 0;
        WORD  align =0;
        unsigned char *pData = (unsigned char *) (temp);
        printk(" len = %lu \n", dwDwordCount << 2);
        for (  len = 0; len < dwDwordCount << 2; len++)
        {
            printk("%02x ",(int) (*pData));
            if (align++ == 16)
            {
                printk("\n");
                align = 0;
            }
            pData++;
        }
    }
#endif

    }
}

//Rx_ReceiverOff, and Rx_ReceiverOn use a reference counter
//  because they are used in both the Rx code and the link management count
void Rx_ReceiverOff(PPRIVATE_DATA privateData)
{
    DWORD dwIntFlags=0;
    VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
    if(privateData->dwRxOffCount==0) {
        DWORD dwMacCr=Mac_GetRegDW(privateData,MAC_CR,keyCode);
        if(!(dwMacCr&MAC_CR_RXEN_)) {
            SMSC_WARNING("Rx_ReceiverOff: Receiver is already Off");
        }
        dwMacCr&=(~MAC_CR_RXEN_);
        Mac_SetRegDW(privateData,MAC_CR,dwMacCr,keyCode);
        //CLEAR_GPIO(GP_RX);
    }
    privateData->dwRxOffCount++;
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
}

//Rx_ReceiverOff, and Rx_ReceiverOn use a reference counter
//  because they are used in both the Rx code and the link management count

void Rx_ReceiverOn(PPRIVATE_DATA privateData, VL_KEY callerKeyCode)
{
    DWORD dwIntFlags=0;
    VL_KEY keyCode=0;

    if (callerKeyCode == 0) {
        keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
    }
    else {
        SMSC_ASSERT(Vl_CheckLock(&(privateData->MacPhyLock),callerKeyCode));
        keyCode = callerKeyCode;
    }
    if(privateData->dwRxOffCount>0) {
        privateData->dwRxOffCount--;
        if(privateData->dwRxOffCount==0) {
            DWORD dwMacCr=Mac_GetRegDW(privateData,MAC_CR,keyCode);
            if(dwMacCr&MAC_CR_RXEN_) {
                SMSC_WARNING("Rx_ReceiverOn: Receiver is already on");
            }
            dwMacCr|=MAC_CR_RXEN_;

#ifdef ENABLE_BAD_PACKET
            dwMacCr |= (MAC_CR_PASSBAD_ | MAC_CR_RXALL_);
#endif

            Mac_SetRegDW(privateData,MAC_CR,dwMacCr,keyCode);
            //SET_GPIO(GP_RX);
        }
    } else {
        SMSC_ASSERT(FALSE);
    }
    if (callerKeyCode == 0) {
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    }
}



void Rx_ProcessPackets(PPRIVATE_DATA privateData)
{

    DWORD dwRxStatus=0;
    PPLATFORM_DATA platformData=NULL;
    DWORD tryAgain=0;
    BOOLEAN ignorePkt=FALSE;
    unsigned char  *skb_offset;
    DWORD dwPacketLength;
#ifdef FULLY_MANAGED
    PTAGGED_DATA_INFO pTaggedInfo;
#endif
    SMSC_ENTER
    //	SET_GPIO(GP_RX);
    privateData->RxCongested=FALSE;
    platformData=&(privateData->PlatformData);
    privateData->RxSkb=NULL;
#ifdef FULLY_MANAGED
    pTaggedInfo = &(privateData->taggedInfo);
#endif
#ifdef TEST_DMA
    nb_rxPorcessInterrupt++;
#endif

    if(privateData->dwRxDmaCh >= TRANSFER_PIO) {
        //Use PIO only
        LanSetDirectRegDW(RX_CFG,0x00000200UL);
        while((!(privateData->RxCongested) )&&(dwRxStatus=Rx_PopRxStatus(privateData))!=0)
        {
            dwPacketLength=((dwRxStatus&0x3FFF0000UL)>>16);
            ignorePkt = Rx_CountErrors(privateData,dwRxStatus);
            // Check if actual frame has error
            if( ((dwRxStatus & RX_STS_ES_) == 0) && (ignorePkt==FALSE) ){
                struct sk_buff *skb=NULL;
                unsigned char  *skb_offset;
                skb=dev_alloc_skb(dwPacketLength+2);
                if(skb!=NULL) {
                    skb->data=skb->head;
                    skb->tail=skb->head;
                    skb_reserve(skb,2); // align IP on 16B boundary
#ifdef FULLY_MANAGED
                    if (dwRxStatus&RX_STS_MCAST_)
                    {
                        pTaggedInfo->isMulticast = TRUE;
                    }
#endif


                    skb_put(skb,dwPacketLength-4UL);

                    //update counters
                    privateData->stats.rx_packets++;
                    privateData->stats.rx_bytes+=(dwPacketLength-4);

                    privateData->RxPacketReadCount++;
                    privateData->RxPioReadCount++;
                    privateData->RxDataDWReadCount+=
                        (dwPacketLength+2+3)>>2;
                    // Add pointer to where data is stored
                    skb_offset = (unsigned char  *) (skb->head);
                    dwPacketLength += (2+3); //2 bytes with 3 bytes of offset for word alignment
                    dwPacketLength >>= 2;
                    Platform_ReadFifo(
                        privateData->dwLanBase,
                        ((DWORD *)(skb_offset)),
                        (dwPacketLength));

                    //SMSC_TRACE("--> 1.Rx_HandOffSkb");
                    Rx_HandOffSkb(privateData,skb);
                    continue;
                } else {
                    SMSC_WARNING("Unable to allocate sk_buff for RX Packet, in PIO path");
                    privateData->stats.rx_dropped++;
                }
            }
            //if we get here then the packet is to be read
            //  out of the fifo and discarded
            {
                dwPacketLength+=(2+3);
                dwPacketLength>>=2;
                Rx_FastForward(privateData,dwPacketLength);
            }
        }/* For while */
    } else {
        //Use DMA and PIO
        DWORD dwDmaCh=privateData->dwRxDmaCh;
        //struct sk_buff *dmaSkb=NULL;//use privateData->RxDmaSkb
        DMA_XFER dmaXfer;
        dmaXfer.dwLanReg=privateData->dwLanBase+RX_DATA_FIFO;
        dmaXfer.pdwBuf=NULL;// this will be reset per dma request
        dmaXfer.dwDmaCh=dwDmaCh;
        dmaXfer.dwDwCnt=0;// this will be reset per dma request
        dmaXfer.fMemWr=TRUE;
//    	printk("1: >Transfer PIO and DMA\n");

        while((!(privateData->RxCongested) )&&(dwRxStatus=Rx_PopRxStatus(privateData))!=0)
        {

            DWORD dwPacketLength;
RUN_AGAIN:
           ignorePkt = Rx_CountErrors(privateData,dwRxStatus);
            dwPacketLength=((dwRxStatus & 0x3FFF0000UL)>>16);
            if( ((dwRxStatus & RX_STS_ES_) == 0) && (ignorePkt == FALSE) )
            {
                struct sk_buff *skb;
                DWORD dwPlatformCacheLineSize =  platformGetCacheLineSize();
                skb = dev_alloc_skb(dwPacketLength+2*dwPlatformCacheLineSize);
                if(skb!=NULL)
                {
                    skb->data=skb->head;
                    skb->tail=skb->head;
                   //align IP on cache line boundary
                    privateData->stats.rx_packets++;
                    privateData->stats.rx_bytes+=(dwPacketLength-4UL);
                    if(dwPacketLength >= privateData->dwRxDmaThreshold)
                    {
                        //use DMA
                        DWORD dwDwordCount;
                        // In FULLY managed, we need to read some data using PIO first.
                        // a nd strip the special tag.
                        unsigned char  *skb_offset;
                        SMSC_TRACE("\n\n\n...........Using DMA \n");
                        skb_reserve(skb,2);
                        skb_put(skb,dwPacketLength-4UL);
                        skb_offset = (unsigned char  *) (skb->head);
                        dwDwordCount=((dwPacketLength+
                                (dwPlatformCacheLineSize - 14)+
                                dwPlatformCacheLineSize-1)&
                                (~(dwPlatformCacheLineSize-1)))>>2;

                        Platform_CacheInvalidate(
                            platformData,
                            skb->head,dwDwordCount<<2);
                        //set end alignment and offset
                        switch(dwPlatformCacheLineSize)
                        {
                        //case 4: LanSetRegDW(RX_CFG,0x00000200UL);break;
                        case 16:LanSetDirectRegDW(RX_CFG,0x40000200UL);break;
                        case 32:LanSetDirectRegDW(RX_CFG,0x80001200UL);break;
                        default:SMSC_ASSERT(FALSE);
                        }

                        if(privateData->RxSkb)
                        {
                            SMSC_TRACE_DMA(".......Waiting for DMA completion 1:  privateData->RxSkb= 0x%lX",
                                    (DWORD)(privateData->RxSkb) );
                            Platform_DmaComplete(platformData,dwDmaCh);
                            Rx_HandOffSkb(privateData,privateData->RxSkb);
                            privateData->RxSkb = NULL;
                        }
                        dmaXfer.pdwBuf=(DWORD *)(skb_offset);
                        dmaXfer.dwDwCnt=dwDwordCount;
                        privateData->RxDataDWReadCount+=dwDwordCount;
                        privateData->RxPacketReadCount++;
                        privateData->RxDmaReadCount++;
                        SMSC_TRACE_DMA("Seting DMA count: Rx Bytes =%lu, dma count =%lu, DMA Addr=0x%lX", dwPacketLength,
                                dwDwordCount,(DWORD ) (dmaXfer.pdwBuf));
                        SMSC_TRACE_DMA(".........  Start DMA for new skb = 0x%lX ",(DWORD)(skb));
#ifdef FULLY_MANAGED
                        if (dwRxStatus&RX_STS_MCAST_)
                        {
                        	pTaggedInfo->isMulticast = TRUE;
                        }
#endif

                        // DMA for RX
#ifdef USE_VALIDATION_DMA
                        if(!DmaStartXfer(platformData,&dmaXfer)) {
                            SMSC_WARNING("Failed Platform_DmaStartXfer");
                        }
#else
                        if(!Platform_DmaStartXfer(platformData,&dmaXfer)) {
                            SMSC_WARNING("Failed Platform_DmaStartXfer \n");
                        }
#endif
                        privateData->RxSkb=skb;
                        privateData->rxStatus= dwRxStatus;

                    }
                    else
                    {
  //                  	printk("\n\n\n2: >Transfer PIO\n");
                        if(privateData->RxSkb) {
                            SMSC_TRACE_DMA(".......Waiting for DMA completion (in PIO) 1:  privateData->RxSkb= 0x%lX",
                                     (DWORD)(privateData->RxSkb) );

                             Platform_DmaComplete(platformData,dwDmaCh);
                             Rx_HandOffSkb(privateData,privateData->RxSkb);
                            // LanSetDirectRegDW(RX_CFG,0x00000200UL);//4 byte end alignment
                             privateData->RxSkb = NULL;
                             SMSC_TRACE_DMA(".....PIO--DMA completed for privateData->RxSkb =0x%lX ...handed over\n", (DWORD)(privateData->RxSkb));
                         }
                       //  privateData->RxSkb=skb;

                         skb_reserve(skb,2); // align IP on 16B boundary

                         skb_put(skb,dwPacketLength-4UL);

                         LanSetDirectRegDW(RX_CFG,0x00000200UL);
                         //update counters
                         privateData->RxPacketReadCount++;
                         privateData->RxPioReadCount++;
                         privateData->RxDataDWReadCount+=
                        	 		(dwPacketLength+2+3)>>2;
                         // Add pointer to where data is stored
                         skb_offset = (unsigned char  *) (skb->head);
                         dwPacketLength += (2+3); //2 bytes with 3 bytes of offset for word alignment
                         dwPacketLength >>= 2;
#ifdef FULLY_MANAGED
                        if (dwRxStatus&RX_STS_MCAST_)
                        {
                        	pTaggedInfo->isMulticast = TRUE;
                        }
#endif
                        Platform_ReadFifo(
                        		privateData->dwLanBase,
                        		((DWORD *)(skb_offset)),
                        		(dwPacketLength));

//                        SMSC_TRACE("--> 1.Rx_HandOffSkb");
                        Rx_HandOffSkb(privateData,skb);
                       // LanSetDirectRegDW(RX_CFG,0x00000200UL);//4 byte end alignment
                       // Rx_FastForward(privateData,dwPacketLength);
                        privateData->RxSkb = NULL;
                    }
                    continue;
                }  // if(skb!=NULL)
                else
                {
                    SMSC_WARNING("Unable to allocate sk_buff for RX Packet, in DMA path");
                    privateData->stats.rx_dropped++;
                }
            } // if((dwRxStatus & RX_STS_ES_) == 0)

            //if we get here then the packet is to be read
            //  out of the fifo and discarded
            SMSC_WARNING("Packet has error ");
            if(privateData->RxSkb) Platform_DmaComplete(platformData,dwDmaCh);
            //delay returning the dmaSkb to OS till later
            dwPacketLength+=(2+3);
            dwPacketLength>>=2;
            LanSetDirectRegDW(RX_CFG,0x00000200UL);//4 byte end alignment
            Rx_FastForward(privateData,dwPacketLength);
        }  //while((dwRxStatus=Rx_PopRxStatus(privateData))!=0)

        if(privateData->RxSkb) {
            //while waiting for dma to complete,
            // check if another packet arrives

            DWORD dwTimeOut=1000000;
            while((Platform_DmaGetDwCnt(platformData,dwDmaCh)) &&
                (dwTimeOut))
            {
                    if((dwRxStatus=Rx_PopRxStatus(privateData))!=0) {
                    tryAgain++;
                    goto RUN_AGAIN;
                }
                udelay(1);
                dwTimeOut--;
            }

            if(dwTimeOut==0) {
                SMSC_WARNING("Timed out while waiting for final Dma to complete");
            }
            if((dwRxStatus=Rx_PopRxStatus(privateData))!=0) {
                tryAgain++;
                goto RUN_AGAIN;
            }
//            printk("Calling outside loop Rx_HandOffSkb\n");
            if(privateData->RxSkb) Platform_DmaComplete(platformData,dwDmaCh);
            Rx_HandOffSkb(privateData,privateData->RxSkb);
            privateData->RxSkb=NULL;

            //check one last time for another packet.
            if((dwRxStatus=Rx_PopRxStatus(privateData))!=0) {
                tryAgain++;
                goto RUN_AGAIN;
            }
        }
    }
        if(privateData->RxCongested)
        	privateData->RxDone = FALSE;
        else
        	privateData->RxDone = TRUE;
    LanSetDirectRegDW(INT_STS,INT_STS_RSFL_);
    SMSC_EXIT

//	CLEAR_GPIO(GP_RX);
}


#ifdef LIMIT_STP_PACKETS_BY_4
int i =0;
#endif
#ifdef FULLY_MANAGED
void Rx_ProcessStpPacket(unsigned long data)
{
    struct sk_buff *skb = NULL;
    SMSC_ASSERT(thisModulePrivate != NULL);

    // Dequeue all packets from STP queue and send them to
    // STP
    do {
            spin_lock_irq(&(thisModulePrivate->stpQueueLock));
            skb=  skb_dequeue(&thisModulePrivate->StpQueue);
            spin_unlock_irq(&(thisModulePrivate->stpQueueLock));
            if ( skb)
            {
                struct net_device  *br, *dev;
                PSWITCH_PPRIVATE_DATA priv =
                        (PSWITCH_PPRIVATE_DATA) (skb->dev->priv);
                br =__dev_get_by_name("bridge");
                if (br &&  priv->rstpEnabled)  {
                    dev =  skb->dev;
                    SMSC_ASSERT(dev!= NULL);


                    skb->dev = br;
                    skb->input_dev = dev;
                    // SMSC_TRACE("!!!!!!!!!!!!!!!! Sending SBK to BRIDGE!!!!!!!!!!!!!!");
                   // printk("sending STP to bridge\n");
#ifdef LIMIT_STP_PACKETS_BY_4
                    if(!(i%4))
                    {
                    	printk("sending STP to bridge =%d\n", i);
                    	dev_queue_xmit(skb);
                    }
                    else
                    	dev_kfree_skb(skb);
                    i++;
#else
                    dev_queue_xmit(skb);
#endif
                }
                else {
                    SMSC_TRACE("Discarding STP frame");
                    dev_kfree_skb(skb);
                }

            }
            else
            {
                break;
            }
    } while (skb != NULL);

}

void Rx_ProcessIgmpPacket(unsigned long data)
{
    struct sk_buff *skb = NULL;
    int portNumber = 0;
    int pktType=0, value=0;
    SMSC_ASSERT(thisModulePrivate != NULL);

    // Dequeue all packets from IGMP queue and send them to
    // IGMP Manager
    do {
        spin_lock_irq(&(thisModulePrivate->igmpQueueLock));
        skb=  skb_dequeue(&thisModulePrivate->IgmpQueue);
        spin_unlock_irq(&(thisModulePrivate->igmpQueueLock));
        if ( skb)
        {
            value =  atomic_read(&(skb->users));
            portNumber = value & 0xF;
            pktType = ( (value >>4) & 0xF);
            //SMSC_TRACE("DRV: sending IGMP Packet to IGMP manager");
            printk("portNumber=%d\n", portNumber);
            printk("skb->len =%d\n", skb->len);
           // dumpSkb(skb);
            sendSnoopedMsgToManager( thisModulePrivate, skb->data,skb->len,pktType, portNumber);
            dev_kfree_skb(skb);
        }

        else
        {
            break;
        }
    } while (skb != NULL);

}
#endif

void Rx_ProcessPacketsTasklet(unsigned long data)
{
    PPRIVATE_DATA privateData=(PPRIVATE_DATA)Rx_TaskletParameter;
    data=data;//make lint happy
    if(privateData==NULL) {
        SMSC_WARNING("Rx_ProcessPacketsTasklet(privateData==NULL)");
        return;
    }
    Rx_ProcessPackets(privateData);
    Lan_ClearIntDeasIntvl(privateData);
    Lan_EnableIRQ(privateData);
}


BOOLEAN Rx_HandleInterrupt(
PPRIVATE_DATA privateData,
DWORD dwIntSts)
{
    BOOLEAN result=FALSE;
    SMSC_ASSERT(privateData!=NULL);

    privateData->LastReasonForReleasingCPU=0;
#ifdef TEST_DMA
    nb_rxInterrupt++;
#endif

    if(dwIntSts&INT_STS_RXE_) {
        // mobbad SMSC_TRACE("Rx_HandleInterrupt: RXE signalled");
        privateData->stats.rx_errors++;
        LanSetDirectRegDW(INT_STS,INT_STS_RXE_);
        result=TRUE;
    }

    if(dwIntSts&INT_STS_RXDFH_INT_) {
            SMSC_TRACE("Interrupt: Frame dropped");
        privateData->stats.rx_dropped+=LanGetDirectRegDW(RX_DROP);
        LanSetDirectRegDW(INT_STS,INT_STS_RXDFH_INT_);
        result=TRUE;
    }

    if(privateData->dwGeneration==0) {
        if(dwIntSts&(INT_STS_RDFL_)) {
            LanSetDirectRegDW(INT_STS,INT_STS_RDFL_);
            Rx_HandleOverrun(privateData);
            result=TRUE;
        }
    } else {
        if(dwIntSts&(INT_STS_RDFO_)) {
            LanSetDirectRegDW(INT_STS,INT_STS_RDFO_);
            Rx_HandleOverrun(privateData);
            result=TRUE;
        }
    }

    if((!(dwIntSts&INT_STS_RSFL_)) && (privateData->RxOverrun==FALSE)) {
        {
                        LanSetDirectRegDW(INT_STS,INT_STS_RSFL_);
            return result;
        }
    }
    result=TRUE;

    if(privateData->MeasuringRxThroughput==FALSE) {
        privateData->MeasuringRxThroughput=TRUE;
        Gpt_ScheduleCallBack(privateData,GptCB_MeasureRxThroughput,1000);
        privateData->RxFlowCurrentThroughput=0;
        privateData->RxFlowCurrentPacketCount=0;
        privateData->RxFlowCurrentWorkLoad=0;
    }
#ifndef NAPI_PROCESS
    if(tasklets) {
        Lan_DisableIRQ(privateData);
        Rx_TaskletParameter=(unsigned long)privateData;
        tasklet_schedule(&Rx_Tasklet);
    } else {
        Rx_ProcessPackets(privateData);
    }
#else
    if(rx_mode==1)
    {
        Lan_DisableIRQ(privateData);
        Rx_TaskletParameter=(unsigned long)privateData;
        privateData->RxProcessingMode=PROCESSING_MODE_TASKLET;
        tasklet_schedule(&Rx_Tasklet);
    }
    else if(rx_mode==2) {
//    	printk("PROCESS_NAPI\n");
    	Lan_DisableIRQ(privateData);
        privateData->RxProcessingMode=PROCESSING_MODE_NAPI;
        //__VL__ added poll schedulled check
        if(netif_rx_schedule_prep(privateData->dev))
        {
            __netif_rx_schedule(privateData->dev);
        }
        else
        {
            printk("DMAC RX Interrupt after __netif_rx_schedule\n");
        }
     } else {
        Rx_ProcessPackets(privateData);
    }
#endif
    return result;
}

////////////////////////////////////////////////
//   Function: RxStop_HandleInterrupt:
//
//
////////////////////////////////////////////////

BOOLEAN RxStop_HandleInterrupt(
PPRIVATE_DATA privateData,
DWORD dwIntSts)
{
    BOOLEAN result=FALSE;
    SMSC_ASSERT(privateData!=NULL);

    if(dwIntSts&INT_STS_RXSTOP_INT_) {
        result=TRUE;
        Gpt_CancelCallBack (privateData, GptCB_RxCompleteMulticast);
        Rx_CompleteMulticastUpdate (privateData);
        LanSetDirectRegDW(INT_STS,INT_STS_RXSTOP_INT_);
        Lan_DisableInterrupt(privateData,INT_EN_RXSTOP_INT_EN_);
    }
    return result;
}

//returns hash bit number for given MAC address
//example:
//   01 00 5E 00 00 01 -> returns bit number 31
static DWORD Rx_Hash(BYTE addr[6])
{
    int i;
    DWORD crc=0xFFFFFFFFUL;
    DWORD poly=0xEDB88320UL;
    DWORD result=0;
    for(i=0;i<6;i++)
    {
        int bit;
        DWORD data=((DWORD)addr[i]);
        for(bit=0;bit<8;bit++)
        {
            DWORD p = (crc^((DWORD)data))&1UL;
            crc >>= 1;
            if(p!=0) crc ^= poly;
            data >>=1;
        }
    }
    result=((crc&0x01UL)<<5)|
        ((crc&0x02UL)<<3)|
        ((crc&0x04UL)<<1)|
        ((crc&0x08UL)>>1)|
        ((crc&0x10UL)>>3)|
        ((crc&0x20UL)>>5);
    return result;
}

void Rx_SetMulticastList(
struct net_device *dev)
{
    PPRIVATE_DATA privateData=NULL;
    VL_KEY keyCode=0;
    DWORD dwIntFlags=0;
    SMSC_ASSERT(dev!=NULL);

    SMSC_TRACE("!!!!!!!!!!!!!!! Rx_SetMulticastList , dev->flags =0x%x!!!!!!!!!!!!!!!", dev->flags );
    privateData=((PPRIVATE_DATA)(dev->priv));
        SMSC_ASSERT(privateData!=NULL);
    keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

    if(dev->flags & IFF_PROMISC) {
    	SMSC_TRACE("Promiscuous Mode is Enabled");
        privateData->set_bits_mask = MAC_CR_PRMS_;
        privateData->clear_bits_mask = (MAC_CR_MCPAS_ | MAC_CR_HPFILT_);

        privateData->HashHi = 0UL;
        privateData->HashLo = 0UL;
        goto PREPARE;
    }

    if(dev->flags & IFF_ALLMULTI) {
        SMSC_TRACE("Receive all Multicast Enabled");
        privateData->set_bits_mask = MAC_CR_MCPAS_;
        privateData->clear_bits_mask = (MAC_CR_PRMS_ | MAC_CR_HPFILT_);

        privateData->HashHi = 0UL;
        privateData->HashLo = 0UL;
        goto PREPARE;
    }


    if(dev->mc_count>0) {
        DWORD dwHashH=0;
        DWORD dwHashL=0;
        DWORD dwCount=0;
        struct dev_mc_list *mc_list=dev->mc_list;
        privateData->set_bits_mask = MAC_CR_HPFILT_;
        privateData->clear_bits_mask = (MAC_CR_PRMS_ | MAC_CR_MCPAS_);

        while(mc_list!=NULL) {
            dwCount++;
            if((mc_list->dmi_addrlen)==6) {
                DWORD dwMask=0x01UL;
                DWORD dwBitNum=Rx_Hash(mc_list->dmi_addr);
            //	SMSC_TRACE("Multicast: enable dwBitNum=%ld,addr=%02X %02X %02X %02X %02X %02X",
            //		dwBitNum,
            //		((BYTE *)(mc_list->dmi_addr))[0],
            //		((BYTE *)(mc_list->dmi_addr))[1],
            //		((BYTE *)(mc_list->dmi_addr))[2],
            //		((BYTE *)(mc_list->dmi_addr))[3],
            //		((BYTE *)(mc_list->dmi_addr))[4],
            //		((BYTE *)(mc_list->dmi_addr))[5]);
                dwMask<<=(dwBitNum&0x1FUL);
                if(dwBitNum&0x20UL) {
                    dwHashH|=dwMask;
                } else {
                    dwHashL|=dwMask;
                }
            } else {
                SMSC_WARNING("dmi_addrlen!=6");
            }
            mc_list=mc_list->next;
        }
        if(dwCount!=((DWORD)(dev->mc_count))) {
            SMSC_WARNING("dwCount!=dev->mc_count");
        }
        SMSC_TRACE("Multicast: HASHH=0x%08lX,HASHL=0x%08lX",dwHashH,dwHashL);
        privateData->HashHi = dwHashH;
        privateData->HashLo = dwHashL;
    }
    else
    {
        privateData->set_bits_mask = 0L;
        privateData->clear_bits_mask = (MAC_CR_PRMS_ | MAC_CR_MCPAS_ | MAC_CR_HPFILT_);

        SMSC_TRACE("Receive own packets only.");
        privateData->HashHi = 0UL;
        privateData->HashLo = 0UL;
    }

    PREPARE:
    if(privateData->dwGeneration<=1) {
        if (privateData->MulticastUpdatePending == FALSE) {
            privateData->MulticastUpdatePending = TRUE;
            // prepare to signal software interrupt
            Lan_SignalSoftwareInterrupt(privateData);
        }
        else {
            // Rx_CompleteMulticastUpdate has not yet been called
            // therefore these latest settings will be used instead
        }
    } else {
        DWORD local_MACCR;
        Mac_SetRegDW(privateData,HASHH,privateData->HashHi,keyCode);
        Mac_SetRegDW(privateData,HASHL,privateData->HashLo,keyCode);
        local_MACCR = Mac_GetRegDW(privateData,MAC_CR,keyCode);
        local_MACCR |= privateData->set_bits_mask;
        local_MACCR &= ~(privateData->clear_bits_mask);
        Mac_SetRegDW(privateData,MAC_CR,local_MACCR,keyCode);
    }
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    return;
}

void Eeprom_EnableAccess(PPRIVATE_DATA privateData)
{
    SMSC_ASSERT(privateData!=NULL);
    if(debug_mode&0x04UL) {
        LanSetDirectRegDW(GPIO_CFG,(g_GpioSetting&0xFF0FFFFFUL));
    } else {
        Lan_ClrBitsDW(GPIO_CFG,0x00F00000UL);
    }
    udelay(100);
}

void Eeprom_DisableAccess(PPRIVATE_DATA privateData)
{
    SMSC_ASSERT(privateData!=NULL);
    if(debug_mode&0x04UL) {
        LanSetDirectRegDW(GPIO_CFG,g_GpioSetting);
    }
}

BOOLEAN Eeprom_IsMacAddressLoaded(PPRIVATE_DATA privateData)
{
#if 0 //fixme mobbad
    SMSC_ASSERT(privateData!=NULL);
    return (LanGetRegDW(E2P_CMD)&
        E2P_CMD_MAC_ADDR_LOADED_)?TRUE:FALSE;
#endif
    printk("!!!!!  Eeprom_IsMacAddressLoaded not implemented !!!!\n");
    return TRUE;
}

BOOLEAN Eeprom_IsBusy(PPRIVATE_DATA privateData)
{
#if 0 //mobad fixme
SMSC_ASSERT(privateData!=NULL);
return (LanGetRegDW(E2P_CMD)&
    E2P_CMD_EPC_BUSY_)?TRUE:FALSE;
#endif
    printk("!!!!!  Eeprom_IsBusy not implemented !!!!\n");
    return FALSE;

}

BOOLEAN Eeprom_Timeout(PPRIVATE_DATA privateData)
{
#if 0 //fixme mobbad
    SMSC_ASSERT(privateData!=NULL);
    return (LanGetRegDW(E2P_CMD)&
        E2P_CMD_EPC_TIMEOUT_)?TRUE:FALSE;
#endif
    printk("!!!!!  Eeprom_Timeout not implemented !!!!\n");
    return FALSE;

}

BOOLEAN Eeprom_ReadLocation(PPRIVATE_DATA privateData,
WORD address, BYTE * data)
{
    DWORD timeout=100000;
    DWORD temp=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(data!=NULL);
    if((temp=LanGetRegDW(E2P_CMD))&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_ReadLocation: Busy at start, E2P_CMD=0x%08lX",temp);
        return FALSE;
    }
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_READ_|((DWORD)address)));
    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    (*data)=(BYTE)(LanGetRegDW(E2P_DATA));
    return TRUE;
}

BOOLEAN Eeprom_EnableEraseAndWrite(
PPRIVATE_DATA privateData)
{
    DWORD timeout=100000;
    SMSC_ASSERT(privateData!=NULL);
    if(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_EnableEraseAndWrite: Busy at start");
        return FALSE;
    }
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_EWEN_));

    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    return TRUE;
}

BOOLEAN Eeprom_DisableEraseAndWrite(
PPRIVATE_DATA privateData)
{
    DWORD timeout=100000;
    SMSC_ASSERT(privateData!=NULL);
    if(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_DisableEraseAndWrite: Busy at start");
        return FALSE;
    }
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_EWDS_));

    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    return TRUE;
}

BOOLEAN Eeprom_WriteLocation(
PPRIVATE_DATA privateData,WORD address,BYTE data)
{
    DWORD timeout=100000;
    SMSC_ASSERT(privateData!=NULL);
    if(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_WriteLocation: Busy at start");
        return FALSE;
    }
    LanSetRegDW(E2P_DATA,((DWORD)data));
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_WRITE_|((DWORD)address)));

    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    return TRUE;
}

BOOLEAN Eeprom_EraseAll(
PPRIVATE_DATA privateData)
{
    DWORD timeout=100000;
    SMSC_ASSERT(privateData!=NULL);
    if(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_EraseAll: Busy at start");
        return FALSE;
    }
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_ERAL_));

    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    return TRUE;
}

BOOLEAN Eeprom_Reload(
PPRIVATE_DATA privateData)
{
    DWORD timeout=100000;
    SMSC_ASSERT(privateData!=NULL);
    if(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_) {
        SMSC_WARNING("Eeprom_Reload: Busy at start");
        return FALSE;
    }
    LanSetRegDW(E2P_CMD,
        (E2P_CMD_EPC_BUSY_|E2P_CMD_EPC_CMD_RELOAD_));

    while((timeout>0)&&
        (LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_))
    {
        udelay(10);
        timeout--;
    }
    if(timeout==0) {
        return FALSE;
    }
    return TRUE;
}

BOOLEAN Eeprom_SaveMacAddress(
PPRIVATE_DATA privateData,
DWORD dwHi16,DWORD dwLo32)
{
    BOOLEAN result=FALSE;
    SMSC_ASSERT(privateData!=NULL);
    Eeprom_EnableAccess(privateData);
    if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
    if(!Eeprom_EraseAll(privateData)) goto DONE;
    if(privateData->dwGeneration==0) {
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,0,0xA5)) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,1,LOBYTE(LOWORD(dwLo32)))) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,2,HIBYTE(LOWORD(dwLo32)))) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,3,LOBYTE(HIWORD(dwLo32)))) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,4,HIBYTE(HIWORD(dwLo32)))) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,5,LOBYTE(LOWORD(dwHi16)))) goto DONE;
        if(!Eeprom_EnableEraseAndWrite(privateData)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,6,HIBYTE(LOWORD(dwHi16)))) goto DONE;
    } else {
        if(!Eeprom_WriteLocation(privateData,0,0xA5)) goto DONE;
        if(!Eeprom_WriteLocation(privateData,1,LOBYTE(LOWORD(dwLo32)))) goto DONE;
        if(!Eeprom_WriteLocation(privateData,2,HIBYTE(LOWORD(dwLo32)))) goto DONE;
        if(!Eeprom_WriteLocation(privateData,3,LOBYTE(HIWORD(dwLo32)))) goto DONE;
        if(!Eeprom_WriteLocation(privateData,4,HIBYTE(HIWORD(dwLo32)))) goto DONE;
        if(!Eeprom_WriteLocation(privateData,5,LOBYTE(LOWORD(dwHi16)))) goto DONE;
        if(!Eeprom_WriteLocation(privateData,6,HIBYTE(LOWORD(dwHi16)))) goto DONE;
    }
    if(!Eeprom_DisableEraseAndWrite(privateData)) goto DONE;

    if(!Eeprom_Reload(privateData)) goto DONE;
    if(!Eeprom_IsMacAddressLoaded(privateData)) goto DONE;
    {
        DWORD dwIntFlags=0;
        VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
        if(dwHi16!=Mac_GetRegDW(privateData,ADDRH,keyCode)) goto DONE;
        if(dwLo32!=Mac_GetRegDW(privateData,ADDRL,keyCode)) goto DONE;
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    }
    result=TRUE;
    DONE:
    Eeprom_DisableAccess(privateData);
    return result;
}

volatile DWORD g_GpioSetting=0x00000000UL;




BOOLEAN Lan_Initialize(
PPRIVATE_DATA privateData,
DWORD dwIntCfg,
DWORD dwTxFifSz,
DWORD dwAfcCfg)
{
    BOOLEAN result=FALSE;
    DWORD dwTimeOut=0;
    DWORD dwTemp=0;
    DWORD dwResetCount=3;

    SMSC_TRACE("-->Lan_Initialize(dwIntCfg=0x%08lX)",dwIntCfg);
    SMSC_ASSERT(privateData!=NULL);


    //Reset the LAN9118
    if(privateData->dwGeneration>0) {
        dwResetCount=1;
    }
    while(dwResetCount>0) {
        LanSetDirectRegDW(HW_CFG,HW_CFG_SRST_);
        dwTimeOut=1000000;
        do {
            udelay(10);
            dwTemp=LanGetDirectRegDW(HW_CFG);
            dwTimeOut--;
        } while((dwTimeOut>0)&&(dwTemp&HW_CFG_SRST_));
        if(dwTemp&HW_CFG_SRST_) {
            SMSC_WARNING("  Failed to complete reset.");
            goto DONE;
        }
        dwResetCount--;
    }

    SMSC_ASSERT(dwTxFifSz>=0x00020000UL);
    SMSC_ASSERT(dwTxFifSz<=0x000E0000UL);
    SMSC_ASSERT((dwTxFifSz&(~HW_CFG_TX_FIF_SZ_))==0);
    LanSetDirectRegDW(HW_CFG,dwTxFifSz);
    privateData->RxMaxDataFifoSize=0;
    switch(dwTxFifSz>>16) {
    case 2:privateData->RxMaxDataFifoSize=13440;break;
    case 3:privateData->RxMaxDataFifoSize=12480;break;
    case 4:privateData->RxMaxDataFifoSize=11520;break;
    case 5:privateData->RxMaxDataFifoSize=10560;break;
    case 6:privateData->RxMaxDataFifoSize=9600;break;
    case 7:privateData->RxMaxDataFifoSize=8640;break;
    case 8:privateData->RxMaxDataFifoSize=7680;break;
    case 9:privateData->RxMaxDataFifoSize=6720;break;
    case 10:privateData->RxMaxDataFifoSize=5760;break;
    case 11:privateData->RxMaxDataFifoSize=4800;break;
    case 12:privateData->RxMaxDataFifoSize=3840;break;
    case 13:privateData->RxMaxDataFifoSize=2880;break;
    case 14:privateData->RxMaxDataFifoSize=1920;break;
    default:SMSC_ASSERT(FALSE);break;
    }

    if(dwAfcCfg==0xFFFFFFFF) {
        switch(dwTxFifSz) {

        //AFC_HI is about ((Rx Data Fifo Size)*2/3)/64
        //AFC_LO is AFC_HI/2
        //BACK_DUR is about 5uS*(AFC_LO) rounded down
        case 0x00020000UL://13440 Rx Data Fifo Size
            dwAfcCfg=0x008C46AF;break;
        case 0x00030000UL://12480 Rx Data Fifo Size
            dwAfcCfg=0x0082419F;break;
        case 0x00040000UL://11520 Rx Data Fifo Size

            dwAfcCfg=0x00783C9F;break;
        case 0x00050000UL://10560 Rx Data Fifo Size
    //			dwAfcCfg=0x006E378F;break;
            dwAfcCfg=0x006E374F;break;
        case 0x00060000UL:// 9600 Rx Data Fifo Size
            dwAfcCfg=0x0064328F;break;
        case 0x00070000UL:// 8640 Rx Data Fifo Size
            dwAfcCfg=0x005A2D7F;break;
        case 0x00080000UL:// 7680 Rx Data Fifo Size
            dwAfcCfg=0x0050287F;break;
        case 0x00090000UL:// 6720 Rx Data Fifo Size
            dwAfcCfg=0x0046236F;break;
        case 0x000A0000UL:// 5760 Rx Data Fifo Size
            dwAfcCfg=0x003C1E6F;break;
        case 0x000B0000UL:// 4800 Rx Data Fifo Size
            dwAfcCfg=0x0032195F;break;

        //AFC_HI is ~1520 bytes less than RX Data Fifo Size
        //AFC_LO is AFC_HI/2
        //BACK_DUR is about 5uS*(AFC_LO) rounded down
        case 0x000C0000UL:// 3840 Rx Data Fifo Size
            dwAfcCfg=0x0024124F;break;
        case 0x000D0000UL:// 2880 Rx Data Fifo Size
            dwAfcCfg=0x0015073F;break;
        case 0x000E0000UL:// 1920 Rx Data Fifo Size
            dwAfcCfg=0x0006032F;break;
        default:SMSC_ASSERT(FALSE);break;
        }
    }
 //   printk("AFC_CFG = 0x%x\n",(unsigned long) dwAfcCfg);
    LanSetDirectRegDW(AFC_CFG,(dwAfcCfg&0xFFFFFFF0UL));

    //make sure EEPROM has finished loading before setting GPIO_CFG
    #if 0 // fixme eeprom has different mapping
    dwTimeOut=1000;
    while((dwTimeOut>0)&&(LanGetRegDW(E2P_CMD)&E2P_CMD_EPC_BUSY_)) {
        udelay(5);
        dwTimeOut--;
    }
    if(dwTimeOut==0) {
        SMSC_WARNING("Lan_Initialize: Timed out waiting for EEPROM busy bit to clear\n");
    }
    #endif

    #if 0 // fixme wrong setting for Hydra/Leo
    if(debug_mode&0x04UL) {
        if(OLD_REGISTERS(privateData))
        {
            g_GpioSetting=0x00270700UL;
        } else {
            g_GpioSetting=0x00670700UL;
        }
    } else {
        g_GpioSetting = 0x70070000UL;
    }
    LanSetDirectRegDW(GPIO_CFG,g_GpioSetting);
    #else
        g_GpioSetting = 0xF000UL;
    #endif

    //initialize interrupts
    LanSetDirectRegDW(INT_EN,0);
    LanSetDirectRegDW(INT_STS,0xFFFFFFFFUL);
    dwIntCfg|=INT_CFG_IRQ_EN_;
    LanSetDirectRegDW(INT_CFG,dwIntCfg);

    Vl_InitLock(&(privateData->MacPhyLock));

    Vl_InitLock(&(privateData->SwitchAccessLock) );
#ifndef CHIP_IS_LAN9303
    Vl_InitLock(&(privateData->PtpAccessLock) );
#endif
    // init_rwsem(privateData);
    spin_lock_init(&(privateData->SwitchRegLock));
    privateData->LanInitialized=TRUE;

    result=TRUE;

    DONE:
    SMSC_TRACE("<--Lan_Initialize");
    return result;
}

void Lan_EnableInterrupt(PPRIVATE_DATA privateData,DWORD dwIntEnMask)
{
    DWORD dwIntFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    Lan_SetBitsDW(INT_EN,dwIntEnMask);
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_DisableInterrupt(PPRIVATE_DATA privateData,DWORD dwIntEnMask)
{
    DWORD dwIntFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    Lan_ClrBitsDW(INT_EN,dwIntEnMask);
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

//Spin locks for the following functions have been commented out
//  because at this time they are not necessary.
//These function are
//  Lan_SetTDFL
//  Lan_SetTSFL
//  Lan_SetRDFL
//  Lan_SetRSFL
//Both the Rx and Tx side of the driver use the FIFO_INT,
//  but the Rx side only touches is during initialization,
//  so it is sufficient that Tx side simple preserve the Rx setting

void Lan_SetTDFL(PPRIVATE_DATA privateData,BYTE level) {
    //	DWORD dwIntFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    //	spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        DWORD temp=LanGetDirectRegDW(FIFO_INT);
        temp&=0x00FFFFFFUL;
        temp|=((DWORD)level)<<24;
        LanSetDirectRegDW(FIFO_INT,temp);
    }
    //	spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_SetTSFL(PPRIVATE_DATA privateData,BYTE level) {
    //	DWORD dwIntFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    //	spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        DWORD temp=LanGetDirectRegDW(FIFO_INT);
        temp&=0xFF00FFFFUL;
        temp|=((DWORD)level)<<16;
        LanSetDirectRegDW(FIFO_INT,temp);
    }
    //	spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_SetRDFL(PPRIVATE_DATA privateData,BYTE level) {
    //	DWORD dwIntFlags=0;
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->LanInitialized==TRUE);
    //	spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        DWORD temp=LanGetDirectRegDW(FIFO_INT);
        temp&=0xFFFF00FFUL;
        temp|=((DWORD)level)<<8;
        LanSetDirectRegDW(FIFO_INT,temp);
    }
    //	spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_SetRSFL(PPRIVATE_DATA privateData,BYTE level) {
//	DWORD dwIntFlags=0;
SMSC_ASSERT(privateData!=NULL);
SMSC_ASSERT(privateData->LanInitialized==TRUE);
//	spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
{
    DWORD temp=LanGetDirectRegDW(FIFO_INT);
    temp&=0xFFFFFF00UL;
    temp|=((DWORD)level);
    LanSetDirectRegDW(FIFO_INT,temp);
}
//	spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_EnableIRQ(PPRIVATE_DATA privateData)
{
    DWORD dwIntFlags=0;
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        Lan_SetBitsDW(INT_CFG,INT_CFG_IRQ_EN_);
    }
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_DisableIRQ(PPRIVATE_DATA privateData)
{
    DWORD dwIntFlags=0;
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        Lan_ClrBitsDW(INT_CFG,INT_CFG_IRQ_EN_);
    }
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_SetIntDeas(PPRIVATE_DATA privateData, DWORD dwIntDeas)
{
    DWORD dwIntFlags=0;
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        Lan_ClrBitsDW(INT_CFG,INT_CFG_INT_DEAS_);
        Lan_SetBitsDW(INT_CFG,(dwIntDeas<<24));
    }
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_ClearIntDeasIntvl(PPRIVATE_DATA privateData)
{
    DWORD dwIntFlags=0;
    spin_lock_irqsave(&(privateData->IntEnableLock),dwIntFlags);
    {
        Lan_ClrBitsDW(INT_CFG,INT_CFG_INT_DEAS_CLR_);
    }
    spin_unlock_irqrestore(&(privateData->IntEnableLock),dwIntFlags);
}

void Lan_SignalSoftwareInterrupt(PPRIVATE_DATA privateData)
{
    SMSC_ASSERT(privateData!=NULL);
    SMSC_ASSERT(privateData->dwLanBase!=0);
    privateData->SoftwareInterruptSignal=FALSE;
    Lan_EnableInterrupt(privateData,INT_EN_SW_INT_EN_);
}

BOOLEAN Lan_HandleSoftwareInterrupt(
                PPRIVATE_DATA privateData,
                DWORD dwIntSts)
{
    if(dwIntSts&INT_STS_SW_INT_) {
        SMSC_ASSERT(privateData!=NULL);
        Lan_DisableInterrupt(privateData,INT_EN_SW_INT_EN_);
        LanSetDirectRegDW(INT_STS,INT_STS_SW_INT_);
        privateData->SoftwareInterruptSignal=TRUE;
        if (privateData->MulticastUpdatePending) {
            Rx_BeginMulticastUpdate (privateData);
        }
        return TRUE;
    }
    return FALSE;
}

typedef struct _SHOW_REG
{
    char  szName[20];
    DWORD dwOffset;
} SHOW_REG;
/*
FUNCTION: Lan_ShowRegs
This function is used to display the registers.
Except the phy.
*/
void Lan_ShowRegs(PPRIVATE_DATA privateData)
{
    //	Make these const struct's static to keep them off the stack.
    //	Otherwise, gcc will try to use _memcpy() to initialize them,
    //	which will *NOT* work in our RunTime environment.
    static const SHOW_REG sysCsr[] = {
        { "ID_REV",			0x50UL		},
        { "INT_CFG",		0x54UL		},
        { "INT_STS",		0x58UL		},
        { "INT_EN",			0x5CUL		},
        { "DMA_CFG",		0x60UL		},
        { "BYTE_TEST",		0x64UL		},
        { "FIFO_INT",		0x68UL		},
        { "RX_CFG",			0x6CUL		},
        { "TX_CFG",			0x70UL		},
        { "HW_CFG",			0x74UL		},
        { "RX_DP_CTRL",		0x78UL		},
        { "RX_FIFO_INF",	0x7CUL		},
        { "TX_FIFO_INF",	0x80UL		},
        { "PMT_CTRL",		0x84UL		},
        { "GPIO_CFG",		0x88UL		},
        { "GPT_CFG",		0x8CUL		},
        { "GPT_CNT",		0x90UL		},
        { "FPGA_REV",		0x94UL		},
        { "ENDIAN",			0x98UL		},
        { "FREE_RUN",		0x9CUL		},
        { "RX_DROP",		0xA0UL		},
        { "MAC_CSR_CMD",	0xA4UL		},
        { "MAC_CSR_DATA",	0xA8UL		},
        { "AFC_CFG",		0xACUL		},
        { "E2P_CMD",		0xB0UL		},
        { "E2P_DATA",		0xB4UL		},
        { "TEST_REG_A",		0xC0UL		}};

    static const SHOW_REG macCsr[] = {
        { "MAC_CR",		MAC_CR		},
        { "ADDRH",		ADDRH		},
        { "ADDRL",		ADDRL		},
        { "HASHH",		HASHH		},
        { "HASHL",		HASHL		},
        { "MII_ACC",	MII_ACC		},
        { "MII_DATA",	MII_DATA	},
        { "FLOW",		FLOW		},
        { "VLAN1",		VLAN1		},
        { "VLAN2",		VLAN2		},
        { "WUFF",		WUFF		},
        { "WUCSR",		WUCSR		}};

    int i, iNumSysRegs, iNumMacRegs;
    DWORD dwOldMacCmdReg, dwOldMacDataReg;

    iNumSysRegs = (int)(sizeof(sysCsr) / sizeof(SHOW_REG));
    iNumMacRegs = (int)(sizeof(macCsr) / sizeof(SHOW_REG));

    // preserve MAC cmd/data reg's
    dwOldMacCmdReg = LanGetDirectRegDW(MAC_CSR_CMD);
    dwOldMacDataReg = LanGetDirectRegDW(MAC_CSR_DATA);

    SMSC_TRACE("");
    SMSC_TRACE("               LAN91C118 CSR's");
    SMSC_TRACE("                     SYS CSR's                     MAC CSR's");

    {
        DWORD dwIntFlags=0;
        VL_KEY keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);
        for (i=0; i<iNumMacRegs; i++)
        {
            SMSC_TRACE(
                "%16s (0x%02lX) = 0x%08lX, %8s (0x%02lX) + 0x%08lX",
                sysCsr[i].szName,
                sysCsr[i].dwOffset,
                *((volatile DWORD *)(privateData->dwLanBase+sysCsr[i].dwOffset)),
                macCsr[i].szName,
                macCsr[i].dwOffset,
                Mac_GetRegDW(privateData,macCsr[i].dwOffset,keyCode));

            // restore original mac cmd/data reg's after each usage
            LanSetDirectRegDW(MAC_CSR_CMD,dwOldMacCmdReg);
            LanSetDirectRegDW(MAC_CSR_DATA,dwOldMacDataReg);
        }
        Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
    }
    for (i=iNumMacRegs; i<iNumSysRegs; i++)
    {
        SMSC_TRACE("%16s (0x%02lX) = 0x%08lX",
            sysCsr[i].szName,
            sysCsr[i].dwOffset,
            *((volatile DWORD *)(privateData->dwLanBase+sysCsr[i].dwOffset)));
    }
}

void Vl_InitLock(PVERIFIABLE_LOCK pVl)
{
    SMSC_ASSERT(pVl!=NULL);
    spin_lock_init(&(pVl->Lock));
    pVl->KeyCode=0;
}

BOOLEAN Vl_CheckLock(PVERIFIABLE_LOCK pVl,VL_KEY keyCode)
{
    BOOLEAN result=FALSE;
    SMSC_ASSERT(pVl!=NULL);
    if(keyCode==pVl->KeyCode)
        result=TRUE;
    return result;
}

VL_KEY Vl_WaitForLock(PVERIFIABLE_LOCK pVl,DWORD *pdwIntFlags)
{
    VL_KEY result=0;
    SMSC_ASSERT(pVl!=NULL);
    spin_lock_irqsave(
        &(pVl->Lock),
        (*pdwIntFlags));
    pVl->KeyCode++;
    if(pVl->KeyCode>0x80000000UL) {
        pVl->KeyCode=1;
    }
    result=pVl->KeyCode;
    return result;
}

void Vl_ReleaseLock(PVERIFIABLE_LOCK pVl,VL_KEY keyCode,DWORD *pdwIntFlags)
{
    SMSC_ASSERT(pVl!=NULL);
    SMSC_ASSERT(pVl->KeyCode==keyCode);
    spin_unlock_irqrestore(&(pVl->Lock),(*pdwIntFlags));
}

void  getSwitchReadLock(PPRIVATE_DATA privateData)
{
    // down_read(mutex);
    spin_lock_irq(&(privateData->SwitchRegLock));
    if (privateData->nbLock)
    {
    SMSC_WARNING("Already Locked: locks=%d",privateData->nbLock);
    SMSC_ASSERT(0);
    }
    privateData->nbLock++;
}

void  releaseSwitchReadLock(PPRIVATE_DATA privateData)
{
    // up_read (mutex);
    spin_unlock_irq(&(privateData->SwitchRegLock));
    privateData->nbLock--;
}

void  getSwitchWriteLock(PPRIVATE_DATA privateData)
{
    // down_write(mutex);
    spin_lock_irq(&(privateData->SwitchRegLock));
    if (privateData->nbLock)
    {
    SMSC_WARNING("Already Locked: locks=%d",privateData->nbLock);
    SMSC_ASSERT(0);
    }
    privateData->nbLock++;
}

void  releaseSwitchWriteLock(PPRIVATE_DATA privateData)
{
    // up_write (mutex);
    spin_unlock_irq(&(privateData->SwitchRegLock));
    privateData->nbLock--;
}

#ifdef FULLY_MANAGED

int  setPortsToCPUTag(PPRIVATE_DATA privateData)
{
	int result=0;
    DWORD dwIntFlags=0;
    DWORD  regValue= 0xFFFFFFFF;
    DWORD  mask    = 0x00030303;
    VL_KEY keyCode=0;
    SMSC_ENTER;
    keyCode=Vl_WaitForLock(&(privateData->SwitchAccessLock),&dwIntFlags);

    regValue=Switch_GetRegDW(privateData,GLB_ID);
    if( ((regValue&0xFFFFFF00UL)==0x00030400UL) || ((regValue&0xFFFFFF00UL)==0x00030500UL) ) {
    	result=1;
    } else {
    	SMSC_WARNING("Switch ID = 0x%08lX",regValue);
    }

    regValue =  Switch_GetRegDW(privateData,BM_PRT_TYP);

    if (regValue != 0xFFFFFFFF)
    {
        regValue &=~ mask;
        regValue |= 0x3; /* Port one and port 2 are dumb and MII port as CPU special tagged */
        Switch_SetRegDW( privateData,BM_PRT_TYP,regValue);
    }

    regValue =  Switch_GetRegDW(privateData,SE_ING_PRT_TYP);
    if (regValue != 0xFFFFFFFF)
    {
        regValue |= 0x3;
        Switch_SetRegDW( privateData,SE_ING_PRT_TYP,regValue);
    }

    Vl_ReleaseLock(&(privateData->SwitchAccessLock),keyCode,&dwIntFlags);

    dwIntFlags=0;
    keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

    Mac_SetRegDW(privateData,VLAN1,0x8200,keyCode);

    Mac_SetRegDW(privateData,VLAN2,0x8100,keyCode);

    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
SMSC_EXIT
    return result;

}


void  InitializeSkbQueues(PPRIVATE_DATA privateData)
{
    skb_queue_head_init (&(privateData->StpQueue));
#ifndef CHIP_IS_LAN9303
    skb_queue_head_init(&(privateData->PtpQueue) );
#endif
    skb_queue_head_init(&(privateData->IgmpQueue) );
}

struct sk_buff *readQueue(struct sk_buff_head *rxQueue)
{
    return  (skb_dequeue(rxQueue) );
}

void stpAddSkpToRxQueue(PPRIVATE_DATA privateData, struct sk_buff *skb)
{
    // Take lock
    spin_lock_irq(&(privateData->stpQueueLock));
    WriteQueue(&(privateData->StpQueue), skb);
    // release lock
    spin_unlock_irq(&(privateData->stpQueueLock));
}


void igmpAddSkpToRxQueue(PPRIVATE_DATA privateData, struct sk_buff *skb)
{
    // Take lock

    spin_lock_irq(&(privateData->igmpQueueLock));
    WriteQueue(&(privateData->IgmpQueue), skb);
    // release lock
    spin_unlock_irq(&(privateData->igmpQueueLock));
}

void WriteQueue(struct sk_buff_head *queue, struct sk_buff *skb)
{
 skb_queue_tail(queue, skb);
}

#endif /** FULLY_MANAGED **/

/////////////////////////////////////////////////////////////////////
////
////     SNMP and CLI suport
////
/////////////////////////////////////////////////////////////////////
DWORD mibReadSwitchRegister(PPRIVATE_DATA privateData, DWORD dwRegOffset)
{
    DWORD regValue   = 0;

    getSwitchReadLock(privateData);
    regValue=Switch_GetRegDW(privateData,dwRegOffset);
    releaseSwitchReadLock(privateData);
    if ( regValue == 0xFFFFFFFF)
        regValue = 0;

    return  regValue;
}


long  switchGetPortAdminState(PPRIVATE_DATA privateData,DWORD portIndex)
{
    return 1;
}

long  switchGetPortOldAdminState(PPRIVATE_DATA privateData,DWORD portIndex)
{
    return 1;
}

/**
switchGetPortState (...) return the state of the port as defined in the MIB.
	up		(1)	-- ready to pass packets
	down		(2)	--
	testing		(3)	-- in some test mode
	unknown		(4)	-- status can not be determined for some reason.
	dormant		(5)	--
	notPresent	(6)	-- some component is missing
	lowerLayerDown	(7) 	-- down due to state of lower-layer interface(s)
**/
long   switchGetPortState(PPRIVATE_DATA privateData,DWORD portIndex)
{

    long portState = 6;


    if ( !privateData->LanInitialized )
    {
        return 2;
}

#ifdef SNMP_SUPPORT
if  (privateData->StopMibPolling)
{
    return 2;
}
#endif
if (portIndex < PHY_NUMBER)
{
    DWORD portRegState =
    mibReadSwitchRegister(privateData,SE_PORT_STATE);

    portRegState  = portRegState >>  (portIndex*2);
    //  0 forwarding
    //  1 blocking
    //  2 learning
    //  3 listening
    switch  (portRegState & 0x3)
    {
        case 0:
            portState = 1;
        break;
        case 1:
            portState = 2;
        break;
        case 2:
        case 3:
            portState = 5;
        break;
    }
}

return portState;
}

long switchGetPortSpeed(PPRIVATE_DATA privateData,DWORD portIndex)
{
return 100;    // for 100 MBITS?S
}

long switchGetPortPromiscuousMode(PPRIVATE_DATA privateData,DWORD portIndex)
{
return 1;
}

long switchGetPortOldPromiscuousMode(PPRIVATE_DATA privateData,DWORD portIndex)
{
return 1;
}

static void  mibClearPortCounter(PPRIVATE_DATA privateData, int portIndex)
{
    SwitchMibCounters  *pSwitchMibCounters =    &( privateData->switchMibCounter[portIndex]);
    memset(pSwitchMibCounters, 0, sizeof(SwitchMibCounters) );
}

static void   mibReadPortCounters( PPRIVATE_DATA privateData, int portIndex)
{
    SwitchMibCounters  *pSwitchMibCounters =
            &( privateData->switchMibCounter[portIndex]);
    pSwitchMibCounters->etherStatsFragments           +=
    		mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_FRAG_CNT);
    pSwitchMibCounters->etherStatsUndersisePkts +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_UNDSZE_CNT);
    pSwitchMibCounters->etherStatsPkts64Octets         +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_64_CNT);
    pSwitchMibCounters->etherStatsPkts65to127Octets    +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_65_TO_127_CNT);
    pSwitchMibCounters->etherStatsPkts128to255Octets   +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_128_TO_255_CNT);
    pSwitchMibCounters->etherStatsPkts256to511Octets   +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_256_TO_511_CNT);
    pSwitchMibCounters->etherStatsPkts512to1023Octets  +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_512_TO_1023_CNT);
    pSwitchMibCounters->etherStatsPkts1024to1518Octets +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_1024_TO_MAX_CNT);
    pSwitchMibCounters->etherStatsOversizePkts         +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_OVRSZE_CNT);
    pSwitchMibCounters->ifInPkts                       +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_PKTOK_CNT);
    pSwitchMibCounters->ifInErrors                     +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_CRCERR_CNT);
    pSwitchMibCounters->etherStatsRxMulticastPkts        +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_MULCST_CNT);
    pSwitchMibCounters->etherStatsRxBroadcastPkts        +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_BRDCST_CNT);
    pSwitchMibCounters->etherMacControlPkts            +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_CTLFRM_CNT);
    pSwitchMibCounters->ifInOctets +=
                mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_PKTLEN_CNT);
    pSwitchMibCounters->etherStatsRxJabbCnt +=
                    mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_JABB_CNT);
    pSwitchMibCounters->etherStatsRxPauseFrms +=
                mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_PAUSE_CNT);

    pSwitchMibCounters->etherStatsTxMulitcastPkts        +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_MULCST_CNT);
    pSwitchMibCounters->etherStatsTxBroadcastPkts        +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_BRDCST_CNT);

    pSwitchMibCounters->ifOutUnderRun    +=
            mibReadSwitchRegister(privateData, portIndex*0x400 +  MII_TX_UNDRUN_CNT);
    pSwitchMibCounters->etherDeferredTransmitted  +=
            mibReadSwitchRegister(privateData, portIndex*0x400 +  MII_TX_DEFER_CNT);
    pSwitchMibCounters->etherPauseTransmitted   +=
            mibReadSwitchRegister(privateData, portIndex*0x400 +  MII_TX_PAUSE_CNT);
    pSwitchMibCounters->ifOutPkts    +=
            mibReadSwitchRegister(privateData, portIndex*0x400 +  MII_TX_PKTOK_CNT);

    pSwitchMibCounters->ifOutUnderSize   +=
            mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_UNDSZE_CNT);

    pSwitchMibCounters->ifOutOctets  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_PKTLEN_CNT);

    pSwitchMibCounters->etherStatsAllignmentErrors  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_ALIGN_CNT);
    pSwitchMibCounters->etherStatsSingleCollisionFrms  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_SNGLECOL_CNT);
    pSwitchMibCounters->etherStatsMultCollisionFrms  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_MULTICOL_CNT);

    pSwitchMibCounters->etherStatsLateCollisionFrms  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_LATECOL_CNT);
    pSwitchMibCounters->etherStatsExcessCollisionFrms  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_EXCCOL_CNT);
    pSwitchMibCounters->etherStatsSymbolErrors  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_RX_SYMBL_CNT);

    pSwitchMibCounters->etherStatsTotCollisionCnt  +=
        mibReadSwitchRegister(privateData, portIndex*0x400 + MII_TX_TOTALCOL_CNT);
   pSwitchMibCounters->SwitchBMDropCnt += Switch_GetRegDW(privateData, (BM_DRP_CNT_SRC_0+portIndex));
   pSwitchMibCounters->SwitchLearnDiscrdCnt 	+= Switch_GetRegDW(privateData, (SE_LRN_DISCRD_CNT_0+portIndex) );
   pSwitchMibCounters->SwitchBMRateDropCnt 	+= Switch_GetRegDW(privateData, (BM_RATE_DRP_CNT_SRC0+portIndex));
   pSwitchMibCounters->SwitchFilteredCnt 	+= Switch_GetRegDW(privateData,(SE_FILTERED_CNT_0+portIndex) );
}

void mib_update_64_counter(U64 *counter,U32 NewValue)
{
LWORD value  = counter->low +  counter->high + NewValue ;

counter->high = (value >> 32 & 0xFFFFFFFF);
counter->low = (value & 0xFFFFFFFF);


}
void  mibUpdateIfxTable(PPRIVATE_DATA privateData, int portIndex)
{
    IfTable_data *pIfTable_data;
    IfXTable_Data *pIfXTable_Data;
    SwitchMibCounters  *pSwitchMibCounters;
    DWORD TxbcastPkt, TxmcastPkt = 0, TxtotalPkt = 0;
    DWORD RxbcastPkt, RxmcastPkt = 0, RxtotalPkt = 0, RxcntrlPkt=0;

    SMSC_ASSERT(portIndex < PHY_NUMBER);

    pIfTable_data  = &(privateData->IfTable_data_array[ portIndex]);
    pIfXTable_Data = &(privateData-> IfXTable_Data_array[ portIndex]);
    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);


#if 0
    pIfTable_data->ifDescr[MAX_OID_LEN];
    pIfTable_data->ifDescr_len;
    pIfTable_data->ifType;
#endif

    pIfTable_data->ifMtu = 1500; // Read Jumbo packet
    pIfTable_data->ifSpeed = getVPhyLinkSpeed(portIndex);  // Read only

    pIfTable_data->ifPhysAddress[0]=portIndex;
    pIfTable_data->ifPhysAddress_len =1;

    pIfTable_data->ifAdminStatus     =
            switchGetPortAdminState(privateData,portIndex);
    pIfTable_data->old_ifAdminStatus =
            switchGetPortOldAdminState(privateData,portIndex);
    pIfTable_data->ifOperStatus =
            switchGetPortState(privateData,portIndex);
    pIfTable_data->ifLastChange = (privateData->interfaceChangeTime[portIndex].tv_sec)/100;

    //bytesL = pSwitchMibCounters->ifInOctetsLow;
    //bytesH = pSwitchMibCounters->ifInOctetsHigh;
    pIfTable_data->ifInOctets   += pSwitchMibCounters->ifInOctets;
    // Unicast/non unicast/broadcast/mcast

    RxbcastPkt = pSwitchMibCounters->etherStatsRxBroadcastPkts;
    RxmcastPkt = pSwitchMibCounters->etherStatsRxMulticastPkts;
    RxtotalPkt = pSwitchMibCounters->ifInPkts;
    RxcntrlPkt = pSwitchMibCounters->etherMacControlPkts;

    pIfTable_data->ifInUcastPkts    += (RxtotalPkt -(RxbcastPkt + RxmcastPkt ));
    pIfTable_data->ifInNUcastPkts   += (RxbcastPkt + RxmcastPkt - RxcntrlPkt);
    pIfTable_data->ifInDiscards     += ((pSwitchMibCounters->SwitchBMDropCnt - pSwitchMibCounters->SwitchBMRateDropCnt) +RxcntrlPkt);        // discarded but no error

    // SMSC_TRACE("pIfTable_data->ifInUcastPkts :%lu", pIfTable_data->ifInUcastPkts);
    pIfTable_data->ifInErrors   +=  pSwitchMibCounters->ifInErrors + pSwitchMibCounters->etherStatsAllignmentErrors + \
    								pSwitchMibCounters->etherStatsOversizePkts + pSwitchMibCounters->etherStatsRxJabbCnt;

    pIfTable_data->ifInUnknownProtos    += 0;  // Unknow ethertype/VLAN/...etc
    /** ifXtabbe  receive side**/
    pIfXTable_Data->ifInMulticastPkts += (RxmcastPkt+ RxcntrlPkt);
    pIfXTable_Data->ifInBroadcastPkts += RxbcastPkt;
    mib_update_64_counter(&(pIfXTable_Data->ifHCInUcastPkts),  (RxtotalPkt -(RxbcastPkt + RxmcastPkt )));
    mib_update_64_counter(&(pIfXTable_Data->ifHCInOctets),   pIfTable_data->ifInOctets );
//    pIfXTable_Data->ifHCInOctets.high += bytesH;
    mib_update_64_counter(&(pIfXTable_Data->ifHCInMulticastPkts), RxmcastPkt) ;
    mib_update_64_counter(&(pIfXTable_Data->ifHCInBroadcastPkts), RxbcastPkt);

    /** Transmit side **/
    // bytesL = pSwitchMibCounters->ifOutOctetsLow;
    // bytesH = pSwitchMibCounters->ifOutOctetsHigh;
    pIfTable_data->ifOutOctets  +=pSwitchMibCounters->ifOutOctets;

    TxtotalPkt = pSwitchMibCounters->ifOutPkts;
    TxbcastPkt = pSwitchMibCounters->etherStatsTxBroadcastPkts;
    TxmcastPkt = pSwitchMibCounters->etherStatsTxMulitcastPkts;

    pIfTable_data->ifOutUcastPkts   += (TxtotalPkt -(TxbcastPkt + TxmcastPkt ));
    pIfTable_data->ifOutNUcastPkts  += ((TxbcastPkt + TxmcastPkt) - (pSwitchMibCounters->etherPauseTransmitted));
    pIfTable_data->ifOutDiscards    += 0;
    pIfTable_data->ifOutErrors      += (pSwitchMibCounters->etherStatsLateCollisionFrms + pSwitchMibCounters->etherStatsExcessCollisionFrms);
    pIfTable_data->ifOutQLen        +=0;         /**  length of egress queue in packets **/

    /** ifXtable transmit side **/
    pIfXTable_Data->ifOutMulticastPkts = (pSwitchMibCounters->etherStatsTxMulitcastPkts- pSwitchMibCounters->etherPauseTransmitted); // Not supported by Hw
    pIfXTable_Data->ifOutBroadcastPkts = pSwitchMibCounters->etherStatsTxBroadcastPkts; // Not Supported by Hw
    mib_update_64_counter(&(pIfXTable_Data->ifHCOutOctets), pIfTable_data->ifOutOctets );
//    pIfXTable_Data->ifHCOutOctets.high += bytesH;

    mib_update_64_counter(&(pIfXTable_Data->ifHCOutUcastPkts), (TxtotalPkt -(TxbcastPkt + TxmcastPkt )));
    mib_update_64_counter(&(pIfXTable_Data->ifHCOutMulticastPkts),  TxmcastPkt );
    mib_update_64_counter(&(pIfXTable_Data->ifHCOutBroadcastPkts),  TxbcastPkt );

    pIfXTable_Data->ifLinkUpDownTrapEnable = 1; /* Enabled by Default*/
    pIfXTable_Data->old_ifLinkUpDownTrapEnable = 1;

    pIfXTable_Data->ifHighSpeed             = getVPhyLinkSpeed(portIndex);
    pIfXTable_Data->ifPromiscuousMode       = switchGetPortPromiscuousMode(privateData,portIndex);
    pIfXTable_Data->old_ifPromiscuousMode   = switchGetPortOldPromiscuousMode(privateData,portIndex);
    pIfXTable_Data->ifConnectorPresent      = portIndex == 0 ? 1:2;
    //char         ifAlias[MAX_OID_LEN];
    //U32          ifAlias_len;
    //char         old_ifAlias[MAX_OID_LEN];
    //U32          old_ifAlias_len;
    pIfXTable_Data->ifCounterDiscontinuityTime = 0;
}



void mibUpdateRmonStatsTable (PPRIVATE_DATA privateData, int portIndex)
{


    EtherStatsTable_data *pRmonStatsTable;
    SwitchMibCounters  *pSwitchMibCounters;

    SMSC_ASSERT(portIndex < PHY_NUMBER);

    pRmonStatsTable = &(privateData->etherStatsTable_data_array[ portIndex]);
    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    pRmonStatsTable->etherStatsDropEvents           += (pSwitchMibCounters->SwitchBMDropCnt- pSwitchMibCounters->SwitchBMRateDropCnt);
    pRmonStatsTable->etherStatsOctets               +=  pSwitchMibCounters->ifInOctets;
    pRmonStatsTable->etherStatsPkts                 +=  (pSwitchMibCounters->ifInPkts \
    													 + ( pSwitchMibCounters->ifInErrors)\
    													 + (pSwitchMibCounters->etherStatsRxJabbCnt) \
    													 + (pSwitchMibCounters->etherStatsAllignmentErrors) \
    													 + (pSwitchMibCounters->etherStatsOversizePkts) \
    													 + (pSwitchMibCounters->etherStatsRxJabbCnt) \
    													 + (pSwitchMibCounters->etherStatsFragments) \
    													 + (pSwitchMibCounters->etherStatsOversizePkts)
    													    );

    pRmonStatsTable->etherStatsBroadcastPkts        +=  pSwitchMibCounters->etherStatsRxBroadcastPkts;
    pRmonStatsTable->etherStatsMulticastPkts        +=  pSwitchMibCounters->etherStatsRxMulticastPkts;
    pRmonStatsTable->etherStatsCRCAlignErrors       +=  (pSwitchMibCounters->ifInErrors + \
    														pSwitchMibCounters->etherStatsAllignmentErrors );
    pRmonStatsTable->etherStatsUndersizePkts        +=  pSwitchMibCounters->ifOutUnderSize;
    pRmonStatsTable->etherStatsOversizePkts         +=  pSwitchMibCounters->etherStatsOversizePkts;
    pRmonStatsTable->etherStatsFragments            +=  pSwitchMibCounters->etherStatsFragments;
    pRmonStatsTable->etherStatsJabbers              +=  pSwitchMibCounters->etherStatsRxJabbCnt;
    pRmonStatsTable->etherStatsCollisions           +=  pSwitchMibCounters->etherStatsTotCollisionCnt;
    pRmonStatsTable->etherStatsPkts64Octets         +=  pSwitchMibCounters->etherStatsPkts64Octets;
    pRmonStatsTable->etherStatsPkts65to127Octets    +=  pSwitchMibCounters->etherStatsPkts65to127Octets;
    pRmonStatsTable->etherStatsPkts128to255Octets   +=  pSwitchMibCounters->etherStatsPkts128to255Octets;
    pRmonStatsTable->etherStatsPkts256to511Octets   +=  pSwitchMibCounters->etherStatsPkts256to511Octets;
    pRmonStatsTable->etherStatsPkts512to1023Octets  +=  pSwitchMibCounters->etherStatsPkts512to1023Octets;
    pRmonStatsTable->etherStatsPkts1024to1518Octets +=  pSwitchMibCounters->etherStatsPkts1024to1518Octets;
}

void mibUpdateIEEE802_3StatsTable (PPRIVATE_DATA privateData, int portIndex)
{
	IEEE802_3StatsTable_data *pMacStatsTable;
    SwitchMibCounters  *pSwitchMibCounters;

    SMSC_ASSERT(portIndex < PHY_NUMBER);

    pMacStatsTable = &(privateData->MACStatsTable_data_array[portIndex]);
    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    /* MAC_TX_PKTTOK_CNT + (MAC_TX_UNDSIZE_CNT) */
	    pMacStatsTable->aFramesTransmittedOk           			+= ((pSwitchMibCounters->ifOutPkts) +(pSwitchMibCounters->ifOutUnderSize));
	    pMacStatsTable->aSingleCollisionFrames               	+= pSwitchMibCounters->etherStatsSingleCollisionFrms;
	    pMacStatsTable->aMultipleCollisionFrames                += pSwitchMibCounters->etherStatsMultCollisionFrms ;

	    pMacStatsTable->aFramesReceivedOk        				+= pSwitchMibCounters->ifInPkts ;
	    pMacStatsTable->aFramesCheckSequeneErrors        		+=  pSwitchMibCounters->ifInErrors  ;
	    pMacStatsTable->aAlignmnetErrors       					+= pSwitchMibCounters->etherStatsAllignmentErrors ;
	    pMacStatsTable->aOctetsTransmittedOk        			+=  ( (pSwitchMibCounters->ifOutOctets) - (18 *pMacStatsTable->aFramesTransmittedOk) );
	    pMacStatsTable->aFrameswithDefferedXmissions         	+=  pSwitchMibCounters->etherDeferredTransmitted;
	    pMacStatsTable->aLateCollisions            				+=   pSwitchMibCounters->etherStatsLateCollisionFrms;
	    pMacStatsTable->aFramesAbortedDueToXSColls              += pSwitchMibCounters->etherStatsExcessCollisionFrms ;
	    pMacStatsTable->aFramesLostDueToIntMACXmitError         += 0 ;
	    pMacStatsTable->aCarrierSenseErrors         			+= 0;
	    pMacStatsTable->aOctetsRecievedOK    					+= ( (pSwitchMibCounters->ifInOctets) - (18 *pMacStatsTable->aFramesReceivedOk) );
	    pMacStatsTable->aFramesLostDuetoIntMACRcvError   		+=  0;
    	pMacStatsTable->aMulticastFramesXmittedOK   			+=  pSwitchMibCounters->etherStatsTxMulitcastPkts;
    	pMacStatsTable->aBroadcastFramesXmittedOK  				+=  pSwitchMibCounters->etherStatsTxBroadcastPkts;
        pMacStatsTable->aFramesWithExcessiveDefferal 			+= 0 ;
        pMacStatsTable->aMulticastFramesReceivedOK 				+=  pSwitchMibCounters->etherStatsRxMulticastPkts;
        pMacStatsTable->aBroadcastFramesReceivedOK  			+=  pSwitchMibCounters->etherStatsRxBroadcastPkts ;
        pMacStatsTable->alnRangeLengthErrors  					+= 0 ;
        pMacStatsTable->aOutRofRangeLengthField 				+= 0 ;

        pMacStatsTable->aFrameTooLongErrors 					+= ((pSwitchMibCounters->etherStatsOversizePkts) +(pSwitchMibCounters->etherStatsRxJabbCnt))   ;
        pMacStatsTable->aCollisionFrames  						+= 0 ;
        pMacStatsTable->aSQETestErrors 							+=  0;
        pMacStatsTable->aSymbolErrorDuringCarrier  				+= pSwitchMibCounters->etherStatsSymbolErrors ;
        pMacStatsTable->aMACControlFramesTransmitted		    += pSwitchMibCounters->etherPauseTransmitted ;

        pMacStatsTable->aMACControlFramesReceived 				+= pSwitchMibCounters->etherMacControlPkts ;
        pMacStatsTable->aUnsupportedOpcodesReceived  			+= (( pSwitchMibCounters->etherMacControlPkts)- pSwitchMibCounters->etherStatsRxPauseFrms);
        pMacStatsTable->aPAUSEMACCtrlFramesTransmitted 			+=  pSwitchMibCounters->etherPauseTransmitted;
        pMacStatsTable->aPAUSEMACCtrlFramesReceived  			+=  pSwitchMibCounters->etherStatsRxPauseFrms;
}

void mibUpdateBridgeStatsTable (PPRIVATE_DATA privateData, int portIndex)
{
	Bridge_Stats_data *pBridgeStatsTable;
    SwitchMibCounters  *pSwitchMibCounters;
    int i;

    SMSC_ASSERT(portIndex < PHY_NUMBER);
    pBridgeStatsTable = &(privateData->BridgeStatsTable_data_array[portIndex]);

    for(i=0; i< PHY_NUMBER; i++)
    {
    	/* This counter is for the Whole Bridge */
    	pSwitchMibCounters =  &(privateData->switchMibCounter[i]);
    	pBridgeStatsTable->dot1dTpLearnedEntryDiscards += pSwitchMibCounters->SwitchLearnDiscrdCnt ;
    }

    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    pBridgeStatsTable->dot1dBasePortDelayExceededDiscards =0;
    pBridgeStatsTable->dot1dBasePortMtuExceededDiscards += pSwitchMibCounters->etherStatsOversizePkts;

    pBridgeStatsTable->dot1dTpPortInFrames += (pSwitchMibCounters->ifInPkts - pSwitchMibCounters->etherMacControlPkts)  ;
    pBridgeStatsTable->dot1dTpPortOutFrames += ((pSwitchMibCounters->ifOutPkts +  pSwitchMibCounters->ifOutUnderSize ) - pSwitchMibCounters->etherPauseTransmitted);
    pBridgeStatsTable->dot1dTpPortInDiscards += ( pSwitchMibCounters->SwitchFilteredCnt + pSwitchMibCounters->SwitchBMRateDropCnt);
}

void mibUpdateDot3StatsTable (PPRIVATE_DATA privateData, int portIndex)
{
    Dot3StatsTable_data *pDot3StatsTable;
    SwitchMibCounters  *pSwitchMibCounters;
    unsigned int phyLinkSpeed, DuplexState;

    phyLinkSpeed = getVPhyLinkSpeed(portIndex);
    if ( (phyLinkSpeed == LINK_SPEED_10HD ) || (phyLinkSpeed == LINK_SPEED_100HD ) )
    		DuplexState = 2; // Half Duplex
    else if ( (phyLinkSpeed == LINK_SPEED_10FD) || (phyLinkSpeed == LINK_SPEED_100FD))
    		DuplexState = 3; // Full Duplex
    else
    		DuplexState = 1; // Unknown State

    SMSC_ASSERT(portIndex < PHY_NUMBER);

    pDot3StatsTable = &(privateData->dot3StatsTable_data_array[ portIndex]);
    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    pDot3StatsTable->dot3StatsIndex = portIndex;
    pDot3StatsTable->dot3StatsAlignmentErrors +=  pSwitchMibCounters->etherStatsAllignmentErrors ;
    pDot3StatsTable->dot3StatsFCSErrors += pSwitchMibCounters->ifInErrors;
    pDot3StatsTable->dot3StatsSingleCollisionFrames += pSwitchMibCounters->etherStatsSingleCollisionFrms;
    pDot3StatsTable->dot3StatsMultipleCollisionFrames += pSwitchMibCounters->etherStatsMultCollisionFrms ;
    pDot3StatsTable->dot3StatsSQETestErrors += 0;                   /** Not impelemneted yet **/
    pDot3StatsTable->dot3StatsDeferredTransmissions +=
            pSwitchMibCounters->etherDeferredTransmitted;
    pDot3StatsTable->dot3StatsLateCollisions += pSwitchMibCounters->etherStatsLateCollisionFrms;
    pDot3StatsTable->dot3StatsExcessiveCollisions += pSwitchMibCounters->etherStatsExcessCollisionFrms;
    pDot3StatsTable->dot3StatsInternalMacTransmitErrors += 0;       /** Not impelemneted yet **/
    pDot3StatsTable->dot3StatsCarrierSenseErrors += 0;              /** Not impelemneted yet **/
    pDot3StatsTable->dot3StatsFrameTooLongs +=  (pSwitchMibCounters->etherStatsOversizePkts + pSwitchMibCounters->etherStatsRxJabbCnt) ;
    pDot3StatsTable->dot3StatsInternalMacReceiveErrors += 0;        /** Not impelemneted yet **/
    pDot3StatsTable->dot3StatsSymbolErrors += pSwitchMibCounters->etherStatsSymbolErrors;
    pDot3StatsTable->dot3StatsDuplexStatus = DuplexState;
    pDot3StatsTable->dot3StatsRateControlAbility = FALSE;
    pDot3StatsTable->dot3StatsRateControlStatus = 1;
}

void mibUpdateDot3CntrlTable (PPRIVATE_DATA privateData, int portIndex)
{
	Dot3ControlTable_data *pDot3CntrlTable;
	SwitchMibCounters  *pSwitchMibCounters;

    SMSC_ASSERT(portIndex < PHY_NUMBER);

    pDot3CntrlTable = &(privateData->dot3ControlTable_data_array[ portIndex]);
    pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    pDot3CntrlTable->dot3ControlFunctionsSupported = 1;
    pDot3CntrlTable->dot3ControlInUnknownOpcodes =   ( pSwitchMibCounters->etherMacControlPkts - \
    												pSwitchMibCounters->etherStatsRxPauseFrms);
    pDot3CntrlTable->dot3HCControlInUnknownOpcodes.high = 0;
    pDot3CntrlTable->dot3HCControlInUnknownOpcodes.low = 0;  /* Only applicable for Gbits */
}

void mibUpdateDot3PauseTable(PPRIVATE_DATA privateData, int portIndex)
{
	Dot3PauseTable_data *pDot3PauseTbl;
	SwitchMibCounters  *pSwitchMibCounters;
	WORD wRegVal;
	VL_KEY keyCode;
	DWORD dwIntFlags=0;
	unsigned int phyLinkSpeed;
	unsigned int PauseOperMode;

	SMSC_ASSERT(privateData != NULL);
	SMSC_ASSERT(portIndex < PHY_NUMBER);
	keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

	pDot3PauseTbl = &(privateData->dot3PauseTable_data_array[ portIndex]);
	pSwitchMibCounters =  &(privateData->switchMibCounter[portIndex]);

    wRegVal = Phy_GetRegW(privateData,portIndex, PHY_BCR,keyCode );
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);

	if (wRegVal & PHY_BCR_AUTO_NEG_ENABLE_)
		 PauseOperMode = PAUSE_OPER_MODE_ENBLTXANDRX ;
	else
	{

	    phyLinkSpeed = getVPhyLinkSpeed(portIndex);
	    if ( (phyLinkSpeed == LINK_SPEED_10HD ) || (phyLinkSpeed == LINK_SPEED_100HD ) )
	    	PauseOperMode = PAUSE_OPER_MODE_DSBLD; // Half Duplex
	    else if ( (phyLinkSpeed == LINK_SPEED_10FD) || (phyLinkSpeed == LINK_SPEED_100FD))
	    		PauseOperMode = PAUSE_OPER_MODE_ENBLTXANDRX; // Full Duplex
	    else
	    	PauseOperMode = PAUSE_OPER_MODE_DSBLD; // Unknown State
	}

	pDot3PauseTbl->dot3PauseAdminMode = PAUSE_ADMIN_MODE_ENBLTXANDRX;
	pDot3PauseTbl->dot3PauseOperMode = PauseOperMode;
	pDot3PauseTbl->dot3InPauseFrames = pSwitchMibCounters->etherStatsRxPauseFrms;
	pDot3PauseTbl->dot3OutPauseFrames = pSwitchMibCounters->etherPauseTransmitted;
	memset(&pDot3PauseTbl->dot3HCInPauseFrames,0, sizeof(U64) ); /* Applicable only to Gigabit */
	memset(&pDot3PauseTbl->dot3HCOutPauseFrames,0, sizeof(U64) ); /* Applicable only to gigabit */
}

void  mibUpdateMibPortCounters( PPRIVATE_DATA privateData )
{
	int portIndex = 0;
	for (portIndex = 0; portIndex < PHY_NUMBER; portIndex ++)
	{
	    mibReadPortCounters(privateData, portIndex);
	    mibUpdateDot3StatsTable(privateData, portIndex);
	    mibUpdateDot3CntrlTable(privateData, portIndex);
	    mibUpdateDot3PauseTable(privateData, portIndex);
	    mibUpdateIEEE802_3StatsTable(privateData,portIndex);
	    mibUpdateBridgeStatsTable (privateData, portIndex);
	    mibUpdateRmonStatsTable(privateData, portIndex);
	    mibUpdateIfxTable(privateData, portIndex);
	}

// Clear counter before reading it
	for (portIndex = 0; portIndex < PHY_NUMBER; portIndex ++)
	{
		mibClearPortCounter(privateData, portIndex);
	}
}


void MibUpdateMibCounters(unsigned long ptr)
{
    PPRIVATE_DATA privateData=(PPRIVATE_DATA)ptr;
    int portIndex = 0;
    for (portIndex = 0; portIndex < PHY_NUMBER; portIndex ++)
    {
        mibReadPortCounters(privateData, portIndex);
        mibUpdateDot3StatsTable (privateData, portIndex);
        mibUpdateDot3CntrlTable (privateData, portIndex);
        mibUpdateIEEE802_3StatsTable(privateData,portIndex);
        mibUpdateBridgeStatsTable (privateData, portIndex);
        mibUpdateRmonStatsTable(privateData, portIndex);
        mibUpdateIfxTable(privateData, portIndex);
    }

    // Clear counter before reading it
    for (portIndex = 0; portIndex < PHY_NUMBER; portIndex ++)
    {
        mibClearPortCounter(privateData, portIndex);
    }

#ifdef  SNMP_SUPPORT
    // restart timer
    if(!(privateData->StopMibPolling)) {
        privateData->MibPollingTimer.expires=jiffies+mibTimer*HZ;
        add_timer(&(privateData->MibPollingTimer));
    }
#endif
}

void mib_start_timer(PPRIVATE_DATA privateData)
{
#ifdef  SNMP_SUPPORT
	if(mibTimer>0) {
		SMSC_TRACE(" !!!!! Starting MIB timer at interval =%lu!!!!! \n", mibTimer*HZ )
		init_timer(&(privateData->MibPollingTimer));
		privateData->MibPollingTimer.function= MibUpdateMibCounters;
		privateData->MibPollingTimer.data=(unsigned long)privateData;
		privateData->MibPollingTimer.expires=jiffies+mibTimer*HZ;
		add_timer(&(privateData->MibPollingTimer));
	}
#endif
return;
}



//////////////////////////////////
////     SNMP and CLI suport ....END
////////////////////////////////

/////////////////////////////////////
///
///   Spanning Tree Block
///
/////////////////////////////////

#ifdef FULLY_MANAGED

int SwitchSendStpCmd(struct net_device *br,
                    struct net_device *dev,
                    unsigned long cmd,
                    unsigned long arg1,
                    unsigned long arg2)
{

    DeviceProtocol pProto;
    struct sk_buff *skb = NULL;

    SMSC_ASSERT(dev !=NULL);
    SMSC_ASSERT(br !=NULL);

   if ((skb = dev_alloc_skb(sizeof(DeviceProtocol) ) ) == NULL)  {
        printk(KERN_INFO "SMSC931x: memory squeeze!\n");
        return -EINVAL;
    }


    skb->dev = br;
    skb->input_dev = dev;


    skb->protocol = SKB_TYPE_IS_STP;
    atomic_set(&skb->users, SKB_TYPE_IS_STP);

     skb->data=skb->head;
     skb->tail=skb->head;
     skb_put(skb, sizeof(DeviceProtocol) );

     pProto.signature = SMSC931X_DRIVER_SIGNATURE;
     pProto.command   = cmd;
     memcpy(skb->data, &pProto, sizeof(DeviceProtocol) );
     SMSC_TRACE(" Sending Command from %s to Bridge,skb->protocol =%x, Command=%lu ",
               skb->input_dev->name,  skb->protocol, cmd - COMMAND_BASE);
     dev_queue_xmit(skb);
    return 0;
}

void SwitchProcessLinkState(unsigned long data)
{
    unsigned long cmd = COMMAND_PORT_LINK_OFF;
    struct net_device  *br, *switch1,*switch2;
    PSWITCH_PPRIVATE_DATA priv;


    br =__dev_get_by_name("bridge");
    SMSC_ASSERT(thisModulePrivate != NULL);

    switch1 = thisModulePrivate->switchDev[1];
    switch2 = thisModulePrivate->switchDev[2];

    SMSC_ASSERT(switch1 != NULL);
    SMSC_ASSERT(switch2 != NULL);


    /** At least one link has changed  has changed **/

    /**  Check switch port 1 */


    if (thisModulePrivate->SwitchPort1StateChanged)
    {

        priv = (PSWITCH_PPRIVATE_DATA) (switch1->priv);

        SMSC_ASSERT(priv != NULL);

        if ( (br != NULL) &&  (priv->rstpEnabled) )  {

            if (thisModulePrivate->SwitchPort1LinkSpeed ==
                LINK_OFF)
            {
                cmd = COMMAND_PORT_LINK_OFF;
            }
            else
            {
                    cmd = COMMAND_PORT_LINK_ON;
            }

          SwitchSendStpCmd(br,switch1,cmd,0,0);
        }

        thisModulePrivate->SwitchPort1StateChanged = FALSE;
    }
    /**  Check switch port 2 */
  if (thisModulePrivate->SwitchPort2StateChanged) {

        priv =  (PSWITCH_PPRIVATE_DATA) (switch2->priv);

        SMSC_ASSERT(priv != NULL);

        if ( (br!= NULL) &&  (priv->rstpEnabled) )
        {
            if (thisModulePrivate->SwitchPort2LinkSpeed ==
                LINK_OFF)
            {
                cmd = COMMAND_PORT_LINK_OFF;
            }
            else
            {
                cmd = COMMAND_PORT_LINK_ON;
            }

          SwitchSendStpCmd(br,switch2,cmd,0,0);
        }

        thisModulePrivate->SwitchPort2StateChanged = FALSE;
   }

}

void  Phy_UpdateSwitchLinkMode(PPRIVATE_DATA privateData)
{
   DWORD dwIntFlags = 0;
   DWORD switchPort1Link, switchPort2Link;
//   static DWORD switchPort1Link, switchPort2Link;
   VL_KEY keyCode;
   DWORD SwitchPortPhyIndex = privateData->dwPhyAddress +1;

 //  printk("Inside the Phy_UpdateSwitchLinkMode\n");

   SMSC_ASSERT(privateData != NULL);

   keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

    switchPort1Link = Phy_GetLinkMode(
                        privateData,
                        SwitchPortPhyIndex,
                        keyCode
                        );
    switchPort2Link = Phy_GetLinkMode(
                        privateData,
                        (SwitchPortPhyIndex +1),
                        keyCode
                        );
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);

    if (switchPort1Link != privateData->SwitchPort1LinkSpeed)
    {
        if ( switchPort1Link == LINK_OFF)
        {
            /** Link becomes off **/
            privateData->SwitchPort1StateChanged = TRUE;
        }
        else if ( privateData->SwitchPort1LinkSpeed == LINK_OFF)
        {
            /** Link becomes on**/
            privateData->SwitchPort1StateChanged = TRUE;
        }
       privateData->SwitchPort1LinkSpeed = switchPort1Link;

    }

    if (switchPort2Link != privateData->SwitchPort2LinkSpeed)
    {
        if ( switchPort2Link == LINK_OFF) {
            /** Link becomes off **/
            privateData->SwitchPort2StateChanged = TRUE;
        }
        else if ( privateData->SwitchPort2LinkSpeed == LINK_OFF)
        {
            /** Link becomes on**/
            privateData->SwitchPort2StateChanged = TRUE;
        }

       privateData->SwitchPort2LinkSpeed = switchPort2Link;

    }



    if ( privateData->SwitchPort1StateChanged ||
         privateData->SwitchPort2StateChanged )
    {
        tasklet_schedule(&STP_LINK_Tasklet);
    }


}


void Phy_CheckSwitchLink(unsigned long ptr)
{
    PPRIVATE_DATA privateData=(PPRIVATE_DATA)ptr;
    if(privateData==NULL) {
        SMSC_WARNING("Phy_CheckSwitchLink(ptr==0)");
        return;
    }

   // SMSC_TRACE(":: ----------> ");
    Phy_UpdateSwitchLinkMode(privateData);


    if(!(privateData->StopSwitchLinkPolling)) {
        privateData->SwitchLinkPollingTimer.expires=jiffies+ HZ;
        add_timer(&(privateData->SwitchLinkPollingTimer));
    }

   // SMSC_TRACE(":: <--------------- ");
}

void  StartSwitchLinkTimer(PPRIVATE_DATA privateData)
{
    init_timer(&(privateData->SwitchLinkPollingTimer));
    privateData->SwitchLinkPollingTimer.function=Phy_CheckSwitchLink;
    privateData->SwitchLinkPollingTimer.data=(unsigned long)privateData;
    privateData->SwitchLinkPollingTimer.expires=jiffies+ HZ;
    if ( !privateData->StopSwitchLinkPolling)
        add_timer(&(privateData->SwitchLinkPollingTimer));

}

void StopSwitchLinkTimer(PPRIVATE_DATA pMaster)
{
    SMSC_ASSERT(pMaster !=NULL );
    pMaster->StopSwitchLinkPolling = TRUE;
}

/* This function adds all the 16 Multicast addreses to the switch table and enable the filter, some
 * of the Bridge Group address are enabled after */
void EnblPermanentDB(PPRIVATE_DATA privateData)
{
	DWORD MacLo,MacHi;
	int i;
	SMSC_ASSERT(privateData !=NULL );
	for (i=0; i<16; i++) {
		MacLo = MAKEDWORD (MAKEWORD(RSVD_MCAST_ADDRESS[i][3],RSVD_MCAST_ADDRESS[i][2]), (DWORD)MAKEWORD(RSVD_MCAST_ADDRESS[i][1], RSVD_MCAST_ADDRESS[i][0]));
  	  	MacHi = MAKEDWORD (MAKEWORD(RSVD_MCAST_ADDRESS[i][5],RSVD_MCAST_ADDRESS[i][4]), 0);
  	  	Switch_Enbl_Fltr_on_ALR(privateData, MacLo, MacHi); // forward to Port1 & Port2
	}
}

void EnableSpanningTree(PSWITCH_PPRIVATE_DATA priv)
{
    PPRIVATE_DATA pMaster;
    ALREntryProp entryProp={0};
    SMSC_ASSERT(priv != NULL);
    priv->rstpEnabled = TRUE;
    pMaster = priv->pMaster;
    SMSC_ASSERT(pMaster != NULL);
    if (!pMaster->stpAlrSet)
    {
        SMSC_TRACE(" SPANNING  TREE IS ENABLED");
        entryProp.port =0;
        entryProp.age_alias_oride=1;
#ifdef CHIP_IS_LAN9303
        entryProp.pri_alia_tag = CPU_TAG_STP_ID;
#endif
        Switch_Add_Static_ALR(pMaster, RSTP_MAC_ADDRESS_LO, RSTP_MAC_ADDRESS_HI, &entryProp);
        pMaster->stpAlrSet = TRUE;
        StartSwitchLinkTimer(pMaster);
    }
    pMaster->rstpEnabled = TRUE;
}

void DisableSpanningTree(PSWITCH_PPRIVATE_DATA priv)
{
    PPRIVATE_DATA pMaster;
    SMSC_ASSERT(priv != NULL);

    priv->rstpEnabled = FALSE;
    pMaster = priv->pMaster;
    SMSC_ASSERT(pMaster != NULL);

    if (pMaster->stpAlrSet)
    {
        Switch_remove_Static_ALR(pMaster, RSTP_MAC_ADDRESS_LO, RSTP_MAC_ADDRESS_HI, 0);
        pMaster->stpAlrSet = FALSE;
        StopSwitchLinkTimer(pMaster);
    }
    priv->rstpState   = STP_FORWARDING ;

}

void STP_Set_Port_State(PSWITCH_PPRIVATE_DATA priv ,PORT_STATE state)
{
    SWITCH_PORT_STATE portState = SWITCH_PORT_INVALID_STATE;
    BOOLEAN   setState= TRUE;

    SMSC_ASSERT(priv != NULL);
    SMSC_ASSERT(priv->pMaster != NULL);
    switch (state)
    {
        case STP_BLOCKING:
            portState = SWITCH_PORT_BLOCKING;
            break;
        case STP_LISTENING:
            portState = SWITCH_PORT_LISTENING;
            break;
        case STP_LEARNING:
            portState = SWITCH_PORT_LEARNING;
            break;
        case STP_FORWARDING:
            portState = SWITCH_PORT_FORWARDING;
            break;
#ifndef CHIP_IS_LAN9303
        case STP_DISABLED:
            SMSC_TRACE("IMPLEMENT SET PORT DISABLE" );
            priv->Disabled = TRUE;
            setState = FALSE;
            break;
#else
        case STP_DISABLED:
        	portState = SWITCH_PORT_DISABLED;
            break;
#endif
        case STP_ENABLED:
            priv->Disabled = FALSE;
            setState = FALSE;
            break;
        default :
            SMSC_ERROR("INVALID SWITCH PORT STATE" );
            setState = FALSE;
        break;

    }
    priv->rstpState = state;

    if (setState)
        Switch_Set_port_state(priv->pMaster, priv->PortNumber, portState);
}

#endif
/**
/////////////////////////////////////
///
///   Spanning Tree Block   END
///
/////////////////////////////////
 */



#ifdef FULLY_MANAGED




/***** VLAN CHECK******/

int isVLANPacket(char *data ,int len, int *proto )
{
    WORD protocol;
   char *p = data;
    int VLANPri;
    p+=12;
    protocol = htons(*((WORD*)p));
       if (protocol == VLAN_TYPE)
       {
    	   p+=2;
    	   VLANPri=htons(*((WORD*)p));
    	   *proto=VLAN_TYPE;
    	   return (VLANPri&0xE000);
        }
     *proto= 0;
     return 0;
}
/**
/////////////////////////////////////
///
///   IGMP Start
///
/////////////////////////////////
 */


int isIGMPPacket(char *data ,int len )
{
	WORD protocol;
	char *p = data;
	int tempVLAN;
	p+=12;
	protocol = htons(*((WORD*)p));
	if ( (protocol == IP_PROTO_TYPE) || (protocol == VLAN_TYPE) ) {
		if (protocol == VLAN_TYPE) {
			p+=2;
			tempVLAN=htons(*((WORD*)p));
			p+=2;
			protocol = htons(*((WORD*)p));
			if (protocol != IP_PROTO_TYPE)
			{
				SMSC_TRACE("%x\n",tempVLAN);
				return tempVLAN;
			}
		}
		/** It's an IP Protocol  */
		p+= 11;
		if (*p == 0x2)
		{
			p+= 11;
			if (*p == 0x11) {
				//	SMSC_TRACE("query");
				return 2;	/* Query*/
			}
			else return 1; /* may be report or Leave group*/
		}
	}
	return FALSE;

}
#ifdef CHIP_IS_LAN9303
/* **************************************************************************************************************
 * **************************************************************************************************************
 * function name: isIPV6Pkt
 * Input: pkt: is a buffer pointing to skb->data
 * 	      len: specifies the skb->len
 * return: type of the PKT (NOT_AN_IPV6PKT, IPV6_NO_ICMP_BUT_RTR_ALRT_MLD_PKT,
 * 							IPV6_MLD_PKT,IPV6_BUT_NOT_AN_MLD_PKT )
 * Note:
 * This function decodes an in-coming packet and identifies as an IPV6 MLD ,IGMP packet, IPV6 options
 * are loosely defined in the hardware, the packets can be in the following format
 * IPV6 base header + Hop-by-Hop option + Routing + Fragment + Destination Options + Authentication +
 * Encapsulated Security Payload + data
 *
 * ***********************************************************************************************************/
int isIPV6Pkt(char* pkt, unsigned int skb_ptktLen)
{
		WORD protocol;
		struct ipv6hdr *ipv6;
		struct ipv6_opt_hdr *ipv6OptHdr;
		char* data;
		unsigned long totPktLen=0;
		TLV *tlv;
		MLDPKTTypes pktType = NOT_AN_IPV6PKT;
		SMSC_ASSERT(pkt);
		data = pkt + (2*ETH_ADDR_SIZE);
		protocol = htons(*((WORD*)data));
		totPktLen  += (2*ETH_ADDR_SIZE);
		SMSC_TRACE("ProtocolType =0x%x\n", protocol);
		if( (protocol == IPV6_PROTO_TYPE) || (protocol==VLAN_TYPE) ) {
			if(protocol == VLAN_TYPE)
			{
				data +=4; /* Increase the VLAN TAG */
				protocol = htons(*((WORD*)data));
				totPktLen +=4;
				if(protocol !=IPV6_PROTO_TYPE)
				{
					SMSC_TRACE("NOT an IPV6 PROTO Type%x\n", protocol);
					return NOT_AN_IPV6PKT;
				}
			}

			data += SIZE_OF_ETH_TYPE;
			totPktLen +=2;
			ipv6 = (struct ipv6hdr *) data;
			if( (ipv6->hop_limit > 1) &  (ipv6HoplimitValidate) )
			{
				SMSC_TRACE("HOP LIMIT is greater than one in IPV6 Pkt Marking as an NOT AN MLD PKT \n");
				goto NOTANMLDPKT;
			}
			if(ipv6->nexthdr != HOP_BY_HOP_OPTION)
			{

				SMSC_TRACE("NO HOP BY HOP HEADER in IPV6 Pkt, Marking as NOT AN MLD PKT \n");
				goto NOTANMLDPKT;
			}

			data += sizeof(struct ipv6hdr);
			ipv6OptHdr = (struct ipv6_opt_hdr *)data;
			totPktLen += sizeof(struct ipv6hdr) ;

			do {
					ipv6OptHdr = (struct ipv6_opt_hdr *)data;
					tlv = (TLV*)(data+2);

					if( (tlv->type = ROUTER_ALRT_ID )&&  (tlv->value == TLV_MLD_HDR_VAL))
					{
						SMSC_TRACE("AN IPV6 RTR ALERT MLD Packet\n")
						pktType=  IPV6_NO_ICMP_BUT_RTR_ALRT_MLD_PKT;
					}
					/* This is a redundand checking to absolutely make sure that it is an MLD packet */
					if( (pktType == IPV6_NO_ICMP_BUT_RTR_ALRT_MLD_PKT) && (ipv6OptHdr->nexthdr == ICMPV6_HDR_ID) )
					{
						SMSC_TRACE("AN IPV6 MLD PACKET\n");
						return IPV6_MLD_PKT;
					}
					switch(ipv6OptHdr->nexthdr)
					{
						case NEXT_HDR_IS_TCP:
						case NEXT_HDR_IS_UDP:
							goto NOTANMLDPKT;
						default:
							break;
					}

					data  +=( (ipv6OptHdr->hdrlen * OPT_HDR_OFFSET) + \
													OPT_HDR_OFFSET);
					totPktLen += ( (ipv6OptHdr->hdrlen * OPT_HDR_OFFSET) + OPT_HDR_OFFSET);
					SMSC_TRACE("Inside the IPV6 LOOP \n");
					if(totPktLen > skb_ptktLen)
						goto NOTANMLDPKT;
				}while( (ipv6OptHdr->nexthdr != NO_NEXT_HDR) && (totPktLen < 1518) );
				SMSC_TRACE("Not an IPV6Pkt protocol =0x%x\n", protocol);
					/* Not a proper Identification, may not be an MLD packet */
				goto NOTANMLDPKT;
		}
		else
		{
			SMSC_TRACE("Not an IPV6Pkt protocol =0x%x\n", protocol);
			return NOT_AN_IPV6PKT;
		}

NOTANMLDPKT:
   return IPV6_BUT_NOT_AN_MLD_PKT;
}
#endif

int IgmpMgrTransmitData(PPRIVATE_DATA privateData, SNOOP_CONFIG_MSG_T *igmpMsg)
{
    struct nlmsghdr *nlh = NULL;
    int ret = 0;
    int dataSize = 0;
    void	*data;
    struct sk_buff	*skb;
    int len = 0;

    SMSC_ASSERT(privateData);
    SMSC_ASSERT(igmpMsg);

    dataSize = sizeof(SNOOP_CONFIG_MSG_T);  /* payload*/
    len = NLMSG_SPACE(dataSize);           /* Msg Hdr plud payload */
    skb = alloc_skb(len, GFP_KERNEL);

    if (skb)
    {
		nlh = NLMSG_PUT(skb, 0,0,igmpMsg->header.cmd_type, len - sizeof(*nlh));
		data = NLMSG_DATA(nlh);
        NETLINK_CB(skb).dst_group = 0;  /* unicast */

        memcpy(data, igmpMsg, dataSize );
        ret = netlink_unicast(privateData->igmp_nls, skb, privateData->igmp_deamon_pid,MSG_DONTWAIT );

        if (ret <= 0 ) {
        	kfree_skb(skb);
        	SMSC_ERROR ("Error Sending Net Link Message");
        	goto nlmsg_failure;
        }

        SMSC_TRACE("DRV to PID %d dataSize=%d, MSG Size=%d, size Tx = %d",
                   privateData->igmp_deamon_pid, dataSize, len, ret);
     }
nlmsg_failure:
    kfree (igmpMsg);
    return 0;
}

static void sendSnoopedMsgToManager( PPRIVATE_DATA privateData, const char *data, DWORD datalen, PACKET_TYPE pktType, int portNumber)
{
	SNOOP_CONFIG_MSG_T *snpMsg;
    DWORD len;
    if (privateData->igmp_deamon_pid == 0) {
        SMSC_WARNING(" No PID to sent IGMP MSG to");
        return;
    }
    snpMsg =  kmalloc(sizeof(SNOOP_CONFIG_MSG_T),GFP_KERNEL);
    if ( snpMsg ) {
        len =  (datalen > MAX_PROTO_LEN) ? MAX_PROTO_LEN : datalen;
        memset (snpMsg, 0, sizeof (SNOOP_CONFIG_MSG_T) ) ;
        snpMsg->header.cmd_type= pktType;
        snpMsg->header.sender_pid= privateData->igmp_deamon_pid ;
        snpMsg->header.port= portNumber;  /**Assuming comming from port 1 **/
        printk("%s , portNumber= %d", __func__, snpMsg->header.port);
        memcpy (snpMsg->body.protoMsg, data, len);
        snpMsg->header.msg_len = len;
        /** we are sending only 4 bytes of data**/
        IgmpMgrTransmitData( privateData, snpMsg);
    }
}

/** **********************************************
 *    igmp_rx_msg_ready is called whenever a message
 *    is waiting on netlink (send by IGMP manager)
 *    It send a signal to IGMP Thread;
 *
 */

void igmp_rx_msg_ready (struct sock *sk, int len)
{
    if (thisModulePrivate != NULL) {
  //      SMSC_TRACE("NetLink MSG Ready");
        if (thisModulePrivate->igmp_thread )
            complete(&(thisModulePrivate->igmp_ready));
    }
}
/** **********************************************
 *    igmpProcessMsg is the thread using netLink.
 *    It waits for Messages from IGMP manager (User Space)
 *    for processing: Commands or IGMP Query messages
 *
 *     It also waits for IGMP Stop/Disable
 */
/** **********************************************
 *    igmpProcessMsg is the thread using netLink.
 *    It waits for Messages from IGMP manager (User Space)
 *    for processing: Commands or IGMP Query messages
 *
 *     It also waits for IGMP Stop/Disable
 */
int  igmpProcessMsg(void *data)
{
    struct sk_buff *skb = NULL;
    struct nlmsghdr *nlh = NULL;
    SNOOP_CONFIG_MSG_T *IGMPMsg;

    int err=0/*,i=0*/;


    PPRIVATE_DATA  privateData = (PPRIVATE_DATA) (data);

    if ( privateData != NULL)
	{
	privateData->igmp_thread = current;
	privateData->igmp_nls= netlink_kernel_create(NETLINK_IGMP_MSG,1,
                igmp_rx_msg_ready, THIS_MODULE);
        if ( privateData->igmp_nls == NULL)
	{
            SMSC_ERROR("Unable to create netlink socket ");
            goto ERROR;
        }
        SMSC_TRACE("NetLink Socket Created");

        while  (1)
	{
            /** Block until a completion MSG is sent (same as semaphore)*/
            wait_for_completion(&(privateData->igmp_ready));
            if (!privateData->SnoopingStarted)
            break;
		 /*          SMSC_TRACE("IGMP NetLink Socket Activity.... ");*/
            skb = skb_recv_datagram( privateData->igmp_nls, 0, 0, &err);
		if (!err)
		{
			nlh = (struct nlmsghdr *)skb->data;
        	        privateData->igmp_deamon_pid = nlh->nlmsg_pid;

        	        printk("Got the nlmsgpid = %d\n",privateData->igmp_deamon_pid);
        	        IGMPMsg =  ( SNOOP_CONFIG_MSG_T *) NLMSG_DATA(nlh);
			if(IGMPMsg->header.cmd_type==MLD_FORWARD_MSG)
			{
			//skb->mac.raw=skb->data;
			skb->len = IGMPMsg->header.msg_len;
			skb->dev=privateData->switchDev[IGMPMsg->header.port];
			skb->data += NLMSG_LENGTH(0)+ sizeof(SNOOP_MSG_HEADER_T);
			skb->priority=0;
 		//	SMSC_TRACE("IGMPMsg->header.port %d\n",IGMPMsg->header.port);
			dev_queue_xmit(skb);
			}
			else
			{
			dev_kfree_skb(skb);
             		/**  Free  SKB**/
			}
		}
     	}
   ERROR:
        SMSC_ERROR("NetLink Thread  EXIT");
	kill_proc(privateData->igmp_thread_pid, SIGTERM, 1);
	privateData->igmp_thread_pid  = 0;
	privateData->igmp_thread = NULL;
	if ( privateData->nls) {
        sock_release(privateData->nls->sk_socket);
        privateData->nls = NULL;
    }
}
	return 0;

}
void SwitchEnableIGMP(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** kernelGlobal Ingress Conf. reg*/

    if (privateData->SnoopingStarted) {
        SMSC_WARNING("IGMP Already started");
        return;
    }

    // Take sem for Writing
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------kernel------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Enable IGMP");
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Global Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
    /**  Enable IGMP Snooping and Disable Forwarding Mcast packets
     *  From here, IGMP manager will set Mcast in ALR.
     *  This will be based on Membership report received
     **/
#ifndef CHIP_IS_LAN9303
    cigr |=  SE_RX_CFG_IGMP_MII_EN_|SE_RX_CFG_IGMP_EN_ | SE_RX_CFG_FLTR_MCST_;
#else
    cigr |= SE_RX_CFG_SNOOP_ECHO_EN_| SE_RX_CFG_IGMP_MII_EN_|SE_RX_CFG_MLD_EN_ | SE_RX_CFG_IGMP_EN_ | SE_RX_CFG_FLTR_MCST_;
#endif
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    /** Start IGMP Thread **/
    privateData->SnoopingStarted = TRUE;
    init_completion(&(privateData->igmp_ready));
    privateData->igmp_thread_pid = kernel_thread(igmpProcessMsg, (void *)(privateData), 0);

DONE:
        releaseSwitchWriteLock(privateData);
    SMSC_TRACE("<----------------------  privateData->igmp_thread_pid %d", privateData->igmp_thread_pid);
    return;
}
void SwitchDisableIGMP(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** Global Ingress Conf. reg*/

    // Take sem for reading
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;

    SMSC_ERROR("ALR Busy, Unable to Enable IGMP");
    releaseSwitchWriteLock(privateData);
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Glfaultobal Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
    /**  Disable IGMP Snooping and Enable Forwarding Mcast packets
     **/
#ifndef CHIP_IS_LAN9303
    cigr &= ~(SE_RX_CFG_IGMP_EN_  | SE_RX_CFG_FLTR_MCST_);
#else
    cigr &= ~(SE_RX_CFG_SNOOP_ECHO_EN_|SE_RX_CFG_IGMP_EN_ | SE_RX_CFG_MLD_EN_ | SE_RX_CFG_FLTR_MCST_);
#endif
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    releaseSwitchWriteLock(privateData);
    /** Now Kill the  thread **/
    privateData->SnoopingStarted = FALSE;
    complete(&(privateData->igmp_ready));

DONE:

        SMSC_TRACE("<----------------------");
    return;
}
#ifdef CHIP_IS_LAN9303
void SwitchEnableMLD(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** kernelGlobal Ingress Conf. reg*/

    if (privateData->SnoopingStarted) {
        SMSC_WARNING("IGMP Already started");
        return;
    }

    // Take sem for Writing
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------kernel------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Enable IGMP");
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Global Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
    /**  Enable IGMP Snooping and Disable Forwarding Mcast packets
     *  From here, IGMP manager will set Mcast in ALR.
     *  This will be based on Membership report received
     **/

    cigr |= (SE_RX_CFG_SNOOP_ECHO_EN_ | SE_RX_CFG_MLD_EN_ | SE_RX_CFG_FLTR_MCST_);

    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    /** Start IGMP Thread **/
    privateData->SnoopingStarted = TRUE;
    init_completion(&(privateData->igmp_ready));
    privateData->igmp_thread_pid = kernel_thread(igmpProcessMsg, (void *)(privateData), 0);

DONE:
        releaseSwitchWriteLock(privateData);
    SMSC_TRACE("<----------------------  privateData->igmp_thread_pid %d", privateData->igmp_thread_pid);
    return;
}
void SwitchDisableMLD(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** Global Ingress Conf. reg*/

    // Take sem for reading
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;

    SMSC_ERROR("ALR Busy, Unable to Enable IGMP");
    releaseSwitchWriteLock(privateData);
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Glfaultobal Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
    /**  Disable IGMP Snooping and Enable Forwarding Mcast packets
     **/

    cigr &= ~(SE_RX_CFG_SNOOP_ECHO_EN_|SE_RX_CFG_MLD_EN_ | SE_RX_CFG_FLTR_MCST_);

    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    releaseSwitchWriteLock(privateData);
    /** Now Kill the  thread **/
    privateData->SnoopingStarted = FALSE;
    complete(&(privateData->igmp_ready));

DONE:

        SMSC_TRACE("<----------------------");
    return;
}
#endif

#endif  /** FULLY_MANAGED **/
void SwitchEnableDAPriority(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** kernelGlobal Ingress Conf. reg*/

    if (privateData->DAHighPriorityEnbl) {
        SMSC_WARNING("DAHighPriority is Already Enabled");
        return;
    }

    // Take sem for Writing
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------kernel------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Enable DAHighPriority at SE_GLOBAL_INGRSS_CFG");
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Global Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
	/* Enable the DA High Priority */
    cigr |= SE_RX_CFG_DA_H_PRI_;
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    privateData->DAHighPriorityEnbl= TRUE;

DONE:
        releaseSwitchWriteLock(privateData);
    SMSC_TRACE("<----------------------  DAHighPriority is Enabled at SE_GLOBAL_INGRSS_CFG");
    return;
}

void SwitchDisableDAPriority(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** kernelGlobal Ingress Conf. reg*/


    // Take sem for Writing
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------kernel------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Enable DAHighPriority at SE_GLOBAL_INGRSS_CFG");
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Global Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
	/* Enable the DA High Priority */
    cigr &= ~(SE_RX_CFG_DA_H_PRI_);
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    privateData->DAHighPriorityEnbl= FALSE;

DONE:
        releaseSwitchWriteLock(privateData);
    SMSC_TRACE("<----------------------  DAHighPriority is DISABLED at SE_GLOBAL_INGRSS_CFG");
    return;
}

void SwitchEnableVLAN(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** kernelGlobal Ingress Conf. reg*/

    if (privateData->VLANEnable) {
        SMSC_WARNING("VLAN Already Enabled");
        return;
    }

    // Take sem for Writing
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------kernel------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;


    SMSC_ERROR("ALR Busy, Unable to Enable VLAN");
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");

    /**  Read Global Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
#ifndef CHIP_IS_LAN9303
	/* Enable the VLAN */
    cigr |= SE_RX_CFG_VLAN_EN_;
#else
    cigr |=(SE_RX_CFG_USE_TAG_EN_| SE_RX_CFG_VLAN_EN_);
#endif
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    privateData->VLANEnable= TRUE;

DONE:
        releaseSwitchWriteLock(privateData);
    SMSC_TRACE("<----------------------  VLAN Enabled");
    return;
}

void SwitchDisableVLAN(PPRIVATE_DATA privateData)
{
    DWORD cigr = 0 ; /** Global Ingress Conf. reg*/

    // Take sem for reading
    getSwitchWriteLock(privateData);
    SMSC_TRACE("---------------------->");
    if ( SwitchALRNotBusy(privateData))
        goto ALR_READY;

    SMSC_ERROR("ALR Busy, Unable to Enable IGMP");
    releaseSwitchWriteLock(privateData);
    goto DONE;
ALR_READY:
        SMSC_TRACE("--------------->, ------------");
    /**  Read Glfaultobal Ingress Config register**/
    cigr = Switch_GetRegDW(privateData,SE_GLOBAL_INGRSS_CFG);
    /**  Disable IGMP VLAN
     **/
#ifndef CHIP_IS_LAN9303
    cigr &= ~(SE_RX_CFG_VLAN_EN_);
#else
    cigr &= ~(SE_RX_CFG_VLAN_EN_ | SE_RX_CFG_USE_TAG_EN_);
#endif
    Switch_SetRegDW(privateData,SE_GLOBAL_INGRSS_CFG,cigr);
    releaseSwitchWriteLock(privateData);
    /** Now Kill the  thread **/

DONE:

        SMSC_TRACE("<----------------------");
    return;
}
#ifdef CHIP_IS_LAN9303
void EnblTurboMode(PPRIVATE_DATA privateData, int portNo)
{

	DWORD vphyVal=0;
	SMSC_TRACE("---------------------->");
	/* Turbo mode is supported for only port 0 and port 1 */
		if(portNo == VirtualPhyIndex)
		{
			vphyVal =LanGetRegDW(VPHY_SPEC_CNTRL_STAT);
			vphyVal |= VPHY_TURBO_MII_ENBL;
			LanSetRegDW(VPHY_SPEC_CNTRL_STAT, vphyVal);
		}
		else if(portNo == 1)
		{
			vphyVal =LanGetRegDW(P1_MII_BASIC_CONTROL);
			vphyVal |= P1_MII_TURBO_MII_ENBL;
			LanSetRegDW(P1_MII_BASIC_CONTROL, vphyVal);
		}
		else
		{
			SMSC_ERROR("Invalid Port No. for Turbo Mode setting");
		}
		 SMSC_TRACE("<----------------------");
}


void DisblTurboMode(PPRIVATE_DATA privateData, int portNo)
{

	DWORD vphyVal=0;
	SMSC_TRACE("---------------------->");
	/* Turbo mode is supported for only port 0 and port 1 */
		if(portNo == VirtualPhyIndex)
		{
			vphyVal =LanGetRegDW(VPHY_SPEC_CNTRL_STAT);
			vphyVal &= ~(VPHY_TURBO_MII_ENBL);
			LanSetRegDW(VPHY_SPEC_CNTRL_STAT, vphyVal);
		}
		else if(portNo == 1)
		{
			vphyVal =LanGetRegDW(P1_MII_BASIC_CONTROL);
			vphyVal &= ~(P1_MII_TURBO_MII_ENBL);
			LanSetRegDW(P1_MII_BASIC_CONTROL, vphyVal);
		}
		else
		{
			SMSC_ERROR("Invalid Port No. for Turbo Mode setting");
		}
		 SMSC_TRACE("<----------------------");

}
#endif

int getVPhyLinkSpeed(int portId)
{
	struct net_device *dev = &SMSC931X;
	DWORD switchPortLinkSpeed;
	DWORD dwIntFlags=0;
    PPRIVATE_DATA privateData = dev->priv;
    VL_KEY keyCode;
    SMSC_ASSERT(privateData != NULL);
    keyCode=Vl_WaitForLock(&(privateData->MacPhyLock),&dwIntFlags);

    switchPortLinkSpeed = Phy_GetLinkMode(
                          privateData,
                          portId,
                          keyCode
                          );
    Vl_ReleaseLock(&(privateData->MacPhyLock),keyCode,&dwIntFlags);
//    printk("switchPortLinkSpeed = %d",switchPortLinkSpeed );
    return switchPortLinkSpeed;

}

EXPORT_SYMBOL(getVPhyLinkSpeed);
#ifndef USING_LINT
module_init(Smsc9312_init_module);
module_exit(Smsc9312_cleanup_module);
#endif


